/* thorn_BAM_Elliptic: Multigrid Elliptic Solver for Cactus
   Copyright (C) 1998 Bernd Bruegmann 
*/
/* BAM_Elliptic_Solve.c */
/* Bernd Bruegmann, 9/97 */

#include "Symmetry.h"
#include "bbmg.h"


#define forallGHc(cctkGH,GH,GHc) \
  for (GHc = GHbyLevel(GH->convlevel,0,0); GHc;       \
       GHc = GHbyLevel(GH->convlevel,GHc->level+1,0)) \

void BAM_Ell_LinMetricSolver(cGH *GH, int *Metric, int FieldIndex, 
               int MIndex, int NIndex, CCTK_REAL *AbsTol, CCTK_REAL *RelTol);
void BAM_Ell_LinConfMetricSolver(cGH *GH, int *MetricPsi, int FieldIndex, 
               int MIndex, int NIndex, CCTK_REAL *AbsTol, CCTK_REAL *RelTol);
void BAM_Elliptic_Linear(cGH *cctkGH, int nu, int nM, int nN, CCTK_REAL tol);
void BAM_Elliptic_VecL(cGH *cctkGH, int *nu, int *nN, CCTK_REAL tol);
void BAM_Elliptic_Vec4(cGH *cctkGH, int *nu, int nnu, CCTK_REAL tol);
void BAM_Maximal_Slicing(cGH *cctkGH, pGH *GH, CCTK_REAL tol);
void BAM_Elliptic_Solve(cGH *cctkGH, pGH *GH, int *nf, int n, int bam_eqn_type,
			CCTK_REAL tol);
void BAM_Init_Grid(cGH *cctkGH);
void flipsign(pGH *GH, int nGF);
void limitations(cGH *cctkGH, pGH *GH); 
void checklinsize(int n, int p);
int adjustlinsize(cGH *cctkGH, int dir, int n, int o);

int cartoon, cartoon_stencil, stencil_symmetric, fisheye;
double fisheyedx, fisheyedy, fisheyedz;
int eqn_type, problem, vb, vbell;
int ox, oy, oz, oxdo, oydo, ozdo, oxup, oyup, ozup;


#define CONFORMAL_METRIC 1

/***************************************************************************/
/* wrapper for initial data */
void BAM_Initial_Data(cGH *cctkGH, pGH *GH) 
{
  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS
  pGH *GHc;
#if 0
  char *s = "model";
  char *l = "----------------------------------------------------------------";
  char *coord[3] = {"x", "y", "z"};
  char t[1024];
#endif
  int i, j, nu, nM, nN, nf[MGFMAX];

  /* BAM or POOF */
#ifdef FIXME
  if (!CCTK_Equals(initial_data, "BAM"))
    return;
#endif
 
  /* problem types */ 
  problem = eqn_type = UNKNOWN;

  /* cactus 4 casualty */
  initGHList(GH);

  /* cactus 4: conformal state is a grid scalar which is a pointer for C */
  /* initialize to non-conformal here for conformal initial data */
  *conformal_state = 0; 
  conformal_metric = 0;

  /* cactus4: check indexing */
#if 0
printf("--------- varindex %d\n", CCTK_VarIndex("bam_elliptic::bam_wsmin"));
printf("--------- varindex %d\n", CCTK_VarIndex("bam_elliptic::bam_ws1"));
printf("--------- varindex %d\n", CCTK_VarIndex("bam_elliptic::bam_wsmax"));
printf("--------- group index %d\n", CCTK_GroupIndex("bam_elliptic::bamwork"));
#endif

  /* make sure we have a valid cactus 
     e.g. if model = "BAM Brill", flat is not called despite what rfr says
     3.0: now it sets flat space
     4.0: now it does not
     --> set it in brbr.c
  */

  /* puncture data */
  for (i = 0; i < N_MWBL; i++) {
    MBL[i] = 0; 
    for (j = 0; j < 3; j++) 
      CBL[i][j] = PBL[i][j] = SBL[i][j] = 0;
  }
  if (CCTK_Equals(initial_data, "BrBr")) {
    int Knonzero = 0;
    CCTK_INFO("Computing black hole initial data (Brandt-Bruegmann)");

    vb = (brbr_verbose == 1) || CCTK_Equals(elliptic_verbose, "yes");
    vbell = 1;
    problem = BHDATA;
    eqn_type = BRBR;
    setnames_brbr();

#if 0
    /* ARGH, cactus 4.0 no longer allows parameter access by string-name */
    /* let's keep this until parameter arrays come along */
    for (i = 0; i < N_MWBL; i++) {
      sprintf(t, "bhm%d", i+1);
      MBL[i] = Getr8(t);
      if (MBL[i] == 0.0)
	break;
      for (j = 0; j < 3; j++) {
	sprintf(t, "bh%s%d", coord[j], i+1);
	CBL[i][j] = Getr8(t);
      }
      for (j = 0; j < 3; j++) {
	sprintf(t, "bhp%s%d", coord[j], i+1);
	PBL[i][j] = Getr8(t);
	if (PBL[i][j] != 0) Knonzero = 1;
      }
      for (j = 0; j < 3; j++) {
	sprintf(t, "bhs%s%d", coord[j], i+1);
	SBL[i][j] = Getr8(t);
	if (SBL[i][j] != 0) Knonzero = 1;
      }
    }
#endif

    /* hardwire for cactus 4.0 */
    MBL[0] = bhm1;
    CBL[0][0] = bhx1;
    CBL[0][1] = bhy1;
    CBL[0][2] = bhz1;
    PBL[0][0] = bhpx1;
    PBL[0][1] = bhpy1;
    PBL[0][2] = bhpz1;
    SBL[0][0] = bhsx1;
    SBL[0][1] = bhsy1;
    SBL[0][2] = bhsz1;

    MBL[1] = bhm2;
    CBL[1][0] = bhx2;
    CBL[1][1] = bhy2;
    CBL[1][2] = bhz2;
    PBL[1][0] = bhpx2;
    PBL[1][1] = bhpy2;
    PBL[1][2] = bhpz2;
    SBL[1][0] = bhsx2;
    SBL[1][1] = bhsy2;
    SBL[1][2] = bhsz2;

    MBL[2] = bhm3;
    CBL[2][0] = bhx3;
    CBL[2][1] = bhy3;
    CBL[2][2] = bhz3;

    MBL[3] = bhm4;
    CBL[3][0] = bhx4;
    CBL[3][1] = bhy4;
    CBL[3][2] = bhz4;

    MBL[4] = bhm5;
    CBL[4][0] = bhx5;
    CBL[4][1] = bhy5;
    CBL[4][2] = bhz5;

    printf("\n");
    for (i = 0; i < N_MWBL; i++) {
      if (MBL[i] == 0.0)
	break;
      printf("m%d  %10.3e\n", i+1, MBL[i]);
      printf("x%d  %10.3e  y%d  %10.3e  z%d  %10.3e\n", 
	     i+1, CBL[i][0], i+1, CBL[i][1], i+1, CBL[i][2]);
      printf("px%d %10.3e  py%d %10.3e  pz%d %10.3e\n",
	     i+1, PBL[i][0], i+1, PBL[i][1], i+1, PBL[i][2]);
      printf("sx%d %10.3e  sy%d %10.3e  sz%d %10.3e\n\n",
	     i+1, SBL[i][0], i+1, SBL[i][1], i+1, SBL[i][2]);
    }
    if (i == N_MWBL) {
      printf("\n\nBAM WARNING: there currently is a hard limit");
      printf("of at most %d punctures\n\n", N_MWBL);
    } 

    /* start with conformal */
    *conformal_state = 3;
    conformal_metric = 1;
    /*FIXME    if (!use_conformal)
      CCTK_WARN(0, "BrBr data needs use_conformal = \"yes\" for now");*/

    /* for testing: force BL */ 
    if (0) {
      for (i = 0; i < N_MWBL; i++) {
	for (j = 0; j < 3; j++) 
	  PBL[i][j] = SBL[i][j] = 0;
      }
    }

    /* initialize to BL
       do it here so that even for nghosts>1 psi et al is set everywhere 
    */
    forallGHc(cctkGH, GH, GHc) {
      set_mwbl_GH(GHc);
      setBM(GHc);
    }

    /* if K_ij is not zero, solve Hamiltonian constraint */
    for (i = 0; i < N_MWBL; i++) { 
      if (MBL[i] == 0.0) break;
      for (j = 0; j < 3; j++) 
	if (PBL[i][j] || SBL[i][j]) Knonzero = 1;
    }
    if (Knonzero) {
      i = CCTK_VarIndex("BAM_Elliptic::bam_wsmax");
      nu = i--;
      nM = i--;
      nN = i--;
      nf[0] = nu; 
      nf[1] = nM;
      nf[2] = nN;
      BAM_Elliptic_Solve(cctkGH, GH, nf, 3, eqn_type, 1e-9);

      /* symmetries for large stencils */
      if (cctkGH->cctk_nghostzones[0] > 1 || 
	  cctkGH->cctk_nghostzones[1] > 1 || 
	  cctkGH->cctk_nghostzones[2] > 1) {
	CartSymGN(cctkGH, "admbase::metric");
	CartSymGN(cctkGH, "admbase::curv");
      }
    } 
  }

  /* Misner, only sitting here because of fisheye option */
  if (CCTK_Equals(initial_data, "misner")) {
    *conformal_state = 3;
    conformal_metric = 1;
    set_misner(cctkGH);
  }

  /* Poisson */
  if (CCTK_Equals(initial_data, "Poisson")) {
    extern int poisson_t;
    poisson_t = CCTK_Equals(poisson_type, "sin*sin*sin");
    CCTK_INFO("Testing multigrid with Poisson equation");
    vb = vbell = 1;
    problem = POISSON;
    eqn_type = FLATLINEAR;
    setnames_poisson();
    i = CCTK_VarIndex("BAM_Elliptic::bam_wsmax");
    nu = i--;
    nM = i--;
    nN = i--;

    nf[0] = nu; 
    nf[1] = nM;
    nf[2] = nN;
    BAM_Elliptic_Solve(cctkGH, GH, nf, 3, eqn_type, 1e-9);
  }

  /* Brill */
  if (CCTK_Equals(initial_data, "Brill")) {
    CCTK_INFO("Computing initial data for Brill wave");
    vb = (brill_verbose == 1) 
      || CCTK_Equals(elliptic_verbose, "yes");
    vbell = 1;
    problem = BRILL;
    if (CCTK_Equals(brill_eqn,"non-flat")) {
      eqn_type = LINEAR;
      errorexit("BAM: port non-flat Brill equation");
    } else {
      eqn_type = FLATLINEAR;
    }
    setnames_brill();
    if (brill_BHmass != 0.0) {
      MBL[0] = brill_BHmass;
      CBL[0][0] = brill_BHx;
      CBL[0][1] = brill_BHy;
      CBL[0][2] = brill_BHz;
      *conformal_state = 3;
      conformal_metric = 1;
      /*FIXME      if (!use_conformal)
	      CCTK_WARN(0, "brill_BHmass!=0 needs use_conformal=\"yes\" for now");*/
    }
    prinfo_brill();

    i = CCTK_VarIndex("BAM_Elliptic::bam_wsmax");
    nu = i--;
    nM = i--;
    nN = i--;

    nf[0] = nu; 
    nf[1] = nM;
    nf[2] = nN;
    BAM_Elliptic_Solve(cctkGH, GH, nf, 3, eqn_type, 1e-9);
  }

  /* lapse */
  if (CCTK_Equals(initial_lapse, "ssapprox")) {
    CCTK_INFO("Setting initial lapse to approximate Schwarzschild");
    forallGHc(cctkGH,GH,GHc)
      set_bbalpha_GH(GHc);
  }
}




/***************************************************************************/
/* check limitations, in particular check/adjust gridsize */
  

/* check/adjust gridsize in Cactus parameter base 
   called in main.c before anything else uses nx, ny, nz, etc
*/
void BAM_Init_Grid(cGH *cctkGH)
{
  DECLARE_CCTK_PARAMETERS
  int nadjust;
  int pr = 0;
  int *fisheye_parm, ptype;

  /* cactus 4 novelty */
  if (cctkGH->cctk_nghostzones[0] != cctkGH->cctk_nghostzones[1] ||
      cctkGH->cctk_nghostzones[0] != cctkGH->cctk_nghostzones[2] ||
      cctkGH->cctk_nghostzones[1] != cctkGH->cctk_nghostzones[2])
    CCTK_WARN(0, "for now need nghostzones to be identical in all directions");

  /* cartoon */
  cartoon = cartoon_stencil = 0;
  cartoon = (bam_cartoon_active == 1);
  if (cartoon) {
    cartoon_stencil = (bam_cartoon_stencil == 1);
    pr = 1;
  }

  /* fisheye */
  fisheye_parm = (int*) CCTK_ParameterGet("fisheye_active", "FishEye", &ptype);

  if (fisheye_parm != NULL)
    fisheye = *fisheye_parm;
  else
    fisheye = 0;

  if (fisheye) {
    fisheyedx = cctkGH->cctk_delta_space[0]/100;
    fisheyedy = cctkGH->cctk_delta_space[1]/100;
    fisheyedz = cctkGH->cctk_delta_space[2]/100;
  }

  /* basic grid offsets */
  /* if there are no ghosts or additional boundary points, offsets are zero */
  oxdo = oydo = ozdo = oxup = oyup = ozup = 0;
  if (CCTK_Equals(domain, "bitant")) {
    ozdo = cctkGH->cctk_nghostzones[2];
  } 
  if (CCTK_Equals(domain, "octant")) {
    oxdo = cctkGH->cctk_nghostzones[0];
    oydo = cctkGH->cctk_nghostzones[1];
    ozdo = cctkGH->cctk_nghostzones[2];
  } 
  if (CCTK_Equals(bam_bound, "bamrobin")) {
    if (!oxdo) oxdo = 1;
    if (!oydo) oydo = 1;
    if (!ozdo) ozdo = 1;
    oxup = oyup = ozup = 1;
  }
  if (cartoon) {
    oxdo = cctkGH->cctk_nghostzones[0];
    oydo = oyup = cctkGH->cctk_gsh[1]/2;
  }

  /* number of points in addition to interior zones */
  ox = oxdo + oxup + 1;
  oy = oydo + oyup + 1;
  oz = ozdo + ozup + 1;
  if (pr) printf("offsets  %d %d %d   %d %d %d  %d %d %d\n", 
		 oxdo, oydo, ozdo, oxup, oyup, ozup, ox, oy, oz);
 
  /* adjust size for powers of two */
  if (pr) printf("nx %d  ny %d  nz %d\n", cctkGH->cctk_gsh[0], 
		 cctkGH->cctk_gsh[1], cctkGH->cctk_gsh[2]);
  if (!CCTK_Equals(bam_autogrid, "no") &&
      (CCTK_Equals(domain, "full") || CCTK_Equals(domain, "octant") ||
       CCTK_Equals(type, "byrange") || CCTK_Equals(domain, "bitant"))) {

    nadjust = adjustlinsize(cctkGH, 1, cctkGH->cctk_gsh[0], ox);
    if (!cartoon) nadjust += adjustlinsize(cctkGH, 2, cctkGH->cctk_gsh[1], oy);
    nadjust += adjustlinsize(cctkGH, 3, cctkGH->cctk_gsh[2], oz);

    if (nadjust) {
      printf("BAM: cactus 4 casualty -- can no longer automatically\n");
      printf("     adjust grid size for boundary and power of 2 gridding\n");
      CCTK_WARN(0, "don't worry, be happy");
    }
  }
  if (pr) {
    printf("nx %d  ny %d  nz %d\n", cctkGH->cctk_gsh[0], 
	   cctkGH->cctk_gsh[1], cctkGH->cctk_gsh[2]);
    /*  printf ("    [%6.3f,%6.3f] [%6.3f,%6.3f] [%6.3f,%6.3f]\n",
	   xmin, xmax,
	   ymin, ymax,
	   zmin, zmax);*/
  }
}


/* adjustlinsize - determine if specified grid size is allowed by BAM.
 *
 * Allowed grids are of sizes
 *    p*2^n + o
 * where p is one of {2,3,5,7,9}, n is an integer, and o is a factor
 * corresponding to the boundary size.
 *
 * If the specified grid is not of an allowed size, then the closest
 * allowed size is chosen. The recommendation is allowed to be at most
 * 3 grid points larger than the specified size.
 *
 * Arguments:
 * dir - axis direction
 *  n  - full size of the grid
 *  o  - size of boundary
 */
int adjustlinsize(cGH *cctkGH, int dir, int n, int o)
{
  DECLARE_CCTK_PARAMETERS

  int match=0;
  int p[5] = {9,7,5,3,2}; /* allowed factorisations of coarse grid */
  int d, m, closest, close_m, close_p;
  int i, j;
  char *x[4] = {"", "x", "y", "z"};

  if (n-o < 16 || CCTK_Equals(bam_autogrid, "mod4")) {
    m = n - n % 4;
  } 
  else {
    m = 2;
    close_m = m;       /* initialise 'closest' m and p values */
    close_p = p[0];
    closest = n;
    for (j=0; j<5 && !match; ++j) { /* loop over the coarse grid factors */
      /* q = (n-o)/p[j];               factorise by p */
      for (i=0; i<20 && !match; ++i) {
	d = n-o-m*p[j];             /* subtract 2^n */
	if (d==0) {                 /* if d==0, the grid is allowed */
	  match = m*p[j];                
	}
	else if ((d>=-3) && (d<closest)) {
	  /* if d is closer than the previous best guess and less than three
	   * larger than the original grid size, set a new 'closest'
	   * factorisation */
	  closest = d;
	  close_m = m;
	  close_p = p[j];
	}
	if (d < 0) /* if the factorisation is too big, give up */
	  break;
	m *= 2;
      }
    }
    if (!match) {
      printf(
"BAM: rerun with n%s changed from %d to e.g. %d (=%d*%d + %d for boundary)\n",
	     x[dir], n, close_p*close_m+o, close_p, close_m, o);
      return 1; /* fail */
    }
  }
  return 0; /* success (specified grid size is allowed) */
}


/* return 1 if n = 2^m */
int ispow2(int n) 
{
  for (; n > 1 && n % 2 == 0; n /= 2);
  return (n == 1);
}




/* check linear sizes with regard to alignment and processor distribution */
void checklinsize(int n, int p)
{
  static int flag = 0;
  int i;

  if (1)
  if (n % (2*p) != 0) {
    printf("In one direction you have P=%d processors and N=%d points\n",
	   p, n);
    printf("(without full/octant).\n");
    printf("For automatic grid size add \"BAM\" to parameter grid.\n");
    errorexit("Sorry, N has to be divisible by 2*P.");
  }

  for (i = n/2/p; i > 1 && i % 2 == 0; i /= 2);

  if (i > 20 && !flag) {
    flag = 1;
    printf("\n");
    printf("BAM WARNING:\n");
    printf("  coarsest grid of the multigrid stack may be too big!\n");
    printf("  good choices for linear sizes are 2^n, 3*2^n, perhaps 5*2^n,\n");
    printf("  plus 1 for full grid, plus 2 for octant\n");
    printf("  note also that 2^n has to be divisible by nproc\n");
    printf("  ok, here are a few examples (+1 or +2):\n");
    printf("  32 64 128 256 512     \n");
    printf("  40 80 160 320 640     \n");
    printf("  48 96 192 384 768     \n");
    printf("\n");
  }
}




/* limitations */
void limitations(cGH *cctkGH, pGH *GH) 
{
  DECLARE_CCTK_PARAMETERS
  pGExtras *extras;
  pConnectivity *connectivity;

  /* FIXME: we assume 3D grid functions here */
  extras = GH->GFExtras [2];
  connectivity = GH->Connectivity [cctkGH->cctk_dim - 1];

  /* only 2^n procs */
  if (!ispow2(CCTK_nProcs(cctkGH))) 
    errorexit(
      "Sorry, but for now multigridding only works for 2^n processors\n");

  /* only these grid modes have been tested */
  if (!CCTK_Equals(type, "byrange") && 
      !CCTK_Equals(domain, "full") &&
      !CCTK_Equals(domain, "bitant") &&
      !(CCTK_Equals(domain, "octant")))
   errorexit("Sorry, but for now use grid = byrange, domain = full, bitant or octant\n");


  /* Hisaaki added grid=resize option 19990409 */
  if (0 && CCTK_Equals(domain, "octant")) {
    if (!CCTK_Equals(type, "BAM") &&  !CCTK_Equals(type,"resize") ) {
      errorexit("Octants need special alignment in BAM \nwhich you can activate with grid = \"BAM octant\"");
    }
  }
  

  /* no idea why this duplicates things, didn't put this here, BB 2/01 */

  /* for now allow only special linear sizes */
  ox = 1; oy = 1; oz = 1;
  if (CCTK_Equals(bam_bound, "bamrobin")) 
    {ox++; oy++; oz++;}
  /* tricky logic for octant-bitant with/without bamrobin */
  if (CCTK_Equals(domain, "octant"))
    {  
      ox += cctkGH->cctk_nghostzones[0];
      oy += cctkGH->cctk_nghostzones[1];
      oz += cctkGH->cctk_nghostzones[2];
    }
  else
    {  if(CCTK_Equals(bam_bound,"bamrobin") && !CCTK_Equals(domain,"bitant"))
      {  ox++; oy++; oz++; }
    }
  if (CCTK_Equals(domain, "bitant"))
    {  oz += cctkGH->cctk_nghostzones[2];
    if(CCTK_Equals(bam_bound,"bamrobin"))
      {  ox++; oy++;  }
    }
  if (cartoon)
    ox++;

  checklinsize(extras->nsize[0] - ox, connectivity->nprocs[0]);
  checklinsize(extras->nsize[1] - oy, connectivity->nprocs[1]);
  checklinsize(extras->nsize[2] - oz, connectivity->nprocs[2]);

  /* convergence mode */
  /* fails for thorn_MoreConvergence, but this happens not only for BAM */

  /* no staggering */
  if (CCTK_StaggerVars() == PUGH_STAGGER)
    errorexit("Staggered grids are not supported.");

  /* limited choice of octant_order */
  if (octant && !(octant_order == 2 || octant_order == 4))
    errorexit("octant_order must be 2 or 4");

  /* limited choice of bitant_order */
  if (bitant && !(bitant_order == 2 || bitant_order == 4))
    errorexit("bitant_order must be 2 or 4");

  /* uniform spacing */
  if (!(dequal(cctkGH->cctk_delta_space[0],cctkGH->cctk_delta_space[1]) && 
	dequal(cctkGH->cctk_delta_space[0],cctkGH->cctk_delta_space[2]) && 
	dequal(cctkGH->cctk_delta_space[1],cctkGH->cctk_delta_space[2])))
    errorexit("need dx = dy = dz, ask BB for fix.");

  /* cartoon */
  if (cartoon) {
    if (cctkGH->cctk_nghostzones[1] > 2 && GH->nprocs > 1)
      errorexit("cartoon needs stencil <= 2 on multi procs in BAM");

    /* cartoon_stagger_x seems not to exist anymore ... [dp]
    if (!(cartoon_stagger_x ==1))
      errorexit("cartoon needs cartoon_stagger_x = \"yes\" for now in BAM");
    */
  }
}




/***************************************************************************/
/* wrapper for maximal slicing */
/* Cactus usually goes through standard linear interface */
void BAM_Maximal_Slicing(cGH *cctkGH, pGH *GH, CCTK_REAL tol)
{
  DECLARE_CCTK_PARAMETERS
  char *l = "----------------------------------------------------------------";
  int i, nu, nM, nN, nf[MGFMAX];
  static int first = 1;

  vb = (maximal_verbose == 1);  
  vbell = CCTK_Equals(elliptic_verbose, "yes");  

  if ((vb || vbell) && !first) CCTK_INFO("Maximal Slicing");
  if (first) {
    first = 0;
    CCTK_INFO("Maximal Slicing");
  }

  problem = MAXIMAL;
  eqn_type = LINEAR;
  setnames_maximal();

  /* cactus alpha already available on top level */
  nu = CCTK_VarIndex("admbase::alp");

  /* worry about using cactus M/N later, but this is the place */
  i = CCTK_VarIndex("BAM_Elliptic::bam_wsmax");
  nM = i--;
  nN = i--;

  /* go for it */
  nf[0] = nu; 
  nf[1] = nM;
  nf[2] = nN;
  BAM_Elliptic_Solve(cctkGH, GH, nf, 3, eqn_type, (double)tol);

  if (vb) printf("%s\n", l);
}





/***************************************************************************/
/* wrapper for generic linear */

/* this corresponds to the Cactus 4 interface defined in 
   arrangements/CactusElliptic/EllBase/src/Ell_Interface.c 

   note that we ignore the metric indices (the StandardEinstein metric is 
   always used), and the same goes for the conformal factor

   only the l2 norm is used as in Cactus 3, other norms come from the
   BAM parameters 
*/

/* fake 1 */
void BAM_Ell_LinMetricSolver(cGH *GH, int *Metric, int FieldIndex, 
               int MIndex, int NIndex, CCTK_REAL *AbsTol, CCTK_REAL *RelTol)
{
  BAM_Elliptic_Linear(GH, FieldIndex, MIndex, NIndex, AbsTol[0]);
}

/* fake 2 */
void BAM_Ell_LinConfMetricSolver(cGH *GH, int *MetricPsi, int FieldIndex, 
               int MIndex, int NIndex, CCTK_REAL *AbsTol, CCTK_REAL *RelTol)
{
  BAM_Elliptic_Linear(GH, FieldIndex, MIndex, NIndex, AbsTol[0]);
}

/* the real thing */
void BAM_Elliptic_Linear(cGH *cctkGH, int nu, int nM, int nN, CCTK_REAL tol)
{
  DECLARE_CCTK_PARAMETERS
  pGH *GH = (pGH *) cctkGH->extensions [CCTK_GHExtensionHandle ("PUGH")];
  char *l = "----------------------------------------------------------------";
  int nf[MGFMAX];
  static int first = 1;

  /* set up */
  vb = (maximal_verbose == 1);  
  vbell = CCTK_Equals(elliptic_verbose, "yes");  
  if ((vb || vbell) && !first) CCTK_INFO("Solve Linear Elliptic");
  if (first) {
    first = 0;
    CCTK_INFO("Solve Linear Elliptic");
  }
  problem = LINEAR;
  eqn_type = LINEAR;
  setnames_linear();

  /* check incoming data but only in verbose mode */
  if (vb||vbell) {
    if (checkfornanGF(GFByIndex(GH, nu)))
      errorexit("BAM_Elliptic_Linear() was called with nan in u");
    if (checkfornanGF(GFByIndex(GH, nM)))
      errorexit("BAM_Elliptic_Linear() was called with nan in M");
    if (checkfornanGF(GFByIndex(GH, nN)))
      errorexit("BAM_Elliptic_Linear() was called with nan in N");
  }

  /* don't know the status of that! BB 1/00 */
#ifdef FIXME /* Were going to comform to BAM signs */
  /* cactus has the opposite sign convention */
  flipsign(GH, nM);
  flipsign(GH, nN);
#endif

  /* go for it */
  nf[0] = nu; 
  nf[1] = nM;
  nf[2] = nN;
  BAM_Elliptic_Solve(cctkGH, GH, nf, 3, eqn_type, (double)tol);

#ifdef FIXME
  /* cactus has the opposite sign convention */
  flipsign(GH, nM);
  flipsign(GH, nN);
#endif

  if (vb) printf("%s\n", l);
}




/* flip sign of cactus field */
void flipsign(pGH *GH, int nGF)
{
  int ijk;
  CCTK_REAL *a;
  pGA *GF = GFByIndex(GH, nGF);

  /*pGA *GF;*/

  a = GFByIndex(GH, nGF)->data;
  /*GF = locateGFbyDataPtr (a);*/

  if (CCTK_IsThornActive("BoxInBox")) {
    /* seems to not be unmaintained ... eg. itime is no longer a member of
       the pGH struct. Note that the following code refers to the
       original version of BoxInBox, which did not make it to Cactus4,
       so don't confuse it with the more recent PAGH version. [dp]
    pGH *GHtmp;
    int l;
    for (l = GH->level;; l++) {
      GHtmp = GHbyLevel(GH->convlevel, l, 0);
      if (!GHtmp || GHtmp->itime != GH->itime) break;
      a = GFByIndex(GHtmp, nGF)->data;
      for (ijk = 0; ijk < GHtmp->npoints; ijk++) 
	a[ijk] = -a[ijk];
      if (0) printf("flipped sign %s l=%d\n", GHtmp->gridFuncs[nGF]->name, l);
    }    
    */
  }
  else {
    for (ijk = 0; ijk < GF->extras->npoints; ijk++) 
      a[ijk] = -a[ijk];
  }

}





/***************************************************************************/
/* wrapper for vector Laplacian: L u  =  vecL u  +  N */

/* the real thing, at the moment there is no vector interface in Cactus 4 */
void BAM_Elliptic_VecL(cGH *cctkGH, int *nu, int *nN, CCTK_REAL tol)
{
  DECLARE_CCTK_PARAMETERS
  pGH *GH = (pGH *) cctkGH->extensions [CCTK_GHExtensionHandle ("PUGH")];
  char *l = "----------------------------------------------------------------";
  int i, nf[MGFMAX];
  static int first = 1;

  if (!CCTK_IsThornActive("BAM_VecLap"))
    errorexit("BAM_Elliptic_VecL(): Please include thorn BAM_VecLap.");

  vb = (veclap_verbose == 1);  
  vbell = CCTK_Equals(elliptic_verbose, "yes");  

  if ((vb || vbell) && !first) printf("\nBAM: Solve VecL Elliptic\n");
  if (first) {
    first = 0;
    printf("\nBAM: Solve VecL Elliptic\n");
  }

  problem = VECLAPLACE;
  eqn_type = VECLAPLACE;
  veclaplacebssn = CCTK_Equals(shift_evolution_method, "bssngauge"); 
  setnames_vecL();

  /* grid function numbers */
  for (i = 0; i < 3; i++) {
    nf[i] = nu[i];
    if ((vb||vbell) && checkfornanGF(GFByIndex(GH, nu[i])))
      errorexit("BAM_Elliptic_VecL() was called with illegal arguments for u");
  }
  for (i = 0; i < 3; i++) {
    nf[i+3] = nN[i];
    if ((vb||vbell) && checkfornanGF(GFByIndex(GH, nN[i])))
      errorexit("BAM_Elliptic_VecL() was called with illegal arguments for N");
  }    

  /* go for it */
  BAM_Elliptic_Solve(cctkGH, GH, nf, 6, eqn_type, (double)tol);

  if (vb) printf("%s\n", l);
}




/* wrapper for 4 equations in 4 variables 
   this assumes that down the line an external stencil function is called 
   nu[] contains nnu external fields
   the first 4 entries in nu[] are the indices of the vector we solve for
   the following entries are static fields
*/
void BAM_Elliptic_Vec4(cGH *cctkGH, int *nu, int nnu, CCTK_REAL tol)
{
  DECLARE_CCTK_PARAMETERS
  pGH *GH = PUGH_pGH(cctkGH);
  char *l = "----------------------------------------------------------------";
  int i;
  static int first = 1;

  if (!CCTK_IsThornActive("BAM_VecLap"))
    errorexit("BAM_Elliptic_VecL(): Please include thorn BAM_VecLap.");

  vb = (veclap_verbose == 1);  
  vbell = CCTK_Equals(elliptic_verbose, "yes");  
  if ((vb || vbell) && !first) printf("\nBAM: Solve VecL Elliptic\n");
  if (first) {
    first = 0;
    printf("\nBAM: Solve VecL Elliptic\n");
  }

  problem = VEC4;
  eqn_type = VEC4;
  setnames_vecL();

  /* check what is passed in */
  for (i = 0; i < nnu; i++) {
    if ((vb||vbell) && checkfornanGF(GFByIndex(GH, nu[i])))
      errorexit("BAM_Elliptic_VecL() was called with illegal arguments for u");
  }

  /* go for it */
  BAM_Elliptic_Solve(cctkGH, GH, nu, nnu, eqn_type, (double)tol);

  if (vb) printf("%s\n", l);
}





/* wrapper for N equations in N variables 
   this assumes that down the line an external stencil function is called 
   nu[] contains the nnu=N external fields
   type defines the problem and eqn_type, which are internal to BAM_Elliptic
*/
void BAM_Elliptic_VecN(cGH *cctkGH, int *nu, int nnu, int t, CCTK_REAL tol)
{
  DECLARE_CCTK_PARAMETERS
  pGH *GH = PUGH_pGH(cctkGH);
  char *l = "----------------------------------------------------------------";
  int i;
  static int first = 1;

  if ((t != TSD) && (t != KSD) && (t != YORK))
    CCTK_WARN(0, "BAM_Elliptic_VecN called with unknown type");

  vb = (brbr_verbose == 1);  
  vbell = CCTK_Equals(elliptic_verbose, "yes");  
  if ((vb || vbell) && !first) printf("\nBAM: Solve VecN Elliptic\n");
  if (first) {
    first = 0;
    printf("\nBAM: Solve VecN Elliptic\n");
  }

  if (t == KSD) {
    problem = KSD;
    eqn_type = KSD;
    setnames_vecL();
  }
 
  if (t == TSD) {
    problem = TSD;
    eqn_type = TSD;
    setnames_vecL();
  }

  if (t==YORK) {
    problem = YORK;
    eqn_type = YORK;
    setnames_vecL();
  }
 
  /* check what is passed in */
  for (i = 0; i < nnu; i++) {
    if ((vb||vbell) && checkfornanGF(GFByIndex(GH, nu[i])))
      errorexit("BAM_Elliptic_VecN() was called with illegal arguments for u");
  }

  /* go for it */
  BAM_Elliptic_Solve(cctkGH, GH, nu, nnu, eqn_type, (double)tol);

  if (vb) printf("%s\n", l);
}





/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/* generic solver */
void BAM_Elliptic_Solve(cGH *cctkGH, pGH *GH, int *nf, int n, 
			int bam_eqn_type, CCTK_REAL tol)
{
  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS
  int nws = 0;
  int bbmg_eqn_type, bbmg_init, bbmg_method;
  int bbmg_npre = 0, bbmg_npost = 0, bbmg_ncycles;
  int ltop, lmax;
  int pr=0;
  pGHList *hlist;
  pGH *h;
  static int firstcall = 1;
  /* int mt = 0; */

  /* cactus 4 casualties */
  if (pr) printf("BAM_Elliptic_Solve(): convlevel %d  mglevel %d  level %d\n", 
		 GH->convlevel, GH->mglevel, GH->level);
  setGFIndices();
  initGHList(GH); 
  conformal_metric = (*conformal_state == 3) ? 1 : 0;

  /* FIXME */
  /*  if (mt) cactus_memtrace();*/

  /* boundary parameters */
  octant = (CCTK_Equals(domain, "octant")) ? 1 : 0;
  bitant = (CCTK_Equals(domain, "bitant")) ? 1 : 0;
  neumann = (CCTK_Equals(bam_bound, "bamrobin")) ? 1 : 0;
  octoffset = (octant) ? cctkGH->cctk_nghostzones[0] - 1 : 0; 
  bitoffset = (bitant) ? cctkGH->cctk_nghostzones[0] - 1 : 0;
  octant_order = bam_octant_order;
  bitant_order = bam_bitant_order;

  robin_limit = bam_robin_limit;
  robin_power = bam_robin_power;

  robin_limit_vec1 = bam_robin_limit_vec1;
  robin_power_vec1 = bam_robin_power_vec1;

  robin_limit_vec2 = bam_robin_limit_vec2;
  robin_power_vec2 = bam_robin_power_vec2;

  robin_limit_vec3 = bam_robin_limit_vec3;
  robin_power_vec3 = bam_robin_power_vec3;

  plane_sx = bam_plane_sx;
  plane_sy = bam_plane_sy;
  plane_sz = bam_plane_sz;

  plane_sx_vec1 = bam_plane_sx_vec1;
  plane_sy_vec1 = bam_plane_sy_vec1;
  plane_sz_vec1 = bam_plane_sz_vec1;

  plane_sx_vec2 = bam_plane_sx_vec2;
  plane_sy_vec2 = bam_plane_sy_vec2;
  plane_sz_vec2 = bam_plane_sz_vec2;

  plane_sx_vec3 = bam_plane_sx_vec3;
  plane_sy_vec3 = bam_plane_sy_vec3;
  plane_sz_vec3 = bam_plane_sz_vec3;

  solve_nmin = bam_solve_nmin;
  solve_nmax = bam_solve_nmax;

  wcycle_factol = bam_wcycle_factol;
  wcycle_nmin = bam_wcycle_nmin;
  wcycle_nmax = bam_wcycle_nmax;
  if (wcycle_nmin > wcycle_nmax) wcycle_nmin = wcycle_nmax;    /* override */  

  /* excision */
  iniboundmask(cctkGH);

  /* sanity check */
  limitations(cctkGH, GH);  

  /* multigrid parameters */
  gridminsize = 9;
  gridmaxlevels = -1;

#ifdef FIXME_WHATISDOBAMRELAXGOODFOR
  if (CCTK_Equals(bam_relax, "unigs")) do_bam_relax = 1;
  else do_bam_relax = 0;
#endif
  rb_decouple = (redblack_decouple == 1);
  fullV = (bam_fullV == 1);
  stencil_symmetric = CCTK_Equals(bam_laplacian, "symmetric");
  weightedinit = bam_weightedinit;
  if (pr) printf("redblack_decouple %d, fullV %d, weightedinit %d\n", 
		 rb_decouple, fullV, weightedinit);

  if (problem == BRILL || problem == POISSON ||
      (CCTK_Equals(initial_data, "brilldata") && firstcall)) {
    gridminsize = brill_minsize;
    gridmaxlevels = brill_nmglevs;
    bbmg_npre = brill_npre;
    bbmg_npost = brill_npost;
    mincycles = brill_mincycles;
    maxcycles = brill_maxcycles;
    tolres2 = brill_tolres2;
    tolresi = brill_tolresi;
    factau2 = brill_factau2;
    factaui = brill_factaui;
  }
  if (problem == BHDATA || problem == TSD || problem == KSD || 
      problem == YORK) {
    gridminsize = brbr_minsize;
    gridmaxlevels = brbr_nmglevs;
    bbmg_npre = brbr_npre;
    bbmg_npost = brbr_npost;
    mincycles = brbr_mincycles;
    maxcycles = brbr_maxcycles;
    tolres2 = brbr_tolres2;
    tolresi = brbr_tolresi;
    factau2 = brbr_factau2;
    factaui = brbr_factaui;
    dalota = CCTK_Equals(brbr_kij, "DainLoustoTakahashi");
  }
  if (problem == MAXIMAL || problem == LINEAR) {
    gridminsize = maximal_minsize;
    gridmaxlevels = maximal_nmglevs;
    bbmg_npre = maximal_npre;
    bbmg_npost = maximal_npost;
    mincycles = maximal_mincycles;
    maxcycles = maximal_maxcycles;
    tolres2 = maximal_tolres2;
    tolresi = maximal_tolresi;
    factau2 = maximal_factau2;
    factaui = maximal_factaui;
    if (tolres2 == 0.0 || tol < tolres2) tolres2 = tol;
  }
  if (problem == POWERS) {
    gridminsize = bamellnonlin_minsize;
    gridmaxlevels = bamellnonlin_nmglevs;
    bbmg_npre =   bamellnonlin_npre;
    bbmg_npost =  bamellnonlin_npost;
    mincycles = bamellnonlin_mincycles;
    maxcycles = bamellnonlin_maxcycles;
    tolres2 = bamellnonlin_tolres2;
    tolresi = bamellnonlin_tolresi;
    factau2 = bamellnonlin_factau2;
    factaui = bamellnonlin_factaui;
    if (tolres2 == 0.0 || tol < tolres2) tolres2 = tol;
  }
  if (problem == VECLAPLACE || problem == VEC4) {
    gridminsize = veclap_minsize;
    gridmaxlevels = veclap_nmglevs;
    bbmg_npre = veclap_npre;
    bbmg_npost = veclap_npost;
    mincycles = veclap_mincycles;
    maxcycles = veclap_maxcycles;
    tolres2 = veclap_tolres2;
    tolresi = veclap_tolresi;
    factau2 = veclap_factau2;
    factaui = veclap_factaui;
    if (tolres2 == 0.0 || tol < tolres2) tolres2 = tol;
  }
  if (pr) printf("tolerances %.1e %.1e %.1e %.1e\n",
		 tolres2, tolresi, factau2, factaui); 
  if (maxcycles < mincycles) maxcycles = mincycles;

  if (bam_solve_normmax > 0)
    solve_normmax = bam_solve_normmax;
  else
    solve_normmax = tolres2;


  /* figure out how many fields we still need */
  eqn_type = bam_eqn_type;
  if (eqn_type == FLATLINEAR)
    nws = 3;
  else if (eqn_type == BRBR) 
    nws = (fisheye) ? 12 : 3;
  else if (eqn_type == LINEAR || eqn_type == POWERS)
    nws = 12;
  else if (eqn_type == VECLAPLACE)
    nws = 9;
  else if (eqn_type == VEC4 || eqn_type == KSD || eqn_type==YORK)
    nws = 12;
  else if (eqn_type == TSD)
    nws = 15;
  if (pr) printf("nws %d, MGF %d, eqn_type %d, min-max %d-%d\n",
		 nws, MGF, eqn_type, 
		 CCTK_VarIndex("BAM_Elliptic::bam_wsmin"), 
		 CCTK_VarIndex("BAM_Elliptic::bam_wsmax"));
  if (nws > CCTK_VarIndex("BAM_Elliptic::bam_wsmax") 
          - CCTK_VarIndex("BAM_Elliptic::bam_wsmin") + 1)
    errorexit("BAM_Elliptic_Solve(): nws too big");

  /* make multigrid stack in PUGH */
  PUGH_makestack(cctkGH, GH, nf, n, nws);

  /* count levels of refinements */
  ltop = lmax = 0;
  if (pr) printf("GHList = \n");
  for (hlist = GHList; hlist; hlist = hlist->next) {
    pGExtras *extras;

    h = hlist->GH;
    /* FIXME: we assume 3D grid functions here */
    extras = h->GFExtras [2];
    if (pr) printf("  %3d %3d %3d  %3d %3d %3d  %10.3f\n", 
		   h->convlevel, h->level, h->mglevel, 
		   extras->nsize[0], extras->nsize[1], extras->nsize[2], 0.0); 
                                                          /* h->phys_time */ 
    if (h->convlevel == GH->convlevel) {
      if (h->level == GH->level && h->mglevel > ltop) 
	ltop = h->mglevel;
      if (h->mglevel < lmax) 
	lmax = h->mglevel;
    }
  }
  ltop = ltop + 1;
  lmax = ltop - lmax;
  if (pr) printf("ltop %d, lmax %d\n", ltop, lmax); 

  /* and translate stack into BAM */
  BAM_translatestack(cctkGH, ltop, lmax, &GH, nf, n, nws);

  /* test */
  if (0) {
    pGExtras *extras;

    /* FIXME: we assume 3D grid functions here */
    extras = GH->GFExtras [2];

    printf("myproc %d\n", GH->myproc);
    printf("lb ub %2d %2d %2d %2d %2d %2d\nowner",
	   extras->lb[GH->myproc][0],
	   extras->ub[GH->myproc][0],
	   extras->lb[GH->myproc][1],
	   extras->ub[GH->myproc][1],
	   extras->lb[GH->myproc][2],
	   extras->ub[GH->myproc][2]);
#ifdef FIXME
    /* FIXME: stagger */
    for (i = 0; i < 6; i++)
      printf(" %2d", extras->ownership[GH->stagger][i/2][i%2]); 
#endif
    printf("\n");
  }


  /* call multigrid driver */
  bbmg_eqn_type = eqn_type;
  bbmg_init = 1;     /* full mg cycles */
  bbmg_method = 0;   /* unused */
  bbmg_ncycles = maxcycles;
  bbmg_elliptic_solve(bbmg_eqn_type, bbmg_init, bbmg_method,
		      bbmg_npre, bbmg_npost, bbmg_ncycles,
		      ltop, lmax, &GH);

  /* clean up */
  BAM_freelevels();
  if (1) PUGH_destroystack(cctkGH,GH, nf, n, nws);
  firstcall = 0;
}
