/* thorn_BAM_Elliptic: Multigrid Elliptic Solver for Cactus
   Copyright (C) 1998 Bernd Bruegmann
*/
/* BIB_inject.c */
/* Bernd Bruegmann and Gerd Lanfermann, 4/98 */

#include "CactusPUGH/PUGH/src/include/pugh.h"
#include "cctk.h"
#include "cctk_Flesh.h"

#include <stdio.h>
#include <stdlib.h>

/* #define VERBOSE */

void BIBInjectGF( pGH *GHfrom, pGH *GHto, double *ufrom, double *uto,
		 int imin, int imax, int jmin, int jmax, int kmin, int kmax);

void BIBInjectGF( pGH *GHfrom, pGH *GHto, double *ufrom, double *uto,
		 int imin, int imax, int jmin, int jmax, int kmin, int kmax) 
{
#if 0
  pGA *GFto=NULL;
  pGA *GFfrom=NULL;
#endif
  double *xr,*yr,*zr,*rr;  
  int myproc;
  int npoints;
  int ii,jj,kk,ll;
  int locimax,locimin,locjmax,locjmin,lockmax,lockmin;
  int xindex,yindex,zindex,timelevel;
  pGExtras *extras;

  myproc = GHto->myproc;

  /* FIXME: GF dimension */
  extras = GHto->GFExtras [GHto->dim - 1];

  /* Get the GF indices */
  xindex = CCTK_VarIndex("grid::x");
  yindex = CCTK_VarIndex("grid::y");
  zindex = CCTK_VarIndex("grid::z");
  timelevel = 0;

  /* subtract lower index bound to get local index, PE dependent */
  locimin=imin-extras->lb[myproc][0];
  locimax=imax-extras->lb[myproc][0];

  locjmin=jmin-extras->lb[myproc][1];
  locjmax=jmax-extras->lb[myproc][1];

  lockmin=kmin-extras->lb[myproc][2]; 
  lockmax=kmax-extras->lb[myproc][2];

  /* patch covering: form intersection of closed intervals */
  if (locimin < 0) locimin = 0;
  if (locjmin < 0) locjmin = 0;
  if (lockmin < 0) lockmin = 0;
  if (locimax > extras->lnsize[0]-1) locimax = extras->lnsize[0]-1;
  if (locjmax > extras->lnsize[1]-1) locjmax = extras->lnsize[1]-1;
  if (lockmax > extras->lnsize[2]-1) lockmax = extras->lnsize[2]-1;

#ifdef VERBOSE
  /* global (onGHto) to PE-local(on GHto) */
  /* get the local imax,imin on GHto(!): local coordinates */
  /* could this give problems bc the region is distributed across the PEs ? */
  /* this assumes that ijk is given on the GHto side, else divide by 2 +1 */
  printf("GHto lb   : %d %d %d \n", 
	 extras->lb[myproc][0], extras->lb[myproc][1], 
	 extras->lb[myproc][2]);
  printf("GHto lnxyz: %d %d %d \n", 
	 extras->lnsize[0], extras->lnsize[1], extras->lnsize[2]);
  
  printf("BAM bib_inject: My PE=%d on bib level=%d\n",
	 myproc,GHto->level);
  printf("BAM bib_inject: global target region [%d,%d,%d]*[%d,%d,%d]\n",
	 imin,jmin,kmin,imax,jmax,kmax);
  printf("BAM bib_inject: local target region: [%d,%d,%d]*[%d,%d,%d]\n",
	 locimin,locjmin,lockmin,locimax,locjmax,lockmax);
  printf(" xstart: %f  ",
         ((pGA ***) GHto->variables) [xindex][timelevel][DATINDEX(extras,locimin,locjmin,lockmin)];
  printf(" ystart: %f  ",
	 GHto->gridFuncs[GFI_Y]->data[DATINDEX(extras,locimin,locjmin,lockmin)]);
  printf(" zstart: %f  \n",
	 GHto->gridFuncs[GFI_Z]->data[DATINDEX(extras,locimin,locjmin,lockmin)]);
  printf(" xend  : %f  ",
	 GHto->gridFuncs[GFI_X]->data[DATINDEX(extras,locimax,locjmax,lockmax)]);
  printf(" yend  : %f  ",
	 GHto->gridFuncs[GFI_Y]->data[DATINDEX(extras,locimax,locjmax,lockmax)]);
  printf(" zend  : %f  \n",
	 GHto->gridFuncs[GFI_Z]->data[DATINDEX(extras,locimax,locjmax,lockmax)]);
  fflush(stdout);
#endif

  /* if there is no overlap, there is nothing to do */
  /* note that e.g. [5,5] is not empty but contains one point */
  if (locimin > locimax || locjmin > locjmax || lockmin > lockmax) {

    if (CCTK_IsThornActive("Interp")) {
      /*
       *  The code in this if loop is unmaintained. Note that the Interp
       *  thorn no longer exists, and PUGHInterp does not include a 
       *  pughGetPoints function. [dp]
       */
	
       /* from thorn_interp/README:
       > It is important to note that *ALL PROCESSORS MUST CALL GETPOINTS*. It
       > is alright for processors to call it with an n of 0, but it must be
       > called on all processors, or your code will hang.

      double x = 0, r;
      pughGetPoints(ufrom, &x, &x, &x, &r, 0);
    
       */
    }
    return;
  }
    
  /* total number of points in the PE-local active target region */
  npoints = (locimax-locimin+1)*(locjmax-locjmin+1)*(lockmax-lockmin+1); 
  
  xr = (CCTK_REAL *)malloc(npoints*sizeof(CCTK_REAL));
  yr = (CCTK_REAL *)malloc(npoints*sizeof(CCTK_REAL));
  zr = (CCTK_REAL *)malloc(npoints*sizeof(CCTK_REAL));
  rr = (CCTK_REAL *)malloc(npoints*sizeof(CCTK_REAL));

  ll=0;
  for (kk=lockmin;kk<=lockmax;kk++)
    for (jj=locjmin;jj<=locjmax;jj++)
      for (ii=locimin;ii<=locimax;ii++) {
	xr[ll] = ((CCTK_REAL *)((pGA ***) GHto->variables)[xindex][timelevel]->data)[DATINDEX(extras,ii,jj,kk)];
	yr[ll] = ((CCTK_REAL *)((pGA ***) GHto->variables)[yindex][timelevel]->data)[DATINDEX(extras,ii,jj,kk)];
	zr[ll] = ((CCTK_REAL *)((pGA ***) GHto->variables)[zindex][timelevel]->data)[DATINDEX(extras,ii,jj,kk)];
	ll++;
      }
  

  if (CCTK_IsThornActive("Interp")) {
    /* Unmaintained, see above [dp]. */

    /* ufrom must be attached to a GF so that GH can be derived */
    /* pughGetPoints(ufrom, xr, yr, zr, rr, ll); */
  }

  ll = 0;
  for (kk=lockmin;kk<=lockmax;kk++)
    for (jj=locjmin;jj<=locjmax;jj++)
      for (ii=locimin;ii<=locimax;ii++) {
	uto[DATINDEX(extras,ii,jj,kk)] = rr[ll++];
      }

  free(xr);
  free(yr);
  free(zr);
  free(rr);
}

