/* thorn_BAM_Elliptic: Multigrid Elliptic Solver for Cactus
   Copyright (C) 1998 Bernd Bruegmann 
*/
/* BIB_to_BAM.c */
/* Bernd Bruegmann, 3/98 */

#include "bbmg.h"



/* For the BoxInBox levels we use the BIB injection operators.          
   Here injection means simply copying every other point.
   Interpolation for prolongation and averaging for restriction is done
   elsewhere. 
*/

/* for multigrid transfers we want the range according to r
   r refers to GLOBAL index space, the loops we would do globally are e.g.
   for (k = kmin; k <= kmax; k++)
   for (j = jmin; j <= jmax; j++)
   for (i = imin; i <= imax; i++)
*/

/* the data is contained in double *u, which is identical to one
   of the GH->gridFuncs[...]->data pointers
*/

/* pughGetPoints() doesn't now whether we go from fine to coarse or the
   other way round, so there is just one BIBInjectGF for both directions
*/
void BIBInjectGF( pGH *GHfrom, pGH *GHto, double *ufrom, double *uto,
		 int imin, int imax, int jmin, int jmax, int kmin, int kmax);




/* inject coarse to fine (refinement factor 2) */
void BIB_injectctof(struct box *gc, struct box *gf, double *uc, double *uf)
{
  BIBInjectGF(gc->GH, gf->GH, uc, uf,  
	      gf->imin, gf->imax, gf->jmin, gf->jmax, gf->kmin, gf->kmax);
}




/* inject fine to coarse (refinement factor 2) */
void BIB_injectftoc(struct box *gf, struct box *gc, double *uf, double *uc)
{
  BIBInjectGF(gf->GH, gc->GH, uf, uc,  
	      gc->imin, gc->imax, gc->jmin, gc->jmax, gc->kmin, gc->kmax);
}






