/* thorn_BAM_Elliptic: Multigrid Elliptic Solver for Cactus
   Copyright (C) 1998 Bernd Bruegmann 
*/
/* Cactus4_utils.c 
   several utilities to interface with Cactus 4, 
   replacement for several Cactus 4 casualties
*/
/* Bernd Bruegmann, 10/99 */
/* many thanks to Gabrielle Allen for getting this started */

#include "bbmg.h"
#include "cctk_WarnLevel.h"

static char *rcsid = "$Header: /numrelcvs/AEIThorns/BAM_Elliptic/src/Cactus4_utils.c,v 1.31 2002/05/05 20:22:13 allen Exp $";

CCTK_FILEVERSION(Development_BAM_Elliptic_Cactus4_utils_c)

/* 3 ways up shutting down: */
#define STOP  exit(0);
#define ABORT exit(1);
#define CORE  exit(2);



int conformal_metric = 1;
pGHList *GHList = NULL;




void addtoGHList(pGH *GH) 
{
  pGHList *tmp;

  tmp = (pGHList *) malloc(sizeof(pGHList));
  tmp->next = GHList;
  tmp->GH = GH;
  GHList = tmp;
}




void initGHList(pGH *GH) 
{
  if (!GHList) addtoGHList(GH);
}




void removefromGHList(pGH *GH)
{
  pGHList *tmp, *x;

  if (GHList->GH == GH) {
    x = GHList;
    GHList = GHList->next;
    free(x);
  } else {
    for (tmp = GHList; tmp->next && tmp->next->GH != GH; tmp = tmp->next);
    if (tmp->next) {
      x = tmp->next;
      tmp->next = tmp->next->next;
      free(x);
    }
  }
}





pGH *GHbyLevel(int convlevel, int biblevel, int mglevel) 
{
  pGHList *tGHList;
  int pr = 0;

  if (pr) 
    printf("GHbyLevel: looking for convlevel %d  mglevel %d  biblevel %d\n", 
	   convlevel, mglevel, biblevel);

  for (tGHList = GHList; tGHList; tGHList = tGHList->next) {

    if (pr) 
      printf("GHbyLevel: found       convlevel %d  mglevel %d  biblevel %d\n", 
	     tGHList->GH->convlevel, tGHList->GH->mglevel, tGHList->GH->level);

    if ((tGHList->GH->convlevel == convlevel) && 
	(tGHList->GH->mglevel   == mglevel)   && 
	(tGHList->GH->level     == biblevel)) {
      return(tGHList->GH);
    }
  }
  return(NULL);
}





/* originally in cactus 3
   routine    locateGFbyDataPtr
   date       Thu Oct  2 14:25:29 1997
   author     Paul Walker
*/
pGA *locateGFbyDataPtr(CCTK_REAL *x) 
{
  pGA *GF = NULL;
  pGHList *ghl;
  int ngf, i;

  ngf = GHList->GH->nvariables;
  for (ghl = GHList; ghl && !GF; ghl = ghl->next) {
    for (i=0; i < ngf && !GF; i++) {
      if (GFByIndex(ghl->GH, i)->data == x) {
	GF = GFByIndex(ghl->GH, i);
      }
    } 
  }
  /* return a core if we didn't find a GH, bc. this is bad !! */
  if (!GF) {
    printf ("locateGFbyDataPtr: Could not locate GF to match %p\n",x);
    printf ("please use core file to debug. \n");
    CORE;
  }
  return (GF);
}




CCTK_REAL *GFDataByName(pGH *GH, char *name) 
{
  int index = CCTK_VarIndex(name);
  if (index < 0) {
    CCTK_WARN(0, "BAM_Elliptic: GFDataByName called with bad name");
    return 0;
  }
  if (0) {
    int storage = GFByIndex(GH, index)->storage;
    printf("GH %p, %d, %s  ", GH, index, name);
    if (!storage) printf("NO STORAGE");
    printf("\n");
  }
  return ((CCTK_REAL *) GFByIndex(GH, index)->data);
  /* new Cactus: return ((CCTK_REAL *) CCTK_VarDataPtrI(GH,0,index)); */
}




pGA *GFByIndex(pGH *GH, int index) 
{
  /* FIXME timelevels */
  int tl = 0;

  return ((pGA ***)GH->variables)[index][tl];
}



/* why not make these ints in the first place?! */
int GFI_psi, GFI_psix, GFI_psixx, GFI_alp, GFI_gxx, GFI_kxx;
int GFI_vl_bssn, GFI_vl_N1;
int GFI_v, GFI_lbeta11;
int GFI_phi, GFI_K, GFI_igxx, GFI_Kx, GFI_Ax, GFI_R;
int GFI_Ygxx, GFI_YABYxx;

void setGFIndices()
{
  GFI_psi = CCTK_VarIndex("staticconformal::psi");
  GFI_psix = CCTK_VarIndex("staticconformal::psix");
  GFI_psixx = CCTK_VarIndex("staticconformal::psixx");
  GFI_alp = CCTK_VarIndex("admbase::alp");
  GFI_gxx = CCTK_VarIndex("admbase::gxx");
  GFI_kxx = CCTK_VarIndex("admbase::kxx");

  if (CCTK_IsThornActive("BAM_VecLap")) {
    GFI_vl_bssn = CCTK_VarIndex("bam_veclap::vlgi11");
    GFI_vl_N1 = CCTK_VarIndex("bam_veclap::vl_N1");
  }
  if (CCTK_IsThornActive("BAM_IniDat")) {
    GFI_v = CCTK_VarIndex("bam_inidat::v");
    GFI_lbeta11 = CCTK_VarIndex("bam_inidat::lbeta11");

    GFI_phi = CCTK_VarIndex("bam_inidat::phi");

    GFI_igxx = CCTK_VarIndex("bam_inidat::igxx");
    GFI_Kx = CCTK_VarIndex("bam_inidat::Ku_x");
    GFI_Ax = CCTK_VarIndex("bam_inidat::Au_x");
    GFI_R = CCTK_VarIndex("bam_inidat::R");
    GFI_K = CCTK_VarIndex("bam_inidat::K");
    GFI_Ygxx = CCTK_VarIndex("bam_inidat::Ygxx");
    GFI_YABYxx = CCTK_VarIndex("bam_inidat::YABYxx");
  }
  if (CCTK_IsThornActive("IDAnalyticBH_PN")) {
    GFI_K = CCTK_VarIndex("bam_inidat::K");
  }
}





/* power of 2 magic for multigrid/BAM a la cactus 3 
   for multigrid, lower boundary points of octants etc are treated as ghosts 
   put them on single processor! 
*/ 
static char *BAM_GetSliceSize (int dir,
                               int nprocs,
                               int nsize,
                               int nghosts)
{
  DECLARE_CCTK_PARAMETERS
  int proc, step;
  char *slice_info;


  /* allocate partition slice string (assuming max 14 characters per number) */
  slice_info = (char *) malloc (15 * nprocs);

  if (slice_info)
  {
    if (! CCTK_Equals (domain, "octant") &&
        ! (CCTK_Equals (domain, "bitant") && dir == 2)) 
    {
      nghosts = 0;
    }

    /* subtract ghosts from nsize in case of symmetry boundary */
    step = (nsize - 1 - nghosts) / nprocs;

    /* first processor gets ghosts added in case of symmetry boundary */
    sprintf (slice_info, "%d", step + 1 + nghosts);

    /* middle processors get standard size */
    for (proc = 1; proc < nprocs-1; proc++)
    {
        sprintf (slice_info, "%s %d", slice_info, step);
    }

    /* last processor gets the rest */
    if (nprocs > 1)
      sprintf (slice_info, "%s %d", 
	       slice_info, nsize - (nprocs-1)*step - 1 - nghosts);

    if (0) printf("slice_info(dir=%d,nprocs=%d,nsize=%d,nghosts=%d) = %s\n", 
		  dir, nprocs, nsize, nghosts, slice_info);
  }

  return (slice_info);
}


void BAM_SetPartitionInfo (int dim, int nsizes[])
{
  int dir;
  int *nghosts, *nprocs;
  char **partition_info;


  nghosts = (int *) malloc (2 * dim * sizeof (int));
  nprocs = nghosts + dim;
  partition_info = (char **) malloc (dim * sizeof (char *));

  PUGH_GFGhostsize (dim, nghosts);
  PUGH_SetupDefaultTopology (dim, nprocs);

  /* compute the partitioning for every direction */
  for (dir = 0 ; dir < dim; dir++)
  {
    /* Check that there are enough grid points in this dimension
     * to make parallelising it worthwhile
     */
    if (nprocs[dir] != 0 && nsizes[dir] <= 2*nghosts[dir] + 1)
    {
      nprocs[dir] = 1;
    }
  }
  PUGH_GenerateTopology (dim, CCTK_nProcs (NULL), nprocs);

  for (dir = 0 ; dir < dim; dir++)
  {
    partition_info[dir] = BAM_GetSliceSize (dir, nprocs[dir], nsizes[dir],
                                            nghosts[dir]);
  }

  /* set the PUGH-internal partition information */
  PUGH_SetPartitionInfo (dim, partition_info);

  /* free allocated resources */
  for (dir = 0 ; dir < dim; dir++)
  {
    free (partition_info[dir]);
  }
  free (partition_info);
  free (nghosts);
}


/* set the partitioning for the initial grid */
void BAM_SetInitialGridPartitioning (void)
{
  int nsizes[3];


  /* get the processor topology */
  PUGH_GFSize (3, nsizes);

  /* set the PUGH-internal partition information */
  BAM_SetPartitionInfo (3, nsizes);
}
