/* thorn_BAM_Elliptic: Multigrid Elliptic Solver for Cactus
   Copyright (C) 1998 Bernd Bruegmann 
*/
/* PUGF_to_BAM.c */
/* Bernd Bruegmann, 10/97 */

#include "bbmg.h"



void setfieldpointers_gab(struct box *g, 
			  double **g11, double **g12, double **g13, 
			  double **g22, double **g23, double **g33)
{
  *g11 = GFDataByName(g->GH, "admbase::gxx");
  *g12 = GFDataByName(g->GH, "admbase::gxy");
  *g13 = GFDataByName(g->GH, "admbase::gxz");
  *g22 = GFDataByName(g->GH, "admbase::gyy");
  *g23 = GFDataByName(g->GH, "admbase::gyz");
  *g33 = GFDataByName(g->GH, "admbase::gzz");
}




void setfieldpointers_Kab(struct box *g,
			  double **K11, double **K12, double **K13, 
			  double **K22, double **K23, double **K33)
{
  *K11 = GFDataByName(g->GH, "admbase::kxx");
  *K12 = GFDataByName(g->GH, "admbase::kxy");
  *K13 = GFDataByName(g->GH, "admbase::kxz");
  *K22 = GFDataByName(g->GH, "admbase::kyy");
  *K23 = GFDataByName(g->GH, "admbase::kyz");
  *K33 = GFDataByName(g->GH, "admbase::kzz");
}




void setfieldpointers_psi(struct box *g, double **psi, 
       double **dpsiopsi1, double **dpsiopsi2, double **dpsiopsi3)
{
  *psi = GFDataByName(g->GH, "staticconformal::psi");
  *dpsiopsi1 = GFDataByName(g->GH, "staticconformal::psix");
  *dpsiopsi2 = GFDataByName(g->GH, "staticconformal::psiy");
  *dpsiopsi3 = GFDataByName(g->GH, "staticconformal::psiz");
}




void setfieldpointers_psi10(struct box *g, double **p) 
{
  *p++ = GFDataByName(g->GH, "staticconformal::psi");
  *p++ = GFDataByName(g->GH, "staticconformal::psix");
  *p++ = GFDataByName(g->GH, "staticconformal::psiy");
  *p++ = GFDataByName(g->GH, "staticconformal::psiz");
  *p++ = GFDataByName(g->GH, "staticconformal::psixx");
  *p++ = GFDataByName(g->GH, "staticconformal::psixy");
  *p++ = GFDataByName(g->GH, "staticconformal::psixz");
  *p++ = GFDataByName(g->GH, "staticconformal::psiyy");
  *p++ = GFDataByName(g->GH, "staticconformal::psiyz");
  *p++ = GFDataByName(g->GH, "staticconformal::psizz");
}




void setfieldpointers_gKalpha(struct box *g, double **u)
{
  *(u++) = GFDataByName(g->GH, "admbase::gxx");
  *(u++) = GFDataByName(g->GH, "admbase::gxy");
  *(u++) = GFDataByName(g->GH, "admbase::gxz");
  *(u++) = GFDataByName(g->GH, "admbase::gyy");
  *(u++) = GFDataByName(g->GH, "admbase::gyz");
  *(u++) = GFDataByName(g->GH, "admbase::gzz");
  *(u++) = GFDataByName(g->GH, "admbase::kxx");
  *(u++) = GFDataByName(g->GH, "admbase::kxy");
  *(u++) = GFDataByName(g->GH, "admbase::kxz");
  *(u++) = GFDataByName(g->GH, "admbase::kyy");
  *(u++) = GFDataByName(g->GH, "admbase::kyz");
  *(u++) = GFDataByName(g->GH, "admbase::kzz");
  *(u++) = GFDataByName(g->GH, "admbase::alp");
  *(u++) = GFDataByName(g->GH, "staticconformal::psi");
  *(u++) = GFDataByName(g->GH, "staticconformal::psix");
  *(u++) = GFDataByName(g->GH, "staticconformal::psiy");
  *(u++) = GFDataByName(g->GH, "staticconformal::psiz");
}




void setfieldpointers_gi(struct box *g, 
			 double **gi11, double **gi12, double **gi13, 
			 double **gi22, double **gi23, double **gi33)
{
  int i = MGF*g->ncomps + 2;

  *gi11 = g->store[i++];
  *gi12 = g->store[i++];
  *gi13 = g->store[i++];
  *gi22 = g->store[i++];
  *gi23 = g->store[i++];
  *gi33 = g->store[i++];
}





void setfieldpointers_gg(struct box *g, 
			 double **gg1, double **gg2, double **gg3)
{
  int i = MGF*g->ncomps + 8;

  *gg1  = g->store[i++];
  *gg2  = g->store[i++];
  *gg3  = g->store[i++];
}




/* inlined by hand into stencil.c, just to be sure ... */
void setfieldpointers_linear(struct box *g, double **M, double **N,
			     double **gi11, double **gi12, double **gi13, 
			     double **gi22, double **gi23, double **gi33,
			     double **gg1, double **gg2, double **gg3)
{
  int i = MGF*g->ncomps;

  *M = g->store[i++];
  *N = g->store[i++];
  *gi11 = g->store[i++];
  *gi12 = g->store[i++];
  *gi13 = g->store[i++];
  *gi22 = g->store[i++];
  *gi23 = g->store[i++];
  *gi33 = g->store[i++];
  *gg1  = g->store[i++];
  *gg2  = g->store[i++];
  *gg3  = g->store[i++];
}





void setfieldpointers_flatlinear(struct box *g, double **M, double **N)
{
  int i = MGF*g->ncomps;

  *M = g->store[i++];
  *N = g->store[i++];
}


