/* thorn_BAM_Elliptic: Multigrid Elliptic Solver for Cactus
   Copyright (C) 1998 Bernd Bruegmann
*/
/* bbmg.h */
/* Bernd Bruegmann, 4/97 */

#ifndef _BBMG_H_
#define _BBMG_H_

#include "cctk.h"
#include "CactusPUGH/PUGH/src/include/pugh.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>


/* debugging level */
#define bbmgPR 0



/* ARGH, cactus 4 removed this from PUGH for no good reason */
typedef struct pGHLIST {
  pGH *GH;
  struct pGHLIST *next;
} pGHList;
extern pGHList *GHList;
void addtoGHList(pGH *GH);
void initGHList(pGH *GH);
void removefromGHList(pGH *GH);
pGH *GHbyLevel(int, int, int);

/* collides with cactus 4, check whether long was really used somewhere!
#define int long
*/

/* more cactus 4 casualties */
pGA *locateGFbyDataPtr(CCTK_REAL *x);
CCTK_REAL *GFDataByName(pGH *GH, char *name);
pGA *GFByIndex(pGH *GH, int index);
void setGFIndices();
extern int conformal_metric;
extern int GFI_psi, GFI_psix, GFI_psixx;
extern int GFI_alp, GFI_gxx, GFI_kxx;
extern int GFI_vl_bssn, GFI_vl_N1;
extern int GFI_v, GFI_lbeta11;
extern int GFI_phi, GFI_K, GFI_igxx, GFI_Kx, GFI_Ax, GFI_R;
extern int GFI_Ygxx, GFI_YABYxx;

/* quick fix */

#ifdef SINGLE_PRECISION
#define double float
#endif



/* translate grid structures */
#define box bbmg_box
#define mgstack PGH

/* resolve multiple instances of names */
#define fieldtitle bbmg_fieldtitle
#define l2norm bbmg_l2norm
#define setboundrobin bbmg_setboundrobin
#define outxmin bbmg_outxmin
#define outymin bbmg_outymin
#define outzmin bbmg_outzmin
#define outxmax bbmg_outxmax
#define outymax bbmg_outymax
#define outzmax bbmg_outzmax
#define errorexit bbmg_errorexit
#define yo bbmg_yo
#define prprim bbmg_prprim
#define prprim2 bbmg_prprim2
#define prprimnflag bbmg_prprimnflag
#define ipolr2o1 bbmg_ipolr2o1
#define setoverlap1d bbmg_setoverlap1d
#define setoverlap bbmg_setoverlap
#define mgrid_restrict bbmg_restrict
#define problem bam_elliptic_problem
#define eqn_type bam_elliptic_eqn_type

#define UNKNOWN    -1
#define NONE        0
#define POISSON     1
#define BRILL       2
#define BHDATA      3
#define MAXIMAL     4

#define FLATLINEAR  101
#define LINEAR      102
#define POWERS      103
#define BRBR        104
#define VECLAPLACE  105
#define VEC4        106
#define TSD         107
#define KSD         108
#define YORK        109

#define BOUND_SCALAR 201
#define BOUND_VECX   202
#define BOUND_VECY   203
#define BOUND_VECZ   204
#define BOUND_TENXX  205
#define BOUND_TENXY  206
#define BOUND_TENXZ  207
#define BOUND_TENYY  208
#define BOUND_TENYZ  209
#define BOUND_TENZZ  210


#define REFBOUND  0
#define NEUMANN   1
#define DIRICHLET 2
#define USEONEBOUND 3       /* Hisaaki added */

#define MGFMAX 40           /* maximal number of fields */
#define MGLMAX 100          /* maximal number of levels */
#define MGCMAX 6            /* maximal number of components */
#define MGF    4            /* number of MG fields, e.g. u, v, w, f */

#define dequaleps 1e-9
#define dless(x,y) ((x)<(y)-dequaleps)
#define dleeq(x,y) ((x)-(y)<dequaleps)
#define dequal(x,y) (dleeq(x,y)&&dleeq(y,x))
#define max(a,b) (((a)<(b))?(b):(a))
#define min(a,b) (((a)<(b))?(a):(b))
#define intof(x,x0,d) ((int)(floor(0.5+((x)-(x0))/(d))))
#define intof2(x,x0,d) ((int)(floor(dequaleps+((x)-(x0))/(d))))
#define doubleof(i,x0,d) ((x0)+(d)*(i))


#ifdef PI
#undef PI
#endif
#define PI 3.14159265358979323846




/* the box */
struct box {
  /* standard bbmg variables: set to the GLOBAL quantities */
  /* we use GLOBAL here so that messy setrange operations don't change */
  struct box *next;         /* for one-sided linked lists */
  double x, y, z;           /* minimal coordinates including ghosts */
  double xmax, ymax, zmax;  /* maximal coordinates including ghosts */
  double d;                 /* uniform grid spacing for now */
  double t;                 /* time */
  int l;                    /* level of this box */
  int ltop, lmax;           /* top level and maximal level for stack */
  int m, n, o;              /* box has range [0,..,m-1], ... */
  int di, dj, dk;           /* linear indexing offsets */
  int imin, jmin, kmin;     /* active subbox */
  int imax, jmax, kmax;
  int bi0, bj0, bk0;        /* boundary flags */
  int bi1, bj1, bk1;
  int nfields;              /* number of scalar fields */
  int ncomps;               /* number of components of mgfields */
  double dx, dy, dz;        /* non-uniform grid spacing coming up */
  double dxm, dym, dzm;     /* reflection ghostzone gridspacing */
  int reffac;               /* absolute refinement factor, 2^|l-lfinest| */
  double *store[MGFMAX];    /* store[nf] = linear field storage */
  double ppow[MGFMAX];      /* exponents for power series, zero terminated */

  /* non-standard variables: LOCAL quantities (for domain decomp) */
  double lx, ly, lz;
  double lxmax, lymax, lzmax;
  int lm, ln, lo;
  int limin, ljmin, lkmin;
  int limax, ljmax, lkmax;
  int lijk;

  /* PUGH stuff so that we can call e.g. Double pGF_Norm2(pGH *GH, pGF *GF) */
  cGH *cctkGH;
  pGH *GH;
  pGA *GF[MGFMAX];          /* grid functions corresponding to store[] */
};



/* implement sweeps for linked list of rectangular grids */

#define forlevel(l,g)\
  for (g = level[l]->next, setrangeall(g), setmgind(g,&u,&v,&w,&f); g;\
       g = g->next, setrangeall(g), setmgind(g,&u,&v,&w,&f))

#define foroverlap(lh,gh,lH,gH)\
  forlevel(lH,gH)\
  forlevel(lh,gh)\
  if (setoverlap(gh,gH)) continue; else

#define gijkof(g,i,j,k) \
  ((i)*g->di + (j)*g->dj + (k)*g->dk - g->lijk)

#define ijkof(i,j,k) \
  ((i)*di + (j)*dj + (k)*dk - lijk)


#define REVERSE_INDEXING
#ifdef REVERSE_INDEXING

/* FORTRAN order */
#define forrange(g) \
  for (k = g->lkmin; k <= g->lkmax; k++) \
  for (j = g->ljmin; j <= g->ljmax; j++) \
  for (i = g->limin, ijk = gijkof(g,i,j,k); i <= g->limax; i++, ijk++)

#define forrangefc(g,gc) \
  for (k = g->lkmin, kc = gc->lkmin; k <= g->lkmax; k+=2, kc++) \
  for (j = g->ljmin, jc = gc->ljmin; j <= g->ljmax; j+=2, jc++) \
  for (i = g->limin, ic = gc->limin; i <= g->limax; i+=2, ic++)

#define ijkto(i,j,k) \
{int lk, lj, li; \
 lk = lijk/dk; lj = (lijk-lk*dk)/dj; li = (lijk-lk*dk-lj*dj)/di; \
  k =  ijk/dk;  j = ( ijk- k*dk)/dj;  i = ( ijk- k*dk- j*dj)/di; \
  k += lk; j += lj; i += li; \
}
/* note that this is different from
{k = (ijk+lijk)/dk; j = (ijk+lijk-k*dk)/dj; i = (ijk+lijk-k*dk-j*dj)/di;}
*/

#else

/* C order */
#define forrange(r) \
  for (i = r->imin; i <= r->imax; i++) \
  for (j = r->jmin; j <= r->jmax; j++) \
  for (k = r->kmin, ijk = gijkof(r,i,j,k); k <= r->kmax; k++, ijk++)

#define forrangefc(r,rc) \
  for (i = r->imin, ic = rc->imin; i <= r->imax; i+=2, ic++) \
  for (j = r->jmin, jc = rc->jmin; j <= r->jmax; j+=2, jc++) \
  for (k = r->kmin, kc = rc->kmin; k <= r->kmax; k+=2, kc++)

#endif



/* externals */
extern struct box *level[MGLMAX];
extern char *fieldtitle[MGFMAX];
extern double outxmin, outymin, outzmin, outxmax, outymax, outzmax;
extern double tolres2, tolresi, factau2, factaui;
extern double solve_normmax, wcycle_factol;
extern int solve_nmin, solve_nmax, wcycle_nmin, wcycle_nmax;
extern int gridminsize, gridmaxlevels, mincycles, maxcycles;
extern int problem, eqn_type, veclaplacebssn, vb, vbell, fullV, weightedinit;
extern int rb_decouple, do_bam_relax, fisheye;
extern double fisheyedx, fisheyedy, fisheyedz;
extern int cartoon, cartoon_stencil, stencil_symmetric, neumann, masks;
extern int octant, octant_order, bitant, bitant_order, octoffset, bitoffset;
extern int ox, oy, oz, oxdo, oydo, ozdo, oxup, oyup, ozup;
extern double robin_limit, robin_power, robin_limit_vec1, robin_power_vec1;
extern double robin_limit_vec2, robin_power_vec2;
extern double robin_limit_vec3, robin_power_vec3;
extern double plane_sx,plane_sy,plane_sz,plane_sx_vec1;
extern double plane_sy_vec1,plane_sz_vec1;
extern double plane_sx_vec2,plane_sy_vec2,plane_sz_vec2;
extern double plane_sx_vec3,plane_sy_vec3,plane_sz_vec3;



/* algebra.c */
void setconst(struct box *g, int nfa, double c);
void copy(struct box *g, int nfa, int nfr);
void add(struct box *g, int nfa, int nfb, int nfr);
void subtract(struct box *g, int nfa, int nfb, int nfr);
double l2norm(struct box *g, int nfa);
double linfnorm(struct box *g, int nfa);
void setconst_scalar(struct box *g, int nfa, double c);
void copy_scalar(struct box *g, int nfa, int nfr);
void add_scalar(struct box *g, int nfa, int nfb, int nfr);
void subtract_scalar(struct box *g, int nfa, int nfb, int nfr);
double l2norm_scalar(struct box *g, int nfa);
double linfnorm_scalar(struct box *g, int nfa);

/* bound.c */
void setouterboxsize(int ltop);
void setboundflags(struct box *g);
void setboundrobin(struct box *g, int nfa);
void setboundrobin_scalar(struct box *g, int nf,
   double sx, double sy, double sz,double rob_lim, double rob_pow);
void setob_0(pGH *GH, int n);
void setob_1(pGH *GH, int n);
void setob_2s(pGH *GH, int n);

/* driver.c */
void errorexit(char *s);
void yo(void);
void bbmg_elliptic_solve(int bbmg_eqn_type, int bbmg_init, int bbmg_method,
			 int bbmg_npre, int bbmg_npost, int bbmg_ncycles,
			 int ltop, int lmax, struct mgstack **s);

/* ellops.c */
void exactsolve(int l);
void lop(struct box *g, int u, int v);
void rb_gauss_seidel(struct box *g, int u, int v, int f);
void relax(int l, int nsweeps);

/* fasmg.c */
void coarsetofine(int lH, int lh);
int finetocoarse(int lh, int lH);
void solve(int l, int lmax, int npre, int npost);
void free_mgstack(void);
void setmgind(struct box *g, int *u, int *v, int *w, int *f);
double findres(int p, struct box *gh);
void restau(int lh, double *res2, double *resi, double *tau2, double *taui,
	    int p);
void prresiduals(int lmax);
void prnormres(int lh);

/* print.c */
void prprim(char *s, struct box *b, int nfield);
void prprimin(char *s, struct box *b, int nfield);
void printrestau(int l, double normres, double normtau);
void prrange(char *s, struct box *gh);
void prboxpar(struct box *g);

/* prolong.c */
void prolong(struct box *rc, int uc, struct box *r, int u);

/* range.c */
int setoverlap(struct box *gh, struct box *gH);
void setrangeall(struct box *g);
void shrinkrange(struct box *g);
void shrinkoverlap(struct box *gh, struct box *gH);
void setrangeftoc(struct box *gh, struct box *gH);
void setrangectof(struct box *gH, struct box *gh);

/* restrict.c */
void mgrid_restrict(struct box *r, int u, struct box *rc, int uc);
void injectftoc(struct box *r, int u, struct box *rc, int uc);
void injectftoc_scalar(struct box *r, struct box *rc, double *u, double *uc);

/* stencil.c */
void stencil(struct box *g, int u, int ijk, double *lu, double *lii);


/****************************************************************************/

/* admcon.c */
void admcon(struct box *g,
	    double *g11, double *g12, double *g13,
	    double *g22, double *g23, double *g33,
	    double *K11, double *K12, double *K13,
	    double *K22, double *K23, double *K33,
	    double *hamil, double *diffeo1, double *diffeo2, double *diffeo3);

/* bhdata */
void make_mgstack_bhdata(int ltop, int lmax, struct mgstack **s);
void init_bhdata(int init, int ltop, int lmax, struct mgstack **s);
void init_bhdata_box(struct box *g);
void stencil_bhdata(struct box *g, int nfu, int ijk, double *lu, double *lii);
void setfieldpointers_bhdata(struct box * g, double **mu, double **K2);

/* brill.c */
void setnames_brill(void);
void prinfo_brill(void);
void setfieldpointers_brill(struct box *g, double **M, double **N);
void init_brill_box(struct box *g);
void init_brill(int init, int ltop, int lmax, struct mgstack **s);
extern double brill_a, brill_b, brill_c, brill_BHmass;
extern double brill_rho0, brill_r0, brill_srho, brill_sr, brill_sz;
extern double brill_dfindif;
extern int brill_q;

/* brbr.c */
#define N_MWBL 30
extern double MBL[N_MWBL], CBL[N_MWBL][3], PBL[N_MWBL][3], SBL[N_MWBL][3];
extern int dalota;
void set_mwbl_box(struct box *g);
void set_mwbl_GH(pGH *GH);
void set_bbalpha_GH(pGH *GH);
void setnames_brbr(void);
void prinfo_brbr(void);
void setfieldpointers_brbr(struct box *g, double **M, double **N);
void init_brbr_box(struct box *g);
void init_brbr(int init, int ltop, int lmax, struct mgstack **s);
void set_brbr_box(struct box *g);
void set_brbr(int ltop, int lmax, struct mgstack **s);

/* coeffs.c */
void set_laplace(struct box *g);
void set_laplace_sym(struct box *g);
void set_K2(struct box *g, double *h, double *driver);

/* maxslice.c */
void setnames_maximal(void);
void init_mg_box(struct box *g);
void init_maximal_box(struct box *g);
void init_maximal_lapse(struct box *g, int onlyfaces);
void init_maxslice(int init, int ltop, int lmax, struct mgstack **s);
void restrict_g(int ltop);
void restrict_g_PUGH(cGH *cctkGH,int ltop);
void restrict_MN(int ltop);
void init_linear(int init, int ltop, int lmax, struct mgstack **s);
void setnames_linear(void);
extern double maxslice_epsdriver;

/* misner.c */
void set_misner(cGH *cctkGH);

/* poisson.c */
void setnames_poisson(void);
void setfieldpointers_poisson(struct box *g, double **lin, double **rho);
void init_poisson_box(struct box *g);
void init_poisson(int init, int ltop, int lmax, struct mgstack **s);

/* powers.c */
void init_powers(int init, int ltop, int lmax, struct mgstack **s);
void setnames_powers(int n);
extern double ppowers[MGFMAX];

/* simpleexcision.c */
void iniboundmask(cGH *);
void setboundmask(struct box *g, int nf, int nonzero);
void setboundmask_scalar(struct box *g, int nf, int nonzero, double *cmask,
			 double (*se)(struct box *, double *,
				      double, double, double),
			 int idx);
/* veclaplace.c */
void setnames_vecL(void);
void restrict_g_PUGH_vecL(int ltop);
void restrict_N_vecL(int ltop);
void init_mg_box_vecL(struct box *g);
void init_box_vecL(struct box *g);
void init_vecL(int init, int ltop, int lmax, struct mgstack **s);
int checkfornan(struct box *g, int ng, int nGH);
int checkfornanGF(pGA *GF);
void nan_vecL(struct box *g);



/****************************************************************************/

/* PUGH_to_BAM.c */
void BAM_translatestack(cGH *cctkGH, int ltop, int lmax, struct mgstack **s,
			int *nf, int n, int nws);
void BAM_freelevels(void);
void setrangePUGH(struct box *g, int onlyowned);
void PUGHsyncGF(pGA *GF);
void PUGHsyncd(double *u);
void PUGHsyncn(struct box *g, int nf, int n);
void PUGHsyncGFn(struct box *g, int nGF, int n);
void PUGHsyncvec(struct box *g, int nf);
void PUGHsyncall(struct box *g);
void prprimGF(char *s, struct box *g);

/* PUGF_to_BAM.c */
void setfieldpointers_gab(struct box *g,
			  double **g11, double **g12, double **g13,
			  double **g22, double **g23, double **g33);
void setfieldpointers_Kab(struct box *g,
			  double **K11, double **K12, double **K13,
			  double **K22, double **K23, double **K33);
void setfieldpointers_psi(struct box *g, double **psi,
       double **dpsiopsi1, double **dpsiopsi2, double **dpsiopsi3);
void setfieldpointers_gi(struct box *g,
			 double **gi11, double **gi12, double **gi13,
			 double **gi22, double **gi23, double **gi33);
void setfieldpointers_gg(struct box *g,
			 double **gg1, double **gg2, double **gg3);
void setfieldpointers_linear(struct box *g, double **M, double **N,
			     double **gi11, double **gi12, double **gi13,
			     double **gi22, double **gi23, double **gi33,
			     double **gg1, double **gg2, double **gg3);
void setfieldpointers_flatlinear(struct box *g, double **M, double **N);
void setfieldpointers_gKalpha(struct box *g, double **u);

/* PUGH_MakeStack.c */
void PUGH_makestack(cGH *cctkGH, pGH *GH, int *nf, int n, int nws);
void PUGH_destroystack(cGH *cctkGH, pGH *GH, int *nf, int n, int nws);

/* fortran.c */
void setflat(pGH *GH);
void setBM(pGH *GH);
void set_laplace_BMcactus(pGH *GH);
void setob(pGH *GH, int n, double symx, double symy, double symz);

#endif /* _BBMG_H_ */
