/* thorn_BAM_Elliptic: Multigrid Elliptic Solver for Cactus
   Copyright (C) 1998 Bernd Bruegmann 
*/
/* bound.c */
/* Bernd Bruegmann, 4/97 */

#include "bbmg.h"

void prboundtype(struct box *g);
double robin(struct box *g, double *u,
             double sx, double sy, double sz, double rob_lim,double rob_pow,
             int i, int j, int k, int ioff, int joff, int koff);
void setneumannflags(struct box *g, int *bi0, int *bj0, int *bk0, 
		     int *bi1, int *bj1, int *bk1);



/* size of top level box WITHOUT ghosts */
/* struct box gives size WITH ghosts */
double outxmin, outymin, outzmin, outxmax, outymax, outzmax;



/* print boundary type */  /* Hisaaki modified */
void prboundtype(struct box *g)
{
  printf("bound cond, l%d: ", g->l);
  printf("%6s ", (g->bi0==DIRICHLET)?"DIR":
	         (g->bi0==NEUMANN)?"NEU":
	         (g->bi0==USEONEBOUND)?"ONE":"REF");
  printf("%6s ", (g->bj0==DIRICHLET)?"DIR":
                 (g->bj0==NEUMANN)?"NEU":
                 (g->bi0==USEONEBOUND)?"ONE":"REF");
  printf("%6s ", (g->bk0==DIRICHLET)?"DIR":
                 (g->bk0==NEUMANN)?"NEU":
	         (g->bi0==USEONEBOUND)?"ONE":"REF");
  printf(" ");
  printf("%6s ", (g->bi1==DIRICHLET)?"DIR":
                 (g->bi1==NEUMANN)?"NEU":
	         (g->bi0==USEONEBOUND)?"ONE":"REF");
  printf("%6s ", (g->bj1==DIRICHLET)?"DIR":
                 (g->bj1==NEUMANN)?"NEU":
	         (g->bi0==USEONEBOUND)?"ONE":"REF");
  printf("%6s ", (g->bk1==DIRICHLET)?"DIR":
                 (g->bk1==NEUMANN)?"NEU":
        	 (g->bi0==USEONEBOUND)?"ONE":"REF");
  printf("\n");
}



/* set outer box size
   ghosts are NOT counted
   not called in PUGH
*/
void setouterboxsize(int ltop)
{
  struct box *g = level[ltop]->next;

  if (1) {
    outxmin = g->x;
    outymin = g->y;
    outzmin = g->z;
    outxmax = g->xmax;
    outymax = g->ymax;
    outzmax = g->zmax;
  }

  /* correct for ghosts */
  if (g->bi0 == NEUMANN) outxmin += g->d;
  if (g->bj0 == NEUMANN) outymin += g->d;
  if (g->bk0 == NEUMANN) outzmin += g->d;
  if (g->bi1 == NEUMANN) outxmax -= g->d;
  if (g->bj1 == NEUMANN) outymax -= g->d;
  if (g->bk1 == NEUMANN) outzmax -= g->d;

  /* print info */
  if (0) {
    printf("\n");
    printf("outer box:  %6.3f %6.3f %6.3f  %6.3f %6.3f %6.3f\n",
	   outxmin, outymin, outzmin, outxmax, outymax, outzmax);
    prboundtype(g);
    printf("\n");
  }
}



/* set boundary flags based on outer box and box size
   ghosts are NOT counted
*/
void setboundflags(struct box *g) 
{

  DECLARE_CCTK_PARAMETERS

  if      (dless(g->x, outxmin)) g->bi0 = NEUMANN;
  else if (dless(outxmin, g->x)) g->bi0 = REFBOUND;
  else                           g->bi0 = DIRICHLET;

  if      (dless(g->y, outymin)) g->bj0 = NEUMANN;
  else if (dless(outymin, g->y)) g->bj0 = REFBOUND;
  else                           g->bj0 = DIRICHLET;

  if      (dless(g->z, outzmin)) g->bk0 = NEUMANN;
  else if (dless(outzmin, g->z)) g->bk0 = REFBOUND;
  else                           g->bk0 = DIRICHLET;

  if      (dless(g->xmax, outxmax)) g->bi1 = REFBOUND;
  else if (dless(outxmax, g->xmax)) g->bi1 = NEUMANN;
  else                              g->bi1 = DIRICHLET;

  if      (dless(g->ymax, outymax)) g->bj1 = REFBOUND;
  else if (dless(outymax, g->ymax)) g->bj1 = NEUMANN;
  else                              g->bj1 = DIRICHLET;

  if      (dless(g->zmax, outzmax)) g->bk1 = REFBOUND;
  else if (dless(outzmax, g->zmax)) g->bk1 = NEUMANN;
  else                              g->bk1 = DIRICHLET;

  if (0) prboundtype(g);

  /* Hisaaki added --> */
  if(CCTK_Equals(type,"customBAM") || customizedBCmode == 1){
    g->bi0 = USEONEBOUND;
    g->bj0 = USEONEBOUND;
    g->bk0 = USEONEBOUND;
    g->bi1 = USEONEBOUND;
    g->bj1 = USEONEBOUND;
    g->bk1 = USEONEBOUND;
  }
  /* Hisaaki added <-- */
}




void setneumannflags(struct box *g, int *bi0, int *bj0, int *bk0, 
		     int *bi1, int *bj1, int *bk1)
{ 
  DECLARE_CCTK_PARAMETERS

  *bi0 = (g->bi0 == NEUMANN) ? 1 : 0;
  *bi1 = (g->bi1 == NEUMANN) ? 1 : 0;

  *bj0 = (g->bj0 == NEUMANN) ? 1 : 0;
  *bj1 = (g->bj1 == NEUMANN) ? 1 : 0;

  *bk0 = (g->bk0 == NEUMANN) ? 1 : 0;
  *bk1 = (g->bk1 == NEUMANN) ? 1 : 0;

  if (cartoon) {
    *bi0 = 0;
    *bj0 = 0;
    *bj1 = 0;
  }

  /* Hisaaki added --> */
  if(CCTK_Equals(type,"customBAM") || customizedBCmode == 1){
    *bi0 = 1;
    *bi1 = 1;
    *bj0 = 1;
    *bj1 = 1;
    *bk0 = 1;
    *bk1 = 1;
  }
  /* Hisaaki added <-- */
}





/* tensor interfaces to thorn_StandardEinstein's onebound */
#define eve  1.0
#define odd -1.0

void setob_0(pGH *GH, int n)
{
  setob(GH, n, eve, eve, eve);
}
void setob_1(pGH *GH, int n)
{
  setob(GH, n++, odd, eve, eve);
  setob(GH, n++, eve, odd, eve);
  setob(GH, n++, eve, eve, odd);
}  
void setob_2s(pGH *GH, int n)
{
  setob(GH, n++, eve, eve, eve);
  setob(GH, n++, odd, odd, eve);
  setob(GH, n++, odd, eve, odd);
  setob(GH, n++, eve, eve, eve);
  setob(GH, n++, eve, odd, odd);
  setob(GH, n++, eve, eve, eve);
}




/* set Robin boundary condition */
/* - especially for vectors, boundary treatment becomes problem dependent
   - but it is simple to replace robin() with say extrapolate(), wave(), ...
   - supports mixed boundaries, weakly tested
*/
void setboundrobin(struct box *g, int nfa)
{

  DECLARE_CCTK_PARAMETERS

  int pr = 0;
  int nf;
  double sx, sy, sz, rob_pow, rob_lim;

  sx = plane_sx;
  sy = plane_sy;
  sz = plane_sz;
  rob_pow = robin_power;
  rob_lim = robin_limit;

  /* for all components */
  for (nf = 0; nf < g->ncomps; nf++) {

    /* fix up vector symmetries and robin conditions */
    if (problem == VECLAPLACE || problem == VEC4 || 
	problem == TSD || problem == KSD || problem == YORK) {  
      int o = 0;
      if (problem == VEC4) o = 1;
      if (problem == KSD)  o = 1;
      if (problem == YORK) o = 1;
      if (problem == TSD)  o = 2;

      if (nf == 0+o) {  
	 sx = plane_sx_vec1;
	 sy = plane_sy_vec1;
	 sz = plane_sz_vec1;
	 rob_pow = robin_power_vec1;
	 rob_lim = robin_limit_vec1;
      }
      if (nf == 1+o) {  
	sx = plane_sx_vec2;
	sy = plane_sy_vec2;
	sz = plane_sz_vec2;
	rob_pow = robin_power_vec2;
	rob_lim = robin_limit_vec2;
      }
      if (nf == 2+o) {  
	sx = plane_sx_vec3;
	sy = plane_sy_vec3;
	sz = plane_sz_vec3;
	rob_pow = robin_power_vec3;
	rob_lim = robin_limit_vec3;
      }
    }  

    /* set boundaries that need update after each sweep */
    if (pr) prprim("before setbound", g, nf+nfa);
    setboundrobin_scalar(g, nf+nfa, sx,sy,sz,rob_lim,rob_pow);
    if (pr) prprim("after  setbound", g, nf+nfa);

    /* Hisaaki added from here -> */
    if (CCTK_Equals(type,"customBAM") || customizedBCmode == 1) {

#ifdef FIXME
#define oneboundfromC FORTRAN_NAME(onebound_, ONEBOUND, onebound)
/*             call onebound(ARENA, var,one,one,one)  */
#define setNeuBCfromC FORTRAN_NAME(setneumannbc_, SETNEUMANNBC, setNeumannBC)
/*             call setNeumannBC(ARENA,phi_solve)     */
      double *u = g->store[nfa];
      CCTK_REAL one = 1.0;
       /* printf(" welcome to His in BAM : %f \n",
	  g->GF[nfa]->data[DATINDEX(g->GH,0,0,0)]); */
       SetupFortranArrays(g->GH);
       oneboundfromC(CREATE_ARENAVARS_ARGS(g->GH),u,&one,&one,&one);
       /*
       setNeuBCfromC(CREATE_ARENAVARS_ARGS(g->GH),u);
       oneboundfromC(CREATE_ARENAVARS_ARGS(GH),GF->data,&symx, &symy, &symz);
       */
#endif
    }
    /* Hisaaki added <--  */
  }

#if 0 /* Cartoon support seems to be broken here -- eg. use of 
	 Cartoon2DBCVar(). [dp] */

/* Tensor types (hack) */
#define TENSOR_TYPE_SCALAR  0
#define TENSOR_TYPE_U       1

  if (cartoon) {
    int pr2 = 0;

    CCTK_INT Cartoon2DBCVar(cGH *GH, 
			    CCTK_INT tensortype, const char *impvarname);

    if (pr2) prprim("before sba", g, nfa);
    if (g->ncomps == 1)
      Cartoon2DBCVar(g->cctkGH, TENSOR_TYPE_SCALAR, g->GF[nfa]->name);
    if (problem == VECLAPLACE)
      Cartoon2DBCVar(g->cctkGH, TENSOR_TYPE_U, g->GF[nfa]->name);
    if (problem == VEC4)
      errorexit("bound.c: implement VEC4");
    if (problem == KSD)
      errorexit("bound.c: implement KSD");
    if (problem == YORK)
      errorexit("bound.c: implement YORK");
    if (problem == TSD)
      errorexit("bound.c: implement TSD");
    if (pr2) prprim("after sba", g, nfa);
  }
#endif

  /* excision */
  setboundmask(g, nfa, 1);
}






#define forfacei0 \
   if (imin == limin) \
   for (i = limin; i <= limin; i++) \
   for (j = ljmin; j <= ljmax; j++) \
   for (k = lkmin; k <= lkmax; k++) \

#define forfacei1 \
   if (imax == limax) \
   for (i = limax; i <= limax; i++) \
   for (j = ljmin; j <= ljmax; j++) \
   for (k = lkmin; k <= lkmax; k++) \


#define forfacej0 \
   if (jmin == ljmin) \
   for (i = limin; i <= limax; i++) \
   for (j = ljmin; j <= ljmin; j++) \
   for (k = lkmin; k <= lkmax; k++) \

#define forfacej1 \
   if (jmax == ljmax) \
   for (i = limin; i <= limax; i++) \
   for (j = ljmax; j <= ljmax; j++) \
   for (k = lkmin; k <= lkmax; k++) \


#define forfacek0 \
   if (kmin == lkmin) \
   for (i = limin; i <= limax; i++) \
   for (j = ljmin; j <= ljmax; j++) \
   for (k = lkmin; k <= lkmin; k++) \

#define forfacek1 \
   if (kmax == lkmax) \
   for (i = limin; i <= limax; i++) \
   for (j = ljmin; j <= ljmax; j++) \
   for (k = lkmax; k <= lkmax; k++) \


#define foredgei0j0 \
   if (imin == limin && jmin == ljmin) \
     for (i = limin, j = ljmin, k = lkmin; k <= lkmax; k++)
#define foredgei0j1 \
   if (imin == limin && jmax == ljmax) \
     for (i = limin, j = ljmax, k = lkmin; k <= lkmax; k++)
#define foredgei1j0 \
   if (imax == limax && jmin == ljmin) \
     for (i = limax, j = ljmin, k = lkmin; k <= lkmax; k++)
#define foredgei1j1 \
   if (imax == limax && jmax == ljmax) \
     for (i = limax, j = ljmax, k = lkmin; k <= lkmax; k++)

#define foredgei0k0 \
   if (imin == limin && kmin == lkmin) \
     for (i = limin, j = ljmin, k = lkmin; j <= ljmax; j++)
#define foredgei0k1 \
   if (imin == limin && kmax == lkmax) \
     for (i = limin, j = ljmin, k = lkmax; j <= ljmax; j++)
#define foredgei1k0 \
   if (imax == limax && kmin == lkmin) \
     for (i = limax, j = ljmin, k = lkmin; j <= ljmax; j++)
#define foredgei1k1 \
   if (imax == limax && kmax == lkmax) \
     for (i = limax, j = ljmin, k = lkmax; j <= ljmax; j++)

#define foredgej0k0 \
   if (jmin == ljmin && kmin == lkmin) \
     for (i = limin, j = ljmin, k = lkmin; i <= limax; i++)
#define foredgej0k1 \
   if (jmin == ljmin && kmax == lkmax) \
     for (i = limin, j = ljmin, k = lkmax; i <= limax; i++)
#define foredgej1k0 \
   if (jmax == ljmax && kmin == lkmin) \
     for (i = limin, j = ljmax, k = lkmin; i <= limax; i++)
#define foredgej1k1 \
   if (jmax == ljmax && kmax == lkmax) \
     for (i = limin, j = ljmax, k = lkmax; i <= limax; i++)


#define ijk ijkof(i,j,k)
#define ijkoff ijkof(i+ioff,j+joff,k+koff)


void setboundrobin_scalar(struct box *g, int nf, 
   double sx, double sy, double sz,double rob_lim, double rob_pow)
{

  DECLARE_CCTK_PARAMETERS

  double *u = g->store[nf];
  int i, j, k;
  int di = g->di, dj = g->dj, dk = g->dk, lijk = g->lijk;
  int imin, jmin, kmin, imax, jmax, kmax;
  int limin, ljmin, lkmin, limax, ljmax, lkmax;
  int bi0, bi1, bj0, bj1, bk0, bk1;

  if (0) printf("rb_scalar: %f %f\n", rob_lim,rob_pow);

  /* set local range to all without changing global range */
  imin = g->imin;
  jmin = g->jmin;
  kmin = g->kmin;
  imax = g->imax;
  jmax = g->jmax;
  kmax = g->kmax;
  setrangeall(g);
  setrangePUGH(g, 0);
  g->imin = imin;
  g->jmin = jmin;
  g->kmin = kmin;
  g->imax = imax;
  g->jmax = jmax;
  g->kmax = kmax;

  /* convenience variables for #defines */
  imin = octoffset;
  jmin = octoffset;
  kmin = octoffset;
  /* bitant just changes kmin (octoffset is 0 in this case) */
  if (bitant) kmin = bitoffset;
  /* cartoon, too */
  if (cartoon) {imin = oxdo-1; jmin = oydo-1;}
  imax = g->m-1;
  jmax = g->n-1;
  kmax = g->o-1;
  if (cartoon) {jmax -= oyup-1;}
  limin = g->limin;
  ljmin = g->ljmin;
  lkmin = g->lkmin;
  limax = g->limax;
  ljmax = g->ljmax;
  lkmax = g->lkmax;

  /* boundary flags */
  setneumannflags(g, &bi0, &bj0, &bk0, &bi1, &bj1, &bk1);

  /* faces */
  if (bi0) forfacei0
    u[ijk] = robin(g,u,sx,sy,sz,rob_lim,rob_pow, i, j, k, 1, 0, 0);
  if (bi1) forfacei1
    u[ijk] = robin(g,u,sx,sy,sz,rob_lim,rob_pow, i, j, k,-1, 0, 0);
  if (bj0) forfacej0
    u[ijk] = robin(g,u,sx,sy,sz,rob_lim,rob_pow, i, j, k, 0, 1, 0);
  if (bj1) forfacej1
    u[ijk] = robin(g,u,sx,sy,sz,rob_lim,rob_pow, i, j, k, 0,-1, 0);
  if (bk0) forfacek0
    u[ijk] = robin(g,u,sx,sy,sz,rob_lim,rob_pow, i, j, k, 0, 0, 1);
  if (bk1) forfacek1
    u[ijk] = robin(g,u,sx,sy,sz,rob_lim,rob_pow, i, j, k, 0, 0,-1);
 
  /* edges */
  if (bi0&&bj0) foredgei0j0
    u[ijk] = 0.5*(robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,1,0,0)+
                  robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,1,0));
  if (bi0&&bj1) foredgei0j1
    u[ijk] = 0.5*(robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,1,0,0)+
                  robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,-1,0));
  if (bi0&&bk0) foredgei0k0
    u[ijk] = 0.5*(robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,1,0,0)+
                  robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,0,1));
  if (bi0&&bk1) foredgei0k1
    u[ijk] = 0.5*(robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,1,0,0)+
                  robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,0,-1));

  if (bi1&&bj0) foredgei1j0
    u[ijk] = 0.5*(robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,-1,0,0)+
                  robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,1,0));
  if (bi1&&bj1) foredgei1j1
    u[ijk] = 0.5*(robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,-1,0,0)+
                  robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,-1,0));
  if (bi1&&bk0) foredgei1k0
    u[ijk] = 0.5*(robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,-1,0,0)+
                  robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,0,1));
  if (bi1&&bk1) foredgei1k1
    u[ijk] = 0.5*(robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,-1,0,0)+
                  robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,0,-1));

  if (bj0&&bk0) foredgej0k0
    u[ijk] = 0.5*(robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,1,0)+
                  robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,0,1));
  if (bj0&&bk1) foredgej0k1
    u[ijk] = 0.5*(robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,1,0)+
                  robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,0,-1));
  if (bj1&&bk0) foredgej1k0
    u[ijk] = 0.5*(robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,-1,0)+
                  robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,0,1));
  if (bj1&&bk1) foredgej1k1
    u[ijk] = 0.5*(robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,-1,0)+
                  robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,0,-1));

  /* corners */
  if (bi0&&bj0&&bk0) if (imin==limin&&jmin==ljmin&&kmin==lkmin) {
    i = imin; j = jmin; k = kmin; u[ijk] = 
    (robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,1,0,0)+
     robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,1,0)+
     robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,0,1))/3.0;
  }
  if (bi0&&bj0&&bk1) if (imin==limin&&jmin==ljmin&&kmax==lkmax) {
    i = imin; j = jmin; k = kmax; u[ijk] = 
    (robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,1,0,0)+
     robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,1,0)+
     robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,0,-1))/3.0;
  }
  if (bi0&&bj1&&bk1) if (imin==limin&&jmax==ljmax&&kmax==lkmax) {
    i = imin; j = jmax; k = kmax; u[ijk] = 
    (robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,1,0,0)+
     robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,-1,0)+
     robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,0,-1))/3.0;
  }
  if (bi0&&bj1&&bk0) if (imin==limin&&jmax==ljmax&&kmin==lkmin) {
    i = imin; j = jmax; k = kmin; u[ijk] = 
    (robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,1,0,0)+
     robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,-1,0)+
     robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,0,1))/3.0;
  }
  if (bi1&&bj1&&bk0) if (imax==limax&&jmax==ljmax&&kmin==lkmin) {
    i = imax; j = jmax; k = kmin; u[ijk] = 
    (robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,-1,0,0)+
     robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,-1,0)+
     robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,0,1))/3.0;
  }
  if (bi1&&bj0&&bk0) if (imax==limax&&jmin==ljmin&&kmin==lkmin) {
    i = imax; j = jmin; k = kmin; u[ijk] = 
    (robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,-1,0,0)+
     robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,1,0)+
     robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,0,1))/3.0;
  }
  if (bi1&&bj0&&bk1) if (imax==limax&&jmin==ljmin&&kmax==lkmax) {
    i = imax; j = jmin; k = kmax; u[ijk] = 
    (robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,-1,0,0)+
     robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,1,0)+
     robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,0,-1))/3.0;
  }
  if (bi1&&bj1&&bk1) if (imax==limax&&jmax==ljmax&&kmax==lkmax) {
    i = imax; j = jmax; k = kmax; u[ijk] = 
    (robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,-1,0,0)+
     robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,-1,0)+
     robin(g,u,sx,sy,sz,rob_lim,rob_pow,i,j,k,0,0,-1))/3.0;
  }
}




/* the basic robin formula for all directions */
double robin(struct box *g, double *u, 
             double sx, double sy, double sz, double rob_lim,double rob_pow,
	     int i, int j, int k, int ioff, int joff, int koff)
{
  double x, y, z, r2, s_apply = 0.0;
  int di = g->di, dj = g->dj, dk = g->dk, lijk = g->lijk;
  int n;

  i += ioff;
  j += joff;
  k += koff;

  /* catch octant */
  if (octant && (ioff > 0 || joff > 0 || koff > 0)) { 
    n = g->reffac;
    if(ioff > 0) s_apply = sx;
    if(joff > 0) s_apply = sy;
    if(koff > 0) s_apply = sz;

    if (octant_order == 2)
      if(s_apply > 0) 
        return u[ijk];
      else
        return (1-2*n)*u[ijk];
    else
      if(s_apply > 0)
        return (2*u[ijk] + (n-1)*u[ijkoff])/(n+1);
      else
        return ((1-2*n)*(2*(2*n+1)*u[ijk] + 
                (n-1)*u[ijkoff]))/((n+1)*(2*n+1));
  }

  /* catch bitant */
  if (bitant && (koff > 0)) {
    n = g->reffac;
    s_apply = sz;
    if (bitant_order == 2)
      if(s_apply > 0)      
        return u[ijk];
      else
        return (1-2*n)*u[ijk];
    else
      if(s_apply > 0)
        return (2*u[ijk] + (n-1)*u[ijkoff])/(n+1);
      else
        return ((1-2*n)*(2*(2*n+1)*u[ijk] +    
                (n-1)*u[ijkoff]))/((n+1)*(2*n+1));
  }


  /* robin */
  x = g->x + i*g->d;
  y = g->y + j*g->d;
  z = g->z + k*g->d;
  r2 = x*x + y*y + z*z;
  if (dequal(r2,0.0)) r2 = dequaleps*dequaleps;

  return 2.0*g->d * (ioff*x + joff*y + koff*z)/r2 * 
    (u[ijk] - rob_lim) * rob_pow + u[ijkoff];
}
