/* thorn_BAM_Elliptic: Multigrid Elliptic Solver for Cactus
   Copyright (C) 1998 Bernd Bruegmann 
*/
/* coeffs.c */
/* Bernd Bruegmann, 10/97 */
/* mostly generated by Mathematica, laplace2.m */



#include "bbmg.h"

#define Cal(x,y,z) ((x)?(y):(z))
#define Power(x,y) (pow((double) (x), (double) (y)))
#define pow2(x)    ((x)*(x))
#define pow2inv(x) (1.0/((x)*(x)))




void set_laplace(struct box *g) 
{
int nocd = (eqn_type == BRBR) ? 1 : !conformal_metric;

double dx = g->dx, dy = g->dy, dz = g->dz;
int di = g->di, dj = g->dj, dk = g->dk;
int i, j, k, ijk;

double oo2dx = 1/(2*dx), oo2dy = 1/(2*dy), oo2dz = 1/(2*dz);

double f;
double fdetg;
double delg111;
double delg112;
double delg113;
double delg122;
double delg123;
double delg133;
double delg211;
double delg212;
double delg213;
double delg222;
double delg223;
double delg233;
double delg311;
double delg312;
double delg313;
double delg322;
double delg323;
double delg333;
double delgb111;
double delgb112;
double delgb113;
double delgb122;
double delgb123;
double delgb133;
double delgb211;
double delgb212;
double delgb213;
double delgb222;
double delgb223;
double delgb233;
double delgb311;
double delgb312;
double delgb313;
double delgb322;
double delgb323;
double delgb333;
double df1;
double df2;
double df3;
double ggdo1;
double ggdo2;
double ggdo3;
double gginv11;
double gginv12;
double gginv13;
double gginv22;
double gginv23;
double gginv33;
double ginv11;
double ginv12;
double ginv13;
double ginv22;
double ginv23;
double ginv33;

double *g11, *g12, *g13, *g22, *g23, *g33;
double *gi11, *gi12, *gi13, *gi22, *gi23, *gi33, *gg1, *gg2, *gg3;
double *psi, *dpop1, *dpop2, *dpop3;

setfieldpointers_gab(g, &g11, &g12, &g13, &g22, &g23, &g33);
setfieldpointers_gi(g, &gi11, &gi12, &gi13, &gi22, &gi23, &gi33); 
setfieldpointers_gg(g, &gg1, &gg2, &gg3);
setfieldpointers_psi(g, &psi, &dpop1, &dpop2, &dpop3);


setrangeall(g);
shrinkrange(g);
setrangePUGH(g, 1);

if (!nocd) {
   
forrange(g) {
     
f
=
Power(psi[ijk],4)
;
     
df1
=
4*f*dpop1[ijk]
;
    
df2
=
4*f*dpop2[ijk]
;
     
df3
=
4*f*dpop3[ijk]
;
     
     
gginv11
=
g22[ijk]*g33[ijk] - pow2(g23[ijk])
;
     
gginv12
=
g13[ijk]*g23[ijk] - g12[ijk]*g33[ijk]
;
     
gginv13
=
-(g13[ijk]*g22[ijk]) + g12[ijk]*g23[ijk]
;
    
gginv22
=
g11[ijk]*g33[ijk] - pow2(g13[ijk])
;
     
gginv23
=
g12[ijk]*g13[ijk] - g11[ijk]*g23[ijk]
;
     
gginv33
=
g11[ijk]*g22[ijk] - pow2(g12[ijk])
;
     
fdetg
=
f*(gginv11*g11[ijk] + gginv12*g12[ijk] + gginv13*g13[ijk])
;
     
gi11[ijk] = ginv11
=
gginv11/fdetg
;
     
gi12[ijk] = ginv12
=
gginv12/fdetg
;
     
gi13[ijk] = ginv13
=
gginv13/fdetg
;
     
gi22[ijk] = ginv22
=
gginv22/fdetg
;
     
gi23[ijk] = ginv23
=
gginv23/fdetg
;
     
gi33[ijk] = ginv33
=
gginv33/fdetg
;
    
delgb111
=
oo2dx*(-g11[-di + ijk] + g11[di + ijk])
;
     
delgb112
=
oo2dx*(-g12[-di + ijk] + g12[di + ijk])
;
     
delgb113
=
oo2dx*(-g13[-di + ijk] + g13[di + ijk])
;
     
delgb122
=
oo2dx*(-g22[-di + ijk] + g22[di + ijk])
;
     
delgb123
=
oo2dx*(-g23[-di + ijk] + g23[di + ijk])
;
     
delgb133
=
oo2dx*(-g33[-di + ijk] + g33[di + ijk])
;
     
delgb211
=
oo2dy*(-g11[-dj + ijk] + g11[dj + ijk])
;
     
delgb212
=
oo2dy*(-g12[-dj + ijk] + g12[dj + ijk])
;
     
delgb213
=
oo2dy*(-g13[-dj + ijk] + g13[dj + ijk])
;
     
delgb222
=
oo2dy*(-g22[-dj + ijk] + g22[dj + ijk])
;
     
delgb223
=
oo2dy*(-g23[-dj + ijk] + g23[dj + ijk])
;
     
delgb233
=
oo2dy*(-g33[-dj + ijk] + g33[dj + ijk])
;
     
delgb311
=
oo2dz*(-g11[-dk + ijk] + g11[dk + ijk])
;
     
delgb312
=
oo2dz*(-g12[-dk + ijk] + g12[dk + ijk])
;
     
delgb313
=
oo2dz*(-g13[-dk + ijk] + g13[dk + ijk])
;
     
delgb322
=
oo2dz*(-g22[-dk + ijk] + g22[dk + ijk])
;
     
delgb323
=
oo2dz*(-g23[-dk + ijk] + g23[dk + ijk])
;
     
delgb333
=
oo2dz*(-g33[-dk + ijk] + g33[dk + ijk])
;
     
delg111
=
delgb111*f + df1*g11[ijk]
;
     
delg112
=
delgb112*f + df1*g12[ijk]
;
     
delg113
=
delgb113*f + df1*g13[ijk]
;
     
delg122
=
delgb122*f + df1*g22[ijk]
;
     
delg123
=
delgb123*f + df1*g23[ijk]
;
     
delg133
=
delgb133*f + df1*g33[ijk]
;
     
delg211
=
delgb211*f + df2*g11[ijk]
;
     
delg212
=
delgb212*f + df2*g12[ijk]
;
     
delg213
=
delgb213*f + df2*g13[ijk]
;

delg222
=
delgb222*f + df2*g22[ijk]
;
     
delg223
=
delgb223*f + df2*g23[ijk]
;
     
delg233
=
delgb233*f + df2*g33[ijk]
;
     
delg311
=
delgb311*f + df3*g11[ijk]
;
     
delg312
=
delgb312*f + df3*g12[ijk]
;
     
delg313
=
delgb313*f + df3*g13[ijk]
;
     
delg322
=
delgb322*f + df3*g22[ijk]
;
     
delg323
=
delgb323*f + df3*g23[ijk]
;
     
delg333
=
delgb333*f + df3*g33[ijk]
;

ggdo1
=
(delg111*ginv11)/2. + delg211*ginv12 + delg311*ginv13 + 
(-delg122/2. + delg212)*ginv22 + (-delg123 + delg213 + delg312)*ginv23 + 
(-delg133/2. + delg313)*ginv33
;
     
ggdo2
=
(delg112 - delg211/2.)*ginv11 + delg122*ginv12 + 
(delg123 - delg213 + delg312)*ginv13 + (delg222*ginv22)/2. + 
delg322*ginv23 + (-delg233/2. + delg323)*ginv33
;
     
ggdo3
=
(delg113 - delg311/2.)*ginv11 + (delg123 + delg213 - delg312)*ginv12 + 
delg133*ginv13 + (delg223 - delg322/2.)*ginv22 + delg233*ginv23 + 
(delg333*ginv33)/2.
;
     
gg1[ijk]
=
-(ggdo1*ginv11) - ggdo2*ginv12 - ggdo3*ginv13
;
     
gg2[ijk]
=
-(ggdo1*ginv12) - ggdo2*ginv22 - ggdo3*ginv23
;
    
gg3[ijk]
=
-(ggdo1*ginv13) - ggdo2*ginv23 - ggdo3*ginv33
;
     
     
}/* loop i, j, k */
   


/* no conformal differencing */
} else {

forrange(g) {
     
/* lazy, should substitute below for optimization */
f = 1.0;
df1 = 0.0;
df2 = 0.0;
df3 = 0.0;
     
     
gginv11
=
g22[ijk]*g33[ijk] - pow2(g23[ijk])
;
     
gginv12
=
g13[ijk]*g23[ijk] - g12[ijk]*g33[ijk]
;
     
gginv13
=
-(g13[ijk]*g22[ijk]) + g12[ijk]*g23[ijk]
;
     
gginv22
=
g11[ijk]*g33[ijk] - pow2(g13[ijk])
;
     
gginv23
=
g12[ijk]*g13[ijk] - g11[ijk]*g23[ijk]
;
     
gginv33
=
g11[ijk]*g22[ijk] - pow2(g12[ijk])
;
     
fdetg
=
f*(gginv11*g11[ijk] + gginv12*g12[ijk] + gginv13*g13[ijk])
;
     
gi11[ijk] = ginv11
=
gginv11/fdetg
;
     
gi12[ijk] = ginv12
=
gginv12/fdetg
;
     
gi13[ijk] = ginv13
=
gginv13/fdetg
;
     
gi22[ijk] = ginv22
=
gginv22/fdetg
;
     
gi23[ijk] = ginv23
=
gginv23/fdetg
;
     
gi33[ijk] = ginv33
=
gginv33/fdetg
;
     
delgb111
=
oo2dx*(-g11[-di + ijk] + g11[di + ijk])
;
     
delgb112
=
oo2dx*(-g12[-di + ijk] + g12[di + ijk])
;
     
delgb113
=
oo2dx*(-g13[-di + ijk] + g13[di + ijk])
;
     
delgb122
=
oo2dx*(-g22[-di + ijk] + g22[di + ijk])
;
     
delgb123
=
oo2dx*(-g23[-di + ijk] + g23[di + ijk])
;
     
delgb133
=
oo2dx*(-g33[-di + ijk] + g33[di + ijk])
;
     
delgb211
=
oo2dy*(-g11[-dj + ijk] + g11[dj + ijk])
;
     
delgb212
=
oo2dy*(-g12[-dj + ijk] + g12[dj + ijk])
;
     
delgb213
=
oo2dy*(-g13[-dj + ijk] + g13[dj + ijk])
;
     
delgb222
=
oo2dy*(-g22[-dj + ijk] + g22[dj + ijk])
;
     
delgb223
=
oo2dy*(-g23[-dj + ijk] + g23[dj + ijk])
;
     
delgb233
=
oo2dy*(-g33[-dj + ijk] + g33[dj + ijk])
;
     
delgb311
=
oo2dz*(-g11[-dk + ijk] + g11[dk + ijk])
;
     
delgb312
=
oo2dz*(-g12[-dk + ijk] + g12[dk + ijk])
;
     
delgb313
=
oo2dz*(-g13[-dk + ijk] + g13[dk + ijk])
;
     
delgb322
=
oo2dz*(-g22[-dk + ijk] + g22[dk + ijk])
;
     
delgb323
=
oo2dz*(-g23[-dk + ijk] + g23[dk + ijk])
;
     
delgb333
=
oo2dz*(-g33[-dk + ijk] + g33[dk + ijk])
;
     
delg111
=
delgb111*f + df1*g11[ijk]
;
     
delg112
=
delgb112*f + df1*g12[ijk]
;
     
delg113
=
delgb113*f + df1*g13[ijk]
;
     
delg122
=
delgb122*f + df1*g22[ijk]
;
     
delg123
=
delgb123*f + df1*g23[ijk]
;
     
delg133
=
delgb133*f + df1*g33[ijk]
;
     
delg211
=
delgb211*f + df2*g11[ijk]
;
     
delg212
=
delgb212*f + df2*g12[ijk]
;
     
delg213
=
delgb213*f + df2*g13[ijk]
;
     
delg222
=
delgb222*f + df2*g22[ijk]
;
     
delg223
=
delgb223*f + df2*g23[ijk]
;
     
delg233
=
delgb233*f + df2*g33[ijk]
;
     
delg311
=
delgb311*f + df3*g11[ijk]
;
     
delg312
=
delgb312*f + df3*g12[ijk]
;
     
delg313
=
delgb313*f + df3*g13[ijk]
;
     
delg322
=
delgb322*f + df3*g22[ijk]
;
     
delg323
=
delgb323*f + df3*g23[ijk]
;
     
delg333
=
delgb333*f + df3*g33[ijk]
;
     
ggdo1
=
(delg111*ginv11)/2. + delg211*ginv12 + delg311*ginv13 + 
(-delg122/2. + delg212)*ginv22 + (-delg123 + delg213 + delg312)*ginv23 + 
(-delg133/2. + delg313)*ginv33
;
     
ggdo2
=
(delg112 - delg211/2.)*ginv11 + delg122*ginv12 + 
(delg123 - delg213 + delg312)*ginv13 + (delg222*ginv22)/2. + 
delg322*ginv23 + (-delg233/2. + delg323)*ginv33
;

ggdo3
=
(delg113 - delg311/2.)*ginv11 + (delg123 + delg213 - delg312)*ginv12 + 
delg133*ginv13 + (delg223 - delg322/2.)*ginv22 + delg233*ginv23 + 
(delg333*ginv33)/2.
;
     
gg1[ijk]
=
-(ggdo1*ginv11) - ggdo2*ginv12 - ggdo3*ginv13
;
    
gg2[ijk]
=
-(ggdo1*ginv12) - ggdo2*ginv22 - ggdo3*ginv23
;
    
gg3[ijk]
=
-(ggdo1*ginv13) - ggdo2*ginv23 - ggdo3*ginv33
;
     
     
   }/* loop i, j, k */
 } /* else */
 
/* laplace2.c */
/* n* = 161,  n/ = 37,  n+ = 275, nvars = 57 */

PUGHsyncn(g, MGF*g->ncomps + 8, 3);
}  
  





void set_K2(struct box *g, double *h, double *driver)
{
int i, j, k, ijk;

/* auxiliary variables */
double Kdi11;
double Kdi12;
double Kdi13;
double Kdi21;
double Kdi22;
double Kdi23;
double Kdi31;
double Kdi32;
double Kdi33;

/* fields */
double *gi11, *gi12, *gi13, *gi22, *gi23, *gi33;
double *K11, *K12, *K13, *K22, *K23, *K33;

setfieldpointers_gi(g, &gi11, &gi12, &gi13, &gi22, &gi23, &gi33);
setfieldpointers_Kab(g, &K11, &K12, &K13, &K22, &K23, &K33);


setrangePUGH(g, 0);
forrange(g) {


Kdi11
=
gi11[ijk]*K11[ijk] + gi12[ijk]*K12[ijk] + gi13[ijk]*K13[ijk]
;

Kdi12
=
gi12[ijk]*K11[ijk] + gi22[ijk]*K12[ijk] + gi23[ijk]*K13[ijk]
;

Kdi13
=
gi13[ijk]*K11[ijk] + gi23[ijk]*K12[ijk] + gi33[ijk]*K13[ijk]
;

Kdi21
=
gi11[ijk]*K12[ijk] + gi12[ijk]*K22[ijk] + gi13[ijk]*K23[ijk]
;

Kdi22
=
gi12[ijk]*K12[ijk] + gi22[ijk]*K22[ijk] + gi23[ijk]*K23[ijk]
;

Kdi23
=
gi13[ijk]*K12[ijk] + gi23[ijk]*K22[ijk] + gi33[ijk]*K23[ijk]
;

Kdi31
=
gi11[ijk]*K13[ijk] + gi12[ijk]*K23[ijk] + gi13[ijk]*K33[ijk]
;

Kdi32
=
gi12[ijk]*K13[ijk] + gi22[ijk]*K23[ijk] + gi23[ijk]*K33[ijk]
;

Kdi33
=
gi13[ijk]*K13[ijk] + gi23[ijk]*K23[ijk] + gi33[ijk]*K33[ijk]
;

h[ijk]
=
-2*Kdi12*Kdi21 - 2*Kdi13*Kdi31 - 2*Kdi23*Kdi32 - pow2(Kdi11) - pow2(Kdi22) - 
   pow2(Kdi33)
;

driver[ijk]
=
(maxslice_epsdriver==0.0) 
?
0.0
:
maxslice_epsdriver*(Kdi11 + Kdi22 + Kdi33)
;


}
}





/* set coefficients for symmetric stencil for laplace operator
      Laplace u  =  1/g del(g gi del u) 
*/
void set_laplace_sym(struct box *g) 
{
int nocd = (eqn_type == BRBR) ? 1 : !conformal_metric;

int i, j, k, ijk;

double f;
double fdetg;
double gginv11;
double gginv12;
double gginv13;
double gginv22;
double gginv23;
double gginv33;
double sqrtdetg;

double *g11, *g12, *g13, *g22, *g23, *g33;
double *gi11, *gi12, *gi13, *gi22, *gi23, *gi33, *gg1, *gg2, *gg3;
double *psi, *dpop1, *dpop2, *dpop3;

setfieldpointers_gab(g, &g11, &g12, &g13, &g22, &g23, &g33);
setfieldpointers_gi(g, &gi11, &gi12, &gi13, &gi22, &gi23, &gi33); 
setfieldpointers_gg(g, &gg1, &gg2, &gg3);
setfieldpointers_psi(g, &psi, &dpop1, &dpop2, &dpop3);


setrangeall(g);
setrangePUGH(g, 0);

forrange(g) {

  f = Cal(nocd,1,Power(psi[ijk],4));

  gginv11 = g22[ijk]*g33[ijk] - pow2(g23[ijk]);
  gginv12 = g13[ijk]*g23[ijk] - g12[ijk]*g33[ijk];
  gginv13 = -(g13[ijk]*g22[ijk]) + g12[ijk]*g23[ijk];
  gginv22 = g11[ijk]*g33[ijk] - pow2(g13[ijk]);
  gginv23 = g12[ijk]*g13[ijk] - g11[ijk]*g23[ijk];
  gginv33 = g11[ijk]*g22[ijk] - pow2(g12[ijk]);

  fdetg = f*(gginv11*g11[ijk] + gginv12*g12[ijk] + gginv13*g13[ijk]);

  gg1[ijk] = sqrtdetg = f * sqrt(fdetg);

  gi11[ijk] = sqrtdetg*gginv11/fdetg;
  gi12[ijk] = sqrtdetg*gginv12/fdetg;
  gi13[ijk] = sqrtdetg*gginv13/fdetg;
  gi22[ijk] = sqrtdetg*gginv22/fdetg;
  gi23[ijk] = sqrtdetg*gginv23/fdetg;
  gi33[ijk] = sqrtdetg*gginv33/fdetg;

}
}  
  




