/* thorn_BAM_Elliptic: Multigrid Elliptic Solver for Cactus
   Copyright (C) 1998 Bernd Bruegmann 
*/
/* dalota.c */
/* Bernd Bruegmann, Ryoji Takahashi 4/02 */


/* puncture data can use different extrinsic curvatures, here
   we use the Kerr extrinsic curvature as discussed by
   Dain, Lousto and Takahashi in gr-qc/0201062
*/


#include "bbmg.h"
#define EPSZERO (dequaleps/1000.0)


extern double MBL[N_MWBL], CBL[N_MWBL][3], PBL[N_MWBL][3], SBL[N_MWBL][3];

#define fori     for (i = 1; i <= 3; i++)
#define forsymij fori for (j = i; j <= 3; j++)
#define delta(i,j) (((i)==(j))?1.0:0.0)




/* compare byKofxyz() in brbr.c:
   compute components of extrinsic curvature at a given point 
*/
void dltKofxyz(double x, double y, double z,
	      double *K11, double *K12, double *K13, 
	      double *K22, double *K23, double *K33)
{
  double xold = x, yold = y, zold = z;
  int i, j, n;
  double xc, yc, zc, r, cos, dxcos, dxcos2, dxcos3, dxcos5;
  double dycos, dycos2, dycos3, dycos5, dzcos, dzcos2, dzcos3, dzcos5;
  double Kmass, byJ, neta;
  double *K[4][4], S[4], N[4], Eta[4], dOmega[4], eNS[4];

  if (fisheye) FishEye_CoordTrans(&x, &y, &z);

  K[1][1] = K11;
  K[1][2] = K12;
  K[1][3] = K13;
  K[2][2] = K22;
  K[2][3] = K23;
  K[3][3] = K33;
  forsymij
    *K[i][j] = 0;

  for (n = 0; MBL[n] != 0.0 && n < N_MWBL; n++) {
    xc = x-CBL[n][0];
    yc = y-CBL[n][1];
    zc = z-CBL[n][2];
    r  = sqrt(xc*xc+yc*yc+zc*zc);
    if (dequal(r, 0.0)) r = EPSZERO;

    /* fori P[i] = PBL[n][i-1]; */
    fori S[i] = SBL[n][i-1];

    byJ = sqrt(S[1]*S[1]+S[2]*S[2]+S[3]*S[3]);

    N[1] = S[1]/byJ;
    N[2] = S[2]/byJ;
    N[3] = S[3]/byJ;

    neta = r*r - ((N[1]*xc)*(N[1]*xc)+(N[2]*yc)*(N[2]*yc)
 	 +(N[3]*zc)*(N[3]*zc));

    cos = (N[1]*xc+N[2]*yc+N[3]*zc)/r;
    
    Eta[1] = N[2]*zc - N[3]*yc;
    Eta[2] = - N[1]*zc + N[3]*xc;
    Eta[3] = N[1]*yc - N[2]*xc;

    Kmass = sqrt(2.0+sqrt(4.0+byJ*byJ));

    dxcos = - xc*(N[1]*xc+N[2]*yc+N[3]*zc)/(r*r*r) + N[1]/r;

    dxcos2 =  - 2*xc*(N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)/
	    (r*r*r*r) 
	    + 2*N[1]*(N[1]*xc+N[2]*yc+N[3]*zc)/(r*r);
  
    dxcos3 =  - 3*xc*(N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)*
	    (N[1]*xc+N[2]*yc+N[3]*zc)/(r*r*r*r*r) 
	    + 3*N[1]*(N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)/
	    (r*r*r);

    dxcos5 = - 5*xc*(N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)*
	    (N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)*
	    (N[1]*xc+N[2]*yc+N[3]*zc)/(r*r*r*r*r*r*r) 
	    + 5*N[1]*(N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)*
	    (N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)/
	    (r*r*r*r*r);

    dycos = - yc*(N[1]*xc+N[2]*yc+N[3]*zc)/(r*r*r) + N[2]/r;

    dycos2 =  - 2*yc*(N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)/
	    (r*r*r*r) 
	    + 2*N[2]*(N[1]*xc+N[2]*yc+N[3]*zc)/(r*r);

    dycos3 =  - 3*yc*(N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)*
	    (N[1]*xc+N[2]*yc+N[3]*zc)/(r*r*r*r*r) 
	    + 3*N[2]*(N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)/
	    (r*r*r);

    dycos5 = - 5*yc*(N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)*
	    (N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)*
	    (N[1]*xc+N[2]*yc+N[3]*zc)/(r*r*r*r*r*r*r) 
	    + 5*N[2]*(N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)*
	    (N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)/
	    (r*r*r*r*r);

    dzcos = - zc*(N[1]*xc+N[2]*yc+N[3]*zc)/(r*r*r) + N[3]/r;

    dzcos2 =  - 2*zc*(N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)/
	    (r*r*r*r) 
	    + 2*N[3]*(N[1]*xc+N[2]*yc+N[3]*zc)/(r*r);

    dzcos3 =  - 3*zc*(N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)*
	    (N[1]*xc+N[2]*yc+N[3]*zc)/(r*r*r*r*r) 
	    + 3*N[3]*(N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)/
	    (r*r*r);

    dzcos5 = - 5*zc*(N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)*
	    (N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)*
	    (N[1]*xc+N[2]*yc+N[3]*zc)/(r*r*r*r*r*r*r) 
	    + 5*N[3]*(N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)*
	    (N[1]*xc+N[2]*yc+N[3]*zc)*(N[1]*xc+N[2]*yc+N[3]*zc)/
	    (r*r*r*r*r);

    
    dOmega[1] = byJ*(dxcos3 - 3*dxcos)-((Kmass*(byJ*byJ*byJ)/
                (Kmass*Kmass*Kmass))*(dxcos - 2*dxcos3 + dxcos5)*((r + Kmass +
                (Kmass*Kmass - (byJ*byJ)/(Kmass*Kmass))/(4*r))*(r + Kmass +
                (Kmass*Kmass - (byJ*byJ)/(Kmass*Kmass))/(4*r))+(byJ*byJ)/
                (Kmass*Kmass)*cos*cos))/((r + Kmass + (Kmass*Kmass -
                (byJ*byJ)/(Kmass*Kmass))/(4*r))*(r + Kmass +(Kmass*Kmass 
                - (byJ*byJ)/(Kmass*Kmass))/(4*r))+(byJ*byJ)/(Kmass*Kmass)*
                cos*cos)/((r + Kmass + (Kmass*Kmass - (byJ*byJ)/(Kmass*Kmass))/
                (4*r))*(r + Kmass + (Kmass*Kmass - (byJ*byJ)/(Kmass*Kmass))/
                (4*r))+(byJ*byJ)/(Kmass*Kmass)*cos*cos) + 
                ((Kmass*(byJ*byJ*byJ)/(Kmass*Kmass*Kmass))*
		(cos - 2*cos*cos*cos + cos*cos*cos*cos*cos)*2*(r + Kmass + 
                (Kmass*Kmass - (byJ*byJ)/(Kmass*Kmass))/(4*r))*(x/r + 
                (Kmass*Kmass - (byJ*byJ)/(Kmass*Kmass))/(2*r*r)) +
	        (byJ*byJ)/(Kmass*Kmass)*dxcos2)/((r + Kmass + (Kmass*Kmass -
                (byJ*byJ)/(Kmass*Kmass))/(4*r))*(r + Kmass + (Kmass*Kmass - 
                (byJ*byJ)/(Kmass*Kmass))/(4*r))+(byJ*byJ)/(Kmass*Kmass)
                *cos*cos)/((r + Kmass + (Kmass*Kmass -
                (byJ*byJ)/(Kmass*Kmass))/(4*r))*(r + Kmass + (Kmass*Kmass - 
                (byJ*byJ)/(Kmass*Kmass))/(4*r))+(byJ*byJ)/(Kmass*Kmass)
	        *cos*cos);

    dOmega[2] =  byJ*(dycos3 - 3*dycos)-((Kmass*(byJ*byJ*byJ)/
                (Kmass*Kmass*Kmass))*(dycos - 2*dycos3 + dycos5)*((r + Kmass +
                (Kmass*Kmass - (byJ*byJ)/(Kmass*Kmass))/(4*r))*(r + Kmass +
                (Kmass*Kmass - (byJ*byJ)/(Kmass*Kmass))/(4*r))+(byJ*byJ)/
                (Kmass*Kmass)*cos*cos))/((r + Kmass + (Kmass*Kmass -
                (byJ*byJ)/(Kmass*Kmass))/(4*r))*(r + Kmass +(Kmass*Kmass 
                - (byJ*byJ)/(Kmass*Kmass))/(4*r))+(byJ*byJ)/(Kmass*Kmass)*
                cos*cos)/((r + Kmass + (Kmass*Kmass - (byJ*byJ)/(Kmass*Kmass))/
                (4*r))*(r + Kmass + (Kmass*Kmass - (byJ*byJ)/(Kmass*Kmass))/
                (4*r))+(byJ*byJ)/(Kmass*Kmass)*cos*cos) + 
                ((Kmass*(byJ*byJ*byJ)/(Kmass*Kmass*Kmass))*
		(cos - 2*cos*cos*cos + cos*cos*cos*cos*cos)*2*(r + Kmass + 
                (Kmass*Kmass - (byJ*byJ)/(Kmass*Kmass))/(4*r))*(y/r + 
                (Kmass*Kmass - (byJ*byJ)/(Kmass*Kmass))/(2*r*r)) +
	        (byJ*byJ)/(Kmass*Kmass)*dycos2)/((r + Kmass + (Kmass*Kmass -
                (byJ*byJ)/(Kmass*Kmass))/(4*r))*(r + Kmass + (Kmass*Kmass - 
                (byJ*byJ)/(Kmass*Kmass))/(4*r))+(byJ*byJ)/(Kmass*Kmass)
                *cos*cos)/((r + Kmass + (Kmass*Kmass -
                (byJ*byJ)/(Kmass*Kmass))/(4*r))*(r + Kmass + (Kmass*Kmass - 
                (byJ*byJ)/(Kmass*Kmass))/(4*r))+(byJ*byJ)/(Kmass*Kmass)
	        *cos*cos);

    dOmega[3] =  byJ*(dzcos3 - 3*dzcos)-((Kmass*(byJ*byJ*byJ)/
                (Kmass*Kmass*Kmass))*(dzcos - 2*dzcos3 + dzcos5)*((r + Kmass +
                (Kmass*Kmass - (byJ*byJ)/(Kmass*Kmass))/(4*r))*(r + Kmass +
                (Kmass*Kmass - (byJ*byJ)/(Kmass*Kmass))/(4*r))+(byJ*byJ)/
                (Kmass*Kmass)*cos*cos))/((r + Kmass + (Kmass*Kmass -
                (byJ*byJ)/(Kmass*Kmass))/(4*r))*(r + Kmass +(Kmass*Kmass 
                - (byJ*byJ)/(Kmass*Kmass))/(4*r))+(byJ*byJ)/(Kmass*Kmass)*
                cos*cos)/((r + Kmass + (Kmass*Kmass - (byJ*byJ)/(Kmass*Kmass))/
                (4*r))*(r + Kmass + (Kmass*Kmass - (byJ*byJ)/(Kmass*Kmass))/
                (4*r))+(byJ*byJ)/(Kmass*Kmass)*cos*cos) + 
                ((Kmass*(byJ*byJ*byJ)/(Kmass*Kmass*Kmass))*
		(cos - 2*cos*cos*cos + cos*cos*cos*cos*cos)*2*(r + Kmass + 
                (Kmass*Kmass - (byJ*byJ)/(Kmass*Kmass))/(4*r))*(z/r + 
                (Kmass*Kmass - (byJ*byJ)/(Kmass*Kmass))/(2*r*r)) +
	        (byJ*byJ)/(Kmass*Kmass)*dzcos2)/((r + Kmass + (Kmass*Kmass -
                (byJ*byJ)/(Kmass*Kmass))/(4*r))*(r + Kmass + (Kmass*Kmass - 
                (byJ*byJ)/(Kmass*Kmass))/(4*r))+(byJ*byJ)/(Kmass*Kmass)
                *cos*cos)/((r + Kmass + (Kmass*Kmass -
                (byJ*byJ)/(Kmass*Kmass))/(4*r))*(r + Kmass + (Kmass*Kmass - 
                (byJ*byJ)/(Kmass*Kmass))/(4*r))+(byJ*byJ)/(Kmass*Kmass)
	        *cos*cos);


    eNS[1] = dOmega[3]*Eta[2] - dOmega[2]*Eta[3];
    eNS[2] = dOmega[1]*Eta[3] - dOmega[3]*Eta[1];
    eNS[3] = dOmega[2]*Eta[1] - dOmega[1]*Eta[2];

    forsymij 
      *K[i][j] += 1/(neta*neta)*(eNS[i]*Eta[j]+eNS[j]*Eta[i]);
  }

  if (fisheye)
    FishEye_IndexTransDDSym(xold, yold, zold, K11, K12, K13, K22, K23, K33);

}	

