/* thorn_BAM_Elliptic: Multigrid Elliptic Solver for Cactus
   Copyright (C) 1998 Bernd Bruegmann 
*/
/* driver.c */
/* Bernd Bruegmann, 4/97 */

#include "bbmg.h"
#include "cctk_Config.h"


#define PR bbmgPR

int straightrestrict = 0;
int weightedinit = 0;
int fullV = 1;
int neumann, octant, octant_order,bitoffset,octoffset,bitant,bitant_order;
int gridminsize, gridmaxlevels, mincycles, maxcycles;
double tolres2, tolresi, factau2, factaui;
double robin_limit, robin_power, robin_limit_vec1, robin_power_vec1;
double robin_limit_vec2, robin_power_vec2, robin_limit_vec3, robin_power_vec3;
double plane_sx,plane_sy,plane_sz,plane_sx_vec1,plane_sy_vec1,plane_sz_vec1;
double plane_sx_vec2,plane_sy_vec2,plane_sz_vec2;
double plane_sx_vec3,plane_sy_vec3,plane_sz_vec3;
int nvcycle, lvcycle;


void bbmg_fullVdriver(int bbmg_eqn_type, int bbmg_init, int bbmg_method,
		      int bbmg_npre, int bbmg_npost, int bbmg_ncycles,
		      int ltop, int lmax, struct mgstack **s);




/* solve an elliptic equation using FAS multigrid */
void bbmg_elliptic_solve(int bbmg_eqn_type, int bbmg_init, int bbmg_method,
			 int bbmg_npre, int bbmg_npost, int bbmg_ncycles,
			 int ltop, int lmax, struct mgstack **s)
{
  int i, l;
#if 0
  int lmin, n;
#endif
  int u, v, w, f;
  struct box *g, *gh, *gH;
  int pr = 0;
  void set_brill(int ltop, int lmax, struct mgstack **s);

  /* say hello */
  if (pr) printf("\nBAM: entering bam_elliptic_solve\n"); 
  if (pr) printf("init %d, method %d, ltop %d, lmax %d\n",
		 bbmg_init, bbmg_method, ltop, lmax);
  if (pr) printf("npre %d, npost %d, mincycles %d, maxcycles %d\n",
		 bbmg_npre, bbmg_npost, mincycles, maxcycles);

  /* problem specific initialization */
  if (problem == POISSON)
    init_poisson(bbmg_init, ltop, lmax, s);
  else if (problem == BRILL)
    init_brill(bbmg_init, ltop, lmax, s);
  else if (problem == BHDATA)
    init_brbr(bbmg_init, ltop, lmax, s);
  else if (problem == MAXIMAL)
    init_maxslice(bbmg_init, ltop, lmax, s);
  else if (problem == LINEAR)
    init_linear(bbmg_init, ltop, lmax, s);
  else if (problem == POWERS)
    init_powers(bbmg_init, ltop, lmax, s);
  else if (problem == VECLAPLACE || problem == VEC4)
    init_vecL(bbmg_init, ltop, lmax, s);
  else if (problem == TSD)
    init_vecL(bbmg_init, ltop, lmax, s);
  else if (problem == KSD)
      init_vecL(bbmg_init, ltop, lmax, s);
  else if (problem == YORK)
      init_vecL(bbmg_init, ltop, lmax, s);
  else
    errorexit("bam_elliptic_solve(): unknown type of problem");

  /* now we have for each level l a linked list of grids at that level
       with pointers to the storage provided in mgstack
     constant fields have been initialized already in mgstack, because
       here it would be a mess to implement a multitude of boundary condition
       for all the constant fields 
  */

  /* perform multigrid V cycles 
     if bbmg_init is 0, then we start the V cycles at lmax
     if bbmg_init is not 0, then we start from scratch with full multigrid
  */

  /* print fields */
  if (PR) {
    for (l = 1; l <= lmax; l++) {
      printf("level %d:\n", l);
      forlevel(l, g) {
	shrinkrange(g); 
	for (i = 0; i < g->nfields; i++) {
	  printf("test input  %.15e %.15e ", 
		 l2norm_scalar(g, i), linfnorm_scalar(g, i));
	  if (0) prprim(" norm2", g, i); 
	  else printf("%s\n", fieldtitle[i]);
	}
      }
    }
  }

  /* test one-level operations */
  if (0) {
    for (l = 1; level[l]->next; l++)
      if (dequal(level[l]->next->d, 1.0)) break;
    lmax = l;
    printf("lmax = %d\n\n", lmax);
    prresiduals(lmax); /* edit out trunc error, which two-level! */
    for (l = 0; l < 100; l++) {
      relax(lmax, 1);
      prresiduals(lmax);
    }
    return;
  }

  /* test two-level operations */
  if (0) {
    int u, v, w, f;
    struct box *gh, *gH;
    int rho = MGF*level[1]->next->ncomps + 6; /* ouch */

    foroverlap(lmax, gh, lmax-1, gH) {
      subtract(gH, rho, rho, rho);
      setrangeftoc(gh, gH);
      mgrid_restrict(gh, rho, gH, rho);
      if (0) prprim("rhoH = R(rhoh)", gH, rho);    
      if (0) prprim("rhoh", gh, rho);    
      printf("|rhoH|_2 = %e\n\n", l2norm(gH, rho));
      setrangeall(gh);
      subtract(gh, rho, rho, rho);
      setrangectof(gH, gh);
      prolong(gH, rho, gh, rho); /* returns proper fine interior */
      if (1) prprim("rhoh = P(rhoH)", gh, rho);    
      printf("|rhoh|_2 = %e\n\n", l2norm(gh, rho));
    }
    return;
  }

  /* solve by V cycles */
  bbmg_fullVdriver(bbmg_eqn_type, bbmg_init, bbmg_method,
		   bbmg_npre, bbmg_npost, bbmg_ncycles,
		   ltop, lmax, s);

  /* leave with injected fields as opposed to solutions to the
     modified multigrid equation */
  for (l = lmax; l > ltop; l--)
    foroverlap(l, gh, l-1, gH) {
      setrangeftoc(gh, gH);
      injectftoc(gh, u, gH, u);
    }

  /* problem specific finish */
  if (problem == BRILL)
    set_brill(ltop, lmax, s);
  if (problem == BHDATA)
    set_brbr(ltop, lmax, s);

  /* print fields */
  if (PR) 
    forlevel(lmax, g)
      for (i = 0; i < g->nfields; i++) {
	printf("test output %.15e %.15e ", 
	       l2norm_scalar(g, i), linfnorm_scalar(g, i));
	if (0) prprim(" norm2", g, i); 
	else printf("%s\n", fieldtitle[i]);
      }

  /* say good bye */
  if (pr) printf("BAM: leaving bam_elliptic_solve\n\n"); 
}





/* should be replaced by application specific version */
void errorexit(char *s) 
{
#ifdef CCTK_MPI
  CCTK_VWarn (0, __LINE__, __FILE__, CCTK_THORNSTRING,
              "Fatal error in BAM: '%s'", s);
#else
  if (1) printf("Fatal error in BAM:\n%s\n", s);
  if (0) fprintf(stderr, "Fatal error in BAM: %s\n", s);
#endif
  exit(1);
}



/* for debugging */
void yo() 
{
  printf("yo!\n");
  fflush(stdout);
}




/**********************************************************************/

/* multigrid V cycles
   full multigrid corresponds to lmin = 1 
*/
void bbmg_fullVdriver(int bbmg_eqn_type, int bbmg_init, int bbmg_method,
		      int bbmg_npre, int bbmg_npost, int bbmg_ncycles,
		      int ltop, int lmax, struct mgstack **s)
{
  int pr = vb || PR;
  int l, lmin, n;
  double res2, resi, tau2, taui;
  void prolong_u(int lH, int lh);

  /* cycle up? */
  lmin = (fullV) ? 1 : lmax;

  for (l = lmin; l <= lmax; l++) {
    lvcycle = l;
    
    /* this is the place where in BAM new refinements may be created */
    /* let's not get into that now */

    /* perform V cycles */
    for (n = 0; n < maxcycles; n++) {
      nvcycle = n;

      /* are we done? */
      if (n >= mincycles) {
	
	/* should also support relative errors! */
	/* check errors */
	res2 = resi = tau2 = taui = 0;
	restau(l, (tolres2)?&res2:0, (tolresi)?&resi:0, 
/* temporary hack: what is the better default? */
#if 1
	          (factau2)?&tau2:0, (factaui)?&taui:0, 0);
#else
	          (factau2-1.0)?&tau2:0, (factaui-1.0)?&taui:0, 0);
#endif
	if (res2 <= tolres2 && 
	    resi <= tolresi &&
#if 1
	    (res2*factau2 <= tau2) &&
	    (resi*factaui <= taui))
#else
	    (factau2 == 1.0 || res2*factau2 <= tau2) &&
	    (factaui == 1.0 || resi*factaui <= taui))
#endif
	  break;
      }
      if (pr) prresiduals(l); /* if factau not 1, will be incorrect on uH */
      if (pr) 
	printf("----- level %d, V-cycle %d -------------------------\n\n", 
	       l, n+1);

      /* solve */
      solve(l, lmax, bbmg_npre, bbmg_npost);
      if (vb) nan_vecL(level[l]->next);

      /* for l == 1, solve is exact */
      if (l == 1) break;
    }
    if (!pr && l == lmax && vbell) printf(
      "------------------------ after %d V-cycles ---------------------\n",n);
    if (pr || l == lmax && vbell) prresiduals(l);

    /* for full multigrid, prolong u for refined initial guess,
       f should be zero from stack initialization
    */
    if (l < lmax)
      prolong_u(l, l+1);
  }
}

