/* thorn_BAM_Elliptic: Multigrid Elliptic Solver for Cactus
   Copyright (C) 1998 Bernd Bruegmann 
*/
/* relax.c */
/* Bernd Bruegmann, 4/97 */

#include "bbmg.h"

#define PR bbmgPR

void rb2_gauss_seidel(struct box *g, int u, int v, int f);
void uni_gauss_seidel(struct box *g, int u, int v, int f);
int rb_decouple, bam_relax;

int solve_nmin, solve_nmax;
double solve_normmax;


/* perform a number of relaxation sweeps */
void relax(int l, int nsweeps)
{
  int i, u, v, w, f;
  struct box *g;
  int pr = 0;

  forlevel(l, g) {
    shrinkrange(g);
    if (pr) printf("before %d sweeps\n", nsweeps);
    if (pr&&l>1) findres(1, g);

    for (i = 1; i <= nsweeps; i++) {
      rb_gauss_seidel(g, u, v, f);
      if (0) printf("relax %4d   normres %10.3e\n", i, findres(0, g));
      if (0)       /* if error small enough, or convergence too slow, or ... */
	break;
    }

    if (pr) printf("after %d sweeps\n", nsweeps);
    if (pr&&l>1) findres(0, g);
    setrangeall(g);
                                  if (pr) prprim("after relax", g, u);
  }
}




/* exactly solve on this level by sufficiently many relaxation sweeps
   applicable only to very small grids
   if you really want a headache, use PETSc here
*/
void exactsolve(int l) 
{
  int i, u, v, w, f;
  struct box *g;
  int nmin = solve_nmin, nmax=solve_nmax;
  double normmax = solve_normmax;
  double normres, normres0;
  int pr = 0||PR;
  int pr2 = 0;
  int o = (problem == VEC4) ? 1 : 0;
  extern int lvcycle, nvcycle;

  forlevel(l, g) {
    shrinkrange(g);
    normres0 = findres(0, g);
    if (pr) printf("exactsolve %4d   normres %.15e\n", 0, normres0);
                               if (pr2) prprim("exact bottom: f", g, f+o);
    for (i = 1; i <= nmax; i++) {
      setrangeall(g);
                               if (pr2&&i<5) prprim("exact bottom: u", g, u+o);
      shrinkrange(g);
      rb_gauss_seidel(g, u, v, f);
      if (i >= nmin) {
	normres = findres(0, g);
#ifdef HAVE_ISNAN
	if (isnan(normres)) 
	  errorexit("exactsolve(): nan");
#endif
	if (pr&&i%nmin==0) 
	  printf("exactsolve %4d   normres %.15e\n", i, normres);
	if (normres < normmax)       /* if error small enough */
	  break;
      }
      if (1 && i < nmin) {
	if (pr) printf("exactsolve %4d   normres %.15e\n", i, findres(0, g));
      }
    }
    setrangeall(g);
                               if (pr2) prprim("after exact : u", g, u+o);
                               if (pr2) prprim("after exact : f", g, f+o);
    if (1) {
      shrinkrange(g);
      normres = findres(0, g);
      if (vb||pr) printf("exactsolve: swept %3d for res %10.3e, V %d, ", 
		     i, normres, (g->m-2)*(g->n-2)*(g->o-2));
      if (vb||pr) printf("vcycle l=%d, n=%d\n\n", lvcycle, nvcycle);
#ifdef HAVE_ISNAN
      if (isnan(normres)) errorexit("exactsolve(): nan");
#endif
      if (0 && normres >= normmax) {
	prprim("after inexact : u", g, u+o);
	prprim("after inexact : f", g, f+o);
      }
    }
  }
}





/* v = L u */
void lop(struct box *g, int u, int v)
{
  int i, j, k, ijk, nc;
  double lu[MGCMAX], *nolii = (double *) NULL;

  /* backward compatibility test */
  if (u) printf("BAM warning: lop with u = %d\n", u);

  setboundrobin(g, u);
  setrangePUGH(g, 1);
  forrange(g) {

    /* finally some physics!! */
    stencil(g, u, ijk, lu, nolii);

    for (nc = 0; nc < g->ncomps; nc++)
      g->store[v+nc][ijk] = lu[nc];
  }

  PUGHsyncvec(g, v);
  setboundmask(g, v, 0);
}




/* red black non-linear Gauss Seidel relaxation
   note that stencils with mixed derivatives overlap, but 
     stencil separation is not strictly required if loss of symmetry allowed,
     tried red, black, green, yellow, ..., didn't really matter
   v is passed in case an intermediate layer is needed
*/
void rb_gauss_seidel(struct box *g, int u, int v, int f)
{
  int pr = 0;
  int i, j, k, ijk, nc;
  double lu[MGCMAX], lii[MGCMAX];
  int o = octoffset + bitoffset + ((cartoon)?oxdo+oydo:0);

  /* may be we don't want red-black at all? */
  if (bam_relax == 1) {
    uni_gauss_seidel(g, u, v, f);
    return;
  }

  /* may be we want to decouple? will be slower */
  if (rb_decouple) {
    rb2_gauss_seidel(g, u, v, f);
    return;
  }

  /* boundary */
  if (1||!cartoon) setboundrobin(g, u);

  /* for odd points */
  setrangePUGH(g, 1);
  forrange(g) {
    if ((i+j+k+o)%2) {
      stencil(g, u, ijk, lu, lii);
      for (nc = 0; nc < g->ncomps; nc++)
	g->store[u+nc][ijk] += (g->store[f+nc][ijk] - lu[nc])/lii[nc];
    }
  }
  PUGHsyncvec(g, u);
  if (masks || (cartoon && !cartoon_stencil)) setboundrobin(g, u);
                               if (pr&&g->l==1) prprim("rbGS after odd", g, u);


  /* for even points */
  setrangePUGH(g, 1);
  forrange(g) {
    if ((i+j+k+o+1)%2) {
      stencil(g, u, ijk, lu, lii);
      for (nc = 0; nc < g->ncomps; nc++)
	g->store[u+nc][ijk] += (g->store[f+nc][ijk] - lu[nc])/lii[nc];
    }
  }
  PUGHsyncvec(g, u);
  setboundrobin(g, u);
                              if (pr&&g->l==1) prprim("rbGS after even", g, u);
}




/* red black non-linear Gauss Seidel relaxation
   decouple by using two levels
*/
void rb2_gauss_seidel(struct box *g, int u, int v, int f)
{
  int pr = 0;
  int i, j, k, ijk, nc;
  double lu[MGCMAX], lii[MGCMAX];
  int o = octoffset + bitoffset + ((cartoon)?oxdo+oydo:0);

  /* boundary */
  setboundrobin(g, u);

  /* make copy */
  setrangeall(g);
  copy(g, u, v);
  shrinkrange(g);

  /* for odd points */
  setrangePUGH(g, 1);
  forrange(g) {
    if ((i+j+k+o)%2) {
      stencil(g, u, ijk, lu, lii);
      for (nc = 0; nc < g->ncomps; nc++)
	g->store[v+nc][ijk] += (g->store[f+nc][ijk] - lu[nc])/lii[nc];
    }
  }
  PUGHsyncvec(g, v);
  if (masks || (cartoon && !cartoon_stencil)) setboundrobin(g, v);
                               if (pr&&g->l==1) prprim("rbGS after odd", g, v);

  /* for even points */
  setrangePUGH(g, 1);
  forrange(g) {
    if ((i+j+k+o+1)%2) {
      stencil(g, v, ijk, lu, lii);
      for (nc = 0; nc < g->ncomps; nc++)
	g->store[u+nc][ijk] += (g->store[f+nc][ijk] - lu[nc])/lii[nc];
    } else {
      for (nc = 0; nc < g->ncomps; nc++)
	g->store[u+nc][ijk] = g->store[v+nc][ijk];
    }
  }
  PUGHsyncvec(g, u);

  /* boundary */
  setboundrobin(g, u);
                              if (pr&&g->l==1) prprim("rbGS after even", g, u);

  /* may have to worry about cleanup */
  setrangeall(g);
  subtract(g, v, v, v);
  shrinkrange(g);
}





/* uniform non-linear Gauss Seidel relaxation
   decouple by using two levels
*/
void uni_gauss_seidel(struct box *g, int u, int v, int f)
{
  int pr = 0;
  int i, j, k, ijk, nc;
  double lu[MGCMAX], lii[MGCMAX];

  /* boundary */
  setboundrobin(g, u);

  /* make copy */
  setrangeall(g);
  copy(g, u, v);
  shrinkrange(g);

  /* for all points */
  setrangePUGH(g, 1);
  forrange(g) {
    stencil(g, v, ijk, lu, lii);
    for (nc = 0; nc < g->ncomps; nc++)
      g->store[u+nc][ijk] += (g->store[f+nc][ijk] - lu[nc])/lii[nc];
  }
  PUGHsyncvec(g, u);
                              if (pr&&g->l==1) prprim("rbGS after uni", g, v);

  /* boundary */
  setboundrobin(g, u);
                              if (pr&&g->l==1) prprim("rbGS after even", g, u);

  /* may have to worry about cleanup */
  setrangeall(g);
  subtract(g, v, v, v);
  shrinkrange(g);

  /* an experiment in filtering */
#ifdef THORN_FILTERGF
  if (0) {
    void FilterGF(pGH *GH);
    FilterGF(g->GH);
  }
  if (0) FilterGF4(g->GH, GFI_BAM_WSMIN+u, (double) 1.0, 0);
#endif
}





