/* thorn_BAM_Elliptic: Multigrid Elliptic Solver for Cactus
   Copyright (C) 1998 Bernd Bruegmann 
*/
/* maxslice.c */
/* Bernd Bruegmann, 10/97 */

#include "bbmg.h"


double maxslice_epsdriver;



/**************************************************************************/

void setnames_maximal() 
{
  int i = 0;

  fieldtitle[i++] = "u";
  fieldtitle[i++] = "v";
  fieldtitle[i++] = "w";
  fieldtitle[i++] = "f";
  fieldtitle[i++] = "K2";
  fieldtitle[i++] = "driver";
  fieldtitle[i++] = "gi11";
  fieldtitle[i++] = "gi12";
  fieldtitle[i++] = "gi13";
  fieldtitle[i++] = "gi22";
  fieldtitle[i++] = "gi23";
  fieldtitle[i++] = "gi33";
  fieldtitle[i++] = "gg1";
  fieldtitle[i++] = "gg2";
  fieldtitle[i++] = "gg3";
}





void setnames_linear() 
{
  int i = 0;

  fieldtitle[i++] = "u";
  fieldtitle[i++] = "v";
  fieldtitle[i++] = "w";
  fieldtitle[i++] = "f";
  fieldtitle[i++] = "M";
  fieldtitle[i++] = "N";
  fieldtitle[i++] = "gi11";
  fieldtitle[i++] = "gi12";
  fieldtitle[i++] = "gi13";
  fieldtitle[i++] = "gi22";
  fieldtitle[i++] = "gi23";
  fieldtitle[i++] = "gi33";
  fieldtitle[i++] = "gg1";
  fieldtitle[i++] = "gg2";
  fieldtitle[i++] = "gg3";
}





/* restrict constant metric data */
void restrict_g_PUGH(cGH *cctkGH, int ltop)
{
  int pr = 0;
  int l, i;
#if 0
  int j, k, ijk;
#endif
  struct box *gh, *gH;
  int u, v, w, f;
  double *uh[17], *uH[17];

  if (vb) printf("................. restrict_g_PUGH, ltop %d\n", ltop);

  for (l = ltop; l >= 2; l--) {
    foroverlap(l, gh, l-1, gH) {

      /* don't, because want boundary, too */
      if (0) setrangeftoc(gh, gH); 

      setfieldpointers_gKalpha(gh, uh);
      setfieldpointers_gKalpha(gH, uH);

      for (i = 0; i < 17; i++) {
	injectftoc_scalar(gh, gH, uh[i], uH[i]);
	if (0) setboundrobin(gH, i);
      }

      /* need sync since inter level */
      CCTK_SyncGroup(cctkGH,"admbase::metric");
      CCTK_SyncGroup(cctkGH,"admbase::curv");
      CCTK_SyncGroup(cctkGH,"admbase::lapse");
      CCTK_SyncGroup(cctkGH,"staticconformal::confac");
      CCTK_SyncGroup(cctkGH,"staticconformal::confac_1derivs");

    }
  }

  /* print */
  for (l = ltop; l >= 1; l--) 
    forlevel(l, gh)
      if (pr) prprimGF("gxx", gh);
  for (l = ltop; l >= 1; l--) 
    forlevel(l, gh)
      if (pr) prprimGF("psi", gh);
}




/* restrict constant coefficients M and N */
void restrict_MN(int ltop)
{
  int pr = 0;
  int l, i;
#if 0
  int j, k, ijk;
#endif
  struct box *gh, *gH;
  int u, v, w, f;
  double *uh[17], *uH[17];

  if (vb) printf("................. restrict_MN, ltop %d\n", ltop);

  for (l = ltop; l >= 2; l--) {
    foroverlap(l, gh, l-1, gH) {

      setfieldpointers_flatlinear(gh, &uh[0], &uh[1]);
      setfieldpointers_flatlinear(gH, &uH[0], &uH[1]);

      for (i = 0; i < 2; i++) {
	injectftoc_scalar(gh, gH, uh[i], uH[i]);
	PUGHsyncd(uH[i]);
      }
    }
  }

  /* print */
  for (l = ltop; l >= 1; l--) 
    forlevel(l, gh)
      if (pr) prprimGF("Mlinear", gh);
  for (l = ltop; l >= 1; l--) 
    forlevel(l, gh)
      if (pr) prprimGF("Nsource", gh);
}




/* restrict constant coefficients and field u */
void restrict_g(int ltop)
{
  int pr = 0;
  int l, i;
#if 0
  int j, k, ijk;
#endif
  struct box *gh, *gH;
  int u, v, w, f;
  void restrict_scalar(struct box *r, struct box *rc, double *u, double *uc);

  if (pr||vb) printf("................. restrict_g, ltop %d\n", ltop);

  /* from fine to coarse levels */
  for (l = ltop; l >= 2; l--) {
    foroverlap(l, gh, l-1, gH) {

      /* u */
      for (i = 0; i < gh->ncomps; i++) {
	/* for all */
	setoverlap(gh, gH); 
	injectftoc_scalar(gh, gH, gh->store[i], gH->store[i]);
	/* for interior we have weighting stencil */
	if (weightedinit) {
	  setrangeftoc(gh, gH);
	  restrict_scalar(gh, gH, gh->store[i], gH->store[i]);
	}
	PUGHsyncn(gH, i, 1);
      }

      /* constant coefficients
         typically not synchronized because not needed except for restrict */
      for (i = MGF*gh->ncomps; i < gh->nfields; i++) {
	setoverlap(gh, gH); 
	injectftoc_scalar(gh, gH, gh->store[i], gH->store[i]);
	if (weightedinit) {
	  if (l == ltop) PUGHsyncn(gh, i, 1);
	  setrangeftoc(gh, gH);
	  restrict_scalar(gh, gH, gh->store[i], gH->store[i]);
	}
	PUGHsyncn(gH, i, 1);
      }

      /* coefficients which figure in stencils need boundary */
      if (stencil_symmetric) {
	i = MGF*gh->ncomps;
	setboundrobin_scalar(gH, i++,  1,  1,  1,  1,  1);   /* gixx */
	setboundrobin_scalar(gH, i++, -1, -1,  1,  0,  1);   /* gixy */
	setboundrobin_scalar(gH, i++, -1,  1, -1,  0,  1);   /* gixz */
	setboundrobin_scalar(gH, i++,  1,  1,  1,  1,  1);   /* giyy */
	setboundrobin_scalar(gH, i++,  1, -1, -1,  0,  1);   /* giyz */
	setboundrobin_scalar(gH, i++,  1,  1,  1,  1,  1);   /* gizz */
	setboundrobin_scalar(gH, i++,  1,  1,  1,  1,  1);   /* gg   */
      }
    }
  }

  if (pr) 
    for (i = 0; i < level[ltop]->next->ncomps; i++)
      for (l = ltop; l >= 1; l--) 
	forlevel(l, gh)
	  prprim("restrict_g", gh, i);
  if (pr) 
    for (i=MGF*level[ltop]->next->ncomps; i < level[ltop]->next->nfields; i++)
      for (l = ltop; l >= 1; l--) 
	forlevel(l, gh)
	  prprim("restrict_g", gh, i);
}




/* init multigrid fields */
void init_mg_box(struct box *g)
{
  int u, v, w, f;
 
  setmgind(g, &u, &v, &w, &f);
  setrangeall(g);
  setconst(g, v, 0.0);
  setconst(g, w, 0.0);
  setconst(g, f, 0.0);
  if (0) printf("inds in init_mg_box: u %d, v %d, w %d, f %d\n", u, v, w, f);
}




/* init one box */
void init_maximal_box(struct box *g)
{
  DECLARE_CCTK_PARAMETERS
  int i, j, k, ijk, nf;
#if 0
  int u, v, w, f;
#endif
  double psi4;
  double *M, *N, *psi, *d;
  
  /* debugging */
  if (0) {
    setrangeall(g);
    setrangePUGH(g, 0);
    for (nf = 1; nf < g->nfields; nf++)
      forrange(g) g->store[nf][ijk] = 0; 
    PUGHsyncall(g);
  }

  /* compute coefficients gi and gg of Laplace operator */
  if (!CCTK_IsThornActive("BonaMasso_Evolution") && 
      CCTK_Equals(bam_laplacian, "BMcactus")) {
    printf("BAM warning: no BM variables but bam_laplacian = \"BMcactus\"\n");
    printf("             using bam_laplacian = \"delginv\"\n");
  }
  if (CCTK_IsThornActive("BonaMasso_Evolution") && 
      CCTK_Equals(bam_laplacian, "BMcactus")) {
    if (vb) printf("--> calling set_laplace_BMcactus()\n");
    set_laplace_BMcactus(g->GH);
  } else {    
    if (CCTK_Equals(bam_laplacian, "symmetric")) {
      if (vb) printf("--> calling set_laplace_sym()\n");
      set_laplace_sym(g);
    } else {
      if (vb) printf("--> calling set_laplace()\n");
      setrangeall(g);
      set_laplace(g);
    }
  }

  /* set M = - gi gi K K, N = driver or 0 */
  if (problem == MAXIMAL) {
    setfieldpointers_flatlinear(g, &M, &N);
    setrangeall(g);
    set_K2(g, M, N);
  }

  /* usually it is a good idea to rescale, do it for all coeffs */
  if (1|| CCTK_Equals(bam_laplacian,"scale"))
    if (conformal_metric) {
      setfieldpointers_psi(g, &psi, &d, &d, &d);
      setrangeall(g);
      setrangePUGH(g, 0);
      forrange(g) {
	psi4 = pow(psi[ijk], (double)4.0);
	if (stencil_symmetric) {
	  if (0) {
	    nf = MGF*g->ncomps;
	    g->store[nf+ 6][ijk] /= psi4;   /* gg = detg, coeff is 1/gg */
	    g->store[nf+ 9][ijk] *= psi4;   /* M */
	    g->store[nf+10][ijk] *= psi4;   /* N */
	  }
	} else {
	  for (nf = MGF*g->ncomps; nf < MGF*g->ncomps + 11; nf++)
	    g->store[nf][ijk] *= psi4;
	}
      }
    }

  /* trivial initializations: v = w = f = 0 */
  init_mg_box(g);

  /* boundary? something smart about alpha? */
}





/* initialize all levels */
void init_maxslice(int init, int ltop, int lmax, struct mgstack **s) 
{
  int l;
  int u, v, w, f;
  struct box *g;
  int pr = 0;

  if (vb) printf("BAM: init_maxslice\n");
  if (!init) return;

  /* the order of restrict-derive or derive-restrict may matter
     octant only allows the latter
  */
  if (0) {
    /* restrict constant metric data */
    restrict_g(ltop);
    
    /* compute coefficients on all levels */
    for (l = lmax; l >= 1; l--)
      forlevel(l, g)
        init_maximal_box(g);
  } else {
    /* compute coefficients for bottom levels */
    for (l = lmax; l >= ltop; l--)
      forlevel(l, g)
	init_maximal_box(g);

    /* restrict constant coefficient data as well as u */
    restrict_g(ltop);

    /* set v = w = f = 0 */
    for (l = lmax-1; l >= 1; l--)
      forlevel(l, g)
	init_mg_box(g);
  }

  for (l = lmax; l >= 1; l--)
    forlevel(l, g) {
      if (pr) prprim("init maximal", g, MGF+0);
      if (pr) prprim("init maximal", g, MGF+1);
      if (pr) prprim("init maximal", g, MGF+2);
      if (pr) prprim("init maximal", g, MGF+10);
    }

  if (vb) printf("BAM: initialized fields, starting solve\n\n");
}





/* initialize all levels */
void init_linear(int init, int ltop, int lmax, struct mgstack **s) 
{
  int l;
  int u, v, w, f;
  struct box *g;
  int pr = 0;
  extern int straightrestrict;

  if (vb) {
    if (eqn_type == POWERS)
      printf("BAM: init_powers\n");
    else
      printf("BAM: init_linear\n");
  }

  /* compute coefficients for bottom levels */
  for (l = lmax; l >= ltop; l--)
    forlevel(l, g)
      init_maximal_box(g);
  
  /* restrict constant coefficient data as well as u */
  straightrestrict = 0;
  if (0) printf("BAM: straightrestrict = %d\n", straightrestrict);
  restrict_g(ltop);
  
  /* set v = w = f = 0 */
  for (l = lmax-1; l >= 1; l--)
    forlevel(l, g)
      init_mg_box(g);

  /* print */
  for (l = lmax; l >= 1; l--)
    forlevel(l, g) {
      if (pr) prprim("init linear", g, MGF+0);
      if (pr) prprim("init linear", g, MGF+1);
      if (pr) prprim("init linear", g, MGF+2);
      if (pr) prprim("init linear", g, MGF+10);
    }

  if (vb) printf("BAM: initialized fields, starting solve\n\n");
}




