/* thorn_BAM_Elliptic: Multigrid Elliptic Solver for Cactus
   Copyright (C) 1998 Bernd Bruegmann 
*/
/* poisson.c */
/* Bernd Bruegmann, 4/97 */

#include "bbmg.h"

int poisson_t;




void setnames_poisson() 
{
  int i = 0;

  fieldtitle[i++] = "u";      /* really loop over comps */
  fieldtitle[i++] = "v";
  fieldtitle[i++] = "w";
  fieldtitle[i++] = "f";
  fieldtitle[i++] = "lin";
  fieldtitle[i++] = "rho";
}



void setfieldpointers_poisson(struct box *g, double **lin, double **rho)
{
  int i = MGF*g->ncomps;

  *lin = g->store[i++];
  *rho = g->store[i++];
}



void init_poisson_box(struct box *g)
{
  int a = 1;    /* odd for nonvanishing center */
  int b = 1;
  int c = 1;
  int i, j, k, ijk;
  int u, v, w, f;
  double r, x, y, z, xx, yy, zz, dx, dy, dz;
  double *lin, *rho;
  
  setfieldpointers_poisson(g, &lin, &rho);
  setmgind(g, &u, &v, &w, &f);

  if (poisson_t) {
    setrangePUGH(g, 0);
    forrange(g) {
      x = g->x + i*g->d;
      y = g->y + j*g->d;
      z = g->z + k*g->d;

      xx = PI * (x - outxmin)/(outxmax-outxmin);
      yy = PI * (y - outxmin)/(outymax-outymin);
      zz = PI * (z - outxmin)/(outzmax-outzmin);
      
      dx = PI*a / (outxmax-outxmin);
      dy = PI*b / (outymax-outymin);
      dz = PI*c / (outzmax-outzmin);

      rho[ijk] = - (dx*dx + dy*dy + dz*dz) * 
                    sin(a*xx) * sin(b*yy) * sin(c*zz);
      
      lin[ijk] = 0;
      g->store[u][ijk] = 0;
      g->store[v][ijk] = 0;
      g->store[w][ijk] = 0;
      g->store[f][ijk] = 0;
    }
  }

  if (poisson_t == 0) {
    double d = 1;
    double xmin = -d;
    double ymin = -d;
    double zmin = -d;
    double xmax = d;
    double ymax = d;
    double zmax = d;

    setrangePUGH(g, 0);
    forrange(g) {
      x = g->x + i*g->d;
      y = g->y + j*g->d;
      z = g->z + k*g->d;
      r = sqrt(x*x + y*y + z*z);
      if (dequal(r, 0)) r = dequaleps;
      g->store[u][ijk] = 1/r;

      if (dless(xmax, x) && dless(ymax, y) && dless(zmax, z) &&
	  dless(x, xmin) && dless(y, ymin) && dless(z, zmin) &&
	  dless(g->x, x) && dless(g->y, y) && dless(g->z, z) &&
	  dless(x, g->xmax) && dless(y, g->ymax) && dless(z, g->zmax))
	g->store[u][ijk] = 0;

      rho[ijk] = 0;
      lin[ijk] = 0;
      g->store[v][ijk] = 0;
      g->store[w][ijk] = 0;
      g->store[f][ijk] = 0;
    }
  }
}




/* initialize
   we don't set one level and then inject because for FMR we can't
*/
void init_poisson(int init, int ltop, int lmax, struct mgstack **s) 
{
  int l;
  int u, v, w, f;
  struct box *g;
  int pr = 0;

  if (!init) return;
  if (pr) printf("BAM: init_poisson, type %d\n", poisson_t);

  for (l = 1; l <= lmax; l++)
    forlevel(l, g) {
      init_poisson_box(g);
      if (0&&pr) prprim("init poisson", g, MGF+1);
    }

  if (pr) printf("BAM: initialized fields, starting solve\n\n");
}



