/* thorn_BAM_Elliptic: Multigrid Elliptic Solver for Cactus
   Copyright (C) 1998 Bernd Bruegmann 
*/
/* powers.c */
/* Bernd Bruegmann, 12/97 */

#include "bbmg.h"

void BAM_Elliptic_Powers(cGH *cctkGH, pGH *GH, int *u, double *p, int n, CCTK_REAL tol);

double ppowers[MGFMAX];





/* wrapper for generic power series */
void BAM_Elliptic_Powers(cGH *cctkGH, pGH *GH, int *u, double *p, int n, CCTK_REAL tol)
{
  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS

  char *l = "----------------------------------------------------------------";
  int i;
  static int first = 1;
  void BAM_Elliptic_Solve(cGH *cctkGH, pGH *GH, int *nf, int n, 
			  int bam_eqn_type, CCTK_REAL tol);

  vb = (bamellnonlin_verbose==1);  
  vbell = (linearelliptic_verbose==1);  

  if ((vb || vbell) && !first) 
    printf("\nBAM: Solve elliptic with various powers\n");
  if (first) {
    first = 0;
    printf("\nBAM: Solve elliptic with various powers\n");
  }

  problem = POWERS;
  eqn_type = POWERS;
  setnames_powers(n);

  /* grid function numbers */
  for (i = 0; i < n && i < MGFMAX; i++) {
    ppowers[i] = p[i];
  }

  /* go for it */
  BAM_Elliptic_Solve(cctkGH, GH, u, n, eqn_type, (double)tol);

  if (vb) printf("%s\n", l);
}




/* bam internal printing */
void setnames_powers(int n) 
{
  int i = 0, j = 3;

  fieldtitle[i++] = "u";
  fieldtitle[i++] = "v";
  fieldtitle[i++] = "w";
  fieldtitle[i++] = "f";
  fieldtitle[i++] = "M";
  fieldtitle[i++] = "N";
  fieldtitle[i++] = "gi11";
  fieldtitle[i++] = "gi12";
  fieldtitle[i++] = "gi13";
  fieldtitle[i++] = "gi22";
  fieldtitle[i++] = "gi23";
  fieldtitle[i++] = "gi33";
  fieldtitle[i++] = "gg1";
  fieldtitle[i++] = "gg2";
  fieldtitle[i++] = "gg3";

  if (n > j++) fieldtitle[i++] = "P1";
  if (n > j++) fieldtitle[i++] = "P2";
  if (n > j++) fieldtitle[i++] = "P3";
  if (n > j++) fieldtitle[i++] = "P4";
  if (n > j++) fieldtitle[i++] = "P5";
}




/* initialize all levels */
/* so far nothing special needed */
void init_powers(int init, int ltop, int lmax, struct mgstack **s) 
{
  init_linear(init, ltop, lmax, s);
}

