/* thorn_BAM_Elliptic: Multigrid Elliptic Solver for Cactus
   Copyright (C) 1998 Bernd Bruegmann 
*/
/* print.c */
/* Bernd Bruegmann, 4/97 */

#include <string.h>
#include "bbmg.h"


/* warning: prim stands for primitive */

void prprim2(struct box *b, double *a, int j0, int j1, int k0, int k1, 
	     int slice, int plane);
void prprimnflag(char *s, struct box *b, int nfield, int inflag);




void prprim2(struct box *b, double *a, int j0, int j1, int k0, int k1, 
	     int slice, int plane)
{
  int j, k;
  double v = 0.0;
  int jmax = j0 + 20;
  int kmax = k0 + 20;

  for (k = k0; k <= k1 && k <= kmax; k++) {
    for (j = j0; j <= j1 && j <= jmax; j++) {

      if (plane == 3)
	v = a[gijkof(b,j,k,slice)];
      else if (plane == 2)
	v = a[gijkof(b,k,slice,j)];
      else if (plane == 1)
	v = a[gijkof(b,slice,j,k)];
      else 
	errorexit("prprim2: invalid plane");

      if (j1-j0 < 10) {
	printf("%10.3e", v);
      }
      else if (j1-j0 < 10) {
	if (v >  99.999) v =  99.999;
	if (v < -99.999) v = -99.999;
	printf("%7.3f", v);
      }
      else {
	if (v >  9.99) v =  9.99;
	if (v < -9.99) v = -9.99;
	printf("%5.2f", v);
      }
    }
    printf("\n");
  }
  fflush(stdout);
}




void prprim(char *s, struct box *b, int nfield)
{
  if (0) printf("GF name: %s\n", locateGFbyDataPtr(b->store[nfield])->name);
  prprimnflag(s, b, nfield, 0);
}


void prprimin(char *s, struct box *b, int nfield)
{
  prprimnflag(s, b, nfield, 1);
}



void prprimnflag(char *s, struct box *b, int nfield, int inflag)
{
  int plane = (cartoon) ? 2 : 3; 
  int i0, j0, k0, i1, j1, k1;
  int li0, lj0, lk0, li1, lj1, lk1;
  int slice;
  double *a = b->store[nfield];
  int showall = (cartoon) ? 1 : 0;

  if (0) {
    plane = 0; 
    showall = 1;
  }

  i0 = b->imin;
  j0 = b->jmin;
  k0 = b->kmin;
  i1 = b->imax;
  j1 = b->jmax;
  k1 = b->kmax;
    
  if (showall) {
    b->imin = 0;
    b->jmin = 0;
    b->kmin = 0;
    b->imax = b->m-1;
    b->jmax = b->n-1;
    b->kmax = b->o-1;

    setrangeall(b);
  }

  if (1) setrangePUGH(b, 0);
  li0 = b->limin;
  lj0 = b->ljmin;
  lk0 = b->lkmin;
  li1 = b->limax;
  lj1 = b->ljmax;
  lk1 = b->lkmax;

  if (inflag) {
    i0++; j0++; k0++; i1--; j1--; k1--;
  }

  if (0) if (strstr(s, "exact bottom")) plane = 0;
  if (0) if (strstr(s, "after inexact")) plane = 0;

  if (1) {
    printf("%.15e, %.15e, %s, %s, t %6.3f, range %d %d  %d %d  %d %d ", 
	   l2norm_scalar(b, nfield), linfnorm_scalar(b, nfield), 
	   s, fieldtitle[nfield], b->t, li0, li1, lj0, lj1, lk0, lk1);
    if (0) return;
  }
  if (0) 
    printf("%s, %s, t %6.3f, range %d %d  %d %d  %d %d  ", 
	   s, fieldtitle[nfield], b->t, li0, li1, lj0, lj1, lk0, lk1);

  if (plane == 3) {
    slice = k0 + (k1-k0)/2;
    printf("k=%d\n", slice); 
    prprim2(b, a, li0, li1, lj0, lj1, slice, plane);
  }

  if (plane == 2) {
    slice = j0 + (j1-j0)/2;
    printf("j=%d\n", slice); 
    prprim2(b, a, lk0, lk1, li0, li1, slice, plane);
  }

  if (plane == 1) {
    slice = i0 + (i1-i0)/2;
    printf("i=%d\n", slice); 
    prprim2(b, a, lj0, lj1, lk0, lk1, slice, plane);
  }

  if (plane == 0) {
    for (slice = lk0; slice <= lk1; slice++) {
      if (slice > lk0) 
	printf("\n%s, %s, t %6.3f, range %d %d  %d %d  %d %d  ", 
	       s, fieldtitle[nfield], b->t, li0, li1, lj0, lj1, lk0, lk1);
      printf("k=%d\n", slice); 
      prprim2(b, a, li0, li1, lj0, lj1, slice, 3);
    }
  }
  printf("\n");

  if (showall) {
    b->imin = i0;
    b->jmin = j0;
    b->kmin = k0;
    b->imax = i1;
    b->jmax = j1;
    b->kmax = k1;
  }
}



void printrestau(int l, double normres, double normtau)
{
  printf("l %2d   |res| %10.3e   |tau| %10.3e\n", l, normres, normtau);
}



void prboxpar(struct box *g)
{
  int i;

  if (!g) {
    printf("struct box *: %p\n", g);
    return;
  }
  printf("struct box *: %p  next %p\n", g, g->next);
  printf("x    %9.6f  y    %9.6f  z    %9.6f\n", g->x, g->y, g->z);
  printf("xmax %9.6f  ymax %9.6f  zmax %9.6f\n", g->xmax, g->ymax, g->zmax);
  printf("d %f  t %f  l %d   m %d  n %d  o %d  di %d  dj %d  dk %d\n", 
	 g->d, g->t, g->l, g->m, g->n, g->o, g->di, g->dj, g->dk);
  printf("bi0  %d  bj0  %d  bk0  %d   bi1  %d  bj1  %d  bk1  %d\n",
	 g->bi0, g->bj0, g->bk0, g->bi1, g->bj1, g->bk1);
  printf(" imin %d   jmin %d   kmin %d    imax %d   jmax %d   kmax %d\n",
	 g->imin, g->jmin, g->kmin, g->imax, g->jmax, g->kmax);
  printf("limin %d  ljmin %d  lkmin %d   limax %d  ljmax %d  lkmax %d",
	 g->limin, g->ljmin, g->lkmin, g->limax, g->ljmax, g->lkmax);
  printf("   lijk %d\n", g->lijk);
  if (0) {
    printf("nfields %d  ncomps %d\n", g->nfields, g->ncomps);
    for (i = 0; g->store[i]; i++)
      printf("%10s %p %p\n", fieldtitle[i], g->store[i], g->GF[i]->data);
  }
}
