/* thorn_BAM_Elliptic: Multigrid Elliptic Solver for Cactus
   Copyright (C) 1998 Bernd Bruegmann 
*/
/* range.c */
/* Bernd Bruegmann, 4/97 */

#include "bbmg.h"


/* return nonzero if x1-x2 is integer multiple of 2*dfine */
#define aligned(x1,x2,dfine) (!(((int)(fabs((x1)-(x2))/(dfine)+0.5))%2)) 


#define pr bbmgPR

int setoverlap1d(double xa1, int m1, double d1, int *imin1, int *imax1,
		 double xa2, int m2, double d2, int *imin2, int *imax2,
                 int dir);




/* set index range for grid transfer operations
   if boundaries are not aligned, move in  
   if no overlap, return 1
   else return 0 and set index ranges
   minimal range will be 1..1
*/
int setoverlap1d(double xa1, int m1, double d1, int *imin1, int *imax1,
		 double xa2, int m2, double d2, int *imin2, int *imax2,
                 int dir)
{
  double xb1, xb2;
  double xa, xb;

  /* what an awful hack to put this here, but since setoverlap1d is only 
     called with xa = gh->x, here we can remove points that are due to stencil 
     width > 1, and put them back before returning below */
  if (octoffset) {
    xa1 += octoffset * d1; m1 -= octoffset;
    xa2 += octoffset * d2; m2 -= octoffset;
  }
  if (bitoffset) {
    if (dir == 2) {
      xa1 += bitoffset * d1; m1 -= bitoffset;
      xa2 += bitoffset * d2; m2 -= bitoffset;
    }
  }
  if (cartoon) {
    if (dir == 0) {
      xa1 += (oxdo-1) * d1; m1-=(oxdo-1);
      xa2 += (oxdo-1) * d2; m2-=(oxdo-1);
    } 
    if (dir == 1) {
      xa1 += (oydo-1) * d1; m1-=2*(oydo-1);
      xa2 += (oydo-1) * d2; m2-=2*(oydo-1);
    } 
  }

  /* should implement opposite order */
  if (dless(d2, d1))
    errorexit("setoverlap1d(): d2 < d1");

  /* upper boundary */
  xb1 = xa1 + (m1-1)*d1; 
  xb2 = xa2 + (m2-1)*d2; 
  
  /* find overlap in double coordinate */
  /* if boundaries are not aligned, MOVE IN fine grid boundary */
  if (aligned(xa1, xa2, d1))
    xa = max(xa1, xa2);
  else
    xa = max(xa1+d1, xa2);
  if (aligned(xb1, xb2, d1))
    xb = min(xb1, xb2);
  else
    xb = min(xb1-d1, xb2);
  if (pr) 
    printf("setoverlap1d: xa1 %f  xb1 %f   xa2 %f  xb2 %f   xa %f  xb %f\n",
	   xa1, xb1, xa2, xb2, xa, xb);
  if (dless(xb, xa)) return 1;

  /* compute and set integer range */
  *imin1 = intof(xa, xa1, d1);
  *imax1 = intof(xb, xa1, d1);
  *imin2 = intof(xa, xa2, d2);
  *imax2 = intof(xb, xa2, d2);

  /* add invisible octant ghosts */
  if (octoffset) {
    *imin1 += octoffset;
    *imax1 += octoffset;
    *imin2 += octoffset;
    *imax2 += octoffset;
  }
  if (bitoffset)
    if (dir == 2) {
      *imin1 += bitoffset;
      *imax1 += bitoffset;
      *imin2 += bitoffset;
      *imax2 += bitoffset;
    }
  if (cartoon) {
    if (dir == 0) {
      *imin1 += oxdo-1;
      *imax1 += oxdo-1;
      *imin2 += oxdo-1;
      *imax2 += oxdo-1;
    }
    if (dir == 1) {
      *imin1 += oydo-1;
      *imax1 += oydo-1;
      *imin2 += oydo-1;
      *imax2 += oydo-1;
    }
  }
  return 0;
}




/* set overlap */
int setoverlap(struct box *gh, struct box *gH)
{
  if (setoverlap1d(gh->x, gh->m, gh->d, &gh->imin, &gh->imax,
                   gH->x, gH->m, gH->d, &gH->imin, &gH->imax,0)) return 1;

  if (setoverlap1d(gh->y, gh->n, gh->d, &gh->jmin, &gh->jmax,
                   gH->y, gH->n, gH->d, &gH->jmin, &gH->jmax,1)) return 1;

  if (setoverlap1d(gh->z, gh->o, gh->d, &gh->kmin, &gh->kmax,
                   gH->z, gH->o, gH->d, &gH->kmin, &gH->kmax,2)) return 1;

  if (pr) prrange("setoverlap, gh", gh);
  if (pr) prrange("setoverlap, gH", gH);
  return 0;
}




/* print range */
void prrange(char *s, struct box *gh)
{
  printf("%s: imin imax ...   %2d %2d  %2d %2d  %2d %2d\n",
	 s, gh->imin, gh->imax, gh->jmin, gh->jmax, gh->kmin, gh->kmax);
  printf("%s: limin limax ... %2d %2d  %2d %2d  %2d %2d\n",
	 s, gh->limin, gh->limax, gh->ljmin, gh->ljmax, gh->lkmin, gh->lkmax);
}




/* set range to the whole box including ghosts */
void setrangeall(struct box *g)
{
  if (!g) return;
  g->imin = g->jmin = g->kmin = 0;
  g->imax = g->m-1;
  g->jmax = g->n-1;
  g->kmax = g->o-1;

  g->imin += octoffset;
  g->jmin += octoffset;
  g->kmin += octoffset;
 
  g->kmin += bitoffset;

  if (cartoon) {
    g->imin += oxdo-1;
    g->jmin += oydo-1;
    g->jmax -= oyup-1;
  }
}




/* shring box by one */
void shrinkrange(struct box *g)
{
  if (!g) return;
  g->imin++;
  g->jmin++;
  g->kmin++;
  g->imax--;
  g->jmax--;
  g->kmax--;
}




/* shrink overlap */
void shrinkoverlap(struct box *gh, struct box *gH)
{
  shrinkrange(gh);
  shrinkrange(gh);
  shrinkrange(gH);
}




/* set range for finetocoarse for given overlap */
void setrangeftoc(struct box *gh, struct box *gH)
{

  DECLARE_CCTK_PARAMETERS

  setoverlap(gh, gH);

  /* Hisaaki added --> */
  if(CCTK_Equals(type,"customBAM") || customizedBCmode == 1){
    gh->imin += 2;
    gH->imin += 1;
    gh->jmin += 2;
    gH->jmin += 1;
    gh->kmin += 2;
    gH->kmin += 1;
    gh->imax -= 2;
    gH->imax -= 1;
    gh->jmax -= 2;
    gH->jmax -= 1;
    gh->kmax -= 2;
    gH->kmax -= 1;
  }
  else{
  /* Hisaaki added <-- */

  if (gh->bi0 != NEUMANN) {
    gh->imin += 2;
    gH->imin += 1;
  }
  if (gh->bj0 != NEUMANN) {
    gh->jmin += 2;
    gH->jmin += 1;
  }
  if (gh->bk0 != NEUMANN) {
    gh->kmin += 2;
    gH->kmin += 1;
  }
  if (gh->bi1 != NEUMANN) {
    gh->imax -= 2;
    gH->imax -= 1;
  }
  if (gh->bj1 != NEUMANN) {
    gh->jmax -= 2;
    gH->jmax -= 1;
  }
  if (gh->bk1 != NEUMANN) {
    gh->kmax -= 2;
    gH->kmax -= 1;
  }
  /* Hisaaki added --> */
  }
  /* Hisaaki added <-- */


  if (pr) prrange("ftoc      , gh", gh);
  if (pr) prrange("ftoc      , gH", gH);
}




/* set range for coarsetofine for given overlap */
void setrangectof(struct box *gH, struct box *gh)
{
  DECLARE_CCTK_PARAMETERS

  setoverlap(gh, gH);

  /* Hisaaki added --> */
  if(CCTK_Equals(type,"customBAM") || customizedBCmode == 1){
    gh->imin += 2;
    gH->imin += 1;
    gh->jmin += 2;
    gH->jmin += 1;
    gh->kmin += 2;
    gH->kmin += 1;
    gh->imax -= 2;
    gH->imax -= 1;
    gh->jmax -= 2;
    gH->jmax -= 1;
    gh->kmax -= 2;
    gH->kmax -= 1;
  }
  else{
  /* Hisaaki added --> */

  if (gh->bi0 == DIRICHLET) {
    gh->imin += 2;
    gH->imin += 1;
  }
  if (gh->bj0 == DIRICHLET) {
    gh->jmin += 2;
    gH->jmin += 1;
  }
  if (gh->bk0 == DIRICHLET) {
    gh->kmin += 2;
    gH->kmin += 1;
  }
  if (gh->bi1 == DIRICHLET) {
    gh->imax -= 2;
    gH->imax -= 1;
  }
  if (gh->bj1 == DIRICHLET) {
    gh->jmax -= 2;
    gH->jmax -= 1;
  }
  if (gh->bk1 == DIRICHLET) {
    gh->kmax -= 2;
    gH->kmax -= 1;
  }
  /* Hisaaki added --> */
  }
  /* Hisaaki added <-- */

  if (pr) prrange("ctof      , gh", gh);
  if (pr) prrange("ctof      , gH", gH);
}
