/* thorn_BAM_Elliptic: Multigrid Elliptic Solver for Cactus
   Copyright (C) 1998 Bernd Bruegmann 
*/
/* restrict.c */
/* Bernd Bruegmann, 4/97 */

#include "bbmg.h"

void restrict_scalar(struct box *r, struct box *rc, double *u, double *uc);
void BIB_injectftoc(struct box *r, struct box *rc, double *u, double *uc);




/* vector restriction operator, refinement factor 2 */
void mgrid_restrict(struct box *r, int u, struct box *rc, int uc)
{
  int nc;

  for (nc = 0; nc < r->ncomps; nc++)
    restrict_scalar(r, rc, r->store[u+nc], rc->store[uc+nc]);
  PUGHsyncvec(rc, uc);
  setboundmask(rc, uc, 1);
}




/* vector inject fine into coarse, refinement factor 2 */
void injectftoc(struct box *r, int u, struct box *rc, int uc)
{
  int nc;

  for (nc = 0; nc < r->ncomps; nc++)
    injectftoc_scalar(r, rc, r->store[u+nc], rc->store[uc+nc]);
  PUGHsyncvec(rc, uc);
}




/* scalar restriction operator, refinement factor 2 
   ranges must allow size 1 boundary layer
   adjoint of trilinear interpolation (?, useful?)
*/
void restrict_scalar(struct box *r, struct box *rc, double *u, double *uc)
{
  DECLARE_CCTK_PARAMETERS

  extern int straightrestrict;
  int i, j, k, ic, jc, kc;
  int ijk, ijkc;
  int di = r->di;
  int dj = r->dj;
  int dk = r->dk;

  if (straightrestrict) {
    injectftoc_scalar(r, rc, u, uc);
    return;
  }

  /* for pure multigrid stack everything is processor local */
  if (rc->l < rc->ltop) {
    setrangePUGH(r, 2);
    setrangePUGH(rc, 1);

    if (cartoon) {
      forrangefc(r, rc) {
	ijk  = gijkof(r, i, j, k);
	ijkc = gijkof(rc,ic,jc,kc);
	uc[ijkc] = 
	  0.25*u[ijk] +
	  0.125*(u[ijk +di] + u[ijk -di] + 
		 u[ijk +dk] + u[ijk -dk]) +
	  0.0625*(u[ijk +di +dk] + u[ijk -di -dk] +
		  u[ijk -di +dk] + u[ijk +di -dk]);
      }
    } else {
      forrangefc(r, rc) {
	ijk  = gijkof(r, i, j, k);
	ijkc = gijkof(rc,ic,jc,kc);
	uc[ijkc] = 
	  0.125*u[ijk] +
	  0.0625*(u[ijk +di] + u[ijk -di] + 
		  u[ijk +dj] + u[ijk -dj] +
		  u[ijk +dk] + u[ijk -dk]) +
	  0.03125*(u[ijk +di +dj] + u[ijk -di -dj] + 
		   u[ijk +dj +dk] + u[ijk -dj -dk] +
		   u[ijk +di +dk] + u[ijk -di -dk] +
		   u[ijk +di -dj] + u[ijk -di +dj] + 
		   u[ijk +dj -dk] + u[ijk -dj +dk] +
		   u[ijk -di +dk] + u[ijk +di -dk]) +
	  0.015625*(u[ijk +di +dj +dk] + u[ijk -di -dj -dk] + 
		    u[ijk -di +dj +dk] + u[ijk +di -dj -dk] +
		    u[ijk +di -dj +dk] + u[ijk -di +dj -dk] +
		    u[ijk +di +dj -dk] + u[ijk -di -dj +dk]);
      }
    }
  }

  /* but for BoxInBox the simplest thing is a local fine double temporary */
  /* ouch */
  if (rc->l >= rc->ltop) {
    double *utmp;
    pGA *GF = locateGFbyDataPtr(u);
    GF->data = utmp = (double *) malloc(sizeof(double)*r->lm*r->ln*r->lo);
    if (!u) errorexit("restrict_scalar(): out of memory");
    if (octant) errorexit("restrict_scalar(): probably wrong for octants");

    setrangePUGH(r, 2);
    forrange(r) {
      utmp[ijk] = 
        0.125*u[ijk] +
        0.0625*(u[ijk +di] + u[ijk -di] + 
                u[ijk +dj] + u[ijk -dj] +
                u[ijk +dk] + u[ijk -dk]) +
        0.03125*(u[ijk +di +dj] + u[ijk -di -dj] + 
  	         u[ijk +dj +dk] + u[ijk -dj -dk] +
  	         u[ijk +di +dk] + u[ijk -di -dk] +
  	         u[ijk +di -dj] + u[ijk -di +dj] + 
	         u[ijk +dj -dk] + u[ijk -dj +dk] +
	         u[ijk -di +dk] + u[ijk +di -dk]) +
        0.015625*(u[ijk +di +dj +dk] + u[ijk -di -dj -dk] + 
		  u[ijk -di +dj +dk] + u[ijk +di -dj -dk] +
		  u[ijk +di -dj +dk] + u[ijk -di +dj -dk] +
		  u[ijk +di +dj -dk] + u[ijk -di -dj +dk]);
    }
    BIB_injectftoc(r, rc, utmp, uc);
    free(utmp);
    GF->data = u;
  }
}





/* inject fine into coarse, refinement factor 2 */
void injectftoc_scalar(struct box *r, struct box *rc, double *u, double *uc)
{
  int pr = 0;
  int i, j, k, ic, jc, kc;
  struct box *g;

  if (rc->l >= rc->ltop) {
    BIB_injectftoc(r, rc, u, uc);
    return;
  }

  setrangePUGH(r, 2);
  g=r;
  if (pr)
  printf("limin %d  ljmin %d  lkmin %d   limax %d  ljmax %d  lkmax %d\n",
	 g->limin, g->ljmin, g->lkmin, g->limax, g->ljmax, g->lkmax);

  setrangePUGH(rc, 1);
  g=rc;
  if (pr)
  printf("limin %d  ljmin %d  lkmin %d   limax %d  ljmax %d  lkmax %d\n",
	 g->limin, g->ljmin, g->lkmin, g->limax, g->ljmax, g->lkmax);

  forrangefc(r, rc) {
    if (0) printf("%d %d %d  %d %d %d   %e %e\n", ic, jc, kc, i, j, k,
		  uc[gijkof(rc,ic,jc,kc)], u[gijkof(r,i,j,k)]);
    uc[gijkof(rc,ic,jc,kc)] = u[gijkof(r,i,j,k)];
  }
}




