/* thorn_BAM_Elliptic: Multigrid Elliptic Solver for Cactus
   Copyright (C) 1998 Bernd Bruegmann 
*/
/* simpleexcision.c */
/* Bernd Bruegmann, 2/00 */

#include "bbmg.h"
 

#define gpleeq(x,y,z) ((x)-(y)<(z)/2)
#define gpequal(x,y,z) (gpleeq(x,y,z)&&gpleeq(y,x,z))

int masks;
double cubemask[12];

double se_oneoverr(struct box *g, double *, double x, double y, double z);

double se_clobber(struct box *g, double *, double x, double y, double z);

double (*se_function)(struct box *g, double *, double x, double y,double z);

int nx, ny, nz;

void
iniboundmask(cGH *cctkGH) 
{
  DECLARE_CCTK_PARAMETERS
  int i, n = 0;

  masks = 0;
  for (i = 0; i < 12; i++)
    cubemask[i] = 0;
 
  nx = cctkGH->cctk_lsh[0];
  ny = cctkGH->cctk_lsh[1];
  nz = cctkGH->cctk_lsh[2];
 
  if (problem == KSD)
    {
      masks = 1;
    }

#ifdef EXCISION_SIMPLEEXCISION
  else
    {
      /* Get the excision masks.
	 (simpleexcision_bam() is found in Development/SimpleExcision) */
      void simpleexcision_bam(cGH *, int *, double *);
      simpleexcision_bam(cctkGH, &masks, cubemask);
    }
#endif

  if (vb) 
    {
      for (n = 1; n <= masks; n++)
	{
	  printf("cubemask%d  ", n);
	  for (i = 0; i < 6; i++)
	    printf("%7.3f", cubemask[i+6*(n-1)]);
	  printf("\n");
	}
    }
}


void
setboundmask(struct box *g, int nf, int nonzero)
{
  int nc, m, ws0_idx;
  int pr = 0;

  if (!masks) return;

  if (problem == BHDATA || problem == BRILL)
    {
      return;
    }
  else if (problem == KSD)
    {
      return;
    }
  else if (problem == POISSON)
    {
      se_function = se_oneoverr;
    }
  else if ((problem == VECLAPLACE) || (problem == VEC4) || (problem == TSD))
    {
      se_function = se_clobber;
    }
  else 
    {
      if (pr) printf("setboundmask(): unknown problem type");
      return;
    }
      
  ws0_idx = CCTK_VarIndex("simpleexcision::se_ws0");

  if (problem == VECLAPLACE) ++ws0_idx;

  for (nc = 0; nc < g->ncomps; nc++)
    { 
      if (pr) prprim("before setboundmask", g, nf+nc);

      for (m = 0; m < masks; m++)
	setboundmask_scalar(g, nf+nc, nonzero, cubemask + 6*m,
			    se_function, ws0_idx+nc);

      if (pr) prprim("after  setboundmask", g, nf+nc);
    }
}

void 
setboundmask_scalar(struct box *g, int nf, int nonzero, double *cmask,
		    double (*se)(struct box *, double *, double, 
				 double, double),
		    int gf_idx)
{
  int i = 0, j, k, ijk;
  double x, y, z;
  double *u = g->store[nf];
  double xmin = cmask[i++];
  double ymin = cmask[i++];
  double zmin = cmask[i++];
  double xmax = cmask[i++];
  double ymax = cmask[i++];
  double zmax = cmask[i++];

  double d;
  double *clobber_data;

  /* find overlap range ... */

  d = g->d;
  clobber_data = GFByIndex(g->GH, gf_idx)->data;

  forrange(g)
    {
      x = g->x + i*d;
      y = g->y + j*d;
      z = g->z + k*d;

      if (gpleeq(x, xmax, d) && gpleeq(y, ymax, d) && gpleeq(z, zmax, d) &&
	  gpleeq(xmin, x, d) && gpleeq(ymin, y, d) && gpleeq(zmin, z, d))
	{

	  /*	  if (nonzero &&
	      (gpequal(x, xmax, d) || gpequal(y, ymax, d) ||
	       gpequal(z, zmax, d) ||
	       gpequal(xmin, x, d) || gpequal(ymin, y, d) ||
	       gpequal(zmin, z, d)))
	  */
	    u[ijk] = se(g, clobber_data, x, y, z);
	    /*
	  else
	    u[ijk] = 0;
	    */
	}
    }
}

/*
 * The gflapse and gfshift functions re-calculate the ijk components based
 * on xyz. In fact, ijk is available to the function from which these
 * are called, so it may eventually be better to replace these function
 * calls with a direct access of the bc_xxx grid function elements. [dp]
 */

double
se_oneoverr(struct box *g, double *data, double x, double y,
	    double z)
{
  double r;

  r = sqrt(x*x + y*y + z*z);

  if (dequal(r, 0)) r = dequaleps;

  return  1/r;
}

double
se_clobber(struct box *g, double *data, double x, double y, double z)
{
  int i, j, k, ijk;

  i = (x - g->x)/g->d;
  j = (y - g->y)/g->d;
  k = (z - g->z)/g->d;

  ijk = i + nx*(j + ny*k);

  return data[ijk];
}
