/* thorn_BAM_Elliptic: Multigrid Elliptic Solver for Cactus
   Copyright (C) 1998 Bernd Bruegmann 
*/
/* veclaplace.c */
/* Bernd Bruegmann, 3/98 */

#include "bbmg.h"

void setboundrobin_GFIndex(struct box *g, int i);
void restrict_vecL_external(struct box *gh, struct box *gH, int i0, int i1);

int veclaplacebssn = 0;


/**************************************************************************/

void setnames_vecL() 
{

  if (problem == VECLAPLACE) {
    fieldtitle[0] = "u1";
    fieldtitle[1] = "u2";
    fieldtitle[2] = "u3";
    fieldtitle[3] = "v1";
    fieldtitle[4] = "v2";
    fieldtitle[5] = "v3";
    fieldtitle[6] = "w1";
    fieldtitle[7] = "w2";
    fieldtitle[8] = "w3";
    fieldtitle[9] = "f1";
    fieldtitle[10] = "f2";
    fieldtitle[11] = "f3";
    fieldtitle[12] = "N1";
    fieldtitle[13] = "N2";
    fieldtitle[14] = "N3";
  } else if (problem == VEC4) {
    fieldtitle[0] = "u0";
    fieldtitle[1] = "u1";
    fieldtitle[2] = "u2";
    fieldtitle[3] = "u3";
    fieldtitle[4] = "v0";
    fieldtitle[5] = "v1";
    fieldtitle[6] = "v2";
    fieldtitle[7] = "v3";
    fieldtitle[8] = "w0";
    fieldtitle[9] = "w1";
    fieldtitle[10] = "w2";
    fieldtitle[11] = "w3";
    fieldtitle[12] = "f0";
    fieldtitle[13] = "f1";
    fieldtitle[14] = "f2";
    fieldtitle[15] = "f3";
  } else if (problem == TSD) {
    fieldtitle[0] = "uv";
    fieldtitle[1] = "ua";
    fieldtitle[2] = "ub1";
    fieldtitle[3] = "ub2";
    fieldtitle[4] = "ub3";
    fieldtitle[5] = "vv";
    fieldtitle[6] = "va";
    fieldtitle[7] = "vb1";
    fieldtitle[8] = "vb2";
    fieldtitle[9] = "vb3";
    fieldtitle[10] = "wv";
    fieldtitle[11] = "wa";
    fieldtitle[12] = "wb1";
    fieldtitle[13] = "wb2";
    fieldtitle[14] = "wb3";
    fieldtitle[15] = "fv";
    fieldtitle[16] = "fa";
    fieldtitle[17] = "fb1";
    fieldtitle[18] = "fb2";
    fieldtitle[19] = "fb3";
  } else if ((problem == KSD) || (problem==YORK)) {
    /* (copied from VEC4 above) */
    fieldtitle[0] = "phi";
    fieldtitle[1] = "wx";
    fieldtitle[2] = "wy";
    fieldtitle[3] = "wz";
    fieldtitle[4] = "v0";
    fieldtitle[5] = "v1";
    fieldtitle[6] = "v2";
    fieldtitle[7] = "v3";
    fieldtitle[8] = "w0";
    fieldtitle[9] = "w1";
    fieldtitle[10] = "w2";
    fieldtitle[11] = "w3";
    fieldtitle[12] = "f0";
    fieldtitle[13] = "f1";
    fieldtitle[14] = "f2";
    fieldtitle[15] = "f3";
  } else {
    errorexit("setnames_vecL(): unknown problem type"); 
  }
}





/* cactus doesn't know about Robin, and we have to put something,
   so let's impose Robin on external cactus fields */
/* Bernd is going to hate me for this, but hey, it is not
   as bad as some other kludges I have seen in here! MM  :-)  */
/* I am speechless, look at that beautiful code 
   anyway, this is much better now that I brought it in line with what certain
   nonconnoiseurs call kludges ... :) */
void setboundrobin_GFIndex(struct box *g, int i)
{
  double *sorg, rporg, rlorg;
  double sx,sy,sz;

  sx = sy = sz = 0.0;

  /* reflection symmetries */
  if( (i == CCTK_VarIndex("staticconformal::psi")) || 
      (i == CCTK_VarIndex("admbase::alp")))
    {  sx = 1.0; sy = 1.0; sz = 1.0; }
  if((i == CCTK_VarIndex("staticconformal::psix")))
    {  sx = -1.0; sy = 1.0; sz = 1.0; }
  if((i == CCTK_VarIndex("staticconformal::psiy")))
    {  sx = 1.0; sy = -1.0; sz = 1.0; }
  if((i == CCTK_VarIndex("staticconformal::psiz")))
    {  sx = 1.0; sy = 1.0; sz = -1.0; }
  if( (i == CCTK_VarIndex("admbase::gxx")) || 
      (i == CCTK_VarIndex("admbase::kxx")) || 
      (i == CCTK_VarIndex("staticconformal::psixx")))
    {  sx = 1.0; sy = 1.0; sz = 1.0; }
  if( (i == CCTK_VarIndex("admbase::gyy")) || 
      (i == CCTK_VarIndex("admbase::kyy")) || 
      (i == CCTK_VarIndex("staticconformal::psiyy")))
    {  sx = 1.0; sy = 1.0; sz = 1.0; }
  if( (i == CCTK_VarIndex("admbase::gzz")) || 
      (i == CCTK_VarIndex("admbase::kzz")) || 
      (i == CCTK_VarIndex("staticconformal::psizz")))
    {  sx = 1.0; sy = 1.0; sz = 1.0; }
  if( (i == CCTK_VarIndex("admbase::gxy")) || 
      (i == CCTK_VarIndex("admbase::kxy")) || 
      (i == CCTK_VarIndex("staticconformal::psixy")))
    {  sx = -1.0; sy = -1.0; sz = 1.0; }
  if( (i == CCTK_VarIndex("admbase::gxz")) || 
      (i == CCTK_VarIndex("admbase::kxz")) || 
      (i == CCTK_VarIndex("staticconformal::psixz")))
    {  sx = -1.0; sy = 1.0; sz = -1.0; }
  if( (i == CCTK_VarIndex("admbase::gyz")) || 
      (i == CCTK_VarIndex("admbase::kyz")) || 
      (i == CCTK_VarIndex("staticconformal::psiyz")))
    {  sx = 1.0; sy = -1.0; sz = -1.0; }

  if (problem == YORK) {
    if (i == CCTK_VarIndex("bam_inidat::YK")   ||
	i == CCTK_VarIndex("bam_inidat::Ygxx") ||
        i == CCTK_VarIndex("bam_inidat::YAxx") ||
	i == CCTK_VarIndex("bam_inidat::Ygyy") ||
        i == CCTK_VarIndex("bam_inidat::YAyy") ||
	i == CCTK_VarIndex("bam_inidat::Ygzz") ||
        i == CCTK_VarIndex("bam_inidat::YAzz")) {
      sx = sy = sz = 1.0;
    }
    if (i == CCTK_VarIndex("bam_inidat::Ygxy") ||
        i == CCTK_VarIndex("bam_inidat::YAxy")) {
      sx = sy = -1.0;
      sz = 1.0;
    }
    if (i == CCTK_VarIndex("bam_inidat::Ygxz") ||
        i == CCTK_VarIndex("bam_inidat::YAxz")) {
      sx = sz = -1.0;
      sy = 1.0;
    }
    if (i == CCTK_VarIndex("bam_inidat::Ygyz") ||
        i == CCTK_VarIndex("bam_inidat::YAyz")) {
      sy = sz = -1.0;
      sx = 1.0;
    }
  }

  /* robin parameters */
  rporg = robin_power;
  rlorg = robin_limit;
  robin_power = 1;
  if (i == CCTK_VarIndex("staticconformal::psi") ||
      i == CCTK_VarIndex("admbase::alp") || 
     (i >= CCTK_VarIndex("admbase::gxx") && 
      i <= CCTK_VarIndex("admbase::gzz")))
    robin_power = robin_limit = 1;
  if (i == CCTK_VarIndex("admbase::gxy") || 
      i == CCTK_VarIndex("admbase::gxz") || 
      i == CCTK_VarIndex("admbase::gyz") )
    robin_limit = 0;
  if ((i >= CCTK_VarIndex("staticconformal::psix") && 
       i <= CCTK_VarIndex("staticconformal::psiz")) || 
      (i >= CCTK_VarIndex("admbase::kxx")  && 
       i <= CCTK_VarIndex("admbase::kzz"))) {
    robin_power = 2; robin_limit = 0;
  }
  if (i >= CCTK_VarIndex("staticconformal::psixx") && 
      i <= CCTK_VarIndex("staticconformal::psizz")) {
    robin_power = 3; robin_limit = 0;
  }
	
  /* call boundary routine */
  sorg = g->store[0];
  g->store[0] = GFByIndex(g->GH, i)->data;
  setboundrobin_scalar(g, 0, sx, sy, sz, robin_limit, robin_power);
  g->store[0] = sorg;
  robin_power = rporg;
  robin_limit = rlorg;
}




/* restrict for external metric data */
void restrict_vecL_external(struct box *gh, struct box *gH, int i0, int i1)
{
  char *name;
  int i;

  /* inject */
  for (i = i0; i <= i1; i++) {
    if (0) printf("CCTK i %d  PUGH gfno %d    CCTK %s  PUGH %s\n", 
		  i, GFByIndex(gH->GH, i)->id, 
		  CCTK_VarName(i), GFByIndex(gH->GH, i)->name);
    setoverlap(gh, gH); 
    injectftoc_scalar(gh, gH, GFByIndex(gh->GH, i)->data,
		      GFByIndex(gH->GH, i)->data);
    PUGHsyncGFn(gH, i, 1);
  }

  /* synchronize */
  /* this fails for some unknown reason */
  if (0) {
    name = CCTK_GroupNameFromVarI(i0);
    if (1) printf("restrict_vecL_external(): %s\n", name); 
    CCTK_SyncGroup(gH->cctkGH, name);
  }

  /* we have to do something ... */ 
  for (i = i0; i <= i1; i++) {
    setboundrobin_GFIndex(gH, i);
  }
}


/* restrict constant metric data */
void restrict_g_PUGH_vecL(int ltop)
{
  DECLARE_CCTK_PARAMETERS
  int pr = 0;
  int l, i, i0, i1;
  struct box *gh, *gH;
  int u, v, w, f;

  if (pr) printf("................. restrict_g_PUGH_vecL, ltop %d\n", ltop);
  if (vb && CCTK_Equals(bam_bound, "bamrobin")) {
    printf("BAM Warning: you have bound=bamrobin\n");
    printf("note that veclaplace.c assumes asymptotic flatness\n");
  }

  for (l = ltop; l >= 2; l--) {
    foroverlap(l, gh, l-1, gH) {

      /* don't, because want boundary, too */
      if (0) setrangeftoc(gh, gH); 

      /* external metric data */
      if (eqn_type == VECLAPLACE) {
	if (!veclaplacebssn) {

	  /* conformal factor */
	  if (conformal_metric) {
	    i0 = CCTK_VarIndex("staticconformal::psi");
	    restrict_vecL_external(gh, gH, i0, i0);
	    i0 = CCTK_VarIndex("staticconformal::psix");
	    i1 = CCTK_VarIndex("staticconformal::psiz");
	    restrict_vecL_external(gh, gH, i0, i1);
	    i0 = CCTK_VarIndex("staticconformal::psixx");
	    i1 = CCTK_VarIndex("staticconformal::psizz");
	    restrict_vecL_external(gh, gH, i0, i1);
	  }
	  
	  /* alpha, g, K */
	  i0 = CCTK_VarIndex("admbase::alp");
	  restrict_vecL_external(gh, gH, i0, i0);
	  i0 = CCTK_VarIndex("admbase::gxx");
	  i1 = CCTK_VarIndex("admbase::gzz");
	  restrict_vecL_external(gh, gH, i0, i1);
	  i0 = CCTK_VarIndex("admbase::kxx");
	  i1 = CCTK_VarIndex("admbase::kzz");
	  restrict_vecL_external(gh, gH, i0, i1);
	}

	/* precomputed for bssn stencil; now we have GFI */
	else {
	  restrict_vecL_external(gh, gH, GFI_vl_bssn, GFI_vl_bssn+11);
	}

	/* N^i */
	i0 = gh->ncomps*MGF;
	i1 = i0 + gh->ncomps;
	setrangeall(gH);
	subtract(gH, i0, i0, i0);
	for (i = i0; i < i1; i++) {
	  setoverlap(gh, gH); 
	  injectftoc_scalar(gh, gH, gh->store[i], gH->store[i]); 
	  PUGHsyncn(gH, i, 1);
	}
      }

      /* external metric data for generic four vector problem */
      if (eqn_type == VEC4) {
	if (problem == VEC4) {
	  restrict_vecL_external(gh, gH, GFI_vl_N1, GFI_vl_N1+2);
	  restrict_vecL_external(gh, gH, GFI_vl_bssn, GFI_vl_bssn+17);
	} else {
	  errorexit("restrict_g_PUGH_vecL(): unfinished code");
	}
      }

      /* external data for thin sandwich data */
      if (eqn_type == TSD) {
	i0 = CCTK_VarIndex("staticconformal::psi");
	restrict_vecL_external(gh, gH, i0, i0);
	i0 = CCTK_VarIndex("staticconformal::psix");
	i1 = CCTK_VarIndex("staticconformal::psiz");
	restrict_vecL_external(gh, gH, i0, i1);
      }

      /* external data for Kerr-Schild data */
      if (eqn_type == KSD) {
	i0 = CCTK_VarIndex("admbase::gxx");
	restrict_vecL_external(gh, gH, i0, i0+5);
	i0 = CCTK_VarIndex("admbase::kxx");
	i1 = CCTK_VarIndex("admbase::kzz");
	restrict_vecL_external(gh, gH, i0, i1);
	i0 = CCTK_VarIndex("bam_inidat::igxx");
	restrict_vecL_external(gh, gH, i0, i0+5);
	i0 = CCTK_VarIndex("bam_inidat::K");
	restrict_vecL_external(gh, gH, i0, i0+0);
	i0 = CCTK_VarIndex("bam_inidat::Ku_x");
	restrict_vecL_external(gh, gH, i0, i0+2);
	i0 = CCTK_VarIndex("bam_inidat::Au_x");
	restrict_vecL_external(gh, gH, i0, i0+2);
	i0 = CCTK_VarIndex("bam_inidat::R");
	restrict_vecL_external(gh, gH, i0, i0+0);
      }
      
      /* external data for the York initial value problem */
      if (eqn_type == YORK) {
	i0 = CCTK_VarIndex("staticconformal::psi");
	restrict_vecL_external(gh, gH, i0, i0);
	i0 = CCTK_VarIndex("staticconformal::psix");
	restrict_vecL_external(gh, gH, i0, i0+2);
	i0 = CCTK_VarIndex("staticconformal::psixx");
	restrict_vecL_external(gh, gH, i0, i0+5);
	i0 = CCTK_VarIndex("admbase::gxx");
	restrict_vecL_external(gh, gH, i0, i0+5);
	i0 = CCTK_VarIndex("admbase::kxx");
	restrict_vecL_external(gh, gH, i0, i0+5);
	i0 = CCTK_VarIndex("bam_inidat::K");
	restrict_vecL_external(gh, gH, i0, i0+0);
	restrict_vecL_external(gh, gH, GFI_Ygxx, GFI_Ygxx + 12);
	restrict_vecL_external(gh, gH, GFI_YABYxx, GFI_YABYxx + 9);
      }

      /* u^i */
      i0 = 0;
      i1 = i0 + gh->ncomps;
      for (i = i0; i < i1; i++) {
	setoverlap(gh, gH); 
	injectftoc_scalar(gh, gH, gh->store[i], gH->store[i]); 
	PUGHsyncn(gH, i, 1);
      }
      setboundrobin(gH, i0);

    }
  }

  /* print */
  for (l = ltop; l >= 1; l--) 
    forlevel(l, gh) {
      if (0) prprimGF("bam_veclap::vlgg1", gh);
      if (0) prprimGF("bam_veclap::vlgh1", gh);
      if (0) prprimGF("bam_inidat::Ygxx", gh);
    }
  if (conformal_metric) {
    for (l = ltop; l >= 1; l--) 
      forlevel(l, gh)
	if (pr) prprimGF("staticconformal::psi", gh);
  }
}





/* check for nan */
int checkfornan(struct box *g, int ng, int nGH) 
{
  int flag = 0;
  int i, j, k, ijk, lijk = g->lijk;
  int di = g->di, dj = g->dj, dk = g->dk;
  double *u;
  char *s;

  if (ng >= 0) {
    u = g->store[ng];
    s = fieldtitle[ng];
  } else if (nGH >= 0) {
    u = GFByIndex(g->GH, nGH)->data;
    s = GFByIndex(g->GH, nGH)->name;
  } else
    return 0;

  /* loop without using range to avoid side effects */
  for (ijk = 0; ijk < g->lm*g->ln*g->lo; ijk++) {
    if (isnan(u[ijk])) {
      ijkto(i,j,k);
      printf("l=%d: %s = nan at (%d,%d,%d)=%d\n", g->l, s, i, j, k, ijk);
      flag = 1;
      break;
    }
  }
  return flag;
}

int checkfornanGF(pGA *GF) 
{
  int ijk, n;
  double *u;
  char *s;

  if (!GF) {
    printf("GF = NULL\n");
    return -1;
  }
  u = GF->data;
  if (!u) {
    printf("GF->data = NULL\n");
    return -2;
  }
  s = GF->name;
  n = GF->extras->npoints;

  for (ijk = 0; ijk < n; ijk++) {
    if (isnan(u[ijk])) {
      printf("grid function %s has NaN at linear index %d\n", s, ijk);
      return 1;
    }
  }
  return 0;
}




/* check all fields for nan */
void nan_vecL(struct box *g) 
{
  int pr = bbmgPR;
  int ex = 0;
  static int first = 1;
  int i, i0, i1, flag = 0;
  
  if (pr && vb) printf("Checking whether there are nans ...\n");
  if (0 || !first) {
    first = 0;
    if (conformal_metric) 
      i0 = CCTK_VarIndex("staticconformal::psi");
    else
      i0 = CCTK_VarIndex("admbase::alp");
    i1 = CCTK_VarIndex("admbase::kzz");
    for (i = i0; i <= i1; i++)
      if (checkfornan(g, -1, i)) flag = 1;
    if (ex && flag) errorexit("nan");
  }

  for (i = 0; i < g->nfields; i++)
    if (checkfornan(g, i, -1)) flag = 1;
  if (ex && flag) errorexit("nan");
  if (pr && vb && !flag) printf("no nan found\n");
}




/* init one box */
void init_box_vecL(struct box *g)
{
  DECLARE_CCTK_PARAMETERS
  int i, j, k, ijk, nf;
  double psi4;
  double *M, *N, *psi, *d;

  /* debugging */
  if (0) {
    setrangeall(g);
    setrangePUGH(g, 0);
    for (nf = 1; nf < g->nfields; nf++)
      forrange(g) g->store[nf][ijk] = 0; 
    PUGHsyncall(g);
  }

  /* compute coefficients gi and gg of Laplace operator */
  if (!CCTK_IsThornActive("BonaMasso_Evolution") && 
      CCTK_Equals(bam_laplacian, "BMcactus")) {
    printf("BAM warning: no BM variables but bam_laplacian = \"BMcactus\"\n");
    printf("             using bam_laplacian = \"delginv\"\n");
  }
  if (CCTK_IsThornActive("BonaMasso_Evolution") && 
      CCTK_Equals(bam_laplacian, "BMcactus")) {
    set_laplace_BMcactus(g->GH);
  } else {    
    setrangeall(g);
    set_laplace(g);
  }

  /* set M = - gi gi K K, N = driver or 0 */
  if (problem == MAXIMAL) {
    setfieldpointers_flatlinear(g, &M, &N);
    setrangeall(g);
    set_K2(g, M, N);
  }

  /* usually it is a good idea to rescale, do it for all coeffs */
  if (1||CCTK_Equals(bam_laplacian,"scale"))
    if (conformal_metric) {
      setfieldpointers_psi(g, &psi, &d, &d, &d);
      setrangeall(g);
      setrangePUGH(g, 0);
      forrange(g) {
	psi4 = pow(psi[ijk], (double)4.0);
	for (nf = MGF*g->ncomps; nf < MGF*g->ncomps + 11; nf++)
	  g->store[nf][ijk] *= psi4;
      }
    }

  /* trivial initializations: v = w = f = 0 */
  init_mg_box(g);

  /* boundary? something smart about alpha? */
}





/* initialize all levels */
void init_vecL(int init, int ltop, int lmax, struct mgstack **s) 
{
  int l;
  int u, v, w, f;
  struct box *g;
  int pr = 0;
  extern int straightrestrict;

  /* welcome */
  if (vb) {
    printf("BAM: init_vecL\n");
    forlevel(lmax, g) {

      /* debug */
      if (pr) prprim("init_vecL", g, 0);
      if (pr) prprim("init_vecL", g, 1);
      if (pr) prprim("init_vecL", g, 2);

      /* check analytic shift directly */
      if (0) {
	shrinkrange(g);
	lop(g, u, v);
	prprim("init_vecL", g, 3);
	prprim("init_vecL", g, 12);
	prprim("init_vecL", g, 4);
	prprim("init_vecL", g, 13);
	prprim("init_vecL", g, 5);
	prprim("init_vecL", g, 14);
      }
    }
  }

  /* compute coefficients for bottom levels */
  /* actually, we don't pre compute for now */
  if (lmax != ltop) errorexit("init_vecL(): lmax != ltop -> update for BiB!");
  if (0) {
    for (l = lmax; l >= ltop; l--)
      forlevel(l, g) init_maximal_box(g);
  }
  
  /* restrict constant coefficient data as well as u */
  straightrestrict = 0;
  restrict_g_PUGH_vecL(ltop);

  /* set v = w = f = 0 */
  for (l = lmax; l >= 1; l--)
    forlevel(l, g)
      init_mg_box(g);

  /* print */
  for (l = lmax; l >= 1; l--)
    forlevel(l, g) {
      if (pr) {
	prprim("init_vecL", g, 0);
	prprim("init_vecL", g, 1);
	prprim("init_vecL", g, 2);
	prprim("init_vecL", g, 3);
	prprim("init_vecL", g, 12);
	prprim("init_vecL", g, 13);
	prprim("init_vecL", g, 14);
      }
    }

  for (l = lmax; l >= 1; l--)
    forlevel(l, g)
      nan_vecL(g);
  if (vb) printf("BAM: initialized fields, starting solve\n\n");
}




