/* bambctgauge.c */
/* Bernd Bruegmann, 15.12.1999 */
/* Produced with Mathematica */

#include "bamveclap.h"
extern int minstrain;

#define Power(x,y) (pow((double) (x), (double) (y)))
#define Log(x)     log((double) (x)))
#define pow2(x)    ((x)*(x))
#define pow2inv(x) (1.0/((x)*(x)))

#define Cal(x,y,z) ((x)?(y):(z))





void bambctgauge(CCTK_ARGUMENTS) {

DECLARE_CCTK_ARGUMENTS
int nocd = (*conformal_state == 0);
int i, j, k, di, dj, dk, ijk;
double dx = cctkGH->cctk_delta_space[0];
double dy = cctkGH->cctk_delta_space[1];
double dz = cctkGH->cctk_delta_space[2];
double oo2dx = 1/(2*dx), oo2dy = 1/(2*dy), oo2dz = 1/(2*dz);
double oodx2 = 1/(dx*dx), oody2 = 1/(dy*dy), oodz2 = 1/(dz*dz);
double oo4dxdy = 1/(4*dx*dy);
double oo4dxdz = 1/(4*dx*dz);
double oo4dydz = 1/(4*dy*dz);

double f;
double fdetg;
double delg111;
double delg112;
double delg113;
double delg122;
double delg123;
double delg133;
double delg211;
double delg212;
double delg213;
double delg222;
double delg223;
double delg233;
double delg311;
double delg312;
double delg313;
double delg322;
double delg323;
double delg333;
double delgb111;
double delgb112;
double delgb113;
double delgb122;
double delgb123;
double delgb133;
double delgb211;
double delgb212;
double delgb213;
double delgb222;
double delgb223;
double delgb233;
double delgb311;
double delgb312;
double delgb313;
double delgb322;
double delgb323;
double delgb333;
double delu11;
double delu12;
double delu13;
double delu21;
double delu22;
double delu23;
double delu31;
double delu32;
double delu33;
double df1;
double df2;
double df3;
double gginv11;
double gginv12;
double gginv13;
double gginv22;
double gginv23;
double gginv33;
double ginv11;
double ginv12;
double ginv13;
double ginv22;
double ginv23;
double ginv33;
double Kup11;
double Kup12;
double Kup13;
double Kup22;
double Kup23;
double Kup33;



#define vlN1 vl_N1
#define vlN2 vl_N2
#define vlN3 vl_N3

forinnerpoints(cctkGH) {


f
=
Cal(nocd,1,Power(psi[ijk],4))
;

df1
=
Cal(nocd,0,4*f*psix[ijk])
;

df2
=
Cal(nocd,0,4*f*psiy[ijk])
;

df3
=
Cal(nocd,0,4*f*psiz[ijk])
;

gginv11
=
gyy[ijk]*gzz[ijk] - pow2(gyz[ijk])
;

gginv12
=
gxz[ijk]*gyz[ijk] - gxy[ijk]*gzz[ijk]
;

gginv13
=
-(gxz[ijk]*gyy[ijk]) + gxy[ijk]*gyz[ijk]
;

gginv22
=
gxx[ijk]*gzz[ijk] - pow2(gxz[ijk])
;

gginv23
=
gxy[ijk]*gxz[ijk] - gxx[ijk]*gyz[ijk]
;

gginv33
=
gxx[ijk]*gyy[ijk] - pow2(gxy[ijk])
;

fdetg
=
f*(gginv11*gxx[ijk] + gginv12*gxy[ijk] + gginv13*gxz[ijk])
;

ginv11
=
gginv11/fdetg
;

ginv12
=
gginv12/fdetg
;

ginv13
=
gginv13/fdetg
;

ginv22
=
gginv22/fdetg
;

ginv23
=
gginv23/fdetg
;

ginv33
=
gginv33/fdetg
;

delgb111
=
oo2dx*(-gxx[-di + ijk] + gxx[di + ijk])
;

delgb112
=
oo2dx*(-gxy[-di + ijk] + gxy[di + ijk])
;

delgb113
=
oo2dx*(-gxz[-di + ijk] + gxz[di + ijk])
;

delgb122
=
oo2dx*(-gyy[-di + ijk] + gyy[di + ijk])
;

delgb123
=
oo2dx*(-gyz[-di + ijk] + gyz[di + ijk])
;

delgb133
=
oo2dx*(-gzz[-di + ijk] + gzz[di + ijk])
;

delgb211
=
oo2dy*(-gxx[-dj + ijk] + gxx[dj + ijk])
;

delgb212
=
oo2dy*(-gxy[-dj + ijk] + gxy[dj + ijk])
;

delgb213
=
oo2dy*(-gxz[-dj + ijk] + gxz[dj + ijk])
;

delgb222
=
oo2dy*(-gyy[-dj + ijk] + gyy[dj + ijk])
;

delgb223
=
oo2dy*(-gyz[-dj + ijk] + gyz[dj + ijk])
;

delgb233
=
oo2dy*(-gzz[-dj + ijk] + gzz[dj + ijk])
;

delgb311
=
oo2dz*(-gxx[-dk + ijk] + gxx[dk + ijk])
;

delgb312
=
oo2dz*(-gxy[-dk + ijk] + gxy[dk + ijk])
;

delgb313
=
oo2dz*(-gxz[-dk + ijk] + gxz[dk + ijk])
;

delgb322
=
oo2dz*(-gyy[-dk + ijk] + gyy[dk + ijk])
;

delgb323
=
oo2dz*(-gyz[-dk + ijk] + gyz[dk + ijk])
;

delgb333
=
oo2dz*(-gzz[-dk + ijk] + gzz[dk + ijk])
;

delg111
=
delgb111*f + df1*gxx[ijk]
;

delg112
=
delgb112*f + df1*gxy[ijk]
;

delg113
=
delgb113*f + df1*gxz[ijk]
;

delg122
=
delgb122*f + df1*gyy[ijk]
;

delg123
=
delgb123*f + df1*gyz[ijk]
;

delg133
=
delgb133*f + df1*gzz[ijk]
;

delg211
=
delgb211*f + df2*gxx[ijk]
;

delg212
=
delgb212*f + df2*gxy[ijk]
;

delg213
=
delgb213*f + df2*gxz[ijk]
;

delg222
=
delgb222*f + df2*gyy[ijk]
;

delg223
=
delgb223*f + df2*gyz[ijk]
;

delg233
=
delgb233*f + df2*gzz[ijk]
;

delg311
=
delgb311*f + df3*gxx[ijk]
;

delg312
=
delgb312*f + df3*gxy[ijk]
;

delg313
=
delgb313*f + df3*gxz[ijk]
;

delg322
=
delgb322*f + df3*gyy[ijk]
;

delg323
=
delgb323*f + df3*gyz[ijk]
;

delg333
=
delgb333*f + df3*gzz[ijk]
;

delu11
=
oo2dx*(-betax[-di + ijk] + betax[di + ijk])
;

delu12
=
oo2dx*(-betay[-di + ijk] + betay[di + ijk])
;

delu13
=
oo2dx*(-betaz[-di + ijk] + betaz[di + ijk])
;

delu21
=
oo2dy*(-betax[-dj + ijk] + betax[dj + ijk])
;

delu22
=
oo2dy*(-betay[-dj + ijk] + betay[dj + ijk])
;

delu23
=
oo2dy*(-betaz[-dj + ijk] + betaz[dj + ijk])
;

delu31
=
oo2dz*(-betax[-dk + ijk] + betax[dk + ijk])
;

delu32
=
oo2dz*(-betay[-dk + ijk] + betay[dk + ijk])
;

delu33
=
oo2dz*(-betaz[-dk + ijk] + betaz[dk + ijk])
;

vlLiebgxx[ijk]
=
delg111*betax[ijk] + delg211*betay[ijk] + delg311*betaz[ijk] + 
   2*f*(delu11*gxx[ijk] + delu12*gxy[ijk] + delu13*gxz[ijk])
;

vlLiebgxy[ijk]
=
delg112*betax[ijk] + delg212*betay[ijk] + delg312*betaz[ijk] + 
   f*(delu21*gxx[ijk] + (delu11 + delu22)*gxy[ijk] + delu23*gxz[ijk] + 
      delu12*gyy[ijk] + delu13*gyz[ijk])
;

vlLiebgxz[ijk]
=
delg113*betax[ijk] + delg213*betay[ijk] + delg313*betaz[ijk] + 
   f*(delu31*gxx[ijk] + delu32*gxy[ijk] + (delu11 + delu33)*gxz[ijk] + 
      delu12*gyz[ijk] + delu13*gzz[ijk])
;

vlLiebgyy[ijk]
=
delg122*betax[ijk] + delg222*betay[ijk] + delg322*betaz[ijk] + 
   2*f*(delu21*gxy[ijk] + delu22*gyy[ijk] + delu23*gyz[ijk])
;

vlLiebgyz[ijk]
=
delg123*betax[ijk] + delg223*betay[ijk] + delg323*betaz[ijk] + 
   f*(delu31*gxy[ijk] + delu21*gxz[ijk] + delu32*gyy[ijk] + 
      (delu22 + delu33)*gyz[ijk] + delu23*gzz[ijk])
;

vlLiebgzz[ijk]
=
delg133*betax[ijk] + delg233*betay[ijk] + delg333*betaz[ijk] + 
   2*f*(delu31*gxz[ijk] + delu32*gyz[ijk] + delu33*gzz[ijk])
;

Kup11
=
2*(ginv11*(ginv12*kxy[ijk] + ginv13*kxz[ijk]) + ginv12*ginv13*kyz[ijk]) + 
   kxx[ijk]*pow2(ginv11) + kyy[ijk]*pow2(ginv12) + kzz[ijk]*pow2(ginv13)
;

Kup12
=
ginv11*(ginv12*kxx[ijk] + ginv22*kxy[ijk] + ginv23*kxz[ijk]) + 
   ginv12*(ginv13*kxz[ijk] + ginv22*kyy[ijk] + ginv23*kyz[ijk]) + 
   ginv13*(ginv22*kyz[ijk] + ginv23*kzz[ijk]) + kxy[ijk]*pow2(ginv12)
;

Kup13
=
ginv11*(ginv13*kxx[ijk] + ginv23*kxy[ijk] + ginv33*kxz[ijk]) + 
   ginv12*(ginv13*kxy[ijk] + ginv23*kyy[ijk] + ginv33*kyz[ijk]) + 
   ginv13*(ginv23*kyz[ijk] + ginv33*kzz[ijk]) + kxz[ijk]*pow2(ginv13)
;

Kup22
=
2*(ginv12*(ginv22*kxy[ijk] + ginv23*kxz[ijk]) + ginv22*ginv23*kyz[ijk]) + 
   kxx[ijk]*pow2(ginv12) + kyy[ijk]*pow2(ginv22) + kzz[ijk]*pow2(ginv23)
;

Kup23
=
ginv13*(ginv22*kxy[ijk] + ginv23*kxz[ijk]) + 
   ginv12*(ginv13*kxx[ijk] + ginv23*kxy[ijk] + ginv33*kxz[ijk]) + 
   ginv22*(ginv23*kyy[ijk] + ginv33*kyz[ijk]) + ginv23*ginv33*kzz[ijk] + 
   kyz[ijk]*pow2(ginv23)
;

Kup33
=
2*(ginv13*(ginv23*kxy[ijk] + ginv33*kxz[ijk]) + ginv23*ginv33*kyz[ijk]) + 
   kxx[ijk]*pow2(ginv13) + kyy[ijk]*pow2(ginv23) + kzz[ijk]*pow2(ginv33)
;

vlKK[ijk]
=
Kup11*kxx[ijk] + Kup22*kyy[ijk] + 
   2*(Kup12*kxy[ijk] + Kup13*kxz[ijk] + Kup23*kyz[ijk]) + Kup33*kzz[ijk]
;

vlKupnxx[ijk]
=
Kup11/sqrt(1.e-12 + vlKK[ijk])
;

vlKupnxy[ijk]
=
Kup12/sqrt(1.e-12 + vlKK[ijk])
;

vlKupnxz[ijk]
=
Kup13/sqrt(1.e-12 + vlKK[ijk])
;

vlKupnyy[ijk]
=
Kup22/sqrt(1.e-12 + vlKK[ijk])
;

vlKupnyz[ijk]
=
Kup23/sqrt(1.e-12 + vlKK[ijk])
;

vlKupnzz[ijk]
=
Kup33/sqrt(1.e-12 + vlKK[ijk])
;

alp[ijk]
=
(Kup11*vlLiebgxx[ijk] + Kup22*vlLiebgyy[ijk] + 
     2*(Kup12*vlLiebgxy[ijk] + Kup13*vlLiebgxz[ijk] + Kup23*vlLiebgyz[ijk]) + 
     Kup33*vlLiebgzz[ijk])/(2.e-12 + 2*vlKK[ijk])
;



}  /* loop i, j, k */

}  /* function */

/* bambctgauge.c */
/* n* = 237,  n/ = 33,  n+ = 267, nvars = 39 */
