/* bambssnstencil.c */
/* Bernd Bruegmann, 31.3.2000 */
/* Produced with Mathematica */

#include "bamveclap.h"
extern int minstrain, mindistortion, bctgauge, confharmonic, nocd;

#define Power(x,y) (pow((double) (x), (double) (y)))
#define Log(x)     log((double) (x)))
#define pow2(x)    ((x)*(x))
#define pow2inv(x) (1.0/((x)*(x)))

#define Cal(x,y,z) ((x)?(y):(z))



void BAM_VecLap_BSSN_Stencil (CCTK_INT ijk,
                              CCTK_INT di,
                              CCTK_INT dj,
                              CCTK_INT dk,
                              CCTK_REAL dx,
                              CCTK_REAL dy,
                              CCTK_REAL dz,
                              CCTK_REAL *ginv11,
                              CCTK_REAL *ginv12,
                              CCTK_REAL *ginv13,
                              CCTK_REAL *ginv22,
                              CCTK_REAL *ginv23,
                              CCTK_REAL *ginv33,
                              CCTK_REAL *g1,
                              CCTK_REAL *g2,
                              CCTK_REAL *g3,
                              CCTK_REAL *G1,
                              CCTK_REAL *G2,
                              CCTK_REAL *G3,
                              CCTK_REAL *beta1,
                              CCTK_REAL *beta2,
                              CCTK_REAL *beta3,
                              CCTK_REAL *n1,
                              CCTK_REAL *n2,
                              CCTK_REAL *n3,
                              CCTK_REAL *Lu1,
                              CCTK_REAL *Lu2,
                              CCTK_REAL *Lu3,
                              CCTK_REAL *dLdu1,
                              CCTK_REAL *dLdu2,
                              CCTK_REAL *dLdu3)
{

double oo2dx = 1/(2*dx), oo2dy = 1/(2*dy), oo2dz = 1/(2*dz);
double oodx2 = 1/(dx*dx), oody2 = 1/(dy*dy), oodz2 = 1/(dz*dz);
double oo4dxdy = 1/(4*dx*dy);
double oo4dxdz = 1/(4*dx*dz);
double oo4dydz = 1/(4*dy*dz);

double db11;
double db12;
double db13;
double db21;
double db22;
double db23;
double db31;
double db32;
double db33;
double ddb111;
double ddb112;
double ddb113;
double ddb121;
double ddb122;
double ddb123;
double ddb131;
double ddb132;
double ddb133;
double ddb221;
double ddb222;
double ddb223;
double ddb231;
double ddb232;
double ddb233;
double ddb331;
double ddb332;
double ddb333;
double dG11;
double dG12;
double dG13;
double dG21;
double dG22;
double dG23;
double dG31;
double dG32;
double dG33;
double LieG1;
double LieG2;
double LieG3;



dG11
=
oo2dx*(-G1[-di + ijk] + G1[di + ijk])
;

dG12
=
oo2dx*(-G2[-di + ijk] + G2[di + ijk])
;

dG13
=
oo2dx*(-G3[-di + ijk] + G3[di + ijk])
;

dG21
=
oo2dy*(-G1[-dj + ijk] + G1[dj + ijk])
;

dG22
=
oo2dy*(-G2[-dj + ijk] + G2[dj + ijk])
;

dG23
=
oo2dy*(-G3[-dj + ijk] + G3[dj + ijk])
;

dG31
=
oo2dz*(-G1[-dk + ijk] + G1[dk + ijk])
;

dG32
=
oo2dz*(-G2[-dk + ijk] + G2[dk + ijk])
;

dG33
=
oo2dz*(-G3[-dk + ijk] + G3[dk + ijk])
;

db11
=
oo2dx*(-beta1[-di + ijk] + beta1[di + ijk])
;

db12
=
oo2dx*(-beta2[-di + ijk] + beta2[di + ijk])
;

db13
=
oo2dx*(-beta3[-di + ijk] + beta3[di + ijk])
;

db21
=
oo2dy*(-beta1[-dj + ijk] + beta1[dj + ijk])
;

db22
=
oo2dy*(-beta2[-dj + ijk] + beta2[dj + ijk])
;

db23
=
oo2dy*(-beta3[-dj + ijk] + beta3[dj + ijk])
;

db31
=
oo2dz*(-beta1[-dk + ijk] + beta1[dk + ijk])
;

db32
=
oo2dz*(-beta2[-dk + ijk] + beta2[dk + ijk])
;

db33
=
oo2dz*(-beta3[-dk + ijk] + beta3[dk + ijk])
;

ddb111
=
oodx2*(-2*beta1[ijk] + beta1[-di + ijk] + beta1[di + ijk])
;

ddb112
=
oodx2*(-2*beta2[ijk] + beta2[-di + ijk] + beta2[di + ijk])
;

ddb113
=
oodx2*(-2*beta3[ijk] + beta3[-di + ijk] + beta3[di + ijk])
;

ddb121
=
oo4dxdy*(beta1[-di - dj + ijk] - beta1[di - dj + ijk] - 
     beta1[-di + dj + ijk] + beta1[di + dj + ijk])
;

ddb122
=
oo4dxdy*(beta2[-di - dj + ijk] - beta2[di - dj + ijk] - 
     beta2[-di + dj + ijk] + beta2[di + dj + ijk])
;

ddb123
=
oo4dxdy*(beta3[-di - dj + ijk] - beta3[di - dj + ijk] - 
     beta3[-di + dj + ijk] + beta3[di + dj + ijk])
;

ddb131
=
oo4dxdz*(beta1[-di - dk + ijk] - beta1[di - dk + ijk] - 
     beta1[-di + dk + ijk] + beta1[di + dk + ijk])
;

ddb132
=
oo4dxdz*(beta2[-di - dk + ijk] - beta2[di - dk + ijk] - 
     beta2[-di + dk + ijk] + beta2[di + dk + ijk])
;

ddb133
=
oo4dxdz*(beta3[-di - dk + ijk] - beta3[di - dk + ijk] - 
     beta3[-di + dk + ijk] + beta3[di + dk + ijk])
;

ddb221
=
oody2*(-2*beta1[ijk] + beta1[-dj + ijk] + beta1[dj + ijk])
;

ddb222
=
oody2*(-2*beta2[ijk] + beta2[-dj + ijk] + beta2[dj + ijk])
;

ddb223
=
oody2*(-2*beta3[ijk] + beta3[-dj + ijk] + beta3[dj + ijk])
;

ddb231
=
oo4dydz*(beta1[-dj - dk + ijk] - beta1[dj - dk + ijk] - 
     beta1[-dj + dk + ijk] + beta1[dj + dk + ijk])
;

ddb232
=
oo4dydz*(beta2[-dj - dk + ijk] - beta2[dj - dk + ijk] - 
     beta2[-dj + dk + ijk] + beta2[dj + dk + ijk])
;

ddb233
=
oo4dydz*(beta3[-dj - dk + ijk] - beta3[dj - dk + ijk] - 
     beta3[-dj + dk + ijk] + beta3[dj + dk + ijk])
;

ddb331
=
oodz2*(-2*beta1[ijk] + beta1[-dk + ijk] + beta1[dk + ijk])
;

ddb332
=
oodz2*(-2*beta2[ijk] + beta2[-dk + ijk] + beta2[dk + ijk])
;

ddb333
=
oodz2*(-2*beta3[ijk] + beta3[-dk + ijk] + beta3[dk + ijk])
;

LieG1
=
dG11*beta1[ijk] + dG21*beta2[ijk] + dG31*beta3[ijk] - db21*g2[ijk] - 
   db31*g3[ijk] + ((-db11 + 2*(db22 + db33))*g1[ijk] + 
      (4*ddb111 + ddb122 + ddb133)*ginv11[ijk] + 
      (7*ddb121 + ddb222 + ddb233)*ginv12[ijk] + 
      (7*ddb131 + ddb232 + ddb333)*ginv13[ijk])/3. + ddb221*ginv22[ijk] + 
   2*ddb231*ginv23[ijk] + ddb331*ginv33[ijk]
;

LieG2
=
dG12*beta1[ijk] + dG22*beta2[ijk] + dG32*beta3[ijk] - db12*g1[ijk] - 
   db32*g3[ijk] + ddb112*ginv11[ijk] + 2*ddb132*ginv13[ijk] + 
   (-(db22*g2[ijk]) + 2*(db11 + db33)*g2[ijk] + 
      (ddb111 + 7*ddb122 + ddb133)*ginv12[ijk] + 
      (ddb121 + 4*ddb222 + ddb233)*ginv22[ijk] + 
      (ddb131 + 7*ddb232 + ddb333)*ginv23[ijk])/3. + ddb332*ginv33[ijk]
;

LieG3
=
dG13*beta1[ijk] + dG23*beta2[ijk] + dG33*beta3[ijk] - db13*g1[ijk] - 
   db23*g2[ijk] + ddb113*ginv11[ijk] + 2*ddb123*ginv12[ijk] + 
   ddb223*ginv22[ijk] + (2*(db11 + db22)*g3[ijk] - db33*g3[ijk] + 
      (ddb111 + ddb122 + 7*ddb133)*ginv13[ijk] + 
      (ddb121 + ddb222 + 7*ddb233)*ginv23[ijk] + 
      (ddb131 + ddb232 + 4*ddb333)*ginv33[ijk])/3.
;

*Lu1
=
LieG1 + n1[ijk]
;

*Lu2
=
LieG2 + n2[ijk]
;

*Lu3
=
LieG3 + n3[ijk]
;



/* conditional */
if (dLdu1) {

*dLdu1
=
dG11 - (8*oodx2*ginv11[ijk])/3. - 2*(oody2*ginv22[ijk] + oodz2*ginv33[ijk])
;

*dLdu2
=
dG22 - (8*oody2*ginv22[ijk])/3. - 2*(oodx2*ginv11[ijk] + oodz2*ginv33[ijk])
;

*dLdu3
=
dG33 - 2*(oodx2*ginv11[ijk] + oody2*ginv22[ijk]) - (8*oodz2*ginv33[ijk])/3.
;

}
/* if (dLdu1) */


}  /* function */

/* bambssnstencil.c */
/* n* = 170,  n/ = 28,  n+ = 332, nvars = 18 */
