/* bamconfharmonic.c */
/* Bernd Bruegmann, 3.3.2000 */
/* Produced with Mathematica */

#include "bamveclap.h"
extern int minstrain, mindistortion, bctgauge, confharmonic, nocd;

#define Power(x,y) (pow((double) (x), (double) (y)))
#define Log(x)     log((double) (x)))
#define pow2(x)    ((x)*(x))
#define pow2inv(x) (1.0/((x)*(x)))

#define Cal(x,y,z) ((x)?(y):(z))



void bamconfharmonic(cGH *GH, int ijk,
int di, int dj, int dk, double dx, double dy, double dz,
double *psi,
double *dpop1,
double *dpop2,
double *dpop3,
double *alpha,
double *beta1,
double *beta2,
double *beta3,
double *g11,
double *g12,
double *g13,
double *g22,
double *g23,
double *g33,
double *A11,
double *A12,
double *A13,
double *A22,
double *A23,
double *A33,
double *phi,
double *K,
double *G1,
double *G2,
double *G3,
double *Lu1,
double *Lu2,
double *Lu3,
double *dLdu1,
double *dLdu2,
double *dLdu3
) {

double oo2dx = 1/(2*dx), oo2dy = 1/(2*dy), oo2dz = 1/(2*dz);
double oodx2 = 1/(dx*dx), oody2 = 1/(dy*dy), oodz2 = 1/(dz*dz);
double oo4dxdy = 1/(4*dx*dy);
double oo4dxdz = 1/(4*dx*dz);
double oo4dydz = 1/(4*dy*dz);

double Cif;
double detg;
double f;
double Adia11;
double Adia12;
double Adia13;
double Adia21;
double Adia22;
double Adia23;
double Adia31;
double Adia32;
double Adia33;
double Ainv11;
double Ainv12;
double Ainv13;
double Ainv22;
double Ainv23;
double Ainv33;
double da1;
double da2;
double da3;
double dA111;
double dA112;
double dA113;
double dA122;
double dA123;
double dA133;
double dA211;
double dA212;
double dA213;
double dA222;
double dA223;
double dA233;
double dA311;
double dA312;
double dA313;
double dA322;
double dA323;
double dA333;
double db11;
double db12;
double db13;
double db21;
double db22;
double db23;
double db31;
double db32;
double db33;
double ddb111;
double ddb112;
double ddb113;
double ddb121;
double ddb122;
double ddb123;
double ddb131;
double ddb132;
double ddb133;
double ddb221;
double ddb222;
double ddb223;
double ddb231;
double ddb232;
double ddb233;
double ddb331;
double ddb332;
double ddb333;
double delg111;
double delg112;
double delg113;
double delg122;
double delg123;
double delg133;
double delg211;
double delg212;
double delg213;
double delg222;
double delg223;
double delg233;
double delg311;
double delg312;
double delg313;
double delg322;
double delg323;
double delg333;
double df1;
double df2;
double df3;
double dG11;
double dG12;
double dG13;
double dG21;
double dG22;
double dG23;
double dG31;
double dG32;
double dG33;
double dginv111;
double dginv112;
double dginv113;
double dginv121;
double dginv122;
double dginv123;
double dginv131;
double dginv132;
double dginv133;
double dginv211;
double dginv212;
double dginv213;
double dginv221;
double dginv222;
double dginv223;
double dginv231;
double dginv232;
double dginv233;
double dginv311;
double dginv312;
double dginv313;
double dginv321;
double dginv322;
double dginv323;
double dginv331;
double dginv332;
double dginv333;
double divAinv1;
double divAinv2;
double divAinv3;
double dK1;
double dK2;
double dK3;
double gamma111;
double gamma112;
double gamma113;
double gamma122;
double gamma123;
double gamma133;
double gamma211;
double gamma212;
double gamma213;
double gamma222;
double gamma223;
double gamma233;
double gamma311;
double gamma312;
double gamma313;
double gamma322;
double gamma323;
double gamma333;
double gammado111;
double gammado112;
double gammado113;
double gammado122;
double gammado123;
double gammado133;
double gammado211;
double gammado212;
double gammado213;
double gammado222;
double gammado223;
double gammado233;
double gammado311;
double gammado312;
double gammado313;
double gammado322;
double gammado323;
double gammado333;
double gginv11;
double gginv12;
double gginv13;
double gginv22;
double gginv23;
double gginv33;
double ginv11;
double ginv12;
double ginv13;
double ginv22;
double ginv23;
double ginv33;
double LieG1;
double LieG2;
double LieG3;
double n1;
double n2;
double n3;



df1
=
oo2dx*(-phi[-di + ijk] + phi[di + ijk])
;

df2
=
oo2dy*(-phi[-dj + ijk] + phi[dj + ijk])
;

df3
=
oo2dz*(-phi[-dk + ijk] + phi[dk + ijk])
;

delg111
=
oo2dx*(-g11[-di + ijk] + g11[di + ijk])
;

delg112
=
oo2dx*(-g12[-di + ijk] + g12[di + ijk])
;

delg113
=
oo2dx*(-g13[-di + ijk] + g13[di + ijk])
;

delg122
=
oo2dx*(-g22[-di + ijk] + g22[di + ijk])
;

delg123
=
oo2dx*(-g23[-di + ijk] + g23[di + ijk])
;

delg133
=
oo2dx*(-g33[-di + ijk] + g33[di + ijk])
;

delg211
=
oo2dy*(-g11[-dj + ijk] + g11[dj + ijk])
;

delg212
=
oo2dy*(-g12[-dj + ijk] + g12[dj + ijk])
;

delg213
=
oo2dy*(-g13[-dj + ijk] + g13[dj + ijk])
;

delg222
=
oo2dy*(-g22[-dj + ijk] + g22[dj + ijk])
;

delg223
=
oo2dy*(-g23[-dj + ijk] + g23[dj + ijk])
;

delg233
=
oo2dy*(-g33[-dj + ijk] + g33[dj + ijk])
;

delg311
=
oo2dz*(-g11[-dk + ijk] + g11[dk + ijk])
;

delg312
=
oo2dz*(-g12[-dk + ijk] + g12[dk + ijk])
;

delg313
=
oo2dz*(-g13[-dk + ijk] + g13[dk + ijk])
;

delg322
=
oo2dz*(-g22[-dk + ijk] + g22[dk + ijk])
;

delg323
=
oo2dz*(-g23[-dk + ijk] + g23[dk + ijk])
;

delg333
=
oo2dz*(-g33[-dk + ijk] + g33[dk + ijk])
;

dG11
=
oo2dx*(-G1[-di + ijk] + G1[di + ijk])
;

dG12
=
oo2dx*(-G2[-di + ijk] + G2[di + ijk])
;

dG13
=
oo2dx*(-G3[-di + ijk] + G3[di + ijk])
;

dG21
=
oo2dy*(-G1[-dj + ijk] + G1[dj + ijk])
;

dG22
=
oo2dy*(-G2[-dj + ijk] + G2[dj + ijk])
;

dG23
=
oo2dy*(-G3[-dj + ijk] + G3[dj + ijk])
;

dG31
=
oo2dz*(-G1[-dk + ijk] + G1[dk + ijk])
;

dG32
=
oo2dz*(-G2[-dk + ijk] + G2[dk + ijk])
;

dG33
=
oo2dz*(-G3[-dk + ijk] + G3[dk + ijk])
;

dK1
=
oo2dx*(-K[-di + ijk] + K[di + ijk])
;

dK2
=
oo2dy*(-K[-dj + ijk] + K[dj + ijk])
;

dK3
=
oo2dz*(-K[-dk + ijk] + K[dk + ijk])
;

dA111
=
oo2dx*(-A11[-di + ijk] + A11[di + ijk])
;

dA112
=
oo2dx*(-A12[-di + ijk] + A12[di + ijk])
;

dA113
=
oo2dx*(-A13[-di + ijk] + A13[di + ijk])
;

dA122
=
oo2dx*(-A22[-di + ijk] + A22[di + ijk])
;

dA123
=
oo2dx*(-A23[-di + ijk] + A23[di + ijk])
;

dA133
=
oo2dx*(-A33[-di + ijk] + A33[di + ijk])
;

dA211
=
oo2dy*(-A11[-dj + ijk] + A11[dj + ijk])
;

dA212
=
oo2dy*(-A12[-dj + ijk] + A12[dj + ijk])
;

dA213
=
oo2dy*(-A13[-dj + ijk] + A13[dj + ijk])
;

dA222
=
oo2dy*(-A22[-dj + ijk] + A22[dj + ijk])
;

dA223
=
oo2dy*(-A23[-dj + ijk] + A23[dj + ijk])
;

dA233
=
oo2dy*(-A33[-dj + ijk] + A33[dj + ijk])
;

dA311
=
oo2dz*(-A11[-dk + ijk] + A11[dk + ijk])
;

dA312
=
oo2dz*(-A12[-dk + ijk] + A12[dk + ijk])
;

dA313
=
oo2dz*(-A13[-dk + ijk] + A13[dk + ijk])
;

dA322
=
oo2dz*(-A22[-dk + ijk] + A22[dk + ijk])
;

dA323
=
oo2dz*(-A23[-dk + ijk] + A23[dk + ijk])
;

dA333
=
oo2dz*(-A33[-dk + ijk] + A33[dk + ijk])
;

da1
=
oo2dx*(-alpha[-di + ijk] + alpha[di + ijk])
;

da2
=
oo2dy*(-alpha[-dj + ijk] + alpha[dj + ijk])
;

da3
=
oo2dz*(-alpha[-dk + ijk] + alpha[dk + ijk])
;

db11
=
oo2dx*(-beta1[-di + ijk] + beta1[di + ijk])
;

db12
=
oo2dx*(-beta2[-di + ijk] + beta2[di + ijk])
;

db13
=
oo2dx*(-beta3[-di + ijk] + beta3[di + ijk])
;

db21
=
oo2dy*(-beta1[-dj + ijk] + beta1[dj + ijk])
;

db22
=
oo2dy*(-beta2[-dj + ijk] + beta2[dj + ijk])
;

db23
=
oo2dy*(-beta3[-dj + ijk] + beta3[dj + ijk])
;

db31
=
oo2dz*(-beta1[-dk + ijk] + beta1[dk + ijk])
;

db32
=
oo2dz*(-beta2[-dk + ijk] + beta2[dk + ijk])
;

db33
=
oo2dz*(-beta3[-dk + ijk] + beta3[dk + ijk])
;

ddb111
=
oodx2*(-2*beta1[ijk] + beta1[-di + ijk] + beta1[di + ijk])
;

ddb112
=
oodx2*(-2*beta2[ijk] + beta2[-di + ijk] + beta2[di + ijk])
;

ddb113
=
oodx2*(-2*beta3[ijk] + beta3[-di + ijk] + beta3[di + ijk])
;

ddb121
=
oo4dxdy*(beta1[-di - dj + ijk] - beta1[di - dj + ijk] - 
     beta1[-di + dj + ijk] + beta1[di + dj + ijk])
;

ddb122
=
oo4dxdy*(beta2[-di - dj + ijk] - beta2[di - dj + ijk] - 
     beta2[-di + dj + ijk] + beta2[di + dj + ijk])
;

ddb123
=
oo4dxdy*(beta3[-di - dj + ijk] - beta3[di - dj + ijk] - 
     beta3[-di + dj + ijk] + beta3[di + dj + ijk])
;

ddb131
=
oo4dxdz*(beta1[-di - dk + ijk] - beta1[di - dk + ijk] - 
     beta1[-di + dk + ijk] + beta1[di + dk + ijk])
;

ddb132
=
oo4dxdz*(beta2[-di - dk + ijk] - beta2[di - dk + ijk] - 
     beta2[-di + dk + ijk] + beta2[di + dk + ijk])
;

ddb133
=
oo4dxdz*(beta3[-di - dk + ijk] - beta3[di - dk + ijk] - 
     beta3[-di + dk + ijk] + beta3[di + dk + ijk])
;

ddb221
=
oody2*(-2*beta1[ijk] + beta1[-dj + ijk] + beta1[dj + ijk])
;

ddb222
=
oody2*(-2*beta2[ijk] + beta2[-dj + ijk] + beta2[dj + ijk])
;

ddb223
=
oody2*(-2*beta3[ijk] + beta3[-dj + ijk] + beta3[dj + ijk])
;

ddb231
=
oo4dydz*(beta1[-dj - dk + ijk] - beta1[dj - dk + ijk] - 
     beta1[-dj + dk + ijk] + beta1[dj + dk + ijk])
;

ddb232
=
oo4dydz*(beta2[-dj - dk + ijk] - beta2[dj - dk + ijk] - 
     beta2[-dj + dk + ijk] + beta2[dj + dk + ijk])
;

ddb233
=
oo4dydz*(beta3[-dj - dk + ijk] - beta3[dj - dk + ijk] - 
     beta3[-dj + dk + ijk] + beta3[dj + dk + ijk])
;

ddb331
=
oodz2*(-2*beta1[ijk] + beta1[-dk + ijk] + beta1[dk + ijk])
;

ddb332
=
oodz2*(-2*beta2[ijk] + beta2[-dk + ijk] + beta2[dk + ijk])
;

ddb333
=
oodz2*(-2*beta3[ijk] + beta3[-dk + ijk] + beta3[dk + ijk])
;

f
=
phi[ijk]
;



/* conditional */
if (!nocd) {

f
=
f + log(psi[ijk])
;

df1
=
df1 + dpop1[ijk]
;

df2
=
df2 + dpop2[ijk]
;

df3
=
df3 + dpop3[ijk]
;

}
/* if (!nocd) */


gginv11
=
g22[ijk]*g33[ijk] - pow2(g23[ijk])
;

gginv12
=
g13[ijk]*g23[ijk] - g12[ijk]*g33[ijk]
;

gginv13
=
-(g13[ijk]*g22[ijk]) + g12[ijk]*g23[ijk]
;

gginv22
=
g11[ijk]*g33[ijk] - pow2(g13[ijk])
;

gginv23
=
g12[ijk]*g13[ijk] - g11[ijk]*g23[ijk]
;

gginv33
=
g11[ijk]*g22[ijk] - pow2(g12[ijk])
;

detg
=
gginv11*g11[ijk] + gginv12*g12[ijk] + gginv13*g13[ijk]
;

ginv11
=
gginv11/detg
;

ginv12
=
gginv12/detg
;

ginv13
=
gginv13/detg
;

ginv22
=
gginv22/detg
;

ginv23
=
gginv23/detg
;

ginv33
=
gginv33/detg
;

gammado111
=
delg111/2.
;

gammado112
=
delg211/2.
;

gammado113
=
delg311/2.
;

gammado122
=
-delg122/2. + delg212
;

gammado123
=
(-delg123 + delg213 + delg312)/2.
;

gammado133
=
-delg133/2. + delg313
;

gammado211
=
delg112 - delg211/2.
;

gammado212
=
delg122/2.
;

gammado213
=
(delg123 - delg213 + delg312)/2.
;

gammado222
=
delg222/2.
;

gammado223
=
delg322/2.
;

gammado233
=
-delg233/2. + delg323
;

gammado311
=
delg113 - delg311/2.
;

gammado312
=
(delg123 + delg213 - delg312)/2.
;

gammado313
=
delg133/2.
;

gammado322
=
delg223 - delg322/2.
;

gammado323
=
delg233/2.
;

gammado333
=
delg333/2.
;

gamma111
=
gammado111*ginv11 + gammado211*ginv12 + gammado311*ginv13
;

gamma112
=
gammado112*ginv11 + gammado212*ginv12 + gammado312*ginv13
;

gamma113
=
gammado113*ginv11 + gammado213*ginv12 + gammado313*ginv13
;

gamma122
=
gammado122*ginv11 + gammado222*ginv12 + gammado322*ginv13
;

gamma123
=
gammado123*ginv11 + gammado223*ginv12 + gammado323*ginv13
;

gamma133
=
gammado133*ginv11 + gammado233*ginv12 + gammado333*ginv13
;

gamma211
=
gammado111*ginv12 + gammado211*ginv22 + gammado311*ginv23
;

gamma212
=
gammado112*ginv12 + gammado212*ginv22 + gammado312*ginv23
;

gamma213
=
gammado113*ginv12 + gammado213*ginv22 + gammado313*ginv23
;

gamma222
=
gammado122*ginv12 + gammado222*ginv22 + gammado322*ginv23
;

gamma223
=
gammado123*ginv12 + gammado223*ginv22 + gammado323*ginv23
;

gamma233
=
gammado133*ginv12 + gammado233*ginv22 + gammado333*ginv23
;

gamma311
=
gammado111*ginv13 + gammado211*ginv23 + gammado311*ginv33
;

gamma312
=
gammado112*ginv13 + gammado212*ginv23 + gammado312*ginv33
;

gamma313
=
gammado113*ginv13 + gammado213*ginv23 + gammado313*ginv33
;

gamma322
=
gammado122*ginv13 + gammado222*ginv23 + gammado322*ginv33
;

gamma323
=
gammado123*ginv13 + gammado223*ginv23 + gammado323*ginv33
;

gamma333
=
gammado133*ginv13 + gammado233*ginv23 + gammado333*ginv33
;

dginv111
=
-2*(delg123*ginv12*ginv13 + ginv11*(delg112*ginv12 + delg113*ginv13)) - 
   delg111*pow2(ginv11) - delg122*pow2(ginv12) - delg133*pow2(ginv13)
;

dginv112
=
-(ginv11*(delg111*ginv12 + delg112*ginv22 + delg113*ginv23)) - 
   ginv12*(delg113*ginv13 + delg122*ginv22 + delg123*ginv23) - 
   ginv13*(delg123*ginv22 + delg133*ginv23) - delg112*pow2(ginv12)
;

dginv113
=
-(ginv11*(delg111*ginv13 + delg112*ginv23 + delg113*ginv33)) - 
   ginv12*(delg112*ginv13 + delg122*ginv23 + delg123*ginv33) - 
   ginv13*(delg123*ginv23 + delg133*ginv33) - delg113*pow2(ginv13)
;

dginv121
=
-(ginv11*(delg111*ginv12 + delg112*ginv22 + delg113*ginv23)) - 
   ginv12*(delg113*ginv13 + delg122*ginv22 + delg123*ginv23) - 
   ginv13*(delg123*ginv22 + delg133*ginv23) - delg112*pow2(ginv12)
;

dginv122
=
-2*(delg123*ginv22*ginv23 + ginv12*(delg112*ginv22 + delg113*ginv23)) - 
   delg111*pow2(ginv12) - delg122*pow2(ginv22) - delg133*pow2(ginv23)
;

dginv123
=
-(ginv13*(delg112*ginv22 + delg113*ginv23)) - delg133*ginv23*ginv33 - 
   ginv12*(delg111*ginv13 + delg112*ginv23 + delg113*ginv33) - 
   ginv22*(delg122*ginv23 + delg123*ginv33) - delg123*pow2(ginv23)
;

dginv131
=
-(ginv11*(delg111*ginv13 + delg112*ginv23 + delg113*ginv33)) - 
   ginv12*(delg112*ginv13 + delg122*ginv23 + delg123*ginv33) - 
   ginv13*(delg123*ginv23 + delg133*ginv33) - delg113*pow2(ginv13)
;

dginv132
=
-(ginv13*(delg112*ginv22 + delg113*ginv23)) - delg133*ginv23*ginv33 - 
   ginv12*(delg111*ginv13 + delg112*ginv23 + delg113*ginv33) - 
   ginv22*(delg122*ginv23 + delg123*ginv33) - delg123*pow2(ginv23)
;

dginv133
=
-2*(delg123*ginv23*ginv33 + ginv13*(delg112*ginv23 + delg113*ginv33)) - 
   delg111*pow2(ginv13) - delg122*pow2(ginv23) - delg133*pow2(ginv33)
;

dginv211
=
-2*(delg223*ginv12*ginv13 + ginv11*(delg212*ginv12 + delg213*ginv13)) - 
   delg211*pow2(ginv11) - delg222*pow2(ginv12) - delg233*pow2(ginv13)
;

dginv212
=
-(ginv11*(delg211*ginv12 + delg212*ginv22 + delg213*ginv23)) - 
   ginv12*(delg213*ginv13 + delg222*ginv22 + delg223*ginv23) - 
   ginv13*(delg223*ginv22 + delg233*ginv23) - delg212*pow2(ginv12)
;

dginv213
=
-(ginv11*(delg211*ginv13 + delg212*ginv23 + delg213*ginv33)) - 
   ginv12*(delg212*ginv13 + delg222*ginv23 + delg223*ginv33) - 
   ginv13*(delg223*ginv23 + delg233*ginv33) - delg213*pow2(ginv13)
;

dginv221
=
-(ginv11*(delg211*ginv12 + delg212*ginv22 + delg213*ginv23)) - 
   ginv12*(delg213*ginv13 + delg222*ginv22 + delg223*ginv23) - 
   ginv13*(delg223*ginv22 + delg233*ginv23) - delg212*pow2(ginv12)
;

dginv222
=
-2*(delg223*ginv22*ginv23 + ginv12*(delg212*ginv22 + delg213*ginv23)) - 
   delg211*pow2(ginv12) - delg222*pow2(ginv22) - delg233*pow2(ginv23)
;

dginv223
=
-(ginv13*(delg212*ginv22 + delg213*ginv23)) - delg233*ginv23*ginv33 - 
   ginv12*(delg211*ginv13 + delg212*ginv23 + delg213*ginv33) - 
   ginv22*(delg222*ginv23 + delg223*ginv33) - delg223*pow2(ginv23)
;

dginv231
=
-(ginv11*(delg211*ginv13 + delg212*ginv23 + delg213*ginv33)) - 
   ginv12*(delg212*ginv13 + delg222*ginv23 + delg223*ginv33) - 
   ginv13*(delg223*ginv23 + delg233*ginv33) - delg213*pow2(ginv13)
;

dginv232
=
-(ginv13*(delg212*ginv22 + delg213*ginv23)) - delg233*ginv23*ginv33 - 
   ginv12*(delg211*ginv13 + delg212*ginv23 + delg213*ginv33) - 
   ginv22*(delg222*ginv23 + delg223*ginv33) - delg223*pow2(ginv23)
;

dginv233
=
-2*(delg223*ginv23*ginv33 + ginv13*(delg212*ginv23 + delg213*ginv33)) - 
   delg211*pow2(ginv13) - delg222*pow2(ginv23) - delg233*pow2(ginv33)
;

dginv311
=
-2*(delg323*ginv12*ginv13 + ginv11*(delg312*ginv12 + delg313*ginv13)) - 
   delg311*pow2(ginv11) - delg322*pow2(ginv12) - delg333*pow2(ginv13)
;

dginv312
=
-(ginv11*(delg311*ginv12 + delg312*ginv22 + delg313*ginv23)) - 
   ginv12*(delg313*ginv13 + delg322*ginv22 + delg323*ginv23) - 
   ginv13*(delg323*ginv22 + delg333*ginv23) - delg312*pow2(ginv12)
;

dginv313
=
-(ginv11*(delg311*ginv13 + delg312*ginv23 + delg313*ginv33)) - 
   ginv12*(delg312*ginv13 + delg322*ginv23 + delg323*ginv33) - 
   ginv13*(delg323*ginv23 + delg333*ginv33) - delg313*pow2(ginv13)
;

dginv321
=
-(ginv11*(delg311*ginv12 + delg312*ginv22 + delg313*ginv23)) - 
   ginv12*(delg313*ginv13 + delg322*ginv22 + delg323*ginv23) - 
   ginv13*(delg323*ginv22 + delg333*ginv23) - delg312*pow2(ginv12)
;

dginv322
=
-2*(delg323*ginv22*ginv23 + ginv12*(delg312*ginv22 + delg313*ginv23)) - 
   delg311*pow2(ginv12) - delg322*pow2(ginv22) - delg333*pow2(ginv23)
;

dginv323
=
-(ginv13*(delg312*ginv22 + delg313*ginv23)) - delg333*ginv23*ginv33 - 
   ginv12*(delg311*ginv13 + delg312*ginv23 + delg313*ginv33) - 
   ginv22*(delg322*ginv23 + delg323*ginv33) - delg323*pow2(ginv23)
;

dginv331
=
-(ginv11*(delg311*ginv13 + delg312*ginv23 + delg313*ginv33)) - 
   ginv12*(delg312*ginv13 + delg322*ginv23 + delg323*ginv33) - 
   ginv13*(delg323*ginv23 + delg333*ginv33) - delg313*pow2(ginv13)
;

dginv332
=
-(ginv13*(delg312*ginv22 + delg313*ginv23)) - delg333*ginv23*ginv33 - 
   ginv12*(delg311*ginv13 + delg312*ginv23 + delg313*ginv33) - 
   ginv22*(delg322*ginv23 + delg323*ginv33) - delg323*pow2(ginv23)
;

dginv333
=
-2*(delg323*ginv23*ginv33 + ginv13*(delg312*ginv23 + delg313*ginv33)) - 
   delg311*pow2(ginv13) - delg322*pow2(ginv23) - delg333*pow2(ginv33)
;

Adia11
=
ginv11*A11[ijk] + ginv12*A12[ijk] + ginv13*A13[ijk]
;

Adia12
=
ginv11*A12[ijk] + ginv12*A22[ijk] + ginv13*A23[ijk]
;

Adia13
=
ginv11*A13[ijk] + ginv12*A23[ijk] + ginv13*A33[ijk]
;

Adia21
=
ginv12*A11[ijk] + ginv22*A12[ijk] + ginv23*A13[ijk]
;

Adia22
=
ginv12*A12[ijk] + ginv22*A22[ijk] + ginv23*A23[ijk]
;

Adia23
=
ginv12*A13[ijk] + ginv22*A23[ijk] + ginv23*A33[ijk]
;

Adia31
=
ginv13*A11[ijk] + ginv23*A12[ijk] + ginv33*A13[ijk]
;

Adia32
=
ginv13*A12[ijk] + ginv23*A22[ijk] + ginv33*A23[ijk]
;

Adia33
=
ginv13*A13[ijk] + ginv23*A23[ijk] + ginv33*A33[ijk]
;

Ainv11
=
Adia11*ginv11 + Adia12*ginv12 + Adia13*ginv13
;

Ainv12
=
Adia21*ginv11 + Adia22*ginv12 + Adia23*ginv13
;

Ainv12
=
Adia11*ginv12 + Adia12*ginv22 + Adia13*ginv23
;

Ainv13
=
Adia31*ginv11 + Adia32*ginv12 + Adia33*ginv13
;

Ainv13
=
Adia11*ginv13 + Adia12*ginv23 + Adia13*ginv33
;

Ainv22
=
Adia21*ginv12 + Adia22*ginv22 + Adia23*ginv23
;

Ainv23
=
Adia31*ginv12 + Adia32*ginv22 + Adia33*ginv23
;

Ainv23
=
Adia21*ginv13 + Adia22*ginv23 + Adia23*ginv33
;

Ainv33
=
Adia31*ginv13 + Adia32*ginv23 + Adia33*ginv33
;



/* conditional */
if (divAfromD) {

divAinv1
=
-6*(Ainv11*df1 + Ainv12*df2 + Ainv13*df3) - Ainv11*gamma111 - 
   Ainv22*gamma122 - 2*(Ainv12*gamma112 + Ainv13*gamma113 + 
      Ainv23*gamma123) - Ainv33*gamma133 + 
   (2*(dK1*ginv11 + dK2*ginv12 + dK3*ginv13))/3.
;

divAinv2
=
-6*(Ainv12*df1 + Ainv22*df2 + Ainv23*df3) - Ainv11*gamma211 - 
   Ainv22*gamma222 - 2*(Ainv12*gamma212 + Ainv13*gamma213 + 
      Ainv23*gamma223) - Ainv33*gamma233 + 
   (2*(dK1*ginv12 + dK2*ginv22 + dK3*ginv23))/3.
;

divAinv3
=
-6*(Ainv13*df1 + Ainv23*df2 + Ainv33*df3) - Ainv11*gamma311 - 
   Ainv22*gamma322 - 2*(Ainv12*gamma312 + Ainv13*gamma313 + 
      Ainv23*gamma323) - Ainv33*gamma333 + 
   (2*(dK1*ginv13 + dK2*ginv23 + dK3*ginv33))/3.
;


} else { /* if (!divAfromD) */

divAinv1
=
Adia21*dginv211 + Adia22*dginv221 + Adia23*dginv231 + Adia31*dginv311 + 
   Adia32*dginv321 + Adia33*dginv331 + 
   ginv11*((2*dA112 + dA211)*ginv12 + (2*dA113 + dA311)*ginv13 + 
      dA212*ginv22 + (dA213 + dA312)*ginv23 + dA313*ginv33) + 
   ginv12*((2*dA123 + dA213 + dA312)*ginv13 + dA222*ginv22 + 
      (dA223 + dA322)*ginv23 + dA323*ginv33) + 
   ginv13*(dA223*ginv22 + (dA233 + dA323)*ginv23 + dA333*ginv33) + 
   Adia11*(dginv111 - G1[ijk]) + Adia12*(dginv121 - G2[ijk]) + 
   Adia13*(dginv131 - G3[ijk]) + dA111*pow2(ginv11) + 
   (dA122 + dA212)*pow2(ginv12) + (dA133 + dA313)*pow2(ginv13)
;

divAinv2
=
Adia11*dginv112 + Adia12*dginv122 + Adia13*dginv132 + Adia31*dginv312 + 
   Adia32*dginv322 + Adia33*dginv332 + 
   ginv11*(dA111*ginv12 + dA112*ginv22 + dA113*ginv23) + 
   ginv13*((dA123 + dA312)*ginv22 + (dA133 + dA313)*ginv23) + 
   dA333*ginv23*ginv33 + ginv12*
    ((dA113 + dA311)*ginv13 + (dA122 + 2*dA212)*ginv22 + 
      (dA123 + 2*dA213 + dA312)*ginv23 + dA313*ginv33) + 
   ginv22*((2*dA223 + dA322)*ginv23 + dA323*ginv33) + 
   Adia21*(dginv212 - G1[ijk]) + Adia22*(dginv222 - G2[ijk]) + 
   Adia23*(dginv232 - G3[ijk]) + (dA112 + dA211)*pow2(ginv12) + 
   dA222*pow2(ginv22) + (dA233 + dA323)*pow2(ginv23)
;

divAinv3
=
Adia11*dginv113 + Adia12*dginv123 + Adia13*dginv133 + Adia21*dginv213 + 
   Adia22*dginv223 + Adia23*dginv233 + (dA233 + 2*dA323)*ginv23*ginv33 + 
   ginv11*(dA111*ginv13 + dA112*ginv23 + dA113*ginv33) + 
   ginv12*((dA112 + dA211)*ginv13 + (dA122 + dA212)*ginv23 + 
      (dA123 + dA213)*ginv33) + ginv22*(dA222*ginv23 + dA223*ginv33) + 
   ginv13*(dA212*ginv22 + (dA123 + dA213 + 2*dA312)*ginv23 + 
      (dA133 + 2*dA313)*ginv33) + Adia31*(dginv313 - G1[ijk]) + 
   Adia32*(dginv323 - G2[ijk]) + Adia33*(dginv333 - G3[ijk]) + 
   (dA113 + dA311)*pow2(ginv13) + (dA223 + dA322)*pow2(ginv23) + 
   dA333*pow2(ginv33)
;

}
/* if (divAfromD) */


n1
=
-2*(Ainv11*da1 + Ainv12*da2 + Ainv13*da3 + divAinv1*alpha[ijk])
;

n2
=
-2*(Ainv12*da1 + Ainv22*da2 + Ainv23*da3 + divAinv2*alpha[ijk])
;

n3
=
-2*(Ainv13*da1 + Ainv23*da2 + Ainv33*da3 + divAinv3*alpha[ijk])
;

LieG1
=
((4*ddb111)/3. + (ddb122 + ddb133)/3.)*ginv11 + 
   ((7*ddb121)/3. + (ddb222 + ddb233)/3.)*ginv12 + 
   ((7*ddb131)/3. + (ddb232 + ddb333)/3.)*ginv13 + ddb221*ginv22 + 
   2*ddb231*ginv23 + ddb331*ginv33 + dG11*beta1[ijk] + dG21*beta2[ijk] + 
   dG31*beta3[ijk] + (-db11/3. + (2*(db22 + db33))/3.)*G1[ijk] - 
   db21*G2[ijk] - db31*G3[ijk]
;

LieG2
=
ddb112*ginv11 + ((7*ddb122)/3. + (ddb111 + ddb133)/3.)*ginv12 + 
   2*ddb132*ginv13 + ((4*ddb222)/3. + (ddb121 + ddb233)/3.)*ginv22 + 
   ((7*ddb232)/3. + (ddb131 + ddb333)/3.)*ginv23 + ddb332*ginv33 + 
   dG12*beta1[ijk] + dG22*beta2[ijk] + dG32*beta3[ijk] - db12*G1[ijk] - 
   (db22*G2[ijk])/3. + (2*(db11 + db33)*G2[ijk])/3. - db32*G3[ijk]
;

LieG3
=
ddb113*ginv11 + 2*ddb123*ginv12 + 
   ((ddb111 + ddb122)/3. + (7*ddb133)/3.)*ginv13 + ddb223*ginv22 + 
   ((ddb121 + ddb222)/3. + (7*ddb233)/3.)*ginv23 + 
   ((ddb131 + ddb232)/3. + (4*ddb333)/3.)*ginv33 + dG13*beta1[ijk] + 
   dG23*beta2[ijk] + dG33*beta3[ijk] - db13*G1[ijk] - db23*G2[ijk] + 
   (2*(db11 + db22)*G3[ijk])/3. - (db33*G3[ijk])/3.
;

*Lu1
=
LieG1 + n1
;

*Lu2
=
LieG2 + n2
;

*Lu3
=
LieG3 + n3
;



/* conditional */
if (dLdu1) {

*dLdu1
=
dG11 - (8*ginv11*oodx2)/3. - 2*(ginv22*oody2 + ginv33*oodz2)
;

*dLdu2
=
dG22 - (8*ginv22*oody2)/3. - 2*(ginv11*oodx2 + ginv33*oodz2)
;

*dLdu3
=
dG33 - 2*(ginv11*oodx2 + ginv22*oody2) - (8*ginv33*oodz2)/3.
;

}
/* if (dLdu1) */


}  /* function */

/* bamconfharmonic.c */
/* n* = 809,  n/ = 86,  n+ = 1026, nvars = 25 */
