(* bamconfharmonic.m, Mathematica script *)

(* given tensor equations, output difference equations in C *)

routine = "bamconfharmonic";
file    = routine <> ".c";

Print["Producing " <> file];
Print[""];



(* variables *)
variables  = {psi, dpop[a], alpha, beta[a], g[a,b], A[a,b], phi, K, G[a]}
returnvariables = {Lu[a], dLdu[a]}


(* compute in this order *)
tocompute = {

  (* partial derivatives *)
  df[a] == del[a,phi],
  delg[c,a,b] == del[c,g[a,b]],
  dG[a,b] == del[a, G[b]],
  dK[a] == del[a, K],
  dA[a,b,c] == del[a, A[b,c]],
  da[a] == del[a,alpha],
  db[a,b] == del[a,beta[b]],
  ddb[a,b,c] == deldel[a,b,beta[c]],

  (* add time independent conformal factor *)
  f == phi,
  Cif == Not[nocd],
    f == f + log[psi],
    df[a] == df[a] + dpop[a],
  Cif == end,

  (* inverse conformal metric *)
  gginv[a,b] == ginvdet[a,b],
  detg == g[1,c] gginv[1,c],
  ginv[a,b] == gginv[a,b] / detg,

  (* connection of conformal metric *) 
  gammado[c,a,b] == 1/2 (delg[a,b,c] + delg[b,a,c] - delg[c,a,b]),
  gamma[c,a,b] == ginv[c,d] gammado[d,a,b], 

  (* derivatives of inverse conformal metric *)
  dginv[a,b,c] == - ginv[b,d] ginv[c,e] delg[a,d,e],
  (* dginv[c,a,b] == - gamma[a,c,e] ginv[e,b] - gamma[b,c,e] ginv[a,e], *)

  (* source term, independent of beta *)
  Adia[a,b] == ginv[a,c] A[c,b],
  Ainv[a,b] == ginv[b,c] Adia[a,c],
  Cif == divAfromD,
    divAinv[a] == 2/3 ginv[a,b] dK[b] - 6 Ainv[a,b] df[b] -
                  gamma[a,b,c] Ainv[b,c],
  Cif == else,
    divAinv[a] == ginv[a,b] ginv[c,d] dA[c,d,b] +
                  Adia[b,c] dginv[b,c,a] - Adia[a,b] G[b],
  Cif == end,
  n[a] == - 2 (Ainv[a,b] da[b] + alpha divAinv[a]),

  (* Lie derivative term *)
  LieG[a] == ginv[b,c] ddb[b,c,a] + 1/3 ginv[a,b] ddb[b,c,c] -
            G[b] db[b,a] + beta[b] dG[b,a] + 2/3 G[a] db[b,b],

  (* Gdot *)
  Lu[a] == n[a] + LieG[a]

  (* diagonal derivative: appended below *)
}



(* symmetries: should use rule that index a and b are always sym, ... *)
g[a_,b_] := g[b,a] /; !OrderedQ[{a,b}]
ginv[a_,b_] := ginv[b,a] /; !OrderedQ[{a,b}]
gginv[a_,b_] := gginv[b,a] /; !OrderedQ[{a,b}]
A[a_,b_] := A[b,a] /; !OrderedQ[{a,b}]
Ainv[a_,b_] := Ainv[b,a] /; !OrderedQ[{a,b}]

deldel[a_,b_,c_] := deldel[b,a,c] /; !OrderedQ[{a,b}]
ddb[a_,b_,c_] := ddb[b,a,c] /; !OrderedQ[{a,b}]

delg[c_,a_,b_] := delg[c,b,a] /; !OrderedQ[{a,b}]
dA[c_,a_,b_] := dA[c,b,a] /; !OrderedQ[{a,b}]




(* hidden symmetries for contraction! *)
gamma[c_,a_,b_] := gamma[c,b,a] /; !OrderedQ[{a,b}]
gammado[c_,a_,b_] := gammado[c,b,a] /; !OrderedQ[{a,b}]
gamma/: gamma[a_, b_, c_] gammado[a_, d_, e_] :=
  gammado[a, b, c] gamma[a, d, e] /; !OrderedQ[{{b,c},{d,e}}]





(* compile-time substitutions *)

collectvar = {
    codel[a_, beta[b_]] -> codelbeta[a,b],
    codel[a_, K[b_,c_]] -> codelK[a,b,c],
    ginv[c_,d_] K[a_,c_] K[b_,d_] -> KK[a,b]
}

expcodel = {
  codel[a_, codel[b_,alpha]] -> 
    deldel[a, b, alpha] - gamma[c,a,b] del[c, alpha],

  codel[a_, beta[b_]] -> 
    del[a, beta[b]] + gamma[b,a,c] beta[c],

  codel[a_, K[b_,c_]] -> 
    del[a, K[b,c]] - gamma[d,a,b] K[d,c] - gamma[d,a,c] K[b,d]
}

expcontraction = {
  s_[a___,b_Symbol,c___,d_Symbol,e___] \
    t_[f___,b_Symbol,g___] u_[h___,d_Symbol,i___] ->
    Sum[s[a,b,c,d,e] t[f,b,g] u[h,d,i], {b,1,3}, {d,1,3}],

  s_[a___,b_Symbol,c___] t_[d___,b_Symbol,e___] ->
    Sum[s[a,b,c] t[d,b,e], {b,1,3}],

  s_[a___,b_Symbol,c___,b_Symbol,e___] ->
    Sum[s[a,b,c,b,e], {b,1,3}]
}

facterms = {
  KK[a_,b_] == x_ :> 
   KK[a,b] == Collect[x, {K[1,a], K[2,a], K[3,a]}],
  (* works: K[a_,b_] x_ + K[a_,b_] y_ -> K[a,b] (x + y), *)

  R[a_,b_] == x_ :> 
    R[a,b] == Collect[x, Union[Flatten[Array[ginv,{3,3}]]]], 

  delt[K[a_,b_]] == x_ :>
    delt[K[a,b]] == Collect[x, {alpha, beta[1], beta[2], beta[3]}]
}  
  

(* careful: Union sorts, so apply to elements for which this is ok or wanted *)
expfreesort[x_,a_Symbol] := Union[Flatten[Table[x, {a,1,3}]]]
expfreesort[x_,{a_Symbol}] := expfreesort[x,a]
expfreesort[x_,{a_Symbol,b__}] := 
  Union[Flatten[Table[expfreesort[x,{b}], {a,1,3}]]]

expexpsign = -x_ :> x
expfreeexp[x_Symbol] := x
expfreeexp[x_[a__]] := 
  DeleteCases[Union[Flatten[expfreesort[x[a], {a}]/.expexpsign]], 0]

(* careful: when called on equations, LHS and RHS must have same syms *)
expequsign = -x_ == y_ :> x == -y
expfreeequ[x_Symbol==y_] := x==y
expfreeequ[delt[x_Symbol]==y_] := delt[x] == y 
expfreeequ[x_[a__]==y_] := 
  DeleteCases[Union[Flatten[expfreesort[x[a]==y,{a}]/.expequsign]], True]
expfreeequ[delt[x_[a__]]==y_] := 
  DeleteCases[Union[Flatten[expfreesort[delt[x[a]]==y,{a}]/.expequsign]], True]

(* expand free indices:
   for equations, expansion is based on lhs
   for expressions, only top level list is handled
*)
expfreeindices = {
  x_ :> Flatten[Map[expfreeequ,x]] /; !FreeQ[x,y_==z_],
  x_ :> Flatten[Map[expfreeexp,x]] /; FreeQ[x,y_==z_]
}



(* utilities *)

gg = Array[g, {3,3}];
gginvdet = Inverse[gg] Det[gg];
ginvdet[a_Integer,b_Integer] := gginvdet[[a,b]]

delta[a_,b_] := 0 /; a != b
delta[a_,b_] := 1 /; a == b

prlist[a_,b_] := Block[{},
  Print[a, "\n"];
  Map[(Print[#,"\n"])&, b];
  Print["\n"];
];

glue[a_,b_] := (* SequenceForm[a,b]; *) 
 ToExpression[ToString[a] <> ToString[b]]; 
glueall[x_] := x //. y_Symbol[a_,b___] :> glue[y,a][b] /. y_[] :> y
gluevariables[variables_] := Module[{varexpanded, vartoglue, varglued},
  varexpanded = Flatten[Map[expfreeexp, variables]];
  vartoglue = Cases[varexpanded, x_[a__]];
  varglued = Map[glueall, vartoglue]; 
  Table[vartoglue[[i]] -> varglued[[i]], {i, 1, Length[varglued]}]
]

colpow = {
  Power[x_,2] -> pow2[x], 
  Power[x_,-2] -> pow2inv[x]
}



(***************************************************************************)
(* transform equations into form that is to be differenced *)

If[!conformaldiff, Module[{},
  runsub = DeleteCases[runsub, delg[a__] == b_];
  runsub = runsub /. psi -> 1 /. delpsi[a_] -> 0 /. deldelpsi[a_,b_] -> 0;
  runsub = runsub /. fdelg[a__] -> delg[a];
  runsub = runsub /. fdel[a_, b_, c_] -> del[a, b]; 
  runsub = runsub /. fdeldel[a_, b_, c_, d_] -> deldel[a, b, c]; 
]]

If[!hamiltonian, Module[{},
  variables = DeleteCases[variables, hamil];
  dyneq = DeleteCases[dyneq, hamil == b_];
]]

If[!diffeomorphism, Module[{},
  variables = DeleteCases[variables, diffeo[a_]];
  dyneq = DeleteCases[dyneq, diffeo[a_] == b_];
]]




(* substitutions *)
all = tocompute /. expcodel
prlist["all = tocompute /. expcodel", all]


(* expand contractions *)
all = all /. expcontraction
all = ExpandAll[ExpandAll[all] //. expcontraction]
all = ExpandAll[ExpandAll[all] //. expcontraction]
If[False, prlist["all = ExpandAll[all] /. expcontraction ...", all]]

(* expand free indices *)
all = all /. expfreeindices
If[False, prlist["all = all /. expfreeindices", all]]


(* various sets of variables *)
constvariables = {r, lr[a]}
auxvariables = Complement[Map[#[[1]]&, tocompute], 
                          Join[variables, returnvariables]];

gluevar = gluevariables[Join[variables, returnvariables, 
                             auxvariables, constvariables]];

vars = variables /. expfreeindices /. gluevar;
auxvars = auxvariables /. expfreeindices /. gluevar;
constvars = constvariables /. expfreeindices /. gluevar;
retvars = returnvariables /. expfreeindices /. gluevar; 

Print["vars"];
Print[vars];
Print["auxvars"];
Print[auxvars];
Print["constvars"];
Print[constvars];

all = all /. gluevar
If[False, prlist["all /. gluevar", all]]



(***************************************************************************)
(* produce differenced equations *)

scheme = {lax, dlf, laxdlf, ftcs, bra}[[2]];

cen[m_, d_, x_[i_,j_,k_]] := Switch[m,
  1, (x[i+d,j,k] - x[i-d,j,k]) / (2 d dx),
  2, (x[i,j+d,k] - x[i,j-d,k]) / (2 d dy),
  3, (x[i,j,k+d] - x[i,j,k-d]) / (2 d dz)
]

rcen[m_,d_] := x_[i_,j_,k_] :> cen[m,d,x[i,j,k]] /; 
  Not[x === Times || x === Plus]


(* "standard" second order, unstaggered leapfrog *)
leapfrog = {

  Switch[scheme,

    (* centered time *)
    dlf,
    delt[x_Symbol[i,j,k]] == y_ :> 
      x[n+1][i,j,k] == x[n-1][i,j,k] + 2 dt y,
 
    (* forward time *)
    ftcs,
    delt[x_Symbol[i,j,k]] == y_ :> 
      x[n+1][i,j,k] == x[i,j,k] + dt y,

    (* Lax *)
    lax,

    If [!conformaldiff,

    delt[x_Symbol[i,j,k]] == y_ :>
      x[n+1][i,j,k] == (x[i+1,j,k] + x[i-1,j,k] + x[i,j+1,k] + x[i,j-1,k] 
        + x[i,j,k+1] + x[i,j,k-1])/6 + dt y,

    delt[x_Symbol[i,j,k]] == y_ :>
      x[n+1][i,j,k] == psi[i,j,k] (
        x[i+1,j,k]/psi[i+1,j,k] + x[i-1,j,k]/psi[i-1,j,k] + 
        x[i,j+1,k]/psi[i,j+1,k] + x[i,j-1,k]/psi[i,j-1,k] +  
        x[i,j,k+1]/psi[i,j,k+1] + x[i,j,k-1]/psi[i,j,k-1])/6 + dt y
    ],

    (* Lax-CT combo *)
    laxdlf,
    delt[x_Symbol[i,j,k]] == y_ :>
      x[n+1][i,j,k] == (x[n-1][i+1,j,k] + x[n-1][i-1,j,k]
        + x[n-1][i,j+1,k] + x[n-1][i,j-1,k] 
        + x[n-1][i,j,k+1] + x[n-1][i,j,k-1])/6 + 2 dt y,

    (* brailovskaya *)
    bra,
    delt[x_Symbol[i,j,k]] == y_ :> 
	x[n+1][i,j,k] == Cal[step, 
	  x[i,j,k] + dt y,                             (* predictor *)     
	  0.5 (x[n-1][i,j,k] + x[i,j,k] + dt y)        (* corrector *)
        ] 
  ],

  delt[delt[x_Symbol[i,j,k]]] == y_ :> 
    x[n+1][i,j,k] == 2 x[i,j,k] - x[n-1][i,j,k] + dt dt y,

  del[m_, x_[i,j,k]] 
    :> (x[i,j,k] /. rcen[m, 1]),

  deldel[m_, m_, x_[i,j,k]] 
    :> Factor[x[i,j,k] /. rcen[m, 1/2] /. rcen[m, 1/2]],

  deldel[m_, n_, x_[i,j,k]]
    :> Factor[x[i,j,k] /. rcen[m, 1] /. rcen[n, 1]]
  (* might use del instead, or other stencil for positivity *)

} 


colfdel = {
  fdel[m_, x_[i,j,k], f_[i,j,k]] -> del[m, (x/f)[i,j,k]],
  fdeldel[m_, n_, x_[i,j,k], f_[i,j,k]] -> deldel[m, n, (x/f)[i,j,k]]
}
expfdel = {
  (x_/f_)[i_,j_,k_] -> x[i,j,k]/f[i,j,k]
}


colfactors = {
  x_ / (2dx) -> x oo2dx,
  x_ / (2dy) -> x oo2dy,
  x_ / (2dz) -> x oo2dz,
  Power[dx, -2] -> oodx2,
  Power[dy, -2] -> oody2,
  Power[dz, -2] -> oodz2,
  1 / (4 dx dy) -> oo4dxdy,
  1 / (4 dx dz) -> oo4dxdz,
  1 / (4 dy dz) -> oo4dydz,
  -1 / (4 dx dy) -> -oo4dxdy,
  -1 / (4 dx dz) -> -oo4dxdz,
  -1 / (4 dy dz) -> -oo4dydz
}


(* put variables on grid *)
gridvars = Join[vars];
putvarsgrid = Map[(# -> (#)[i,j,k])&, gridvars]
dif = all /. putvarsgrid;
If[False, prlist["dif = all /. putvarsgrid", dif]]

(* difference equations *)
dif = dif /. colfdel
dif = dif //. leapfrog /. x_[n+1] :> glue[x,adv] /. x_[n-1] :> glue[x,ret]; 
dif = dif /. colfactors /. expfdel




(***********************************************************************)
(* special for relaxation: compute diagonal terms and add to list *)

diffeqns[x_] := 
  x //. expcontraction /. expfreeindices /. gluevar /.
  putvarsgrid //. leapfrog /. colfactors

diffrules[x_] := diffeqns[x] /. Equal -> Rule        

tmp = diffeqns[{ 
        Lu[a] == ginv[b,c] ddb[b,c,a] + 1/3 ginv[a,b] ddb[b,c,c] -
                 G[b] db[b,a] + beta[b] dG[b,a] + 2/3 G[a] db[b,b] }];

sub = diffrules[{
  db[a,b] == del[a,beta[b]],
  ddb[a,b,c] == deldel[a,b,beta[c]]
}];

tmp = tmp //. sub;

diagonalterms = {
  Cif == dLdu1,
    dLdu1 == D[tmp[[1,2]], beta1[i,j,k]],
    dLdu2 == D[tmp[[2,2]], beta2[i,j,k]],
    dLdu3 == D[tmp[[3,2]], beta3[i,j,k]],
  Cif == end
};

dif = Join[dif, diagonalterms]



(***********************************************************************)
(* introduce linear index *)
lininvars = Union[vars, 
                  Map[(glue[#,adv])&, vars], 
                  Map[(glue[#,ret])&, vars]];
dif = dif /. f_[i_,j_,k_] :> f[Expand[i*di+j*dj+k*dk]] /; MemberQ[lininvars, f]
dif = dif /. i*di+j*dj+k*dk -> ijk

If[False, 
  prlist["dif = dif/.putvarsgrid/.colfdel/.diffrule/.expfdel", dif]]

(* finally, use cactus variable names *)
(*
dif = dif /. Inner[Rule, vars, cactusvars, List]
If[True, prlist["dif = dif /. replacebycactusvars", dif]]
*)


(***************************************************************************)
(* trivial optimization: factor as much as you can linearly *)

(* to make formulas clearer, get out inverse metric first *)
ginvvars = {ginv[a,b]} /. expfreeindices /. gluevar
collginv = x_ == y_ :> x == Collect[y, ginvvars]

(* one of the deepest Mathematica insights I ever had, just try it without *)
rule = a_ x_ + a_ y_ -> a (x + y)
sule = a_ /; a < 0 -> minus (- a)
tule = minus -> -1

dif = dif /. collginv
dif = dif //. rule      (* sufficient for numbers with different signs *) 
dif = dif /. sule //. rule /. tule



(***************************************************************************)
(***************************************************************************)
(* write C code *)

Off[Part::partd]
Off[Part::pspec]

Print["writing to ", file, "\n"];
DeleteFile[file];
filepointer = OpenAppend[file];
pr[x_] := Module[{s},
  WriteString[filepointer, x];
]

pr["/* "<>file<>" */\n"];
pr["/* Bernd Bruegmann, "<>
       ToString[Date[][[3]]]<>"."<>
       ToString[Date[][[2]]]<>"."<>
       ToString[Date[][[1]]]<>" */\n"];
pr["/* Produced with Mathematica */\n"];
pr["\n"];
pr["#include \"bamveclap.h\"\n"];
pr["extern int minstrain, mindistortion, bctgauge, confharmonic, nocd;\n"];
pr["\n"];
pr["#define Power(x,y) (pow((double) (x), (double) (y)))\n"];
pr["#define Log(x)     log((double) (x)))\n"];
pr["#define pow2(x)    ((x)*(x))\n"];
pr["#define pow2inv(x) (1.0/((x)*(x)))\n\n"];
pr["#define Cal(x,y,z) ((x)?(y):(z))\n\n"];
pr["\n\n"];





(***************************************************************************)
(* write definition of C variables *)


writeCdef[vars_, auxvars_] := Module[{cvars, s, nf, nfields, nl},

  pr["void "<>routine<>"(cGH *GH, int ijk,\n"];
  pr["int di, int dj, int dk, double dx, double dy, double dz,\n"];
  nfields = Length[vars];
  For[nf = 1, nf < nfields, nf++,
    s = StringForm["double *``,\n", vars[[nf]]];
    pr[s];
  ];
  pr[StringForm["double *``\n", vars[[nfields]]]];
  pr[") {\n\n"];

  pr["double oo2dx = 1/(2*dx), oo2dy = 1/(2*dy), oo2dz = 1/(2*dz);\n"];
  pr["double oodx2 = 1/(dx*dx), oody2 = 1/(dy*dy), oodz2 = 1/(dz*dz);\n"];
  pr["double oo4dxdy = 1/(4*dx*dy);\n"];
  pr["double oo4dxdz = 1/(4*dx*dz);\n"];
  pr["double oo4dydz = 1/(4*dy*dz);\n"];
  pr["\n"];
  
  nfields = Length[auxvars];
  For[nf = 1, nf <= nfields, nf++,
    s = StringForm["double ``;\n", auxvars[[nf]]];
    pr[s];
  ];
  pr["\n\n\n"]
]

writeCdef[Join[vars, retvars], auxvars]




(***************************************************************************)
(* write equations *)

tocover = Join[constvars, {log, pow, pow2, pow2inv, Cal}]
coverpsi   = Map[ (#[i__] :> #[{i}])&, tocover]
uncoverpsi = Map[ (#[[{i__}]] :> #[i])&, tocover]



writeCassign[e_] := Module[{x, nfields, nf},

  If[e[[1]] =!= Cif,
    x = e /. coverpsi;
    x = ToString[InputForm[x]];
    x = StringReplace[x, "]" -> "]]"];
    x = StringReplace[x, "[" -> "[["];
    x = ToExpression[x];
    x = x /. uncoverpsi /. colpow;
    Print[CForm[x[[1]]]];
    If[MemberQ[retvars, x[[1]]], pr["*"]];
    PutAppend[CForm[x[[1]]], file];
    pr["=\n"];        (* to avoid assignment in CForm *)
    PutAppend[CForm[x[[2]]], file];
    pr[";\n\n"];
  ];

  If[e[[1]] === Cif,
    If[e[[2]] === end,
      pr["}\n"]; 
      pr[StringForm["/* if (``) */\n\n\n", lastif]]
    ];
    If[e[[2]] === else,
      pr["\n"]; 
      pr[StringForm["} else { /* if (!``) */\n\n", lastif]]
    ];
    If[e[[2]] =!= end && e[[2]] =!= else,
      pr["\n\n"]; 
      pr["/* conditional */\n"];
      pr[StringForm["if (``) {\n\n", e[[2]]]];
      lastif = e[[2]]
    ] 
  ]
]

Map[writeCassign, dif];


(***************************************************************************)
(* end function *)

pr["}  /* function */\n\n"];


(* count operations *)

char = ReadList[file, Character];
nmul = Count[char, "*"];
ndiv = Count[char, "/"];
nsum = Count[char, "+"] + Count[char, "-"];
count = StringForm["/* n* = ``,  n/ = ``,  n+ = ``, nvars = `` */\n", 
          nmul, ndiv, nsum, Length[vars]];
Print[""];
Print[file];
Print[count];
pr["/* "<>file<>" */\n"];
pr[count];

Close[filepointer];





