/* bamveclaplace.c */
/* Bernd Bruegmann, 16.2.2000 */
/* Produced with Mathematica */

#include "bamveclap.h"
extern int minstrain, mindistortion, bctgauge, confharmonic, nocd;

#define Power(x,y) (pow((double) (x), (double) (y)))
#define Log(x)     log((double) (x)))
#define pow2(x)    ((x)*(x))
#define pow2inv(x) (1.0/((x)*(x)))

#define Cal(x,y,z) ((x)?(y):(z))



void BAM_VecLap_VecLaplace (CCTK_INT ijk,
                            CCTK_INT di,
                            CCTK_INT dj,
                            CCTK_INT dk,
                            CCTK_REAL dx,
                            CCTK_REAL dy,
                            CCTK_REAL dz,
                            CCTK_REAL *psi,
                            CCTK_REAL *dpop1,
                            CCTK_REAL *dpop2,
                            CCTK_REAL *dpop3,
                            CCTK_REAL *ddpop11,
                            CCTK_REAL *ddpop12,
                            CCTK_REAL *ddpop13,
                            CCTK_REAL *ddpop22,
                            CCTK_REAL *ddpop23,
                            CCTK_REAL *ddpop33,
                            CCTK_REAL *alpha,
                            CCTK_REAL *g11,
                            CCTK_REAL *g12,
                            CCTK_REAL *g13,
                            CCTK_REAL *g22,
                            CCTK_REAL *g23,
                            CCTK_REAL *g33,
                            CCTK_REAL *K11,
                            CCTK_REAL *K12,
                            CCTK_REAL *K13,
                            CCTK_REAL *K22,
                            CCTK_REAL *K23,
                            CCTK_REAL *K33,
                            CCTK_REAL *u1,
                            CCTK_REAL *u2,
                            CCTK_REAL *u3,
                            CCTK_REAL *n1,
                            CCTK_REAL *n2,
                            CCTK_REAL *n3,
                            CCTK_REAL *Lu1,
                            CCTK_REAL *Lu2,
                            CCTK_REAL *Lu3,
                            CCTK_REAL *dLdu1,
                            CCTK_REAL *dLdu2,
                            CCTK_REAL *dLdu3)
{
double oo2dx = 1/(2*dx), oo2dy = 1/(2*dy), oo2dz = 1/(2*dz);
double oodx2 = 1/(dx*dx), oody2 = 1/(dy*dy), oodz2 = 1/(dz*dz);
double oo4dxdy = 1/(4*dx*dy);
double oo4dxdz = 1/(4*dx*dz);
double oo4dydz = 1/(4*dy*dz);

double Cif;
double divu;
double f;
double fdetg;
double KK;
double totch;
double B1111;
double B1112;
double B1113;
double B1122;
double B1123;
double B1133;
double B1212;
double B1213;
double B1222;
double B1223;
double B1233;
double B1313;
double B1322;
double B1323;
double B1333;
double B2222;
double B2223;
double B2233;
double B2323;
double B2333;
double B3333;
double BB111;
double BB112;
double BB113;
double BB122;
double BB123;
double BB133;
double BB211;
double BB212;
double BB213;
double BB222;
double BB223;
double BB233;
double BB311;
double BB312;
double BB313;
double BB322;
double BB323;
double BB333;
double cdK111;
double cdK112;
double cdK113;
double cdK122;
double cdK123;
double cdK133;
double cdK211;
double cdK212;
double cdK213;
double cdK222;
double cdK223;
double cdK233;
double cdK311;
double cdK312;
double cdK313;
double cdK322;
double cdK323;
double cdK333;
double cdKK1;
double cdKK2;
double cdKK3;
double cdKup111;
double cdKup112;
double cdKup113;
double cdKup122;
double cdKup123;
double cdKup133;
double cdKup211;
double cdKup212;
double cdKup213;
double cdKup222;
double cdKup223;
double cdKup233;
double cdKup311;
double cdKup312;
double cdKup313;
double cdKup322;
double cdKup323;
double cdKup333;
double ddf11;
double ddf12;
double ddf13;
double ddf22;
double ddf23;
double ddf33;
double ddivu1;
double ddivu2;
double ddivu3;
double deldelg1111;
double deldelg1112;
double deldelg1113;
double deldelg1122;
double deldelg1123;
double deldelg1133;
double deldelg1211;
double deldelg1212;
double deldelg1213;
double deldelg1222;
double deldelg1223;
double deldelg1233;
double deldelg1311;
double deldelg1312;
double deldelg1313;
double deldelg1322;
double deldelg1323;
double deldelg1333;
double deldelg2211;
double deldelg2212;
double deldelg2213;
double deldelg2222;
double deldelg2223;
double deldelg2233;
double deldelg2311;
double deldelg2312;
double deldelg2313;
double deldelg2322;
double deldelg2323;
double deldelg2333;
double deldelg3311;
double deldelg3312;
double deldelg3313;
double deldelg3322;
double deldelg3323;
double deldelg3333;
double deldelu111;
double deldelu112;
double deldelu113;
double deldelu121;
double deldelu122;
double deldelu123;
double deldelu131;
double deldelu132;
double deldelu133;
double deldelu211;
double deldelu212;
double deldelu213;
double deldelu221;
double deldelu222;
double deldelu223;
double deldelu231;
double deldelu232;
double deldelu233;
double deldelu311;
double deldelu312;
double deldelu313;
double deldelu321;
double deldelu322;
double deldelu323;
double deldelu331;
double deldelu332;
double deldelu333;
double delg111;
double delg112;
double delg113;
double delg122;
double delg123;
double delg133;
double delg211;
double delg212;
double delg213;
double delg222;
double delg223;
double delg233;
double delg311;
double delg312;
double delg313;
double delg322;
double delg323;
double delg333;
double delgb111;
double delgb112;
double delgb113;
double delgb122;
double delgb123;
double delgb133;
double delgb211;
double delgb212;
double delgb213;
double delgb222;
double delgb223;
double delgb233;
double delgb311;
double delgb312;
double delgb313;
double delgb322;
double delgb323;
double delgb333;
double DelLieug111;
double DelLieug112;
double DelLieug113;
double DelLieug122;
double DelLieug123;
double DelLieug133;
double DelLieug211;
double DelLieug212;
double DelLieug213;
double DelLieug222;
double DelLieug223;
double DelLieug233;
double DelLieug311;
double DelLieug312;
double DelLieug313;
double DelLieug322;
double DelLieug323;
double DelLieug333;
double delu11;
double delu12;
double delu13;
double delu21;
double delu22;
double delu23;
double delu31;
double delu32;
double delu33;
double df1;
double df2;
double df3;
double dLdubct1;
double dLdubct2;
double dLdubct3;
double gamma111;
double gamma112;
double gamma113;
double gamma122;
double gamma123;
double gamma133;
double gamma211;
double gamma212;
double gamma213;
double gamma222;
double gamma223;
double gamma233;
double gamma311;
double gamma312;
double gamma313;
double gamma322;
double gamma323;
double gamma333;
double gammado111;
double gammado112;
double gammado113;
double gammado122;
double gammado123;
double gammado133;
double gammado211;
double gammado212;
double gammado213;
double gammado222;
double gammado223;
double gammado233;
double gammado311;
double gammado312;
double gammado313;
double gammado322;
double gammado323;
double gammado333;
double gginv11;
double gginv12;
double gginv13;
double gginv22;
double gginv23;
double gginv33;
double ginv11;
double ginv12;
double ginv13;
double ginv22;
double ginv23;
double ginv33;
double Kup11;
double Kup12;
double Kup13;
double Kup22;
double Kup23;
double Kup33;
double Lieug11;
double Lieug12;
double Lieug13;
double Lieug22;
double Lieug23;
double Lieug33;



f
=
Cal(nocd,1,Power(psi[ijk],4))
;

df1
=
Cal(nocd,0,4*f*dpop1[ijk])
;

df2
=
Cal(nocd,0,4*f*dpop2[ijk])
;

df3
=
Cal(nocd,0,4*f*dpop3[ijk])
;

ddf11
=
Cal(nocd,0,f*(4*ddpop11[ijk] + 12*pow2(dpop1[ijk])))
;

ddf12
=
Cal(nocd,0,f*(4*ddpop12[ijk] + 12*dpop1[ijk]*dpop2[ijk]))
;

ddf13
=
Cal(nocd,0,f*(4*ddpop13[ijk] + 12*dpop1[ijk]*dpop3[ijk]))
;

ddf22
=
Cal(nocd,0,f*(4*ddpop22[ijk] + 12*pow2(dpop2[ijk])))
;

ddf23
=
Cal(nocd,0,f*(4*ddpop23[ijk] + 12*dpop2[ijk]*dpop3[ijk]))
;

ddf33
=
Cal(nocd,0,f*(4*ddpop33[ijk] + 12*pow2(dpop3[ijk])))
;

gginv11
=
g22[ijk]*g33[ijk] - pow2(g23[ijk])
;

gginv12
=
g13[ijk]*g23[ijk] - g12[ijk]*g33[ijk]
;

gginv13
=
-(g13[ijk]*g22[ijk]) + g12[ijk]*g23[ijk]
;

gginv22
=
g11[ijk]*g33[ijk] - pow2(g13[ijk])
;

gginv23
=
g12[ijk]*g13[ijk] - g11[ijk]*g23[ijk]
;

gginv33
=
g11[ijk]*g22[ijk] - pow2(g12[ijk])
;

fdetg
=
f*(gginv11*g11[ijk] + gginv12*g12[ijk] + gginv13*g13[ijk])
;

ginv11
=
gginv11/fdetg
;

ginv12
=
gginv12/fdetg
;

ginv13
=
gginv13/fdetg
;

ginv22
=
gginv22/fdetg
;

ginv23
=
gginv23/fdetg
;

ginv33
=
gginv33/fdetg
;

delgb111
=
oo2dx*(-g11[-di + ijk] + g11[di + ijk])
;

delgb112
=
oo2dx*(-g12[-di + ijk] + g12[di + ijk])
;

delgb113
=
oo2dx*(-g13[-di + ijk] + g13[di + ijk])
;

delgb122
=
oo2dx*(-g22[-di + ijk] + g22[di + ijk])
;

delgb123
=
oo2dx*(-g23[-di + ijk] + g23[di + ijk])
;

delgb133
=
oo2dx*(-g33[-di + ijk] + g33[di + ijk])
;

delgb211
=
oo2dy*(-g11[-dj + ijk] + g11[dj + ijk])
;

delgb212
=
oo2dy*(-g12[-dj + ijk] + g12[dj + ijk])
;

delgb213
=
oo2dy*(-g13[-dj + ijk] + g13[dj + ijk])
;

delgb222
=
oo2dy*(-g22[-dj + ijk] + g22[dj + ijk])
;

delgb223
=
oo2dy*(-g23[-dj + ijk] + g23[dj + ijk])
;

delgb233
=
oo2dy*(-g33[-dj + ijk] + g33[dj + ijk])
;

delgb311
=
oo2dz*(-g11[-dk + ijk] + g11[dk + ijk])
;

delgb312
=
oo2dz*(-g12[-dk + ijk] + g12[dk + ijk])
;

delgb313
=
oo2dz*(-g13[-dk + ijk] + g13[dk + ijk])
;

delgb322
=
oo2dz*(-g22[-dk + ijk] + g22[dk + ijk])
;

delgb323
=
oo2dz*(-g23[-dk + ijk] + g23[dk + ijk])
;

delgb333
=
oo2dz*(-g33[-dk + ijk] + g33[dk + ijk])
;

delg111
=
delgb111*f + df1*g11[ijk]
;

delg112
=
delgb112*f + df1*g12[ijk]
;

delg113
=
delgb113*f + df1*g13[ijk]
;

delg122
=
delgb122*f + df1*g22[ijk]
;

delg123
=
delgb123*f + df1*g23[ijk]
;

delg133
=
delgb133*f + df1*g33[ijk]
;

delg211
=
delgb211*f + df2*g11[ijk]
;

delg212
=
delgb212*f + df2*g12[ijk]
;

delg213
=
delgb213*f + df2*g13[ijk]
;

delg222
=
delgb222*f + df2*g22[ijk]
;

delg223
=
delgb223*f + df2*g23[ijk]
;

delg233
=
delgb233*f + df2*g33[ijk]
;

delg311
=
delgb311*f + df3*g11[ijk]
;

delg312
=
delgb312*f + df3*g12[ijk]
;

delg313
=
delgb313*f + df3*g13[ijk]
;

delg322
=
delgb322*f + df3*g22[ijk]
;

delg323
=
delgb323*f + df3*g23[ijk]
;

delg333
=
delgb333*f + df3*g33[ijk]
;

deldelg1111
=
2*delgb111*df1 + ddf11*g11[ijk] + 
   f*oodx2*(-2*g11[ijk] + g11[-di + ijk] + g11[di + ijk])
;

deldelg1112
=
2*delgb112*df1 + ddf11*g12[ijk] + 
   f*oodx2*(-2*g12[ijk] + g12[-di + ijk] + g12[di + ijk])
;

deldelg1113
=
2*delgb113*df1 + ddf11*g13[ijk] + 
   f*oodx2*(-2*g13[ijk] + g13[-di + ijk] + g13[di + ijk])
;

deldelg1122
=
2*delgb122*df1 + ddf11*g22[ijk] + 
   f*oodx2*(-2*g22[ijk] + g22[-di + ijk] + g22[di + ijk])
;

deldelg1123
=
2*delgb123*df1 + ddf11*g23[ijk] + 
   f*oodx2*(-2*g23[ijk] + g23[-di + ijk] + g23[di + ijk])
;

deldelg1133
=
2*delgb133*df1 + ddf11*g33[ijk] + 
   f*oodx2*(-2*g33[ijk] + g33[-di + ijk] + g33[di + ijk])
;

deldelg1211
=
delgb211*df1 + delgb111*df2 + ddf12*g11[ijk] + 
   f*oo4dxdy*(g11[-di - dj + ijk] - g11[di - dj + ijk] - 
      g11[-di + dj + ijk] + g11[di + dj + ijk])
;

deldelg1212
=
delgb212*df1 + delgb112*df2 + ddf12*g12[ijk] + 
   f*oo4dxdy*(g12[-di - dj + ijk] - g12[di - dj + ijk] - 
      g12[-di + dj + ijk] + g12[di + dj + ijk])
;

deldelg1213
=
delgb213*df1 + delgb113*df2 + ddf12*g13[ijk] + 
   f*oo4dxdy*(g13[-di - dj + ijk] - g13[di - dj + ijk] - 
      g13[-di + dj + ijk] + g13[di + dj + ijk])
;

deldelg1222
=
delgb222*df1 + delgb122*df2 + ddf12*g22[ijk] + 
   f*oo4dxdy*(g22[-di - dj + ijk] - g22[di - dj + ijk] - 
      g22[-di + dj + ijk] + g22[di + dj + ijk])
;

deldelg1223
=
delgb223*df1 + delgb123*df2 + ddf12*g23[ijk] + 
   f*oo4dxdy*(g23[-di - dj + ijk] - g23[di - dj + ijk] - 
      g23[-di + dj + ijk] + g23[di + dj + ijk])
;

deldelg1233
=
delgb233*df1 + delgb133*df2 + ddf12*g33[ijk] + 
   f*oo4dxdy*(g33[-di - dj + ijk] - g33[di - dj + ijk] - 
      g33[-di + dj + ijk] + g33[di + dj + ijk])
;

deldelg1311
=
delgb311*df1 + delgb111*df3 + ddf13*g11[ijk] + 
   f*oo4dxdz*(g11[-di - dk + ijk] - g11[di - dk + ijk] - 
      g11[-di + dk + ijk] + g11[di + dk + ijk])
;

deldelg1312
=
delgb312*df1 + delgb112*df3 + ddf13*g12[ijk] + 
   f*oo4dxdz*(g12[-di - dk + ijk] - g12[di - dk + ijk] - 
      g12[-di + dk + ijk] + g12[di + dk + ijk])
;

deldelg1313
=
delgb313*df1 + delgb113*df3 + ddf13*g13[ijk] + 
   f*oo4dxdz*(g13[-di - dk + ijk] - g13[di - dk + ijk] - 
      g13[-di + dk + ijk] + g13[di + dk + ijk])
;

deldelg1322
=
delgb322*df1 + delgb122*df3 + ddf13*g22[ijk] + 
   f*oo4dxdz*(g22[-di - dk + ijk] - g22[di - dk + ijk] - 
      g22[-di + dk + ijk] + g22[di + dk + ijk])
;

deldelg1323
=
delgb323*df1 + delgb123*df3 + ddf13*g23[ijk] + 
   f*oo4dxdz*(g23[-di - dk + ijk] - g23[di - dk + ijk] - 
      g23[-di + dk + ijk] + g23[di + dk + ijk])
;

deldelg1333
=
delgb333*df1 + delgb133*df3 + ddf13*g33[ijk] + 
   f*oo4dxdz*(g33[-di - dk + ijk] - g33[di - dk + ijk] - 
      g33[-di + dk + ijk] + g33[di + dk + ijk])
;

deldelg2211
=
2*delgb211*df2 + ddf22*g11[ijk] + 
   f*oody2*(-2*g11[ijk] + g11[-dj + ijk] + g11[dj + ijk])
;

deldelg2212
=
2*delgb212*df2 + ddf22*g12[ijk] + 
   f*oody2*(-2*g12[ijk] + g12[-dj + ijk] + g12[dj + ijk])
;

deldelg2213
=
2*delgb213*df2 + ddf22*g13[ijk] + 
   f*oody2*(-2*g13[ijk] + g13[-dj + ijk] + g13[dj + ijk])
;

deldelg2222
=
2*delgb222*df2 + ddf22*g22[ijk] + 
   f*oody2*(-2*g22[ijk] + g22[-dj + ijk] + g22[dj + ijk])
;

deldelg2223
=
2*delgb223*df2 + ddf22*g23[ijk] + 
   f*oody2*(-2*g23[ijk] + g23[-dj + ijk] + g23[dj + ijk])
;

deldelg2233
=
2*delgb233*df2 + ddf22*g33[ijk] + 
   f*oody2*(-2*g33[ijk] + g33[-dj + ijk] + g33[dj + ijk])
;

deldelg2311
=
delgb311*df2 + delgb211*df3 + ddf23*g11[ijk] + 
   f*oo4dydz*(g11[-dj - dk + ijk] - g11[dj - dk + ijk] - 
      g11[-dj + dk + ijk] + g11[dj + dk + ijk])
;

deldelg2312
=
delgb312*df2 + delgb212*df3 + ddf23*g12[ijk] + 
   f*oo4dydz*(g12[-dj - dk + ijk] - g12[dj - dk + ijk] - 
      g12[-dj + dk + ijk] + g12[dj + dk + ijk])
;

deldelg2313
=
delgb313*df2 + delgb213*df3 + ddf23*g13[ijk] + 
   f*oo4dydz*(g13[-dj - dk + ijk] - g13[dj - dk + ijk] - 
      g13[-dj + dk + ijk] + g13[dj + dk + ijk])
;

deldelg2322
=
delgb322*df2 + delgb222*df3 + ddf23*g22[ijk] + 
   f*oo4dydz*(g22[-dj - dk + ijk] - g22[dj - dk + ijk] - 
      g22[-dj + dk + ijk] + g22[dj + dk + ijk])
;

deldelg2323
=
delgb323*df2 + delgb223*df3 + ddf23*g23[ijk] + 
   f*oo4dydz*(g23[-dj - dk + ijk] - g23[dj - dk + ijk] - 
      g23[-dj + dk + ijk] + g23[dj + dk + ijk])
;

deldelg2333
=
delgb333*df2 + delgb233*df3 + ddf23*g33[ijk] + 
   f*oo4dydz*(g33[-dj - dk + ijk] - g33[dj - dk + ijk] - 
      g33[-dj + dk + ijk] + g33[dj + dk + ijk])
;

deldelg3311
=
2*delgb311*df3 + ddf33*g11[ijk] + 
   f*oodz2*(-2*g11[ijk] + g11[-dk + ijk] + g11[dk + ijk])
;

deldelg3312
=
2*delgb312*df3 + ddf33*g12[ijk] + 
   f*oodz2*(-2*g12[ijk] + g12[-dk + ijk] + g12[dk + ijk])
;

deldelg3313
=
2*delgb313*df3 + ddf33*g13[ijk] + 
   f*oodz2*(-2*g13[ijk] + g13[-dk + ijk] + g13[dk + ijk])
;

deldelg3322
=
2*delgb322*df3 + ddf33*g22[ijk] + 
   f*oodz2*(-2*g22[ijk] + g22[-dk + ijk] + g22[dk + ijk])
;

deldelg3323
=
2*delgb323*df3 + ddf33*g23[ijk] + 
   f*oodz2*(-2*g23[ijk] + g23[-dk + ijk] + g23[dk + ijk])
;

deldelg3333
=
2*delgb333*df3 + ddf33*g33[ijk] + 
   f*oodz2*(-2*g33[ijk] + g33[-dk + ijk] + g33[dk + ijk])
;

delu11
=
oo2dx*(-u1[-di + ijk] + u1[di + ijk])
;

delu12
=
oo2dx*(-u2[-di + ijk] + u2[di + ijk])
;

delu13
=
oo2dx*(-u3[-di + ijk] + u3[di + ijk])
;

delu21
=
oo2dy*(-u1[-dj + ijk] + u1[dj + ijk])
;

delu22
=
oo2dy*(-u2[-dj + ijk] + u2[dj + ijk])
;

delu23
=
oo2dy*(-u3[-dj + ijk] + u3[dj + ijk])
;

delu31
=
oo2dz*(-u1[-dk + ijk] + u1[dk + ijk])
;

delu32
=
oo2dz*(-u2[-dk + ijk] + u2[dk + ijk])
;

delu33
=
oo2dz*(-u3[-dk + ijk] + u3[dk + ijk])
;

deldelu111
=
oodx2*(-2*u1[ijk] + u1[-di + ijk] + u1[di + ijk])
;

deldelu112
=
oodx2*(-2*u2[ijk] + u2[-di + ijk] + u2[di + ijk])
;

deldelu113
=
oodx2*(-2*u3[ijk] + u3[-di + ijk] + u3[di + ijk])
;

deldelu121
=
oo4dxdy*(u1[-di - dj + ijk] - u1[di - dj + ijk] - u1[-di + dj + ijk] + 
     u1[di + dj + ijk])
;

deldelu122
=
oo4dxdy*(u2[-di - dj + ijk] - u2[di - dj + ijk] - u2[-di + dj + ijk] + 
     u2[di + dj + ijk])
;

deldelu123
=
oo4dxdy*(u3[-di - dj + ijk] - u3[di - dj + ijk] - u3[-di + dj + ijk] + 
     u3[di + dj + ijk])
;

deldelu131
=
oo4dxdz*(u1[-di - dk + ijk] - u1[di - dk + ijk] - u1[-di + dk + ijk] + 
     u1[di + dk + ijk])
;

deldelu132
=
oo4dxdz*(u2[-di - dk + ijk] - u2[di - dk + ijk] - u2[-di + dk + ijk] + 
     u2[di + dk + ijk])
;

deldelu133
=
oo4dxdz*(u3[-di - dk + ijk] - u3[di - dk + ijk] - u3[-di + dk + ijk] + 
     u3[di + dk + ijk])
;

deldelu211
=
oo4dxdy*(u1[-di - dj + ijk] - u1[di - dj + ijk] - u1[-di + dj + ijk] + 
     u1[di + dj + ijk])
;

deldelu212
=
oo4dxdy*(u2[-di - dj + ijk] - u2[di - dj + ijk] - u2[-di + dj + ijk] + 
     u2[di + dj + ijk])
;

deldelu213
=
oo4dxdy*(u3[-di - dj + ijk] - u3[di - dj + ijk] - u3[-di + dj + ijk] + 
     u3[di + dj + ijk])
;

deldelu221
=
oody2*(-2*u1[ijk] + u1[-dj + ijk] + u1[dj + ijk])
;

deldelu222
=
oody2*(-2*u2[ijk] + u2[-dj + ijk] + u2[dj + ijk])
;

deldelu223
=
oody2*(-2*u3[ijk] + u3[-dj + ijk] + u3[dj + ijk])
;

deldelu231
=
oo4dydz*(u1[-dj - dk + ijk] - u1[dj - dk + ijk] - u1[-dj + dk + ijk] + 
     u1[dj + dk + ijk])
;

deldelu232
=
oo4dydz*(u2[-dj - dk + ijk] - u2[dj - dk + ijk] - u2[-dj + dk + ijk] + 
     u2[dj + dk + ijk])
;

deldelu233
=
oo4dydz*(u3[-dj - dk + ijk] - u3[dj - dk + ijk] - u3[-dj + dk + ijk] + 
     u3[dj + dk + ijk])
;

deldelu311
=
oo4dxdz*(u1[-di - dk + ijk] - u1[di - dk + ijk] - u1[-di + dk + ijk] + 
     u1[di + dk + ijk])
;

deldelu312
=
oo4dxdz*(u2[-di - dk + ijk] - u2[di - dk + ijk] - u2[-di + dk + ijk] + 
     u2[di + dk + ijk])
;

deldelu313
=
oo4dxdz*(u3[-di - dk + ijk] - u3[di - dk + ijk] - u3[-di + dk + ijk] + 
     u3[di + dk + ijk])
;

deldelu321
=
oo4dydz*(u1[-dj - dk + ijk] - u1[dj - dk + ijk] - u1[-dj + dk + ijk] + 
     u1[dj + dk + ijk])
;

deldelu322
=
oo4dydz*(u2[-dj - dk + ijk] - u2[dj - dk + ijk] - u2[-dj + dk + ijk] + 
     u2[dj + dk + ijk])
;

deldelu323
=
oo4dydz*(u3[-dj - dk + ijk] - u3[dj - dk + ijk] - u3[-dj + dk + ijk] + 
     u3[dj + dk + ijk])
;

deldelu331
=
oodz2*(-2*u1[ijk] + u1[-dk + ijk] + u1[dk + ijk])
;

deldelu332
=
oodz2*(-2*u2[ijk] + u2[-dk + ijk] + u2[dk + ijk])
;

deldelu333
=
oodz2*(-2*u3[ijk] + u3[-dk + ijk] + u3[dk + ijk])
;

totch
=
(2*confharmonic)/3.
;



/* conditional */
if (confharmonic) {

gamma111
=
0
;

gamma112
=
0
;

gamma113
=
0
;

gamma122
=
0
;

gamma123
=
0
;

gamma133
=
0
;

gamma211
=
0
;

gamma212
=
0
;

gamma213
=
0
;

gamma222
=
0
;

gamma223
=
0
;

gamma233
=
0
;

gamma311
=
0
;

gamma312
=
0
;

gamma313
=
0
;

gamma322
=
0
;

gamma323
=
0
;

gamma333
=
0
;

divu
=
(2*(delu11 + delu22 + delu33))/3.
;

ddivu1
=
(2*(deldelu111 + deldelu122 + deldelu133))/3.
;

ddivu2
=
(2*(deldelu211 + deldelu222 + deldelu233))/3.
;

ddivu3
=
(2*(deldelu311 + deldelu322 + deldelu333))/3.
;


} else { /* if (!confharmonic) */

gammado111
=
delg111/2.
;

gammado112
=
delg211/2.
;

gammado113
=
delg311/2.
;

gammado122
=
-delg122/2. + delg212
;

gammado123
=
(-delg123 + delg213 + delg312)/2.
;

gammado133
=
-delg133/2. + delg313
;

gammado211
=
delg112 - delg211/2.
;

gammado212
=
delg122/2.
;

gammado213
=
(delg123 - delg213 + delg312)/2.
;

gammado222
=
delg222/2.
;

gammado223
=
delg322/2.
;

gammado233
=
-delg233/2. + delg323
;

gammado311
=
delg113 - delg311/2.
;

gammado312
=
(delg123 + delg213 - delg312)/2.
;

gammado313
=
delg133/2.
;

gammado322
=
delg223 - delg322/2.
;

gammado323
=
delg233/2.
;

gammado333
=
delg333/2.
;

gamma111
=
gammado111*ginv11 + gammado211*ginv12 + gammado311*ginv13
;

gamma112
=
gammado112*ginv11 + gammado212*ginv12 + gammado312*ginv13
;

gamma113
=
gammado113*ginv11 + gammado213*ginv12 + gammado313*ginv13
;

gamma122
=
gammado122*ginv11 + gammado222*ginv12 + gammado322*ginv13
;

gamma123
=
gammado123*ginv11 + gammado223*ginv12 + gammado323*ginv13
;

gamma133
=
gammado133*ginv11 + gammado233*ginv12 + gammado333*ginv13
;

gamma211
=
gammado111*ginv12 + gammado211*ginv22 + gammado311*ginv23
;

gamma212
=
gammado112*ginv12 + gammado212*ginv22 + gammado312*ginv23
;

gamma213
=
gammado113*ginv12 + gammado213*ginv22 + gammado313*ginv23
;

gamma222
=
gammado122*ginv12 + gammado222*ginv22 + gammado322*ginv23
;

gamma223
=
gammado123*ginv12 + gammado223*ginv22 + gammado323*ginv23
;

gamma233
=
gammado133*ginv12 + gammado233*ginv22 + gammado333*ginv23
;

gamma311
=
gammado111*ginv13 + gammado211*ginv23 + gammado311*ginv33
;

gamma312
=
gammado112*ginv13 + gammado212*ginv23 + gammado312*ginv33
;

gamma313
=
gammado113*ginv13 + gammado213*ginv23 + gammado313*ginv33
;

gamma322
=
gammado122*ginv13 + gammado222*ginv23 + gammado322*ginv33
;

gamma323
=
gammado123*ginv13 + gammado223*ginv23 + gammado323*ginv33
;

gamma333
=
gammado133*ginv13 + gammado233*ginv23 + gammado333*ginv33
;

divu
=
0
;

ddivu1
=
0
;

ddivu2
=
0
;

ddivu3
=
0
;

}
/* if (confharmonic) */


Lieug11
=
f*((2*delu11 - divu)*g11[ijk] + 2*(delu12*g12[ijk] + delu13*g13[ijk])) + 
   delg111*u1[ijk] + delg211*u2[ijk] + delg311*u3[ijk]
;

Lieug12
=
f*(delu21*g11[ijk] + (delu11 + delu22 - divu)*g12[ijk] + delu23*g13[ijk] + 
      delu12*g22[ijk] + delu13*g23[ijk]) + delg112*u1[ijk] + 
   delg212*u2[ijk] + delg312*u3[ijk]
;

Lieug13
=
f*(delu31*g11[ijk] + delu32*g12[ijk] + (delu11 + delu33 - divu)*g13[ijk] + 
      delu12*g23[ijk] + delu13*g33[ijk]) + delg113*u1[ijk] + 
   delg213*u2[ijk] + delg313*u3[ijk]
;

Lieug22
=
f*(-(divu*g22[ijk]) + 2*(delu21*g12[ijk] + delu22*g22[ijk] + 
         delu23*g23[ijk])) + delg122*u1[ijk] + delg222*u2[ijk] + 
   delg322*u3[ijk]
;

Lieug23
=
f*(delu31*g12[ijk] + delu21*g13[ijk] + delu32*g22[ijk] + 
      (delu22 + delu33 - divu)*g23[ijk] + delu23*g33[ijk]) + 
   delg123*u1[ijk] + delg223*u2[ijk] + delg323*u3[ijk]
;

Lieug33
=
f*(-(divu*g33[ijk]) + 2*(delu31*g13[ijk] + delu32*g23[ijk] + 
         delu33*g33[ijk])) + delg133*u1[ijk] + delg233*u2[ijk] + 
   delg333*u3[ijk]
;

DelLieug111
=
(2*delg112 + delg211)*delu12 + (2*delg113 + delg311)*delu13 + 
   delg111*(3*delu11 - divu) - 2*
    (gamma111*Lieug11 + gamma211*Lieug12 + gamma311*Lieug13) + 
   f*((-ddivu1 + 2*deldelu111)*g11[ijk] + 
      2*(deldelu112*g12[ijk] + deldelu113*g13[ijk])) + deldelg1111*u1[ijk] + 
   deldelg1211*u2[ijk] + deldelg1311*u3[ijk]
;

DelLieug112
=
(delg122 + delg212)*delu12 + (delg123 + delg312)*delu13 + delg111*delu21 + 
   delg113*delu23 + delg112*(2*delu11 + delu22 - divu) - gamma112*Lieug11 - 
   (gamma111 + gamma212)*Lieug12 - gamma312*Lieug13 - gamma211*Lieug22 - 
   gamma311*Lieug23 - ddivu1*f*g12[ijk] + 
   f*(deldelu121*g11[ijk] + (deldelu111 + deldelu122)*g12[ijk] + 
      deldelu123*g13[ijk] + deldelu112*g22[ijk] + deldelu113*g23[ijk]) + 
   deldelg1112*u1[ijk] + deldelg1212*u2[ijk] + deldelg1312*u3[ijk]
;

DelLieug113
=
(delg123 + delg213)*delu12 + (delg133 + delg313)*delu13 + delg111*delu31 + 
   delg112*delu32 + delg113*(2*delu11 + delu33 - divu) - gamma113*Lieug11 - 
   gamma213*Lieug12 - (gamma111 + gamma313)*Lieug13 - gamma211*Lieug23 - 
   gamma311*Lieug33 - ddivu1*f*g13[ijk] + 
   f*(deldelu131*g11[ijk] + deldelu132*g12[ijk] + 
      (deldelu111 + deldelu133)*g13[ijk] + deldelu112*g23[ijk] + 
      deldelu113*g33[ijk]) + deldelg1113*u1[ijk] + deldelg1213*u2[ijk] + 
   deldelg1313*u3[ijk]
;

DelLieug122
=
delg222*delu12 + delg322*delu13 + delg122*(delu11 + 2*delu22 - divu) + 
   2*(delg112*delu21 + delg123*delu23 - gamma112*Lieug12 - gamma212*Lieug22 - 
      gamma312*Lieug23 + f*(deldelu121*g12[ijk] + deldelu122*g22[ijk])) + 
   f*(-(ddivu1*g22[ijk]) + 2*deldelu123*g23[ijk]) + deldelg1122*u1[ijk] + 
   deldelg1222*u2[ijk] + deldelg1322*u3[ijk]
;

DelLieug123
=
delg223*delu12 + delg323*delu13 + delg113*delu21 + delg133*delu23 + 
   delg112*delu31 + delg122*delu32 + 
   delg123*(delu11 + delu22 + delu33 - divu) - gamma113*Lieug12 - 
   gamma112*Lieug13 - gamma213*Lieug22 - (gamma212 + gamma313)*Lieug23 - 
   gamma312*Lieug33 - ddivu1*f*g23[ijk] + 
   f*(deldelu131*g12[ijk] + deldelu121*g13[ijk] + deldelu132*g22[ijk] + 
      (deldelu122 + deldelu133)*g23[ijk] + deldelu123*g33[ijk]) + 
   deldelg1123*u1[ijk] + deldelg1223*u2[ijk] + deldelg1323*u3[ijk]
;

DelLieug133
=
delg233*delu12 + delg333*delu13 + delg133*(delu11 + 2*delu33 - divu) - 
   2*(gamma113*Lieug13 + gamma213*Lieug23 + gamma313*Lieug33) - 
   ddivu1*f*g33[ijk] + 2*(delg113*delu31 + delg123*delu32 + 
      f*(deldelu131*g13[ijk] + deldelu132*g23[ijk] + deldelu133*g33[ijk])) + 
   deldelg1133*u1[ijk] + deldelg1233*u2[ijk] + deldelg1333*u3[ijk]
;

DelLieug211
=
delg111*delu21 + delg311*delu23 + delg211*(delu22 - divu) + 
   f*(-(ddivu2*g11[ijk]) + 2*deldelu212*g12[ijk]) + 
   2*(delg211*delu11 + delg212*delu12 + delg213*delu13 - gamma112*Lieug11 - 
      gamma212*Lieug12 - gamma312*Lieug13 + 
      f*(deldelu211*g11[ijk] + deldelu213*g13[ijk])) + deldelg1211*u1[ijk] + 
   deldelg2211*u2[ijk] + deldelg2311*u3[ijk]
;

DelLieug212
=
delg222*delu12 + delg223*delu13 + (delg112 + delg211)*delu21 + 
   (delg213 + delg312)*delu23 + delg212*(delu11 + 2*delu22 - divu) - 
   gamma122*Lieug11 - (gamma112 + gamma222)*Lieug12 - gamma322*Lieug13 - 
   gamma212*Lieug22 - gamma312*Lieug23 - ddivu2*f*g12[ijk] + 
   f*(deldelu221*g11[ijk] + (deldelu211 + deldelu222)*g12[ijk] + 
      deldelu223*g13[ijk] + deldelu212*g22[ijk] + deldelu213*g23[ijk]) + 
   deldelg1212*u1[ijk] + deldelg2212*u2[ijk] + deldelg2312*u3[ijk]
;

DelLieug213
=
delg223*delu12 + delg233*delu13 + delg113*delu21 + delg313*delu23 + 
   delg211*delu31 + delg212*delu32 + 
   delg213*(delu11 + delu22 + delu33 - divu) - gamma123*Lieug11 - 
   gamma223*Lieug12 - (gamma112 + gamma323)*Lieug13 - gamma212*Lieug23 - 
   gamma312*Lieug33 - ddivu2*f*g13[ijk] + 
   f*(deldelu231*g11[ijk] + deldelu232*g12[ijk] + 
      (deldelu211 + deldelu233)*g13[ijk] + deldelu212*g23[ijk] + 
      deldelu213*g33[ijk]) + deldelg1213*u1[ijk] + deldelg2213*u2[ijk] + 
   deldelg2313*u3[ijk]
;

DelLieug222
=
(delg122 + 2*delg212)*delu21 + (2*delg223 + delg322)*delu23 + 
   delg222*(3*delu22 - divu) - 2*
    (gamma122*Lieug12 + gamma222*Lieug22 + gamma322*Lieug23) + 
   f*(-(ddivu2*g22[ijk]) + 2*(deldelu221*g12[ijk] + deldelu222*g22[ijk] + 
         deldelu223*g23[ijk])) + deldelg1222*u1[ijk] + deldelg2222*u2[ijk] + 
   deldelg2322*u3[ijk]
;

DelLieug223
=
(delg123 + delg213)*delu21 + (delg233 + delg323)*delu23 + delg212*delu31 + 
   delg222*delu32 + delg223*(2*delu22 + delu33 - divu) - gamma123*Lieug12 - 
   gamma122*Lieug13 - gamma223*Lieug22 - (gamma222 + gamma323)*Lieug23 - 
   gamma322*Lieug33 - ddivu2*f*g23[ijk] + 
   f*(deldelu231*g12[ijk] + deldelu221*g13[ijk] + deldelu232*g22[ijk] + 
      (deldelu222 + deldelu233)*g23[ijk] + deldelu223*g33[ijk]) + 
   deldelg1223*u1[ijk] + deldelg2223*u2[ijk] + deldelg2323*u3[ijk]
;

DelLieug233
=
delg133*delu21 + delg333*delu23 + delg233*(delu22 + 2*delu33 - divu) - 
   2*(gamma123*Lieug13 + gamma223*Lieug23 + gamma323*Lieug33) - 
   ddivu2*f*g33[ijk] + 2*(delg213*delu31 + delg223*delu32 + 
      f*(deldelu231*g13[ijk] + deldelu232*g23[ijk] + deldelu233*g33[ijk])) + 
   deldelg1233*u1[ijk] + deldelg2233*u2[ijk] + deldelg2333*u3[ijk]
;

DelLieug311
=
delg111*delu31 + delg211*delu32 + delg311*(delu33 - divu) + 
   f*(-(ddivu3*g11[ijk]) + 2*deldelu312*g12[ijk]) + 
   2*(delg311*delu11 + delg312*delu12 + delg313*delu13 - gamma113*Lieug11 - 
      gamma213*Lieug12 - gamma313*Lieug13 + 
      f*(deldelu311*g11[ijk] + deldelu313*g13[ijk])) + deldelg1311*u1[ijk] + 
   deldelg2311*u2[ijk] + deldelg3311*u3[ijk]
;

DelLieug312
=
delg322*delu12 + delg323*delu13 + delg311*delu21 + delg313*delu23 + 
   delg112*delu31 + delg212*delu32 + 
   delg312*(delu11 + delu22 + delu33 - divu) - gamma123*Lieug11 - 
   (gamma113 + gamma223)*Lieug12 - gamma323*Lieug13 - gamma213*Lieug22 - 
   gamma313*Lieug23 - ddivu3*f*g12[ijk] + 
   f*(deldelu321*g11[ijk] + (deldelu311 + deldelu322)*g12[ijk] + 
      deldelu323*g13[ijk] + deldelu312*g22[ijk] + deldelu313*g23[ijk]) + 
   deldelg1312*u1[ijk] + deldelg2312*u2[ijk] + deldelg3312*u3[ijk]
;

DelLieug313
=
delg323*delu12 + delg333*delu13 + (delg113 + delg311)*delu31 + 
   (delg213 + delg312)*delu32 + delg313*(delu11 + 2*delu33 - divu) - 
   gamma133*Lieug11 - gamma233*Lieug12 - (gamma113 + gamma333)*Lieug13 - 
   gamma213*Lieug23 - gamma313*Lieug33 - ddivu3*f*g13[ijk] + 
   f*(deldelu331*g11[ijk] + deldelu332*g12[ijk] + 
      (deldelu311 + deldelu333)*g13[ijk] + deldelu312*g23[ijk] + 
      deldelu313*g33[ijk]) + deldelg1313*u1[ijk] + deldelg2313*u2[ijk] + 
   deldelg3313*u3[ijk]
;

DelLieug322
=
delg122*delu31 + delg222*delu32 + delg322*(delu33 - divu) + 
   2*(delg312*delu21 + delg322*delu22 + delg323*delu23 - gamma123*Lieug12 - 
      gamma223*Lieug22 - gamma323*Lieug23 + 
      f*(deldelu321*g12[ijk] + deldelu322*g22[ijk])) + 
   f*(-(ddivu3*g22[ijk]) + 2*deldelu323*g23[ijk]) + deldelg1322*u1[ijk] + 
   deldelg2322*u2[ijk] + deldelg3322*u3[ijk]
;

DelLieug323
=
delg313*delu21 + delg333*delu23 + (delg123 + delg312)*delu31 + 
   (delg223 + delg322)*delu32 + delg323*(delu22 + 2*delu33 - divu) - 
   gamma133*Lieug12 - gamma123*Lieug13 - gamma233*Lieug22 - 
   (gamma223 + gamma333)*Lieug23 - gamma323*Lieug33 - ddivu3*f*g23[ijk] + 
   f*(deldelu331*g12[ijk] + deldelu321*g13[ijk] + deldelu332*g22[ijk] + 
      (deldelu322 + deldelu333)*g23[ijk] + deldelu323*g33[ijk]) + 
   deldelg1323*u1[ijk] + deldelg2323*u2[ijk] + deldelg3323*u3[ijk]
;

DelLieug333
=
(delg133 + 2*delg313)*delu31 + (delg233 + 2*delg323)*delu32 + 
   delg333*(3*delu33 - divu) - 2*
    (gamma133*Lieug13 + gamma233*Lieug23 + gamma333*Lieug33) + 
   f*(-(ddivu3*g33[ijk]) + 2*(deldelu331*g13[ijk] + deldelu332*g23[ijk] + 
         deldelu333*g33[ijk])) + deldelg1333*u1[ijk] + deldelg2333*u2[ijk] + 
   deldelg3333*u3[ijk]
;



/* conditional */
if (minstrain) {

B1111
=
pow2(ginv11)
;

B1112
=
ginv11*ginv12
;

B1113
=
ginv11*ginv13
;

B1122
=
pow2(ginv12)
;

B1123
=
ginv12*ginv13
;

B1133
=
pow2(ginv13)
;

B1212
=
(ginv11*ginv22 + pow2(ginv12))/2.
;

B1213
=
(ginv12*ginv13 + ginv11*ginv23)/2.
;

B1222
=
ginv12*ginv22
;

B1223
=
(ginv13*ginv22 + ginv12*ginv23)/2.
;

B1233
=
ginv13*ginv23
;

B1313
=
(ginv11*ginv33 + pow2(ginv13))/2.
;

B1322
=
ginv12*ginv23
;

B1323
=
(ginv13*ginv23 + ginv12*ginv33)/2.
;

B1333
=
ginv13*ginv33
;

B2222
=
pow2(ginv22)
;

B2223
=
ginv22*ginv23
;

B2233
=
pow2(ginv23)
;

B2323
=
(ginv22*ginv33 + pow2(ginv23))/2.
;

B2333
=
ginv23*ginv33
;

B3333
=
pow2(ginv33)
;

}
/* if (minstrain) */




/* conditional */
if (mindistortion || confharmonic) {

B1111
=
(2*pow2(ginv11))/3.
;

B1112
=
(2*ginv11*ginv12)/3.
;

B1113
=
(2*ginv11*ginv13)/3.
;

B1122
=
-(ginv11*ginv22)/3. + pow2(ginv12)
;

B1123
=
ginv12*ginv13 - (ginv11*ginv23)/3.
;

B1133
=
-(ginv11*ginv33)/3. + pow2(ginv13)
;

B1212
=
(ginv11*ginv22)/2. + pow2(ginv12)/6.
;

B1213
=
(ginv12*ginv13)/6. + (ginv11*ginv23)/2.
;

B1222
=
(2*ginv12*ginv22)/3.
;

B1223
=
(ginv13*ginv22)/2. + (ginv12*ginv23)/6.
;

B1233
=
ginv13*ginv23 - (ginv12*ginv33)/3.
;

B1313
=
(ginv11*ginv33)/2. + pow2(ginv13)/6.
;

B1322
=
-(ginv13*ginv22)/3. + ginv12*ginv23
;

B1323
=
(ginv13*ginv23)/6. + (ginv12*ginv33)/2.
;

B1333
=
(2*ginv13*ginv33)/3.
;

B2222
=
(2*pow2(ginv22))/3.
;

B2223
=
(2*ginv22*ginv23)/3.
;

B2233
=
-(ginv22*ginv33)/3. + pow2(ginv23)
;

B2323
=
(ginv22*ginv33)/2. + pow2(ginv23)/6.
;

B2333
=
(2*ginv23*ginv33)/3.
;

B3333
=
(2*pow2(ginv33))/3.
;

}
/* if (mindistortion || confharmonic) */




/* conditional */
if (bctgauge) {

B1111
=
pow2(ginv11)
;

B1112
=
ginv11*ginv12
;

B1113
=
ginv11*ginv13
;

B1122
=
pow2(ginv12)
;

B1123
=
ginv12*ginv13
;

B1133
=
pow2(ginv13)
;

B1212
=
(ginv11*ginv22 + pow2(ginv12))/2.
;

B1213
=
(ginv12*ginv13 + ginv11*ginv23)/2.
;

B1222
=
ginv12*ginv22
;

B1223
=
(ginv13*ginv22 + ginv12*ginv23)/2.
;

B1233
=
ginv13*ginv23
;

B1313
=
(ginv11*ginv33 + pow2(ginv13))/2.
;

B1322
=
ginv12*ginv23
;

B1323
=
(ginv13*ginv23 + ginv12*ginv33)/2.
;

B1333
=
ginv13*ginv33
;

B2222
=
pow2(ginv22)
;

B2223
=
ginv22*ginv23
;

B2233
=
pow2(ginv23)
;

B2323
=
(ginv22*ginv33 + pow2(ginv23))/2.
;

B2333
=
ginv23*ginv33
;

B3333
=
pow2(ginv33)
;

Kup11
=
B1111*K11[ijk] + B1122*K22[ijk] + 
   2*(B1112*K12[ijk] + B1113*K13[ijk] + B1123*K23[ijk]) + B1133*K33[ijk]
;

Kup12
=
B1112*K11[ijk] + B1222*K22[ijk] + 
   2*(B1212*K12[ijk] + B1213*K13[ijk] + B1223*K23[ijk]) + B1233*K33[ijk]
;

Kup13
=
B1113*K11[ijk] + B1322*K22[ijk] + 
   2*(B1213*K12[ijk] + B1313*K13[ijk] + B1323*K23[ijk]) + B1333*K33[ijk]
;

Kup22
=
B1122*K11[ijk] + B2222*K22[ijk] + 
   2*(B1222*K12[ijk] + B1322*K13[ijk] + B2223*K23[ijk]) + B2233*K33[ijk]
;

Kup23
=
B1123*K11[ijk] + B2223*K22[ijk] + 
   2*(B1223*K12[ijk] + B1323*K13[ijk] + B2323*K23[ijk]) + B2333*K33[ijk]
;

Kup33
=
B1133*K11[ijk] + B2233*K22[ijk] + 
   2*(B1233*K12[ijk] + B1333*K13[ijk] + B2333*K23[ijk]) + B3333*K33[ijk]
;

KK
=
Kup11*K11[ijk] + Kup22*K22[ijk] + 
   2*(Kup12*K12[ijk] + Kup13*K13[ijk] + Kup23*K23[ijk]) + Kup33*K33[ijk]
;

cdK111
=
oo2dx*(-K11[-di + ijk] + K11[di + ijk]) - 
   2*(gamma111*K11[ijk] + gamma211*K12[ijk] + gamma311*K13[ijk])
;

cdK112
=
-(gamma112*K11[ijk]) - (gamma111 + gamma212)*K12[ijk] + 
   oo2dx*(-K12[-di + ijk] + K12[di + ijk]) - gamma312*K13[ijk] - 
   gamma211*K22[ijk] - gamma311*K23[ijk]
;

cdK113
=
-(gamma113*K11[ijk]) - gamma213*K12[ijk] - (gamma111 + gamma313)*K13[ijk] + 
   oo2dx*(-K13[-di + ijk] + K13[di + ijk]) - gamma211*K23[ijk] - 
   gamma311*K33[ijk]
;

cdK122
=
oo2dx*(-K22[-di + ijk] + K22[di + ijk]) - 
   2*(gamma112*K12[ijk] + gamma212*K22[ijk] + gamma312*K23[ijk])
;

cdK123
=
-(gamma113*K12[ijk]) - gamma112*K13[ijk] - gamma213*K22[ijk] - 
   (gamma212 + gamma313)*K23[ijk] + oo2dx*(-K23[-di + ijk] + K23[di + ijk]) - 
   gamma312*K33[ijk]
;

cdK133
=
-2*(gamma113*K13[ijk] + gamma213*K23[ijk] + gamma313*K33[ijk]) + 
   oo2dx*(-K33[-di + ijk] + K33[di + ijk])
;

cdK211
=
oo2dy*(-K11[-dj + ijk] + K11[dj + ijk]) - 
   2*(gamma112*K11[ijk] + gamma212*K12[ijk] + gamma312*K13[ijk])
;

cdK212
=
-(gamma122*K11[ijk]) - (gamma112 + gamma222)*K12[ijk] + 
   oo2dy*(-K12[-dj + ijk] + K12[dj + ijk]) - gamma322*K13[ijk] - 
   gamma212*K22[ijk] - gamma312*K23[ijk]
;

cdK213
=
-(gamma123*K11[ijk]) - gamma223*K12[ijk] - (gamma112 + gamma323)*K13[ijk] + 
   oo2dy*(-K13[-dj + ijk] + K13[dj + ijk]) - gamma212*K23[ijk] - 
   gamma312*K33[ijk]
;

cdK222
=
oo2dy*(-K22[-dj + ijk] + K22[dj + ijk]) - 
   2*(gamma122*K12[ijk] + gamma222*K22[ijk] + gamma322*K23[ijk])
;

cdK223
=
-(gamma123*K12[ijk]) - gamma122*K13[ijk] - gamma223*K22[ijk] - 
   (gamma222 + gamma323)*K23[ijk] + oo2dy*(-K23[-dj + ijk] + K23[dj + ijk]) - 
   gamma322*K33[ijk]
;

cdK233
=
-2*(gamma123*K13[ijk] + gamma223*K23[ijk] + gamma323*K33[ijk]) + 
   oo2dy*(-K33[-dj + ijk] + K33[dj + ijk])
;

cdK311
=
oo2dz*(-K11[-dk + ijk] + K11[dk + ijk]) - 
   2*(gamma113*K11[ijk] + gamma213*K12[ijk] + gamma313*K13[ijk])
;

cdK312
=
-(gamma123*K11[ijk]) - (gamma113 + gamma223)*K12[ijk] + 
   oo2dz*(-K12[-dk + ijk] + K12[dk + ijk]) - gamma323*K13[ijk] - 
   gamma213*K22[ijk] - gamma313*K23[ijk]
;

cdK313
=
-(gamma133*K11[ijk]) - gamma233*K12[ijk] - (gamma113 + gamma333)*K13[ijk] + 
   oo2dz*(-K13[-dk + ijk] + K13[dk + ijk]) - gamma213*K23[ijk] - 
   gamma313*K33[ijk]
;

cdK322
=
oo2dz*(-K22[-dk + ijk] + K22[dk + ijk]) - 
   2*(gamma123*K12[ijk] + gamma223*K22[ijk] + gamma323*K23[ijk])
;

cdK323
=
-(gamma133*K12[ijk]) - gamma123*K13[ijk] - gamma233*K22[ijk] - 
   (gamma223 + gamma333)*K23[ijk] + oo2dz*(-K23[-dk + ijk] + K23[dk + ijk]) - 
   gamma323*K33[ijk]
;

cdK333
=
-2*(gamma133*K13[ijk] + gamma233*K23[ijk] + gamma333*K33[ijk]) + 
   oo2dz*(-K33[-dk + ijk] + K33[dk + ijk])
;

cdKup111
=
B1111*cdK111 + B1122*cdK122 + 2*
    (B1112*cdK112 + B1113*cdK113 + B1123*cdK123) + B1133*cdK133
;

cdKup112
=
B1112*cdK111 + B1222*cdK122 + 2*
    (B1212*cdK112 + B1213*cdK113 + B1223*cdK123) + B1233*cdK133
;

cdKup113
=
B1113*cdK111 + B1322*cdK122 + 2*
    (B1213*cdK112 + B1313*cdK113 + B1323*cdK123) + B1333*cdK133
;

cdKup122
=
B1122*cdK111 + B2222*cdK122 + 2*
    (B1222*cdK112 + B1322*cdK113 + B2223*cdK123) + B2233*cdK133
;

cdKup123
=
B1123*cdK111 + B2223*cdK122 + 2*
    (B1223*cdK112 + B1323*cdK113 + B2323*cdK123) + B2333*cdK133
;

cdKup133
=
B1133*cdK111 + B2233*cdK122 + 2*
    (B1233*cdK112 + B1333*cdK113 + B2333*cdK123) + B3333*cdK133
;

cdKup211
=
B1111*cdK211 + B1122*cdK222 + 2*
    (B1112*cdK212 + B1113*cdK213 + B1123*cdK223) + B1133*cdK233
;

cdKup212
=
B1112*cdK211 + B1222*cdK222 + 2*
    (B1212*cdK212 + B1213*cdK213 + B1223*cdK223) + B1233*cdK233
;

cdKup213
=
B1113*cdK211 + B1322*cdK222 + 2*
    (B1213*cdK212 + B1313*cdK213 + B1323*cdK223) + B1333*cdK233
;

cdKup222
=
B1122*cdK211 + B2222*cdK222 + 2*
    (B1222*cdK212 + B1322*cdK213 + B2223*cdK223) + B2233*cdK233
;

cdKup223
=
B1123*cdK211 + B2223*cdK222 + 2*
    (B1223*cdK212 + B1323*cdK213 + B2323*cdK223) + B2333*cdK233
;

cdKup233
=
B1133*cdK211 + B2233*cdK222 + 2*
    (B1233*cdK212 + B1333*cdK213 + B2333*cdK223) + B3333*cdK233
;

cdKup311
=
B1111*cdK311 + B1122*cdK322 + 2*
    (B1112*cdK312 + B1113*cdK313 + B1123*cdK323) + B1133*cdK333
;

cdKup312
=
B1112*cdK311 + B1222*cdK322 + 2*
    (B1212*cdK312 + B1213*cdK313 + B1223*cdK323) + B1233*cdK333
;

cdKup313
=
B1113*cdK311 + B1322*cdK322 + 2*
    (B1213*cdK312 + B1313*cdK313 + B1323*cdK323) + B1333*cdK333
;

cdKup322
=
B1122*cdK311 + B2222*cdK322 + 2*
    (B1222*cdK312 + B1322*cdK313 + B2223*cdK323) + B2233*cdK333
;

cdKup323
=
B1123*cdK311 + B2223*cdK322 + 2*
    (B1223*cdK312 + B1323*cdK313 + B2323*cdK323) + B2333*cdK333
;

cdKup333
=
B1133*cdK311 + B2233*cdK322 + 2*
    (B1233*cdK312 + B1333*cdK313 + B2333*cdK323) + B3333*cdK333
;

cdKK1
=
4*(cdK112*Kup12 + cdK113*Kup13 + cdK123*Kup23) + 
   2*(cdK111*Kup11 + cdK122*Kup22 + cdK133*Kup33)
;

cdKK2
=
4*(cdK212*Kup12 + cdK213*Kup13 + cdK223*Kup23) + 
   2*(cdK211*Kup11 + cdK222*Kup22 + cdK233*Kup33)
;

cdKK3
=
4*(cdK312*Kup12 + cdK313*Kup13 + cdK323*Kup23) + 
   2*(cdK311*Kup11 + cdK322*Kup22 + cdK333*Kup33)
;

B1111
=
B1111 - pow2(Kup11)/KK
;

B1112
=
B1112 - (Kup11*Kup12)/KK
;

B1113
=
B1113 - (Kup11*Kup13)/KK
;

B1122
=
B1122 - (Kup11*Kup22)/KK
;

B1123
=
B1123 - (Kup11*Kup23)/KK
;

B1133
=
B1133 - (Kup11*Kup33)/KK
;

B1212
=
B1212 - pow2(Kup12)/KK
;

B1213
=
B1213 - (Kup12*Kup13)/KK
;

B1222
=
B1222 - (Kup12*Kup22)/KK
;

B1223
=
B1223 - (Kup12*Kup23)/KK
;

B1233
=
B1233 - (Kup12*Kup33)/KK
;

B1313
=
B1313 - pow2(Kup13)/KK
;

B1322
=
B1322 - (Kup13*Kup22)/KK
;

B1323
=
B1323 - (Kup13*Kup23)/KK
;

B1333
=
B1333 - (Kup13*Kup33)/KK
;

B2222
=
B2222 - pow2(Kup22)/KK
;

B2223
=
B2223 - (Kup22*Kup23)/KK
;

B2233
=
B2233 - (Kup22*Kup33)/KK
;

B2323
=
B2323 - pow2(Kup23)/KK
;

B2333
=
B2333 - (Kup23*Kup33)/KK
;

B3333
=
B3333 - pow2(Kup33)/KK
;

Kup11
=
Kup11/KK
;

Kup12
=
Kup12/KK
;

Kup13
=
Kup13/KK
;

Kup22
=
Kup22/KK
;

Kup23
=
Kup23/KK
;

Kup33
=
Kup33/KK
;

BB111
=
cdKup211*Kup12 + Kup11*(2*cdKup111 + cdKup212 + cdKup313 - cdKK2*Kup12) + 
   (cdKup311 - cdKK3*Kup11)*Kup13 - cdKK1*pow2(Kup11)
;

BB112
=
Kup11*(cdKup112 - cdKK1*Kup12) + cdKup312*Kup13 + 
   Kup12*(cdKup111 + 2*cdKup212 + cdKup313 - cdKK3*Kup13) - cdKK2*pow2(Kup12)
;

BB113
=
(cdKup111 + cdKup212 + 2*cdKup313)*Kup13 + Kup11*(cdKup113 - cdKK1*Kup13) + 
   Kup12*(cdKup213 - cdKK2*Kup13) - cdKK3*pow2(Kup13)
;

BB122
=
(cdKup111 + cdKup212 + cdKup313)*Kup22 + Kup11*(cdKup122 - cdKK1*Kup22) + 
   Kup12*(cdKup222 - cdKK2*Kup22) + Kup13*(cdKup322 - cdKK3*Kup22)
;

BB123
=
(cdKup111 + cdKup212 + cdKup313)*Kup23 + Kup11*(cdKup123 - cdKK1*Kup23) + 
   Kup12*(cdKup223 - cdKK2*Kup23) + Kup13*(cdKup323 - cdKK3*Kup23)
;

BB133
=
(cdKup111 + cdKup212 + cdKup313)*Kup33 + Kup11*(cdKup133 - cdKK1*Kup33) + 
   Kup12*(cdKup233 - cdKK2*Kup33) + Kup13*(cdKup333 - cdKK3*Kup33)
;

BB211
=
cdKup111*Kup12 + cdKup211*Kup22 + cdKup311*Kup23 + 
   Kup11*(cdKup112 + cdKup222 + cdKup323 - cdKK1*Kup12 - cdKK2*Kup22 - 
      cdKK3*Kup23)
;

BB212
=
cdKup212*Kup22 + Kup12*(2*cdKup112 + cdKup222 + cdKup323 - cdKK2*Kup22) + 
   (cdKup312 - cdKK3*Kup12)*Kup23 - cdKK1*pow2(Kup12)
;

BB213
=
Kup12*(cdKup113 - cdKK1*Kup13) + cdKup213*Kup22 + cdKup313*Kup23 + 
   Kup13*(cdKup112 + cdKup222 + cdKup323 - cdKK2*Kup22 - cdKK3*Kup23)
;

BB222
=
Kup12*(cdKup122 - cdKK1*Kup22) + cdKup322*Kup23 + 
   Kup22*(cdKup112 + 2*cdKup222 + cdKup323 - cdKK3*Kup23) - cdKK2*pow2(Kup22)
;

BB223
=
(cdKup112 + cdKup222 + 2*cdKup323)*Kup23 + Kup12*(cdKup123 - cdKK1*Kup23) + 
   Kup22*(cdKup223 - cdKK2*Kup23) - cdKK3*pow2(Kup23)
;

BB233
=
(cdKup112 + cdKup222 + cdKup323)*Kup33 + Kup12*(cdKup133 - cdKK1*Kup33) + 
   Kup22*(cdKup233 - cdKK2*Kup33) + Kup23*(cdKup333 - cdKK3*Kup33)
;

BB311
=
cdKup111*Kup13 + cdKup211*Kup23 + cdKup311*Kup33 + 
   Kup11*(cdKup113 + cdKup223 + cdKup333 - cdKK1*Kup13 - cdKK2*Kup23 - 
      cdKK3*Kup33)
;

BB312
=
cdKup112*Kup13 + cdKup212*Kup23 + cdKup312*Kup33 + 
   Kup12*(cdKup113 + cdKup223 + cdKup333 - cdKK1*Kup13 - cdKK2*Kup23 - 
      cdKK3*Kup33)
;

BB313
=
cdKup213*Kup23 + Kup13*(2*cdKup113 + cdKup223 + cdKup333 - cdKK2*Kup23) + 
   (cdKup313 - cdKK3*Kup13)*Kup33 - cdKK1*pow2(Kup13)
;

BB322
=
Kup13*(cdKup122 - cdKK1*Kup22) + cdKup222*Kup23 + cdKup322*Kup33 + 
   Kup22*(cdKup113 + cdKup223 + cdKup333 - cdKK2*Kup23 - cdKK3*Kup33)
;

BB323
=
Kup13*(cdKup123 - cdKK1*Kup23) + cdKup323*Kup33 + 
   Kup23*(cdKup113 + 2*cdKup223 + cdKup333 - cdKK3*Kup33) - cdKK2*pow2(Kup23)
;

BB333
=
(cdKup113 + cdKup223 + 2*cdKup333)*Kup33 + Kup13*(cdKup133 - cdKK1*Kup33) + 
   Kup23*(cdKup233 - cdKK2*Kup33) - cdKK3*pow2(Kup33)
;

n1[ijk]
=
-(BB111*Lieug11) - BB122*Lieug22 - 
   2*(BB112*Lieug12 + BB113*Lieug13 + BB123*Lieug23) - BB133*Lieug33
;

n2[ijk]
=
-(BB211*Lieug11) - BB222*Lieug22 - 
   2*(BB212*Lieug12 + BB213*Lieug13 + BB223*Lieug23) - BB233*Lieug33
;

n3[ijk]
=
-(BB311*Lieug11) - BB322*Lieug22 - 
   2*(BB312*Lieug12 + BB313*Lieug13 + BB323*Lieug23) - BB333*Lieug33
;

}
/* if (bctgauge) */


*Lu1
=
B1111*DelLieug111 + B1122*DelLieug122 + B1133*DelLieug133 + 
   B1112*DelLieug211 + B1222*DelLieug222 + B1233*DelLieug233 + 
   B1113*DelLieug311 + B1322*DelLieug322 + 
   2*(B1112*DelLieug112 + B1113*DelLieug113 + B1123*DelLieug123 + 
      B1212*DelLieug212 + B1223*DelLieug223 + 
      B1213*(DelLieug213 + DelLieug312) + B1313*DelLieug313 + 
      B1323*DelLieug323) + B1333*DelLieug333 + n1[ijk]
;

*Lu2
=
B1112*DelLieug111 + B1233*DelLieug133 + B1122*DelLieug211 + 
   B1222*(DelLieug122 + 2*DelLieug212) + B2222*DelLieug222 + 
   B2233*DelLieug233 + B1123*DelLieug311 + B2223*DelLieug322 + 
   2*(B1212*DelLieug112 + B1213*DelLieug113 + B1322*DelLieug213 + 
      B2223*DelLieug223 + B1223*(DelLieug123 + DelLieug312) + 
      B1323*DelLieug313 + B2323*DelLieug323) + B2333*DelLieug333 + n2[ijk]
;

*Lu3
=
B1113*DelLieug111 + B1322*DelLieug122 + B1123*DelLieug211 + 
   B2223*DelLieug222 + B1133*DelLieug311 + 
   2*(B1213*DelLieug112 + B1313*DelLieug113 + B1223*DelLieug212 + 
      B1323*(DelLieug123 + DelLieug213) + B2323*DelLieug223 + 
      B1233*DelLieug312) + B1333*(DelLieug133 + 2*DelLieug313) + 
   B2233*DelLieug322 + B2333*(DelLieug233 + 2*DelLieug323) + 
   B3333*DelLieug333 + n3[ijk]
;

dLdubct1
=
0
;

dLdubct2
=
0
;

dLdubct3
=
0
;



/* conditional */
if (bctgauge) {

dLdubct1
=
-(BB111*delg111) - BB122*delg122 - 
   2*(BB112*delg112 + BB113*delg113 + BB123*delg123) - BB133*delg133
;

dLdubct2
=
-(BB211*delg211) - BB222*delg222 - 
   2*(BB212*delg212 + BB213*delg213 + BB223*delg223) - BB233*delg233
;

dLdubct3
=
-(BB311*delg311) - BB322*delg322 - 
   2*(BB312*delg312 + BB313*delg313 + BB323*delg323) - BB333*delg333
;

}
/* if (bctgauge) */




/* conditional */
if (dLdu1) {

*dLdu1
=
dLdubct1 + B1322*(deldelg1322 - 
      2*(delg112*gamma123 + delg122*gamma223 + delg123*gamma323)) + 
   B1233*(deldelg1233 - 2*(delg113*gamma123 + delg123*gamma223 + 
         delg133*gamma323)) + B1111*
    (deldelg1111 - 2*(delg111*gamma111 + delg112*gamma211 + 
         delg113*gamma311) + f*oodx2*(-4 + 2*totch)*g11[ijk]) + 
   B1222*(deldelg1222 - 2*(delg112*gamma122 + delg122*gamma222 + 
         delg123*gamma322) - 4*f*oody2*g12[ijk]) + 
   B1112*(deldelg1211 + 2*(deldelg1112 - delg122*gamma211 - 
         delg112*(gamma111 + 2*gamma212) - delg123*gamma311 + 
         2*f*oodx2*totch*g12[ijk] - 
         2*(delg111*gamma112 + delg113*gamma312 + f*oodx2*g12[ijk]))) + 
   B1333*(deldelg1333 - 2*(delg113*gamma133 + delg123*gamma233 + 
         delg133*gamma333) - 4*f*oodz2*g13[ijk]) + 
   B1113*(deldelg1311 + 2*(deldelg1113 - delg123*gamma211 - 
         delg133*gamma311 - delg113*(gamma111 + 2*gamma313) + 
         2*f*oodx2*totch*g13[ijk] - 
         2*(delg111*gamma113 + delg112*gamma213 + f*oodx2*g13[ijk]))) + 
   B1122*(deldelg1122 + 2*(-(delg112*gamma112) - delg122*gamma212 - 
         delg123*gamma312 + f*oodx2*totch*g22[ijk])) + 
   2*(B1213*(deldelg1213 + deldelg1312 - 2*delg111*gamma123 - 
         delg122*gamma213 - delg112*(gamma113 + 2*gamma223) - 
         delg133*gamma312 - delg123*(gamma212 + gamma313) - 
         delg113*(gamma112 + 2*gamma323)) + 
      B1212*(deldelg1212 - delg111*gamma122 - delg122*gamma212 - 
         delg112*(gamma112 + gamma222) - delg123*gamma312 - 
         delg113*gamma322 - 2*f*oody2*g11[ijk]) + 
      B1313*(deldelg1313 - delg111*gamma133 - delg123*gamma213 - 
         delg112*gamma233 - delg133*gamma313 - 
         delg113*(gamma113 + gamma333) - 2*f*oodz2*g11[ijk]) + 
      B1323*(deldelg1323 - delg113*gamma123 - delg112*gamma133 - 
         delg122*gamma233 - delg133*gamma323 - 
         delg123*(gamma223 + gamma333) - 2*f*oodz2*g12[ijk]) + 
      B1223*(deldelg1223 - delg113*gamma122 - delg112*gamma123 - 
         delg122*gamma223 - delg133*gamma322 - 
         delg123*(gamma222 + gamma323) - 2*f*oody2*g13[ijk]) + 
      B1123*(deldelg1123 - delg113*gamma112 - delg112*gamma113 - 
         delg122*gamma213 - delg133*gamma312 - 
         delg123*(gamma212 + gamma313) + 2*f*oodx2*totch*g23[ijk])) + 
   B1133*(deldelg1133 + 2*(-(delg113*gamma113) - delg123*gamma213 - 
         delg133*gamma313 + f*oodx2*totch*g33[ijk]))
;

*dLdu2
=
dLdubct2 + B1123*(deldelg2311 - 
      2*(delg211*gamma113 + delg212*gamma213 + delg213*gamma313)) + 
   B1233*(deldelg1233 - 2*(delg213*gamma113 + delg223*gamma213 + 
         delg233*gamma313)) + B1122*
    (deldelg2211 + 2*(-(delg211*gamma112) - delg212*gamma212 - 
         delg213*gamma312 + f*oody2*totch*g11[ijk])) + 
   B1112*(deldelg1211 - 2*(delg211*gamma111 + delg212*gamma211 + 
         delg213*gamma311) - 4*f*oodx2*g12[ijk]) + 
   B1222*(deldelg1222 + 2*(deldelg2212 - delg211*gamma122 - 
         delg212*gamma222 - delg213*gamma322 + 2*f*oody2*totch*g12[ijk] - 
         2*(delg212*gamma112 + delg222*gamma212 + delg223*gamma312 + 
            f*oody2*g12[ijk]))) + 
   B2222*(deldelg2222 - 2*(delg212*gamma122 + delg222*gamma222 + 
         delg223*gamma322) + f*oody2*(-4 + 2*totch)*g22[ijk]) + 
   B2333*(deldelg2333 - 2*(delg213*gamma133 + delg223*gamma233 + 
         delg233*gamma333) - 4*f*oodz2*g23[ijk]) + 
   2*(B1223*(deldelg1223 + deldelg2312 - delg211*gamma123 - 
         2*(delg212*gamma113 + delg222*gamma213) - delg212*gamma223 - 
         delg233*gamma312 - delg223*(gamma212 + 2*gamma313) - 
         delg213*(gamma112 + gamma323)) + 
      B1323*(deldelg2313 - delg211*gamma133 - delg223*gamma213 - 
         delg212*gamma233 - delg233*gamma313 - 
         delg213*(gamma113 + gamma333) - 2*f*oodz2*g12[ijk]) + 
      B1322*(deldelg2213 - delg211*gamma123 - delg223*gamma212 - 
         delg212*gamma223 - delg233*gamma312 - 
         delg213*(gamma112 + gamma323) + 2*f*oody2*totch*g13[ijk]) + 
      B1212*(deldelg1212 - delg211*gamma112 - delg222*gamma211 - 
         delg212*(gamma111 + gamma212) - delg223*gamma311 - 
         delg213*gamma312 - 2*f*oodx2*g22[ijk]) + 
      B2323*(deldelg2323 - delg213*gamma123 - delg212*gamma133 - 
         delg222*gamma233 - delg233*gamma323 - 
         delg223*(gamma223 + gamma333) - 2*f*oodz2*g22[ijk]) + 
      B1213*(deldelg1213 - delg211*gamma113 - delg223*gamma211 - 
         delg212*gamma213 - delg233*gamma311 - 
         delg213*(gamma111 + gamma313) - 2*f*oodx2*g23[ijk])) + 
   B2223*(deldelg2322 + 2*(deldelg2223 - delg213*gamma122 - 
         delg233*gamma322 - delg223*(gamma222 + 2*gamma323) + 
         2*f*oody2*totch*g23[ijk] - 
         2*(delg212*gamma123 + delg222*gamma223 + f*oody2*g23[ijk]))) + 
   B2233*(deldelg2233 + 2*(-(delg213*gamma123) - delg223*gamma223 - 
         delg233*gamma323 + f*oody2*totch*g33[ijk]))
;

*dLdu3
=
dLdubct3 + B1123*(deldelg2311 - 
      2*(delg311*gamma112 + delg312*gamma212 + delg313*gamma312)) + 
   B1322*(deldelg1322 - 2*(delg312*gamma112 + delg322*gamma212 + 
         delg323*gamma312)) + B1133*
    (deldelg3311 + 2*(-(delg311*gamma113) - delg312*gamma213 - 
         delg313*gamma313 + f*oodz2*totch*g11[ijk])) + 
   B1113*(deldelg1311 - 2*(delg311*gamma111 + delg312*gamma211 + 
         delg313*gamma311) - 4*f*oodx2*g13[ijk]) + 
   B1333*(deldelg1333 + 2*(deldelg3313 - delg311*gamma133 - 
         delg312*gamma233 - delg313*gamma333 + 2*f*oodz2*totch*g13[ijk] - 
         2*(delg313*gamma113 + delg323*gamma213 + delg333*gamma313 + 
            f*oodz2*g13[ijk]))) + 
   B2233*(deldelg3322 + 2*(-(delg312*gamma123) - delg322*gamma223 - 
         delg323*gamma323 + f*oodz2*totch*g22[ijk])) + 
   B2223*(deldelg2322 - 2*(delg312*gamma122 + delg322*gamma222 + 
         delg323*gamma322) - 4*f*oody2*g23[ijk]) + 
   B2333*(deldelg2333 + 2*(deldelg3323 - delg312*gamma133 - 
         delg322*gamma233 - delg323*gamma333 + 2*f*oodz2*totch*g23[ijk] - 
         2*(delg313*gamma123 + delg323*gamma223 + delg333*gamma323 + 
            f*oodz2*g23[ijk]))) + 
   B3333*(deldelg3333 - 2*(delg313*gamma133 + delg323*gamma233 + 
         delg333*gamma333) + f*oodz2*(-4 + 2*totch)*g33[ijk]) + 
   2*(B1323*(deldelg1323 + deldelg2313 - delg311*gamma123 - 
         delg322*gamma213 - delg312*(gamma113 + gamma223) - 
         2*(delg313*gamma112 + delg323*gamma212 + delg333*gamma312) - 
         delg323*gamma313 - delg313*gamma323) + 
      B1233*(deldelg3312 - delg311*gamma123 - delg322*gamma213 - 
         delg312*(gamma113 + gamma223) - delg323*gamma313 - 
         delg313*gamma323 + 2*f*oodz2*totch*g12[ijk]) + 
      B1223*(deldelg2312 - delg311*gamma122 - delg322*gamma212 - 
         delg312*(gamma112 + gamma222) - delg323*gamma312 - 
         delg313*gamma322 - 2*f*oody2*g13[ijk]) + 
      B1213*(deldelg1312 - delg311*gamma112 - delg322*gamma211 - 
         delg312*(gamma111 + gamma212) - delg323*gamma311 - 
         delg313*gamma312 - 2*f*oodx2*g23[ijk]) + 
      B1313*(deldelg1313 - delg311*gamma113 - delg323*gamma211 - 
         delg312*gamma213 - delg333*gamma311 - 
         delg313*(gamma111 + gamma313) - 2*f*oodx2*g33[ijk]) + 
      B2323*(deldelg2323 - delg313*gamma122 - delg312*gamma123 - 
         delg322*gamma223 - delg333*gamma322 - 
         delg323*(gamma222 + gamma323) - 2*f*oody2*g33[ijk]))
;

}
/* if (dLdu1) */


}  /* function */

/* bamveclaplace.c */
/* n* = 1936,  n/ = 139,  n+ = 2283, nvars = 29 */
