/* minshift.c */
/* Bernd Bruegmann, 3/98 */

#include "Symmetry.h"
#include "bamveclap.h"

void swapADMwithBSSN(const cGH *cctkGH);
void readparams(CCTK_ARGUMENTS);

 


/* cactus 4 util */
int nocd, vlevery;
int minstrain = 0, mindistortion = 1, bctgauge = 0;
int confharmonic = 0; 
int bssngauge = 0, bssngauge4 = 0, bssnmaximal = 0;
int bssndivAfromD = 0, bssnuseGamma = 1;

void readparams(CCTK_ARGUMENTS) 
{
  DECLARE_CCTK_PARAMETERS
  DECLARE_CCTK_ARGUMENTS

  mindistortion = 0;
  if (CCTK_Equals(shift_evolution_method,"mindist")) mindistortion = 1;
  if (CCTK_Equals(shift_evolution_method,"minstrain")) minstrain = 1;
  if (CCTK_Equals(shift_evolution_method,"bctgauge")) bctgauge = 1;
  if (CCTK_Equals(shift_evolution_method,"confharmonic")) confharmonic= 1;

  if (CCTK_Equals(shift_evolution_method,"bssngauge")) bssngauge = 1;
  if (CCTK_Equals(shift_evolution_method,"bssngauge1")) bssnmaximal = 1;
  if (CCTK_Equals(shift_evolution_method,"bssngauge3")) bssngauge = 1;
  if (CCTK_Equals(shift_evolution_method,"bssngauge4")) bssngauge4 = 1;
  /* if (CCTK_Equals(slicing, "bssnmaximal")) bssnmaximal = 1; */
  if (bssngauge && bssnmaximal) bssngauge4 = 1;
  if (bssnmaximal || bssngauge4) bssngauge = 1;

  if (confharmonic && !CCTK_Equals(evolution_method, "adm_bssn"))
    CCTK_WARN(0, "shift confharmonic requires evolution_method adm_bssn");
  if ((bssngauge || bssngauge4) && !CCTK_Equals(evolution_method, "adm_bssn"))
    CCTK_WARN(0, "shift bssngauge requires evolution_method adm_bssn");
  if (bssnmaximal && !bssngauge)
    CCTK_WARN(0, "lapse bssnmaximal requires for now also bssngauge");
  if (bssngauge) {
    bssndivAfromD = vl_divAfromD;
    bssnuseGamma = vl_useGamma;
  }
  nocd = (*conformal_state != 3);
  vlevery = vl_every;
}




/* check for nans */
void checknan(cGH *cctkGH, int i) 
{
  double *u = GFDataByIndex(cctkGH, i);
  int npoints = cctkGH->cctk_lsh[0]*cctkGH->cctk_lsh[1]*cctkGH->cctk_lsh[2];
  int ijk;

  for (ijk = 0; ijk < npoints; ijk++) {
    if (isnan(u[ijk])) {
      printf("%s = nan at (%d,%d,%d)=%d\n", CCTK_VarName(i), 0, 0, 0, ijk);
      break;
    }
  }
}





/* solve for the shift vector */
void BAM_VecLap_Shift(CCTK_ARGUMENTS) 
{
  int i0 = CCTK_VarIndex("bam_veclap::vl_n1");
  int i1 = CCTK_VarIndex("bam_veclap::vl_n3");
  int j0 = CCTK_VarIndex("admbase::betax");
  int k0 = CCTK_VarIndex("bam_veclap::vlKK");
  int k1 = CCTK_VarIndex("bam_veclap::vlLiebgzz");
  int GFI_GXX = CCTK_VarIndex("admbase::gxx");
  int GFI_VLKK = CCTK_VarIndex("bam_veclap::vlKK");
  int l0 = CCTK_VarIndex("bam_veclap::vlgi11");
  int l1 = CCTK_VarIndex("bam_veclap::vlacon");
  int GFI_alpha = CCTK_VarIndex("admbase::alp");
  int GFI_beta = CCTK_VarIndex("admbase::betax");
  int i, k;
  int ijk;
  CCTK_REAL *a, *b, *c;
  pGH *GH = PUGH_pGH(cctkGH);
  int npoints = GH->GFExtras[2]->npoints;
  int u[100], N[3], nu;
  static int ncall = -1;

  /* return if nothing to do */
  readparams(CCTK_PASS_CTOC);
  if (!(minstrain || mindistortion || bctgauge || bssngauge || bssngauge4 
        || confharmonic))
    return;
  ++ncall;
  if (ncall % vlevery) {
    return;
  }

  /* make list of grid functions */
  for (i = i0, k = 0; k < 3;) N[k++] = i++;
  for (i = j0, k = 0; k < 3;) u[k++] = i++;
  if (bssngauge4) {
    i = 0;
    u[i++] = GFI_alpha;
    for (k = 0; k < 3; k++) u[i++] = GFI_beta + k;
    for (k = l0; k <= l1; k++) u[i++] = k;
    for (k = i0; k <= i1; k++) u[i++] = k;
    nu = i;
  }

  /* ininitalize source */
  if (1) {
    for (i = i0; i <= i1; i++) {
      a = GFDataByIndex(cctkGH, i);
      for (ijk = 0; ijk < npoints; ijk++)
	a[ijk] = 0;
    }
    if (bssngauge4) {
      a = GFDataByIndex(cctkGH, CCTK_VarIndex("admbase::alp"));
      b = GFDataByIndex(cctkGH, CCTK_VarIndex("bam_veclap::vldtalp"));
      for (ijk = 0; ijk < npoints; ijk++)
	b[ijk] = a[ijk];
    }
    if (0&&bssngauge4) {
      for (i = l0; i <= l1; i++) {
	a = GFDataByIndex(cctkGH, i);
	for (ijk = 0; ijk < npoints; ijk++)
	  a[ijk] = 0;
      }
      a = GFDataByIndex(cctkGH, GFI_alpha);
      for (ijk = 0; ijk < npoints; ijk++)
	a[ijk] = 0;
    }
  }

  /* conformal 3-harmonic needs the conformal/tracesplit metric/curvature */
  if (confharmonic)
    swapADMwithBSSN(cctkGH);

  /* set source */
  if (!bctgauge && !bssngauge && !bssngauge4) {
    bamminshiftsource(CCTK_PASS_CTOC);
    CCTK_SyncGroup(cctkGH, "bam_veclap::vl_sources");
  }
  if (bssngauge || bssngauge4) {
    i0 = CCTK_VarIndex("adm_bssn::adm_bs_gx");
    j0 = CCTK_VarIndex("bam_veclap::vlgg1");
    k0 = CCTK_VarIndex("bam_veclap::vlgh1");
    for (i = 0; i < 3; i++) { 
      a = GFDataByIndex(cctkGH, i0+i);
      b = GFDataByIndex(cctkGH, j0+i);
      c = GFDataByIndex(cctkGH, k0+i);
      for (ijk = 0; ijk < npoints; ijk++)
	c[ijk] = b[ijk] = a[ijk];
    }
    if (bssngauge4)
      bambssngauge4(CCTK_PASS_CTOC);
    else
      bambssngauge(CCTK_PASS_CTOC);
    CCTK_SyncGroup(cctkGH, "bam_veclap::vl_sources");
    CCTK_SyncGroup(cctkGH, "bam_veclap::vl_bssn");

    /* this is how we would do symmetry boundaries for things
       that have to be differentiated twice */
    if (0) for (i = 0; i < 3; i++) CartSymVI(cctkGH, i+k0);
  }

  /* nans? */
  if (0) {
    for (i = l0; i <= l1; i++) checknan(cctkGH, i);
    for (i = GFI_beta; i < GFI_beta+3; i++) checknan(cctkGH, i);
    checknan(cctkGH, GFI_alpha);
  }
 

  /********/
  /* work */
  /********/
  if (bssngauge4)
    BAM_Elliptic_Vec4(cctkGH, u, nu, 0.00001);
  else
    BAM_Elliptic_VecL(cctkGH, u, N, 0.00001);


  /* undo conformal/tracesplit if necessary */
  if (confharmonic)
    swapADMwithBSSN(cctkGH);

  /* save change in lapse */
  if (bssngauge4) {
    a = GFDataByIndex(cctkGH, CCTK_VarIndex("admbase::alp"));
    b = GFDataByIndex(cctkGH, CCTK_VarIndex("bam_veclap::vldtalp"));
    for (ijk = 0; ijk < npoints; ijk++)
      b[ijk] = a[ijk] - b[ijk];
  }

  /* bct gauge sets the lapse here */
  if (bctgauge) {
    bambctgauge(CCTK_PASS_CTOC);
    CCTK_SyncGroup(cctkGH, "bam_veclap::vl_check");
#if 0
    for (i = k1-5; i <= k1; i++) 
      PUGH_SyncGFs(GH, GFDataByIndex(cctkGH, i)); 
#endif
  }

  /* symmetries for stencil size 2 */
  if (cctkGH->cctk_nghostzones[0] > 1 || 
      cctkGH->cctk_nghostzones[1] > 1 || 
      cctkGH->cctk_nghostzones[2] > 1) {
    if (bssngauge4 || bssnmaximal)
      CartSymGN(cctkGH, "admbase::lapse");
    CartSymGN(cctkGH, "admbase::shift");
  }
}




/* swap variables for conformal 3-harmonic
   done like this because there is no conditional inherit in cactus 4
*/
void swapADMwithBSSN(const cGH *cctkGH) 
{
  SwapADM(cctkGH);
}





#if 0 
  /* conformal 3-harmonic needs the conformal metric 
     - recompute here to be independent of conformal formulation
     - could be part of a general scheme for "do it with phys/unphys metric"
     - for ADM_BS/ADMC there may be two conformal factors, so it still
       makes sense to take out the determinant here
  */

    CCTK_REAL *detgfac, f; 
    int n = GFI_GXX;

    gxx = GFDataByIndex(cctkGH, n++);
    gxy = GFDataByIndex(cctkGH, n++);
    gxz = GFDataByIndex(cctkGH, n++);
    gyy = GFDataByIndex(cctkGH, n++);
    gyz = GFDataByIndex(cctkGH, n++);
    gzz = GFDataByIndex(cctkGH, n++);

    EnableGFDataStorage(GH, GH->gridFuncs[GFI_VLKK]);
    detgfac = GFDataByIndex(cctkGH, GFI_VLKK);
    for (ijk = 0; ijk < npoints; ijk++) {
      f = detgfac[ijk] = pow(gxx[ijk]*gyy[ijk]*gzz[ijk]
			     + 2.0*gxy[ijk]*gxz[ijk]*gyz[ijk]
			     - gxx[ijk]*pow(gyz[ijk],2.0)
			     - gyy[ijk]*pow(gxz[ijk],2.0)
			     - gzz[ijk]*pow(gxy[ijk],2.0), -1.0/3.0);
      gxx[ijk] *= f; 
      gxy[ijk] *= f; 
      gxz[ijk] *= f; 
      gyy[ijk] *= f; 
      gyz[ijk] *= f; 
      gzz[ijk] *= f; 
    }

    for (ijk = 0; ijk < npoints; ijk++) {
      f = 1.0/detgfac[ijk];
      gxx[ijk] *= f; 
      gxy[ijk] *= f; 
      gxz[ijk] *= f; 
      gyy[ijk] *= f; 
      gyz[ijk] *= f; 
      gzz[ijk] *= f; 
    }
    DisableGFDataStorage(GH, GH->gridFuncs[GFI_VLKK]);
#endif
