 /*@@
   @file      Boundary.F
   @date      April 1999
   @author    Gabrielle Allen + Miguel Alcubierre
   @desc
     Apply boundary conditions.
   @enddesc
   @version $Header: /numrelcvs/AEIThorns/BSSN_MoL/src/Boundaries.F,v 1.21 2005/09/26 00:36:03 schnetter Exp $
@@*/

#include "cctk.h"
#include "util_Table.h"

#include "cctk_Parameters.h"
#include "cctk_Arguments.h"
#include "cctk_Functions.h"



      subroutine ADM_BSSN_Excision1(CCTK_ARGUMENTS)

      implicit none

      DECLARE_CCTK_ARGUMENTS
      DECLARE_CCTK_PARAMETERS
      DECLARE_CCTK_FUNCTIONS

      CCTK_INT, parameter :: ione=1
      CCTK_INT, parameter :: faces=CCTK_ALL_FACES
      integer :: ierr
      integer :: nx,ny,nz

!     Grid parameters.

      nx = cctk_lsh(1)
      ny = cctk_lsh(2)
      nz = cctk_lsh(3)

!     Call Excision routines before symmetries are applied.

      if (excise.eq.1) then

         if (CCTK_EQUALS(excisiontype,'lego')) then

!           Lego sphere or polyhedra.  For excising a figure other than
!           a cube, we call thorn LegoExcision.  Notice that this could
!           also be called for an excised cube and should work OK.

            call ExcisionFindBoundary(ierr,emask,nx,ny,nz)

            ierr = Boundary_SelectGroupForBC (cctkGH, faces, ione, -ione, "spacemask::mask", "None")

         end if

      end if
      
      end subroutine ADM_BSSN_Excision1



      subroutine ADM_BSSN_Excision2(CCTK_ARGUMENTS)

      implicit none

      DECLARE_CCTK_ARGUMENTS
      DECLARE_CCTK_PARAMETERS
      DECLARE_CCTK_FUNCTIONS

      CCTK_INT, parameter :: ione=1
      CCTK_INT, parameter :: faces=CCTK_ALL_FACES

      integer :: ierr
      integer :: nx,ny,nz

!     Grid parameters.

      nx = cctk_lsh(1)
      ny = cctk_lsh(2)
      nz = cctk_lsh(3)
!     Call Excision routines before symmetries are applied.

      if (excise.eq.1) then

         if (CCTK_EQUALS(excisiontype,'lego')) then

!           Lego sphere or polyhedra.  For excising a figure other than
!           a cube, we call thorn LegoExcision.  Notice that this could
!           also be called for an excised cube and should work OK.

            call ExcisionFindNormals(ierr,emask,
     .           ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz)
           
            ierr = Boundary_SelectGroupForBC (cctkGH, faces, ione, -ione, "adm_bssn::ADM_BSSN_dir", "None")

         end if

      end if

      end subroutine ADM_BSSN_Excision2



      subroutine ADM_BSSN_Boundaries(CCTK_ARGUMENTS)

      use ADM_BSSN_Scalars

      implicit none

      DECLARE_CCTK_ARGUMENTS
      DECLARE_CCTK_PARAMETERS
      DECLARE_CCTK_FUNCTIONS

      CCTK_INT, parameter :: ione=1
      CCTK_INT, parameter :: faces=CCTK_ALL_FACES

      integer :: i
      integer :: ierr
      integer :: nx,ny,nz
      integer, save :: scalar_zero_table = -2
      integer, save :: scalar_one_table  = -2
      integer, save :: lim_1_spd_v0_table = -2
      integer, save :: lim_0_spd_v0_table = -2
      integer, save :: lim_0_spd_iscale_table = -2
      integer, save :: lim_1_spd_iscale_table = -2
      integer, save :: lim_aux_spd_v0_table = -2

      integer, dimension(3) :: sw

      CCTK_REAL v0
      CCTK_REAL scale,iscale
      CCTK_REAL dt,dx,dy,dz
      CCTK_REAL aux

      CCTK_REAL, PARAMETER :: one  = 1.0D0
      CCTK_REAL, PARAMETER :: zero = 0.0D0

!     Define sw.

      sw = 1

!     Grid parameters.

      nx = cctk_lsh(1)
      ny = cctk_lsh(2)
      nz = cctk_lsh(3)

      dt = CCTK_DELTA_TIME
      dx = CCTK_DELTA_SPACE(1)
      dy = CCTK_DELTA_SPACE(2)
      dz = CCTK_DELTA_SPACE(3)

!     FishEye scale.  When using FishEye, the physical metric far away
!     is not diag(1,1,1), but rather diag(s**2,s**2,s**2) with s the
!     "deresolution" parameter, which is contained in the parameter
!     "fisheye_scale".  Since in BSSN we are use a conformal metric,
!     it turns out that only the asymptotic value of the conformal
!     factor ADM_BS_phi is affected.  However, the speed of light is
!     also affected so the radiative boundaries need to know this.

      scale  = fisheye_scale
      iscale = 1.0D0/scale

!     Call Excision routines before symmetries are applied.

      if (excise.eq.1) then

         if (CCTK_EQUALS(excisiontype,'lego')) then

!           Lego sphere or polyhedra.  For excising a figure other than
!           a cube, we call thorn LegoExcision.  Notice that this could
!           also be called for an excised cube and should work OK.

            if (evolveLapse) then
               call ExcisionExtrapolate(ierr,alp,alp_p,emask,
     .              ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,one)
               call ExcisionExtrapolate(ierr,ADM_BS_dtalp,ADM_BS_dtalp_p,emask,
     .              ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
            end if
           
            if (evolveShift) then
               call ExcisionExtrapolate(ierr,betax,betax_p,emask,
     .              ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
               call ExcisionExtrapolate(ierr,betay,betay_p,emask,
     .              ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
               call ExcisionExtrapolate(ierr,betaz,betaz_p,emask,
     .              ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
               call ExcisionExtrapolate(ierr,ADM_BS_Bx,ADM_BS_Bx_p,emask,
     .              ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
               call ExcisionExtrapolate(ierr,ADM_BS_By,ADM_BS_By_p,emask,
     .              ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
               call ExcisionExtrapolate(ierr,ADM_BS_Bz,ADM_BS_Bz_p,emask,
     .              ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
            end if
           
            call ExcisionExtrapolate(ierr,ADM_BS_phi,ADM_BS_phi_p,emask,
     .           ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
           
            call ExcisionExtrapolate(ierr,ADM_BS_gxx,ADM_BS_gxx_p,emask,
     .           ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,one)
            call ExcisionExtrapolate(ierr,ADM_BS_gyy,ADM_BS_gyy_p,emask,
     .           ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,one)
            call ExcisionExtrapolate(ierr,ADM_BS_gzz,ADM_BS_gzz_p,emask,
     .           ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,one)
           
            call ExcisionExtrapolate(ierr,ADM_BS_gxy,ADM_BS_gxy_p,emask,
     .           ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
            call ExcisionExtrapolate(ierr,ADM_BS_gxz,ADM_BS_gxz_p,emask,
     .           ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
            call ExcisionExtrapolate(ierr,ADM_BS_gyz,ADM_BS_gyz_p,emask,
     .           ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
           
            if (evolveGamma) then
               call ExcisionExtrapolate(ierr,ADM_BS_Gx,ADM_BS_Gx_p,emask,
     .              ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
               call ExcisionExtrapolate(ierr,ADM_BS_Gy,ADM_BS_Gy_p,emask,
     .              ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
               call ExcisionExtrapolate(ierr,ADM_BS_Gz,ADM_BS_Gz_p,emask,
     .              ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
            end if

            if (evolveK) then
               call ExcisionExtrapolate(ierr,ADM_BS_K,ADM_BS_K_p,emask,
     .              ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
            end if
           
            call ExcisionExtrapolate(ierr,ADM_BS_Axx,ADM_BS_Axx_p,emask,
     .           ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
            call ExcisionExtrapolate(ierr,ADM_BS_Ayy,ADM_BS_Ayy_p,emask,
     .           ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
            call ExcisionExtrapolate(ierr,ADM_BS_Azz,ADM_BS_Azz_p,emask,
     .           ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
           
            call ExcisionExtrapolate(ierr,ADM_BS_Axy,ADM_BS_Axy_p,emask,
     .           ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
            call ExcisionExtrapolate(ierr,ADM_BS_Axz,ADM_BS_Axz_p,emask,
     .           ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)
            call ExcisionExtrapolate(ierr,ADM_BS_Ayz,ADM_BS_Ayz_p,emask,
     .           ADM_BS_dirx,ADM_BS_diry,ADM_BS_dirz,nx,ny,nz,zero)

         end if

      end if

!     Apply boundary conditions.

      if (CCTK_EQUALS(bound,'none').or.
     $    CCTK_EQUALS(bound,'static').or.
     $    CCTK_EQUALS(bound,'zero').or.
     $    CCTK_EQUALS(bound,'newrad')) then

!        None or static: Do nothing.
!        Zero.  (Also do nothing, for some reason.)
         if (evolveLapse) then
            ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $           "admbase::lapse", "None")
            ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $           "adm_bssn::ADM_BSSN_dtlapse", "None")
         end if

        if (evolveShift) then
           ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $          "admbase::shift", "None")
           ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $          "adm_bssn::ADM_BSSN_B", "None")
        end if

        ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $       "adm_bssn::ADM_BSSN_phi", "None")
        ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $       "adm_bssn::ADM_BSSN_metric", "None")

        if (evolveGamma) then
           ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $          "adm_bssn::ADM_BSSN_gamma", "None")
        end if

        if (evolveK) then
           ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     .          "adm_bssn::ADM_BSSN_K", "None")
        end if

        ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     .       "adm_bssn::ADM_BSSN_curv", "None")

      else if (CCTK_EQUALS(bound,'flat')) then

!        Flat.
         if (evolveLapse) then
            ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $           "admbase::lapse", "Flat")
            ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $           "adm_bssn::ADM_BSSN_dtlapse", "Flat")
         end if

        if (evolveShift) then
           ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $          "admbase::shift", "Flat")
           ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $          "adm_bssn::ADM_BSSN_B", "Flat")
        end if

        ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $       "adm_bssn::ADM_BSSN_phi", "Flat")
        ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $       "adm_bssn::ADM_BSSN_metric", "Flat")

        if (evolveGamma) then
           ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $          "adm_bssn::ADM_BSSN_gamma", "Flat")
        end if

        if (evolveK) then
           ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     .          "adm_bssn::ADM_BSSN_K", "Flat")
        end if

        ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     .       "adm_bssn::ADM_BSSN_curv", "Flat")

      else if (CCTK_EQUALS(bound,'minkowski')) then

!        Minkowski.

         if (scalar_zero_table .eq. -2) then
            call Util_TableCreateFromString(scalar_zero_table, "SCALAR=0.0")
         end if
         if (scalar_one_table .eq. -2) then
            call Util_TableCreateFromString(scalar_one_table, "SCALAR=1.0")
         end if

         if (evolveLapse) then
            ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, scalar_one_table , "admbase::lapse", "Scalar")
            ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, scalar_zero_table, "adm_bssn::adm_bssn_dtlapse", "Scalar")
         end if

         if (evolveShift) then
            ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, scalar_zero_table, "admbase::shift", "Scalar")
            ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, scalar_zero_table, "adm_bssn::adm_bssn_B", "Scalar")
         end if

         ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, scalar_zero_table, "adm_bssn::adm_bssn_phi", "Scalar")
         ierr = Boundary_SelectVarForBC  (cctkGH, faces, ione, scalar_one_table , "adm_bssn::ADM_BS_gxx", "Scalar")
         ierr = Boundary_SelectVarForBC  (cctkGH, faces, ione, scalar_zero_table, "adm_bssn::ADM_BS_gxy", "Scalar")
         ierr = Boundary_SelectVarForBC  (cctkGH, faces, ione, scalar_zero_table, "adm_bssn::ADM_BS_gxz", "Scalar")
         ierr = Boundary_SelectVarForBC  (cctkGH, faces, ione, scalar_one_table , "adm_bssn::ADM_BS_gyy", "Scalar")
         ierr = Boundary_SelectVarForBC  (cctkGH, faces, ione, scalar_zero_table, "adm_bssn::ADM_BS_gyz", "Scalar")
         ierr = Boundary_SelectVarForBC  (cctkGH, faces, ione, scalar_one_table , "adm_bssn::ADM_BS_gzz", "Scalar")

         if (evolveGamma) then
            ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, scalar_zero_table, "adm_bssn::adm_bssn_gamma", "Scalar")
         end if

         if (evolveK) then
            ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, scalar_zero_table, "adm_bssn::adm_bssn_K", "Scalar")
         end if

         ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, scalar_zero_table, "adm_bssn::adm_bssn_curv", "Scalar")

      else if (CCTK_EQUALS(bound,'radiative')) then

!        Radiative.

         if (lim_1_spd_v0_table .eq. -2) then

!           Create tables to pass to radiative bcs
            call Util_TableCreate(lim_1_spd_v0_table, UTIL_TABLE_FLAGS_DEFAULT)
            call Util_TableCreate(lim_0_spd_v0_table, UTIL_TABLE_FLAGS_DEFAULT)
            call Util_TableCreate(lim_0_spd_iscale_table, 
     $           UTIL_TABLE_FLAGS_DEFAULT)
            call Util_TableCreate(lim_1_spd_iscale_table, 
     $           UTIL_TABLE_FLAGS_DEFAULT)
            call Util_TableCreate(lim_aux_spd_v0_table, 
     $           UTIL_TABLE_FLAGS_DEFAULT)

            if (lim_1_spd_v0_table < 0 .or. lim_0_spd_v0_table < 0 .or.
     $          lim_0_spd_iscale_table < 0 .or. lim_1_spd_iscale_table < 0 .or.
     $          lim_aux_spd_v0_table < 0) then
               call CCTK_WARN(1, "Error creating table to hold arguments to radiative boundary condition")
            end if


!           Find gauge speed far away.  The gauge speed is sqrt(harmonic_f).
!           When using FishEye, this speed has to be divided by the 
!           de-resolution factor.

            v0 = sqrt(harmonic_f)*iscale
            call Util_TableSetReal(ierr, lim_0_spd_v0_table, v0, "SPEED")
            call Util_TableSetReal(ierr, lim_1_spd_v0_table, v0, "SPEED")
            call Util_TableSetReal(ierr, lim_1_spd_v0_table, one, "LIMIT")


!           Call the radiative boundaries on the shift using an asymptotic
!           value of 0, and a speed of 1/scale.  This speed is not quite
!           correct, it really should depend on the value of "ShiftGammaCoeff",
!           but for the moment I will just use 1/scale.  Moreover, the wave
!           speed is in fact different for transverse and longitudinal
!           components of the shift, which is even worse.

!           Notice also that I do not apply boundary conditions to the shift
!           itself, but only to its time derivative.  For hyperbolic shift
!           conditions, the shift can be evolved all the way to the boundaries.
!           For parabolic conditions, I simply do not evolve the shift at all
!           at the boundary.
            
            call Util_TableSetReal(ierr, lim_0_spd_iscale_table, iscale, "SPEED")

!           Call the radiative boundaries on diagonal components
!           of the metric using an asymptotic value of 1, and a
!           speed of 1/scale.

            call Util_TableSetReal(ierr, lim_1_spd_iscale_table, iscale, "SPEED")
            call Util_TableSetReal(ierr, lim_1_spd_iscale_table, one, "LIMIT")


!           Call the radiative boundaries on phi using an asymptotic
!           value of log(scale)/2, and a speed v0 (the conformal factor
!           travels with the gauge speed).

            aux = 0.5D0*log(scale)

            call Util_TableSetReal(ierr, lim_aux_spd_v0_table, aux, "LIMIT")
            call Util_TableSetReal(ierr, lim_aux_spd_v0_table, v0, "SPEED")

         end if

         if (cctk_iteration.ge.1) then

!           Call the radiative boundaries on the lapse using an asymptotic
!           value of 1, and a speed v0.

            if (evolveLapse) then
               ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione,
     $              lim_1_spd_v0_table, "admbase::lapse", "Radiation")
               ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione,
     $              lim_0_spd_v0_table, "adm_bssn::ADM_BSSN_dtlapse", 
     $              "Radiation")
            end if

            if (evolveShift) then

!              lim_0_spd_iscale_table: speed = iscale, limit = 0 (default)

               ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $              "admbase::shift", "None")
               ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione,
     $              lim_0_spd_iscale_table, "adm_bssn::ADM_BSSN_B", "Radiation")
            end if

!           lim_1_spd_iscale_table: speed = iscale, limit = one

            ierr = Boundary_SelectVarForBC(cctkGH, faces, ione,
     $           lim_1_spd_iscale_table, "adm_bssn::ADM_BS_gxx", "Radiation")
            ierr = Boundary_SelectVarForBC(cctkGH, faces, ione,
     $           lim_1_spd_iscale_table, "adm_bssn::ADM_BS_gyy", "Radiation")
            ierr = Boundary_SelectVarForBC(cctkGH, faces, ione,
     $           lim_1_spd_iscale_table, "adm_bssn::ADM_BS_gzz", "Radiation")

!           Call the radiative boundaries on off-diagonal components
!           of the metric using an asymptotic value of 0, and a
!           speed of 1/scale.
            ierr = Boundary_SelectVarForBC(cctkGH, faces, ione,
     $           lim_0_spd_iscale_table, "adm_bssn::ADM_BS_gxy", "Radiation")
            ierr = Boundary_SelectVarForBC(cctkGH, faces, ione,
     $           lim_0_spd_iscale_table, "adm_bssn::ADM_BS_gxz", "Radiation")
            ierr = Boundary_SelectVarForBC(cctkGH, faces, ione,
     $           lim_0_spd_iscale_table, "adm_bssn::ADM_BS_gyz", "Radiation")

!           Call the radiative boundaries on the Gammas using
!           an asymptotic value of 0, and a speed of 1/scale.

            if (evolveGamma) then
               ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione,
     $              lim_0_spd_iscale_table, "adm_bssn::ADM_BSSN_gamma", 
     $              "Radiation")
            end if

            ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione,
     $           lim_aux_spd_v0_table, "adm_bssn::ADM_BSSN_phi", 
     $           "Radiation")

!           If we are not forcing trK to be zero, call the
!           radiative boundaries on trK using an asymptotic
!           value of 0, and a speed v0 (trK travels with the
!           gauge speed).

            if (evolveK) then
               ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione,
     $              lim_0_spd_v0_table, "adm_bssn::ADM_BSSN_K", "Radiation")
            end if
         
            ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione,
     $           lim_0_spd_iscale_table, "adm_bssn::ADM_BSSN_curv", 
     $           "Radiation")

         end if

      else if (CCTK_EQUALS(bound,'staticrad')) then

!        Static radiative.

         if (cctk_iteration.ge.1) then

            v0 = sqrt(harmonic_f)

            if (evolveLapse) then
               call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .            alp,alp_p,ADM_BS_alp0,v0)
               call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .            ADM_BS_dtalp,ADM_BS_dtalp_p,ADM_BS_dtalp0,v0)
            end if

            if (evolveShift) then
               call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .            betax,betax_p,ADM_BS_betax0,one)
               call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .            betay,betay_p,ADM_BS_betay0,one)
               call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .            betaz,betaz_p,ADM_BS_betaz0,one)
               call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .            ADM_BS_Bx,ADM_BS_Bx_p,ADM_BS_Bx0,v0)
               call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .            ADM_BS_By,ADM_BS_By_p,ADM_BS_By0,v0)
               call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .            ADM_BS_Bz,ADM_BS_Bz_p,ADM_BS_Bz0,v0)
            end if

            call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .      ADM_BS_phi,ADM_BS_phi_p,ADM_BS_phi0,v0)

            call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .         ADM_BS_gxx,ADM_BS_gxx_p,ADM_BS_gxx0,one)
            call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .         ADM_BS_gyy,ADM_BS_gyy_p,ADM_BS_gyy0,one)
            call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .         ADM_BS_gzz,ADM_BS_gzz_p,ADM_BS_gzz0,one)
            call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .         ADM_BS_gxy,ADM_BS_gxy_p,ADM_BS_gxy0,one)
            call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .         ADM_BS_gxz,ADM_BS_gxz_p,ADM_BS_gxz0,one)
            call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .         ADM_BS_gyz,ADM_BS_gyz_p,ADM_BS_gyz0,one)

            if (evolveGamma.and.evolveShift) then
               call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .            ADM_BS_Gx,ADM_BS_Gx_p,ADM_BS_Gx0,one)
               call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .            ADM_BS_Gy,ADM_BS_Gy_p,ADM_BS_Gy0,one)
               call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .            ADM_BS_Gz,ADM_BS_Gz_p,ADM_BS_Gz0,one)
            end if

            if (evolveK) then
               call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .         ADM_BS_K,ADM_BS_K_p,ADM_BS_K0,v0)
            end if

            call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .      ADM_BS_Axx,ADM_BS_Axx_p,ADM_BS_Axx0,one)
            call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .      ADM_BS_Ayy,ADM_BS_Ayy_p,ADM_BS_Ayy0,one)
            call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .      ADM_BS_Azz,ADM_BS_Azz_p,ADM_BS_Azz0,one)
            call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .      ADM_BS_Axy,ADM_BS_Axy_p,ADM_BS_Axy0,one)
            call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .      ADM_BS_Axz,ADM_BS_Axz_p,ADM_BS_Axz0,one)
            call BSSN_MoL_staticrad(cctk_bbox,nx,ny,nz,dt,dx,dy,dz,x,y,z,r,
     .      ADM_BS_Ayz,ADM_BS_Ayz_p,ADM_BS_Ayz0,one)

         end if

!        Must execute "None" boundary condition on all variables, so
!        that symmetry bcs will be properly executed on them
         if (evolveLapse) then
            ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $           "admbase::lapse", "None")
            ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $           "adm_bssn::ADM_BSSN_dtlapse", "None")
         end if
         
        if (evolveShift) then
           ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $          "admbase::shift", "None")
           ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $          "adm_bssn::ADM_BSSN_B", "None")
        end if
        
        ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $       "adm_bssn::ADM_BSSN_phi", "None")
        ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $       "adm_bssn::ADM_BSSN_metric", "None")

        if (evolveGamma) then
           ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     $          "adm_bssn::ADM_BSSN_gamma", "None")
        end if

        if (evolveK) then
           ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     .          "adm_bssn::ADM_BSSN_K", "None")
        end if

        ierr = Boundary_SelectGroupForBC(cctkGH, faces, ione, -ione, 
     .       "adm_bssn::ADM_BSSN_curv", "None")

      end if

!     End.

      end subroutine ADM_BSSN_Boundaries



/*@@
   @routine    ADM_BSSN_NewRad
   @date       Tue Feb 19 23:26:33 2002
   @author     
   @desc 
   Wrapper for the call to NewRad
   @enddesc 
   @calls     
   @calledby   
   @history 
 
   @endhistory 
@@*/

      subroutine ADM_BSSN_NewRad(CCTK_ARGUMENTS)

      use ADM_BSSN_Scalars

      implicit none

      DECLARE_CCTK_ARGUMENTS
      DECLARE_CCTK_FUNCTIONS
      DECLARE_CCTK_PARAMETERS

      CCTK_INT  :: symtable
      CCTK_INT, dimension(1:6) :: symbnd
      logical,  dimension(1:6) :: doBC

      CCTK_REAL :: power, scale, iscale, v0, aux, dx, dy, dz
      CCTK_INT  :: sx, sy, sz, nx, ny, nz
      CCTK_INT  :: i,j,k

      integer   :: ierr

      nx = cctk_lsh(1)
      ny = cctk_lsh(2)
      nz = cctk_lsh(3)

      dx = CCTK_DELTA_SPACE(1)
      dy = CCTK_DELTA_SPACE(2)
      dz = CCTK_DELTA_SPACE(3)

      sx = 0
      sy = 0
      sz = 0

      symtable = SymmetryTableHandleForGrid (cctkGH)
      if (symtable < 0) call CCTK_WARN (0, "internal error")
      call Util_TableGetIntArray (ierr, int(symtable), 6, symbnd, "symmetry_handle")
      if (ierr /= 6) call CCTK_WARN (0, "internal error")
      doBC = cctk_bbox /= 0 .and. symbnd < 0

      power = radpower

      scale = fisheye_scale
      iscale = 1.d0 / scale

      v0 = sqrt(harmonic_f) * iscale
      aux = 0.5d0 * log(scale)
            
      if (evolveLapse) then
        call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .       alp,adm_bs_salp,1.0D0,v0,power)
        call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .       ADM_BS_dtalp,adm_bs_sdtalp,0.0D0,v0,power)
      end if
        
      call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .     ADM_BS_phi,adm_bs_sphi,aux,v0,power)
      
      if (evolveK) then
        call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .       ADM_BS_K,adm_bs_sK,0.0D0,v0,power)
      end if
      
c     Shift.
      
      if (evolveShift) then
        call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .       ADM_BS_Bx,adm_bs_sBx,0.0D0,iscale,power)
        call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .       ADM_BS_By,adm_bs_sBy,0.0D0,iscale,power)
        call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .       ADM_BS_Bz,adm_bs_sBz,0.0D0,iscale,power)
      end if
      
c     Metric.
      
      call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .     ADM_BS_gxx,adm_bs_sgxx,1.0D0,iscale,power)
      call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .     ADM_BS_gyy,adm_bs_sgyy,1.0D0,iscale,power)
      call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .     ADM_BS_gzz,adm_bs_sgzz,1.0D0,iscale,power)
      
      call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .     ADM_BS_gxy,adm_bs_sgxy,0.0D0,iscale,power)
      call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .     ADM_BS_gxz,adm_bs_sgxz,0.0D0,iscale,power)
      call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .     ADM_BS_gyz,adm_bs_sgyz,0.0D0,iscale,power)
      
c     Aij.
      
      call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .     ADM_BS_Axx,adm_bs_sAxx,0.0D0,iscale,power)
      call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .     ADM_BS_Ayy,adm_bs_sAyy,0.0D0,iscale,power)
      call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .     ADM_BS_Azz,adm_bs_sAzz,0.0D0,iscale,power)
      
      call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .     ADM_BS_Axy,adm_bs_sAxy,0.0D0,iscale,power)
      call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .     ADM_BS_Axz,adm_bs_sAxz,0.0D0,iscale,power)
      call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .     ADM_BS_Ayz,adm_bs_sAyz,0.0D0,iscale,power)
      
c     Gammas.
      
      if (evolveGamma) then
        call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .       ADM_BS_Gx,adm_bs_sGx,0.0D0,iscale,power)
        call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .       ADM_BS_Gy,adm_bs_sGy,0.0D0,iscale,power)
        call NewRad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .       ADM_BS_Gz,adm_bs_sGz,0.0D0,iscale,power)
      end if
      
      end subroutine ADM_BSSN_NewRad
