/*@@
  @file    ShiftSource.F
  @date    February 2001
  @author  Miguel Alcubierre
  @desc 
     Source term for shift vector evolution equation.
  @enddesc
  @version $Header: /numrelcvs/AEIThorns/BSSN_MoL/src/ShiftSource.F,v 1.12 2005/05/28 19:18:48 pollney Exp $
@@*/

#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"

      subroutine ADM_BSSN_ShiftSource(CCTK_ARGUMENTS)

      implicit none 

      DECLARE_CCTK_ARGUMENTS
      DECLARE_CCTK_PARAMETERS

      integer :: i,j,k,nx,ny,nz
      integer :: CCTK_Equals

      CCTK_REAL :: aux1,aux2,aux3,aux0

#include "CactusEinstein/ADMMacros/src/macro/ADM_Spacing_declare.h"

#include "macro/GAMMAMINDIS_declare.h"

c     Grid parameters.

      nx = cctk_lsh(1)
      ny = cctk_lsh(2)
      nz = cctk_lsh(3)

#include "CactusEinstein/ADMMacros/src/macro/ADM_Spacing.h"

c     Calculate shift source.

      if (CCTK_Equals(shift_evolution_method,"gamma0").eq.1) then

c        First time derivative of the shift proportional
c        to (Gamma-Gamma(t=0)).

         do k=1,nz
            do j=1,ny
               do i=1,nx

                  aux1 = exp(-ShiftPhiPower*adm_bs_phi(i,j,k))

                  if (conformal_state == 3) then
                     aux1 = aux1/psi(i,j,k)**ShiftPsiPower
                  end if

                  aux1 = aux1*alp(i,j,k)**ShiftAlpPower*ShiftGammaCoeff

                  adm_bs_sbetax(i,j,k) = aux1*adm_bs_Bx(i,j,k)
                  adm_bs_sbetay(i,j,k) = aux1*adm_bs_By(i,j,k)
                  adm_bs_sbetaz(i,j,k) = aux1*adm_bs_Bz(i,j,k)

               end do
            end do
         end do

         do k=2,nz-1
            do j=2,ny-1
               do i=2,nx-1

                  aux2 = exp(-BetaDriverPhi*adm_bs_phi(i,j,k))

                  if (conformal_state == 3) then
                     aux2 = aux2/psi(i,j,k)**BetaDriverPsi
                  end if

                  if ( BetaDriverSpatial > 0 ) then
                    if (conformal_state > 0) then
                      aux3 = BetaDriverPunc - (BetaDriverPunc - BetaDriverInf) /
     &                          ( BetaDriverFac * ( psi(i,j,k) - 1 )**2 + 1 )
                    else
                      aux3 = BetaDriver
                    end if
                  else
                    aux3 = BetaDriver
                  end if

                  aux2 = aux3*(2.0D0-aux2)*alp(i,j,k)**BetaDriverAlpPower

                  adm_bs_sBx(i,j,k) = adm_bs_sGx(i,j,k)
     &               - aux2*adm_bs_Bx(i,j,k)
                  adm_bs_sBy(i,j,k) = adm_bs_sGy(i,j,k)
     &               - aux2*adm_bs_By(i,j,k)
                  adm_bs_sBz(i,j,k) = adm_bs_sGz(i,j,k)
     &               - aux2*adm_bs_Bz(i,j,k)

               end do
            end do
         end do

      else if (CCTK_Equals(shift_evolution_method,"gamma1").eq.1) then

c        First time derivative of the shift proportional
c        to time derivative of Gammas (parabolic gamma driver).

         adm_bs_sbetax(1,:,:) = 0.0D0
         adm_bs_sbetax(:,1,:) = 0.0D0
         adm_bs_sbetax(:,:,1) = 0.0D0

         adm_bs_sbetay(1,:,:) = 0.0D0
         adm_bs_sbetay(:,1,:) = 0.0D0
         adm_bs_sbetay(:,:,1) = 0.0D0

         adm_bs_sbetaz(1,:,:) = 0.0D0
         adm_bs_sbetaz(:,1,:) = 0.0D0
         adm_bs_sbetaz(:,:,1) = 0.0D0

         adm_bs_sbetax(nx,:,:) = 0.0D0
         adm_bs_sbetax(:,ny,:) = 0.0D0
         adm_bs_sbetax(:,:,nz) = 0.0D0

         adm_bs_sbetay(nx,:,:) = 0.0D0
         adm_bs_sbetay(:,ny,:) = 0.0D0
         adm_bs_sbetay(:,:,nz) = 0.0D0

         adm_bs_sbetaz(nx,:,:) = 0.0D0
         adm_bs_sbetaz(:,ny,:) = 0.0D0
         adm_bs_sbetaz(:,:,nz) = 0.0D0

         do k=2,nz-1
            do j=2,ny-1
               do i=2,nx-1
                  adm_bs_sbetax(i,j,k) = ShiftGammaCoeff*adm_bs_sGx(i,j,k)
                  adm_bs_sbetay(i,j,k) = ShiftGammaCoeff*adm_bs_sGy(i,j,k)
                  adm_bs_sbetaz(i,j,k) = ShiftGammaCoeff*adm_bs_sGz(i,j,k)
               end do
            end do
         end do

         adm_bs_sBx = 0.0D0
         adm_bs_sBy = 0.0D0
         adm_bs_sBz = 0.0D0

      else if (CCTK_Equals(shift_evolution_method,"gamma2").eq.1) then

c        Second time derivative of the shift proportional
c        to time derivative of Gammas (hyperbolic gamma driver).

         adm_bs_sbetax = adm_bs_Bx
         adm_bs_sbetay = adm_bs_By
         adm_bs_sbetaz = adm_bs_Bz

         do k=2,nz-1
            do j=2,ny-1
               do i=2,nx-1

                  aux1 = exp(-ShiftPhiPower*adm_bs_phi(i,j,k))
                  aux2 = exp(-BetaDriverPhi*adm_bs_phi(i,j,k))

                  if (conformal_state == 3) then
                     aux1 = aux1/psi(i,j,k)**ShiftPsiPower
                     aux2 = aux2/psi(i,j,k)**BetaDriverPsi
                  end if

                  if ( BetaDriverSpatial > 0 ) then
                    if (conformal_state > 0) then
                      aux3 = BetaDriverPunc - (BetaDriverPunc - BetaDriverInf) /
     &                          ( BetaDriverFac * ( psi(i,j,k) - 1 )**2 + 1 )
                    else
                      aux3 = BetaDriver
                    end if
                  else
                    aux3 = BetaDriver
                  end if

                  aux1 = aux1*alp(i,j,k)**ShiftAlpPower*ShiftGammaCoeff
                  aux2 = aux3*(2.0D0-aux2)*alp(i,j,k)**BetaDriverAlpPower

                  adm_bs_sBx(i,j,k) = aux1*adm_bs_sGx(i,j,k)
     &               - aux2*adm_bs_Bx(i,j,k)
                  adm_bs_sBy(i,j,k) = aux1*adm_bs_sGy(i,j,k)
     &               - aux2*adm_bs_By(i,j,k)
                  adm_bs_sBz(i,j,k) = aux1*adm_bs_sGz(i,j,k)
     &               - aux2*adm_bs_Bz(i,j,k)

               end do
            end do
         end do

      else if (CCTK_Equals(shift_evolution_method,"hyp_mindist").eq.1) then

c        Hyperbolic version of minimal distortion.

         do k=1,nz
            do j=1,ny
               do i=1,nx

                  aux1 = exp(-ShiftPhiPower*adm_bs_phi(i,j,k))

                  if (conformal_state == 3) then
                     aux1 = aux1/psi(i,j,k)**ShiftPsiPower
                  end if

                  aux1 = aux1*alp(i,j,k)**ShiftAlpPower*ShiftGammaCoeff

                  adm_bs_sbetax(i,j,k) = aux1*adm_bs_Bx(i,j,k)
                  adm_bs_sbetay(i,j,k) = aux1*adm_bs_By(i,j,k)
                  adm_bs_sbetaz(i,j,k) = aux1*adm_bs_Bz(i,j,k)

               end do
            end do
         end do

         do k=2,nz-1
            do j=2,ny-1
               do i=2,nx-1

                  aux2 = exp(-BetaDriverPhi*adm_bs_phi(i,j,k))

                  if (conformal_state == 3) then
                     aux2 = aux2/psi(i,j,k)**BetaDriverPsi
                  end if

                  aux2 = BetaDriver*(2.0D0-aux2)

#include "macro/GAMMAMINDIS_guts.h"

                  adm_bs_sBx(i,j,k) = (adm_bs_sGx(i,j,k)
     &               - GAMMAMINDIS_SOURCEX) - aux2*adm_bs_Bx(i,j,k)
                  adm_bs_sBy(i,j,k) = (adm_bs_sGy(i,j,k)
     &               - GAMMAMINDIS_SOURCEY) - aux2*adm_bs_By(i,j,k)
                  adm_bs_sBz(i,j,k) = (adm_bs_sGz(i,j,k)
     &               - GAMMAMINDIS_SOURCEZ) - aux2*adm_bs_Bz(i,j,k)

#include "macro/GAMMAMINDIS_undefine.h"

               end do
            end do
         end do

      else if (CCTK_Equals(shift_evolution_method,"gammaS").eq.1) then
         
         if (cctk_time < GammaSStartTime) then
            aux0 = 0.0d0
         else
            aux0 = 1.0d0
         endif
         
         do k=1,nz
            do j=1,ny
               do i=1,nx

                  aux2 = CCTK_DELTA_TIME*GammaDriver

                  aux1 = exp(-ShiftPhiPower*adm_bs_phi(i,j,k))
                  aux1 = aux1*alp(i,j,k)**ShiftAlpPower*ShiftGammaCoeff

                  adm_bs_sbetax(i,j,k) =   aux0*aux1 * ( aux2*adm_bs_sGx(i,j,k)
     &               + adm_bs_Gx(i,j,k) )
                  adm_bs_sbetay(i,j,k) =   aux0*aux1 * ( aux2*adm_bs_sGy(i,j,k)
     &               + adm_bs_Gy(i,j,k) )
                  adm_bs_sbetaz(i,j,k) =   aux0*aux1*( aux2*adm_bs_sGz(i,j,k)
     &               + adm_bs_Gz(i,j,k) )

               end do
            end do
         end do

      end if

c     End.

      end subroutine ADM_BSSN_ShiftSource
