#! /bin/sh
# /*@@
#  @file   gsl.sh
#  @date   2005-04-18
#  @author Erik Schnetter <schnetter@aei.mpg.de>
#  @desc
#          Setup GSL as a thorn
#  @enddesc
# @@*/



# /*@@
#   @routine    CCTK_Search
#   @date       Wed Jul 21 11:16:35 1999
#   @author     Tom Goodale
#   @desc
#   Used to search for something in various directories
#   @enddesc
#@@*/

CCTK_Search()
{
  eval  $1=""
  if test $# -lt 4 ; then
    cctk_basedir=""
  else
    cctk_basedir="$4/"
  fi
  for cctk_place in $2
    do
#      echo $ac_n "  Looking in $cctk_place""...$ac_c" #1>&6
      if test -r "$cctk_basedir$cctk_place/$3" ; then
#        echo "$ac_t""... Found" #1>&6
        eval $1="$cctk_place"
        break
      fi
      if test -d "$cctk_basedir$cctk_place/$3" ; then
#        echo "$ac_t""... Found" #1>&6
        eval $1="$cctk_place"
        break
      fi
#      echo "$ac_t"" No" #1>&6
    done

  return
}


# Work out which variation of GSL is installed
# and set the GSL libs, libdirs and includedirs
if [ -z "$GSL_DIR" ]; then
  echo "BEGIN MESSAGE"
  echo 'GSL selected but no GSL_DIR set.  Checking some places...'
  echo "END MESSAGE"

  CCTK_Search GSL_DIR "/usr /usr/local /usr/local/gsl /usr/local/packages/gsl /usr/local/apps/gsl $HOME c:/packages/gsl" include/gsl/gsl_math.h
  if [ -z "$GSL_DIR" ]; then
     echo "BEGIN ERROR" 
     echo 'Unable to locate the GSL directory -- please set GSL_DIR'
     echo "END ERROR" 
     exit 2
  fi
  echo "BEGIN MESSAGE"
  echo "Found a GSL package in $GSL_DIR"
  echo "END MESSAGE"

  # don't explicitely add standard include and library search paths
  if [ "$GSL_DIR" != '/usr' -a "$GSL_DIR" != '/usr/local' ]; then
    GSL_LIB_DIRS="$GSL_DIR/lib"
    GSL_INC_DIRS="$GSL_DIR/include"
  fi
else
  echo "BEGIN MESSAGE" 
  echo "Using GSL package in $GSL_DIR"
  echo "END MESSAGE"

  GSL_LIB_DIRS="$GSL_DIR/lib"
  GSL_INC_DIRS="$GSL_DIR/include"
fi
GSL_LIBS="gsl gslcblas"



# Check whether we run Windows or not
perl -we 'exit (`uname` =~ /^CYGWIN/)'
is_windows=$?



# Finally, add the math lib which might not be linked against by default
if [ $is_windows -eq 0 ]; then
  GSL_LIBS="$GSL_LIBS m"
fi

echo "INCLUDE_DIRECTORY $GSL_INC_DIRS"
echo "LIBRARY $GSL_LIBS"
echo "LIBRARY_DIRECTORY $GSL_LIB_DIRS"

# Remember GSL_DIR setting so that it can be used in later reconfigs
echo "BEGIN MAKE_DEFINITION"
echo "GSL_DIR = $GSL_DIR"
echo "END MAKE_DEFINITION"
