! $Header: /numrelcvs/AEIThorns/HyperWave/src/Init.F90,v 1.2 2004/12/29 22:40:57 schnetter Exp $

! Create initial data:
! Set up a sine wave.
! u(t,x,y,z) = A sin(kx) sin(ky) sin(kz) sin(omega t)
! rho = D_t u
! v_i = D_i u

! We use the Fortran array notation here.
! Note that u, rho, vx, vy, and vz are 3D arrays.

#include "cctk.h"
#include "cctk_Arguments.h"
#include "cctk_Parameters.h"

subroutine HW_Init (CCTK_ARGUMENTS)
  implicit none
  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS
  
  ! The constaint pi
  CCTK_REAL, parameter :: one = 1
  CCTK_REAL :: pi
  
  ! Wave number
  CCTK_REAL :: k, omega
  
  ! Calculate pi
  pi = 4 * atan(one)
  
  ! Calculate the wave number
  k = 2*pi / wavelength
  omega = sqrt(3 * k**2)
  
  ! Calculate the initial data
  u = amplitude * sin(k*x) * sin(k*y) * sin(k*z) * sin(omega*cctk_time)
  rho = omega * amplitude * sin(k*x) * sin(k*y) * sin(k*z) * cos(omega*cctk_time)
  vx = k * amplitude * cos(k*x) * sin(k*y) * sin(k*z) * sin(omega*cctk_time)
  vy = k * amplitude * sin(k*x) * cos(k*y) * sin(k*z) * sin(omega*cctk_time)
  vz = k * amplitude * sin(k*x) * sin(k*y) * cos(k*z) * sin(omega*cctk_time)
  
end subroutine HW_Init
