! $Header: /numrelcvs/AEIThorns/HyperWave/src/RHS.F90,v 1.3 2005/02/27 12:53:23 schnetter Exp $

! Calculate the right hand side of the evolution equations:
! D_t u   = rho
! D_t rho = D_i v_i
! D_t v_i = D_i rho

! We use the Fortran array notation here.
! Note that u, rho, vx, vy, vz, and their right hand sides are 3D arrays.

#include "cctk.h"
#include "cctk_Arguments.h"

subroutine HW_RHS (CCTK_ARGUMENTS)
  implicit none
  DECLARE_CCTK_ARGUMENTS
  
  ! Some local arrays
  CCTK_REAL, dimension(cctk_lsh(1), cctk_lsh(2), cctk_lsh(3)) &
       :: dxrho, dyrho, dzrho, dxvx, dyvy, dzvz
  
  ! This one is easy
  u_rhs = rho
  
  ! Calculate some derivatives
  call Diff_gv (cctkGH, 0, vx, dxvx, -1)
  call Diff_gv (cctkGH, 1, vy, dyvy, -1)
  call Diff_gv (cctkGH, 2, vz, dzvz, -1)
  ! Set the RHS
  rho_rhs = dxvx + dyvy + dzvz
  
  ! Calculate some more derivatives
  call Diff_gv (cctkGH, 0, rho, dxrho, -1)
  call Diff_gv (cctkGH, 1, rho, dyrho, -1)
  call Diff_gv (cctkGH, 2, rho, dzrho, -1)
  ! These are also easy
  vx_rhs = dxrho
  vy_rhs = dyrho
  vz_rhs = dzrho
  
end subroutine HW_RHS
