/* $Header: /numrelcvs/AEIThorns/HyperWave/src/SelectBCs.c,v 1.1 2004/12/29 22:40:57 schnetter Exp $ */

/* Select the boundary conditions for the evolved variables.
   
   The name of the boundary condition itself is given through
   the parameter "boundary".
   
   This selects the boundary conditions, but does not apply them.
   
   Note that the boundary width must be chosen to be consistent
   with the size of the finite differencing stencil.
*/

#include "cctk.h"
#include "cctk_Arguments.h"
#include "cctk_Parameters.h"

void
HW_SelectBCs (CCTK_ARGUMENTS)
{
  DECLARE_CCTK_ARGUMENTS;
  DECLARE_CCTK_PARAMETERS;
  
  int const width = 1;          /* One grid point wide */
  int const table = -1;         /* There is no table */
  
  int ierr1, ierr2, ierr3;      /* Error codes */
  
  /* Select the boundary conditions */
  ierr1 = Boundary_SelectGroupForBC
    (cctkGH, CCTK_ALL_FACES, width, table, "HyperWave::scalar", boundary);
  ierr2 = Boundary_SelectGroupForBC
    (cctkGH, CCTK_ALL_FACES, width, table, "HyperWave::scalar_dot", boundary);
  ierr3 = Boundary_SelectGroupForBC
    (cctkGH, CCTK_ALL_FACES, width, table, "HyperWave::scalar_grad", boundary);
  
  /* Abort if there was an error */
  if (ierr1 != 0 || ierr2 != 0 || ierr3 != 0) {
    CCTK_WARN (0, "Could not select boundary condition");
  }
}
