! $Header: /numrelcvs/AEIThorns/LightCone2/src/calculate.F90,v 1.1 2005/05/26 17:27:45 schnetter Exp $

#include "cctk.h"
#include "cctk_Arguments.h"
#include "cctk_Functions.h"
#include "cctk_Parameters.h"



subroutine LightCone2_Calculate (CCTK_ARGUMENTS)
  implicit none
  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_FUNCTIONS
  DECLARE_CCTK_PARAMETERS
  
  CCTK_REAL :: alfa, beta(3), gama(3,3)
  CCTK_REAL :: offset(3)
  
  integer, parameter :: lwork = 100
  CCTK_REAL :: work(lwork)
  CCTK_REAL :: evec(3,3), eval(3)
  integer   :: info
  
  integer   :: i, j, k
  integer   :: n
  
  
  
  ! Calculate eigenvectors for symmetric matrices
  
  interface syev
     subroutine ssyev (jobz, uplo, n, a, lda, w, work, lwork, info)
       character jobz, uplo
       integer   info, lda, lwork, n
       real      a(lda,n), w(n), work(lwork)
     end subroutine ssyev
     subroutine dsyev (jobz, uplo, n, a, lda, w, work, lwork, info)
       character        jobz, uplo
       integer          info, lda, lwork, n
       double precision a(lda,n), w(n), work(lwork)
     end subroutine dsyev
  end interface
  
  
  
  do k = 1, cctk_lsh(3)
     do j = 1, cctk_lsh(2)
        do i = 1, cctk_lsh(1)
           
           ! Read grid functions into local variables
           
           alfa = alp(i,j,k)
           
           if (shift_state /= 0) then
              beta(1) = betax(i,j,k)
              beta(2) = betay(i,j,k)
              beta(3) = betaz(i,j,k)
           else
              beta = 0
           end if
           
           gama(1,1) = gxx(i,j,k)
           gama(1,2) = gxy(i,j,k)
           gama(1,3) = gxz(i,j,k)
           gama(2,2) = gyy(i,j,k)
           gama(2,3) = gyz(i,j,k)
           gama(3,3) = gzz(i,j,k)
           gama(2,1) = gama(1,2)
           gama(3,1) = gama(1,3)
           gama(3,2) = gama(2,3)
           
           if (conformal_state > 0) then
              gama = gama * psi(i,j,k)**4
           end if
           
           
           
           ! Find vectors x^a for which
           !    g_ab x^a x^b = 0
           ! with x^0 = 1
           ! This leads to
           !    - alpha^2 + beta_i beta^i + 2 beta_i x^i + gamma_ij x^i x^j = 0
           !    - alpha^2 + gamma_ij beta^i beta^j
           !              + 2 gamma_ij beta^i x^j + gamma_ij x^i x^j = 0
           !    - alpha^2 + gamma_ij (beta^i + x^i) (beta^j + x^j) = 0
           !    gamma_ij (beta^i + x^i) (beta^j + x^j) = alpha^2
           !             (beta^i + x^i) (beta^j + x^j) = alpha^2 gamma^ij 
           
           
           
           ! Offset of the ellipsoid
           
           offset = - beta
           
           
           
           ! Shape of the ellipsoid
           
           evec = gama
           call syev ('v', 'u', 3, evec, 3, eval, work, lwork, info)
           
           forall (n=1:3)
              evec(:,n) = evec(:,n) * alfa / sqrt (eval(n))
           end forall
           
           
           
           ! Store local results into grid functions
           
           if (calculate_ellipsoids /= 0) then
              
              offset_x(i,j,k) = offset(1)
              offset_y(i,j,k) = offset(2)
              offset_z(i,j,k) = offset(3)
              
              major_axis_x(i,j,k) = evec(1,1)
              major_axis_y(i,j,k) = evec(2,1)
              major_axis_z(i,j,k) = evec(3,1)
              
              medium_axis_x(i,j,k) = evec(1,2)
              medium_axis_y(i,j,k) = evec(2,2)
              medium_axis_z(i,j,k) = evec(3,2)
              
              minor_axis_x(i,j,k) = evec(1,3)
              minor_axis_y(i,j,k) = evec(2,3)
              minor_axis_z(i,j,k) = evec(3,3)
              
           end if
           
           maximum_speed(i,j,k) = alfa / eval(1)
           
        end do
     end do
  end do
  
end subroutine LightCone2_Calculate
