/* $Header: /numrelcvs/AEIThorns/TensorTypes/src/tensortypes.h,v 1.2 2004/08/31 21:31:00 schnetter Exp $ */

#ifndef TENSORTYPES_H
#define TENSORTYPES_H

#ifdef	__cplusplus
extern "C" {
#endif



#include "cctk.h"



/*
 * A tensor type has a name, which is an arbitrary descriptive string.
 * It has a dimension and a rank.
 * It has dim^rank components.
 * A tensor has symmetries, which means that not all of its components
 *    are stored.  The stored components are called "vars".
 * Each component is either stored as var,
 *    or is equivalent to a var times a parity (+1 or -1),
 *    or is zero, indicated by a parity of 0.
 * There is a mapping from var to the component,
 *    and another mapping back from component to var and parity.
 */



/* A tensor description */
struct tensor
{
  const char * name;            /* description */
  int dim;                      /* dimension */
  int rank;                     /* rank */
  int ncomps;                   /* dim^rank */
  const int * restrict vars;    /* map component to variable */
  const int * restrict parity;  /* parity for the above */
  int nvars;                    /* depends on symmetries */
  const int * restrict comps;   /* map variable to component */
};



/* Find the tensor type for a grid function */
struct tensor const *
TT_varindex2tensortype (int const varindex);

/* Map between variable indices and tensor type vars */
int
TT_varindex2var (int const varindex);
int
TT_var2varindex (int const old_varindex, int const var);

/* Map between tensor components and tensor indices */
void
TT_component2indices (int const dim, int const rank,
                      int component,
                      int * restrict const indices);
int
TT_indices2component (int const dim, int const rank,
                      int const * restrict const indices);

/* Convert interpolator derivative codes to the number of derivatives */
int
TT_derivcode2nderivs (int const derivcode);

/* Find derivatives of tensor types */
struct tensor const *
TT_derivative (struct tensor const * const tensortype, int const nderivs);



#ifdef	__cplusplus
}
#endif

#endif /* TENSORTYPES_H */
