//
// This function dumps all the interpolation results to the specified files.
//
namespace {
void dump_interp_results(const char real_dump_file_name[],
			 const char real4_dump_file_name[],
			 const char real8_dump_file_name[],
			 const grid& interp_grid)
{
CCTK_VInfo(CCTK_THORNSTRING,
	   "dumping interpolation results:");
CCTK_VInfo(CCTK_THORNSTRING,
           "   CCTK_REAL, CCTK_COMPLEX --> \"%s\"",
	   real_dump_file_name);
CCTK_VInfo(CCTK_THORNSTRING,
           "   CCTK_REAL4, CCTK_COMPLEX8 --> \"%s\"",
	   real4_dump_file_name);
CCTK_VInfo(CCTK_THORNSTRING,
           "   CCTK_REAL8, CCTK_COMPLEX16 --> \"%s\"",
	   real8_dump_file_name);

FILE * real_fileptr = fopen( real_dump_file_name, "w");
FILE *real4_fileptr = fopen(real4_dump_file_name, "w");
FILE *real8_fileptr = fopen(real8_dump_file_name, "w");
if (    ( real_fileptr == NULL)
     || (real4_fileptr == NULL)
     || (real8_fileptr == NULL)    )
   then {
	CCTK_VWarn(-1, __LINE__, __FILE__, CCTK_THORNSTRING,
   "\n"
   "***** can't open one or more of the dump files!\n"
   "      CCTK_REAL  --> \"%s\"\n"
   "      CCTK_REAL4 --> \"%s\"\n"
   "      CCTK_REAL8 --> \"%s\"\n"
   ,
		   real_dump_file_name, real4_dump_file_name,
					real8_dump_file_name);	/*NOTREACHED*/
	CCTK_Abort(NULL, -1);					/*NOTREACHED*/
	}

fprintf(real_fileptr, "# dump of CCTK_REAL-precision interpolation results\n");
fprintf(real_fileptr, "# column 1 = CCTK_REAL\n");
fprintf(real_fileptr, "# column 2 = CCTK_COMPLEX.Re\n");
fprintf(real_fileptr, "# column 3 = CCTK_COMPLEX.Im\n");

fprintf(real4_fileptr, "# dump of CCTK_REAL4-precision interpolation results\n");
fprintf(real4_fileptr, "# column 1 = CCTK_REAL4\n");
fprintf(real4_fileptr, "# column 2 = CCTK_COMPLEX8.Re\n");
fprintf(real4_fileptr, "# column 3 = CCTK_COMPLEX8.Im\n");

fprintf(real8_fileptr, "# dump of CCTK_REAL8-precision interpolation results\n");
fprintf(real8_fileptr, "# column 1 = CCTK_REAL8\n");
fprintf(real8_fileptr, "# column 2 = CCTK_COMPLEX16.Re\n");
fprintf(real8_fileptr, "# column 3 = CCTK_COMPLEX16.Im\n");

	for (int posn = 0 ; posn < interp_grid.N_points ; ++posn)
	{
	fprintf(real_fileptr,
		"%.16f\t%.16f\t%.16f\n",
		double(interp_grid.array_real[posn]),
		double(interp_grid.array_complex[posn].Re),
		double(interp_grid.array_complex[posn].Im));
	fprintf(real4_fileptr,
		"%.7f\t%.7f\t%.7f\n",
		double(interp_grid.array_real4[posn]),
		double(interp_grid.array_complex8[posn].Re),
		double(interp_grid.array_complex8[posn].Im));
	fprintf(real8_fileptr,
		"%.16f\t%.16f\t%.16f\n",
		double(interp_grid.array_real8[posn]),
		double(interp_grid.array_complex16[posn].Re),
		double(interp_grid.array_complex16[posn].Im));
	}

fclose(real_fileptr);
fclose(real4_fileptr);
fclose(real8_fileptr);
}
	  }
