// random.cc - generate random numbers
// $Header: /numrelcvs/AEIThorns/TestInterpBase/archive/random.cc,v 1.1 2003/01/06 12:54:35 jthorn Exp $
//
// drandom - generate a random floating point number in [0,1)
//

#include <cstdlib>
#include <climits>

#include "stdc.h"
#include "util.hh"

//******************************************************************************

//
// This function computes a random number drawn from a uniform
// distribution on [0,1), using the  random(3)  non-linear additive
// feedback random number generator.  This is a much higher-quality
// random number generator than either the  rand(3)  or the  drand48(3)
// linear congruential one (both of whose low-order bits are strictly
// periodic).
//
//  std::srandom()  sets the seed for this random number generator.
// Note that on at least some versions of GNU/Linux/x86,  srandom(0)
// and srandom(1) seem to give identical random number sequences.
//
namespace jtutil
	  {
double drandom()
{
//
// Implementation Notes:
//
// Looking at man pages for Linux/x86, OSF1/Alpha, and Irix/mips64,
// it seems that  random()  may return either  int  or  long .  Thus
// we can only count on getting 32 bits for it, so we need 2 calls
// to get a standard double-precision floating-point number.
//

// 2^31 and 2^62, as floating point numbers
const double d_two_31 = 2.0 * 1024.0 * 1024.0 * 1024.0;
const double d_two_62 = d_two_31 * d_two_31;

// integers in [0, 2^31)
const int i1 = std::random() & 0x7fffffff;
const int i2 = std::random() & 0x7fffffff;

// floating point integer values in [0, 2^31)
const double d1 = double(i1);
const double d2 = double(i2);

return d1/d_two_31 + d2/d_two_62;
}
	  }
