#!/usr/bin/perl
# $Header: /numrelcvs/AEIThorns/TestInterpBase/make-test/make-testsuite.perl,v 1.5 2003/01/22 15:16:29 jthorn Exp $

################################################################################

my $help_msg = <<'EOF';
This program generates an Cactus testsuite for interpolator testing,
including a  README  and a  config  file.

Usage:
   make-tests [ --testsuite-dir directory-name ]  \
              [ --testsuite-name name ] \
              [ --template template-file-name ]   \
              <variables-file
or:
   make-tests --help

The --template option (default "template.par") specifies a template
file, in which the variables
	$testsuite_name		e.g. "TestInterpLocal_Large"
	$test_name		e.g. "InterpLocalUniform.Lagrange.2d.order4.par"
	$order			e.g. 2
	$ghost_size		e.g. 1
	$interpolator_type	e.g. "InterpLocalUniform"
	$interpolation_operator	e.g. "Lagrange polynomial interpolation"
	$interpolation_axes	e.g. "XY XZ YX YZ ZX ZY"
	$operation_codes	e.g. "0 1 2 11 12 22"
will be substituted.  $testsuite_name is set from the command line
option, the others come from a "variables file" read from standard input.

The standard input should be a "variables file", with the following format:
Any blank or comment lines ('#' as the first non-whitespace character)
are ignored.
Each noncomment line is split into 7 whitespace-separated fields:
- interpolator_type
- interpolation_operator
- N_dims
- ghost_size
- order_list
- interpolation_axes
- operation_codes
Within each field, any "_" characters are converted to blanks.
The  order_list  field should be a comma-separated list of orders; a
separate test is generated for each order.
EOF

################################################################################

my $README_msg = <<'EOF';
This interpolator testsuite is machine-generated from the Makefile
in the  make-test  directory of thorn TestInterpBase, using a Cactus
binary for the configuration described by the  config-info  file in
this directory.

Don't edit this testsuite by hand (your changes will be overwritten);
rather edit the scripts in TestInterpBase.
EOF

my $config_msg = <<'EOF';
EXTENSIONS hash hash4 hash8
EOF

################################################################################

use strict;
use Getopt::Long;
use FileHandle;

my $false = 0;
my $true = 1;

my $help_flag = $false;
my $N_fields = 7;
my $testsuite_dir  = '../test';
my $testsuite_name = undef();
my $template_name = 'template.par';
GetOptions(
	  'help'             => \$help_flag,
	  'testsuite-dir=s'  => \$testsuite_dir,
	  'testsuite-name=s' => \$testsuite_name,
	  'template=s'       => \$template_name,
	  )
	|| die($help_msg);

if ($help_flag)
	{ print $help_msg; exit 1; }

# generate a README file in the testsuite directory
my $README_name = "$testsuite_dir/README";
print "writing $README_name\n";
my $README_fh = new FileHandle("> $README_name")
   || die("can't open testsuite README file \"$README_name\" for writing!");
print $README_fh $README_msg;
$README_fh->close();

# generate a config file in the testsuite directory
my $config_name = "$testsuite_dir/config";
print "writing $config_name\n";
my $config_fh = new FileHandle("> $config_name")
   || die("can't open testsuite config file \"$config_name\" for writing!");
print $config_fh $config_msg;
$config_fh->close();

	while (my $line = <>)
	{
	if ($line =~ /^\s*#/)
		{ next; }
	if ($line =~ /^\s*$/)
		{ next; }
	chomp($line);

	my @fields = split(/\s+/, $line);
		for (my $i = 0 ; $i < $N_fields ; ++$i)
		{
		$fields[$i] =~ s/_/ /g;
		}
	##print "got ", join(':', @fields), "\n";
	my ($interpolator_type,
	    $interpolation_operator,
	    $N_dims,
	    $ghost_size,
	    $order_list_str,
	    $interpolation_axes,
	    $operation_codes) = @fields;
	my @order_list = split(/,/, $order_list_str);

		foreach my $order (@order_list)
		{
		# generate a single test
		my $operator_name = $interpolation_operator;
		$operator_name =~ s/ .*$//g;
		my $test_name =
   "${interpolator_type}.${operator_name}.${N_dims}d.order${order}";
		my $par_name = "$testsuite_dir/$test_name.par";
		print "writing $par_name\n";

		# write the par file
		my $par_fh = new FileHandle("> $par_name")
   || die("can't open testsuite par file \"$par_name\" for writing!");
		my $template_fh = new FileHandle("< $template_name")
   || die("can't open template file \"$template_name\" for reading!");
			while (my $tline = <$template_fh>)
			{
			$tline =~ s/\$testsuite_name/$testsuite_name/g;
			$tline =~ s/\$test_name/$test_name/g;
			$tline =~ s/\$ghost_size/$ghost_size/g;
			$tline =~ s/\$order/$order/g;
			$tline =~ s/\$interpolator_type/$interpolator_type/g;
			$tline =~ s/\$interpolation_operator/$interpolation_operator/g;
			$tline =~ s/\$interpolation_axes/$interpolation_axes/g;
			$tline =~ s/\$operation_codes/$operation_codes/g;
			print $par_fh $tline;
			}
		$template_fh->close();
		$par_fh->close();
		}
	}
