// InterpGridArrays.cc -- subdriver for CCTK_InterpGridArrays()
// $Header: /numrelcvs/AEIThorns/TestInterpBase/src/InterpGridArrays.cc,v 1.6 2003/01/22 15:30:50 tradke Exp $
//
// do_InterpGridArrays
//

#include <assert.h>
#include <math.h>

#include "cctk.h"
#include "util_ErrorCodes.h"
#include "util_Table.h"

#include "stdc.h"
#include "fp.h"
#include "util.hh"

#include "TestInterpBase.hh"
#include "data_ptrs.hh"
#include "axis_list.hh"
#include "input_grid.hh"
#include "interp_grid.hh"

#undef DEBUG_ARRAY_INDEXING

//******************************************************************************

//
// This function is a subdriver to do the actual interpolation using
// CCTK_InterpGridArrays().
//
// At present this only supports the contiguous-in-memory axes specifiers,
// i.e. {x, xy, xyz}.
//
void do_InterpGridArrays(const cGH *GH, int timer_handle,
			 const char coord_system[],
			 const input_grid& input_grid,
			 interp_grid& interp_grid,
			 const char interp_operator[], int interp_order,
						       const char interp_pars[],
			 int N_operations, const int opcodes[max_N_operations])
{
int status, status1, status2;

const int N_output = N_output_arrays * N_operations;

const int param_table_handle
	= setup_interp_param_table(interp_pars, interp_order);
const int interp_handle = get_interp_handle_from_name(interp_operator);
const int coord_system_handle = get_coord_system_handle(coord_system);

input_grid.input_axes.verify_contiguous();

setup_derivs_in_param_table(param_table_handle,
			    N_operations, opcodes);



//
// set up the interpolation coordinates
//
const void *const interp_coords[max_N_dims]
	= { static_cast<const void *>(interp_grid.interp_x),
	    static_cast<const void *>(interp_grid.interp_y),
	    static_cast<const void *>(interp_grid.interp_z) };

//
// set up the input arrays
//
const int input_array_variable_indices[N_input_arrays]
	= {
	  input_grid.varindex_real,
	  input_grid.varindex_real4,
	  input_grid.varindex_real8,
	  input_grid.varindex_complex,
	  input_grid.varindex_complex8,
	  input_grid.varindex_complex16,
	  };

//
// set up the output arrays
//
CCTK_INT output_array_type_codes[max_N_output];
void*    output_arrays          [max_N_output];
setup_output_arrays(interp_grid,
		    output_array_type_codes, output_arrays);

#ifdef DEBUG_ARRAY_INDEXING
printf("### interp_grid.interp_xyz[0] = %g %g %g\n",
       interp_grid.interp_x[0], interp_grid.interp_y[0], interp_grid.interp_z[0]);
printf("### interp_grid.interp_xyz[1] = %g %g %g\n",
       interp_grid.interp_x[1], interp_grid.interp_y[1], interp_grid.interp_z[1]);
printf("### interp_grid.interp_xyz[2] = %g %g %g\n",
       interp_grid.interp_x[2], interp_grid.interp_y[2], interp_grid.interp_z[2]);
printf("### interp_grid.interp_xyz[10] = %g %g %g\n",
       interp_grid.interp_x[10], interp_grid.interp_y[10], interp_grid.interp_z[10]);
#endif

//
// do the actual interpolation
//
CCTK_VInfo(CCTK_THORNSTRING,
	   "do_InterpGridArrays(): about to call interpolator (%d-D)",
	   interp_grid.N_dims);

CCTK_TimerStartI(timer_handle);
status = CCTK_InterpGridArrays(GH,
			       interp_grid.N_dims,
			       interp_handle,
			       param_table_handle,
			       coord_system_handle,
			       interp_grid.N_points,
				 CCTK_VARIABLE_REAL,
				 interp_coords,
			       N_input_arrays,
				 input_array_variable_indices,
			       N_output,
				 output_array_type_codes,
				 output_arrays);
CCTK_TimerStopI(timer_handle);

#ifdef DEBUG_ARRAY_INDEXING
printf("### interp_grid.output_arrays[0].ptr_real[0] = %g\n",
       interp_grid.output_arrays[0].ptr_real[0]);
printf("### interp_grid.output_arrays[0].ptr_real[1] = %g\n",
       interp_grid.output_arrays[0].ptr_real[1]);
printf("### interp_grid.output_arrays[0].ptr_real[2] = %g\n",
       interp_grid.output_arrays[0].ptr_real[2]);
printf("### interp_grid.output_arrays[0].ptr_real[10] = %g\n",
       interp_grid.output_arrays[0].ptr_real[10]);
#endif

if (status < 0)
   then CCTK_VWarn(-1, __LINE__, __FILE__, CCTK_THORNSTRING,
"do_InterpGridArrays(): status=%d return from interpolator!",
		   status);					/*NOTREACHED*/
CCTK_VInfo(CCTK_THORNSTRING,
	   "do_InterpGridArrays(): status=%d return from interpolator",
	   status);
}
