// Jacobian.hh -- storage for Jacobian matrix
// $Header: /numrelcvs/AEIThorns/TestInterpBase/src/Jacobian.hh,v 1.1 2003/01/06 12:57:19 jthorn Exp $

//
// A Jacobian object holds all the data describing the Jacobian of the
// interpolation at all the interpolation-grid points.
//
// For all the [max_N_dims] arrays, we only use the first N_dims elements.
//
class	Jacobian
	{
public:
	//
	// ***** data *****
	//
	int N_dims;
	CCTK_INT MSS_is_fn_of_interp_coords,
		 MSS_is_fn_of_which_operation,
		 MSS_is_fn_of_input_array_values,
		 Jacobian_is_fn_of_input_array_values;

	// unused entries are zeroed
	CCTK_INT molecule_min_m[max_N_dims],
		 molecule_max_m[max_N_dims];

	// array of --> new[interp_grid.N_points]-allocated arrays
	// ... molecule_position_pointers[axis] is a CCTK_INT*
	CCTK_INT *molecule_position_pointers[max_N_dims];
	// ... molecule_position_voidptrs[axis] is the same pointer,
	//     cast to CCTK_POINTER
	CCTK_POINTER molecule_position_voidptrs[max_N_dims];

	// strides for the Jacobian array
	CCTK_INT Jacobian_interp_point_stride;
	CCTK_INT Jacobian_m_strides[max_N_dims];
	CCTK_INT Jacobian_part_stride;

	// total number of numbers in Jacobian for each out
	int N_Jacobian_values;

	// --> new[N_output_arrays]-allocated arrays
	// of --> new[N_Jacobian_values]-allocated arrays
	CCTK_REAL **Jacobian_pointer;	// Jacobian_pointer[out] is a CCTK_REAL*
	CCTK_POINTER *Jacobian_voidptr;	// Jacobian_voidptr[out] is the same
					// pointer, cast to CCTK_POINTER

	// --> new[]-allocated array
	// or NULL if we don't use offsets
	CCTK_INT *Jacobian_offset;

	//
	// ***** member functions *****
	//
public:
	// Jacobian subscripting computation
	// ... assumes unused Jacobian_m_strides[] are zeroed
	int subscript(int out, int pt,
		      int mi, int mj, int mk,
		      int part=0)
		const
		{
		return Jacobian_offset[out]
		       + pt*Jacobian_interp_point_stride
		       + mi*Jacobian_m_strides[0]
		       + mj*Jacobian_m_strides[1]
		       + mk*Jacobian_m_strides[2]
		       + part*Jacobian_part_stride;
		}

	// setup molecule-posn and Jacobian query in interpolator par table
	void setup_interp_query(const grid& interp_grid,
				int param_table_handle)
		const;

	// delete molecule-posn and Jacobian query from interpolator par table
	void delete_interp_query(int param_table_handle)
		const;

	// compute "linear hash" of all Jacobian values,
	//	hash = sum_k w[k]*array[k]/N_Jacobian_values
	// where the w[k] are pseudorandom weights in the range
	// [hash_weight_min, hash_weight_max).
	fp hash(const grid& interp_grid,
		fp hash_weight_min, fp hash_weight_max)
		const;

	// do two Jacobian objects have the same molecule size?
	static
	  bool same_molecule_size(const Jacobian& A, const Jacobian& B);

public:
	//
	// ***** constructor, destructor *****
	//

	// sets molecule_min_m[] and molecule_max_m[]
	Jacobian(const grid& input_grid, const grid& interp_grid,
		 int interp_handle, int param_table_handle);

	~Jacobian();
	};
