// axis_list.cc -- class to represent list of axes
// $Header: /numrelcvs/AEIThorns/TestInterpBase/src/axis_list.cc,v 1.2 2003/01/17 15:23:13 jthorn Exp $

//
// axis_list::axis_list
// axis_list::setup
// axis_list::is_prefix_of_xyz
// axis_list::verify_contiguous
// axis_list::XYZ_of_xyz_deriv_opcode
//

#include <assert.h>
#include <math.h>
#include <string.h>
#include <stdio.h>

#include "cctk.h"

#include "stdc.h"
#include "fp.h"
#include "util.hh"

#include "TestInterpBase.hh"
#include "data_ptrs.hh"
#include "axis_list.hh"

//******************************************************************************

//
// This function constructs an  axis_list  object from a string
// specifier.
//
axis_list::axis_list(const char string_in[])
	: N_dims_(strlen(string_in)),
	  string_(string_in)
{
if (N_dims_ > max_N_dims)
   then CCTK_VWarn(-1, __LINE__, __FILE__, CCTK_THORNSTRING,
"***** axis_list::axis_list(\"%s\"): list specifies N_dims=%d\n"
"      which is > max_N_dims=%d is valid!"
,
		   string_in, N_dims_,
		   max_N_dims);					/*NOTREACHED*/

// implementation restriction
assert(max_N_dims == 3);

if	(is_prefix("XYZ", string_in)) then setup(0,1,2);
else if (is_prefix("XZY", string_in)) then setup(0,2,1);
else if (is_prefix("YXZ", string_in)) then setup(1,0,2);
else if (is_prefix("YZX", string_in)) then setup(1,2,0);
else if (is_prefix("ZXY", string_in)) then setup(2,0,1);
else if (is_prefix("ZYX", string_in)) then setup(2,1,0);
else	CCTK_VWarn(-1, __LINE__, __FILE__, CCTK_THORNSTRING,
"***** axis_list::axis_list(): unrecognized string_in=\"%s\"!",
		   string_in);					/*NOTREACHED*/
}

//******************************************************************************

//
// This is a helper function for the constructor.  It's called with
// arguments giving the extended axes.
//
void axis_list::setup(int axis0, int axis1, int axis2)
{
axis_list_[0] = axis0;
axis_list_[1] = axis1;
axis_list_[2] = axis2;
	for (int axis = 0 ; axis < max_N_dims ; ++axis)
	{
	where_is_axis_[axis_list_[axis]] = axis;
	}

CCTK_VInfo(CCTK_THORNSTRING,
	   "axis_list::setup(): axis_list_     = [%d,%d,%d]",
	   axis_list_[0], axis_list_[1], axis_list_[2]);
CCTK_VInfo(CCTK_THORNSTRING,
	   "                    where_is_axis_ = [%d,%d,%d]",
	   where_is_axis_[0], where_is_axis_[1], where_is_axis_[2]);
}

//******************************************************************************

//
// This function determines whether or not our list-of-axes is a
// prefix of the contiguous list "XYZ".
//
bool axis_list::is_prefix_of_xyz() const
{
	for (int axis = 0 ; axis < N_dims_ ; ++axis)
	{
	if (axis_list_[axis] != axis)
	   then return false;
	}

return true;
}

//******************************************************************************

//
// This function verifies that our extended axes list is indeed the
// contiguous one "XYZ".  If so, this function is a no-op.  If not, this
// function does a CCTK_VWarn(-1, ...) (and doesn't return).
//
void axis_list::verify_contiguous() const
{
if (! is_prefix_of_xyz())
   then CCTK_VWarn(-1, __LINE__, __FILE__, CCTK_THORNSTRING,
"axis_list::verify_contiguous(): axis list \"%s\" isn't contiguous!",
		   string());					/*NOTREACHED*/
}

//******************************************************************************

//
// Given an xyz-semantics derivative opcode, this function computes
// the corresponding XYZ-semantics derivative opcode.
//
int axis_list::XYZ_of_xyz_deriv_opcode(int xyz_deriv_opcode) const
{
int deriv_axes_xyz[max_N_derivs];
const int N_derivs = decode_derivative_code(xyz_deriv_opcode, deriv_axes_xyz);

int deriv_axes_XYZ[max_N_derivs];
	for (int axis = 0 ; axis < N_derivs ; ++axis)
	{
	const int xyz_axis = deriv_axes_xyz[axis] - 1;  // n.b. 1- vs 0-origin
	const int XYZ_axis = extended_axis(xyz_axis);
	deriv_axes_XYZ[axis] = 1 + XYZ_axis;            // n.b. 1- vs 0-origin
	}
return encode_derivative_code(N_derivs, deriv_axes_XYZ);
}
