// data_ptrs.cc -- store pointers to data arrays of all the different data types
// $Header: /numrelcvs/AEIThorns/TestInterpBase/src/data_ptrs.cc,v 1.1 2003/01/17 15:23:13 jthorn Exp $

//
// data_ptrs::data_ptrs
// data_ptrs::ptr_select
//

#include <assert.h>

#include "cctk.h"

#include "stdc.h"
#include "fp.h"
#include "util.hh"

#include "TestInterpBase.hh"
#include "data_ptrs.hh"

//******************************************************************************

//
// This function constructs a  data_ptrs  object, setting the pointers
// to point to the Cactus data arrays on this processor.
//
data_ptrs::data_ptrs(const cGH* GH)
{
const int timelevel = 0;

ptr_real      = static_cast<CCTK_REAL*>
			   (CCTK_VarDataPtr(GH, timelevel,
					    "TestInterpBase::fn_real"));
assert( ptr_real != NULL );

ptr_real4     = static_cast<CCTK_REAL4*>
			   (CCTK_VarDataPtr(GH, timelevel,
					    "TestInterpBase::fn_real4"));
assert( ptr_real4 != NULL );

ptr_real8     = static_cast<CCTK_REAL8*>
			   (CCTK_VarDataPtr(GH, timelevel,
					    "TestInterpBase::fn_real8"));
assert( ptr_real8 != NULL );

ptr_complex   = static_cast<CCTK_COMPLEX*>
			   (CCTK_VarDataPtr(GH, timelevel,
					    "TestInterpBase::fn_complex"));
assert( ptr_complex != NULL );

ptr_complex8  = static_cast<CCTK_COMPLEX8*>
			   (CCTK_VarDataPtr(GH, timelevel,
					    "TestInterpBase::fn_complex8"));
assert( ptr_complex8 != NULL );

ptr_complex16 = static_cast<CCTK_COMPLEX16*>
			   (CCTK_VarDataPtr(GH, timelevel,
					    "TestInterpBase::fn_complex16"));
assert( ptr_complex16 != NULL );
}

//******************************************************************************

//
// This function selects one of the  ptr_*  pointers and casts it to  void* .
//
void* data_ptrs::select_ptr(int itype) const
{
switch	(itype)
	{
case select_datatype_real:	return static_cast<void *>(ptr_real);
case select_datatype_real4:	return static_cast<void *>(ptr_real4);
case select_datatype_real8:	return static_cast<void *>(ptr_real8);
case select_datatype_complex:	return static_cast<void *>(ptr_complex);
case select_datatype_complex8:	return static_cast<void *>(ptr_complex8);
case select_datatype_complex16:	return static_cast<void *>(ptr_complex16);
default:
	CCTK_VWarn(-1, __LINE__, __FILE__, CCTK_THORNSTRING,
		   "data_ptrs::ptr_select(): bad itype=%d!",
		   itype);					/*NOTREACHED*/
	}
}
