// input_grid.cc -- input grid for interpolation
// $Header: /numrelcvs/AEIThorns/TestInterpBase/src/input_grid.cc,v 1.3 2003/01/18 18:41:34 jthorn Exp $

//
// input_grid::input_grid
// input_grid::verify_axes_contiguous
//

#include <assert.h>
#include <math.h>

#include "cctk.h"

#include "stdc.h"
#include "fp.h"
#include "util.hh"

#include "TestInterpBase.hh"
#include "data_ptrs.hh"
#include "axis_list.hh"
#include "input_grid.hh"

//******************************************************************************

//
// This function constructs an  input_grid  object.
//
input_grid::input_grid(const cGH* GH,
		       const CCTK_INT Cactus_grid_size[max_N_dims],
		       const fp Cactus_local_origin[], const fp Cactus_delta[],
		       const char axes_spec[])
	: input_axes(axes_spec),
	  data_ptrs(GH)
{
CCTK_VInfo(CCTK_THORNSTRING,
	   "input_grid::input_grid(): setting up xyz=%s",
	   axes_spec);

N_dims = input_axes.N_dims();

varindex_real      = CCTK_VarIndex("TestInterpBase::fn_real");
assert( varindex_real >= 0 );
varindex_real4     = CCTK_VarIndex("TestInterpBase::fn_real4");
assert( varindex_real4 >= 0 );
varindex_real8     = CCTK_VarIndex("TestInterpBase::fn_real8");
assert( varindex_real8 >= 0 );
varindex_complex   = CCTK_VarIndex("TestInterpBase::fn_complex");
assert( varindex_complex >= 0 );
varindex_complex8  = CCTK_VarIndex("TestInterpBase::fn_complex8");
assert( varindex_complex8 >= 0 );
varindex_complex16 = CCTK_VarIndex("TestInterpBase::fn_complex16");
assert( varindex_complex16 >= 0 );

const int Cactus_strides[max_N_dims]
	= {
	  1,
	  Cactus_grid_size[0],
	  Cactus_grid_size[0] * Cactus_grid_size[1],
	  };

	for (int axis = 0 ; axis < max_N_dims ; ++axis)
	{
	const int Cactus_axis = input_axes.extended_axis(axis);
	CCTK_VInfo(CCTK_THORNSTRING,
		   "            axis=%d Cactus_axis=%d",
		   axis, Cactus_axis);

	array_dims[axis] = input_axes.axis_is_unextended(axis)
			   ? Cactus_grid_size[Cactus_axis]
			   : 1;
	array_strides[axis] = Cactus_strides[Cactus_axis];
	CCTK_VInfo(CCTK_THORNSTRING,
		   "                   array dim=%d stride=%d",
		   array_dims[axis], array_strides[axis]);

	xyz_origin[axis] = Cactus_local_origin[Cactus_axis];
	xyz_delta[axis]  = Cactus_delta[Cactus_axis];
	CCTK_VInfo(CCTK_THORNSTRING,
		   "                   xyz_origin=%g xyz_delta=%g",
		   double(xyz_origin[axis]), double(xyz_delta[axis]));
	CCTK_VInfo(CCTK_THORNSTRING,
		   "                   xyz_[min,max]=[%g,%g]",
		   double(xyz_origin[axis]),
		   double(xyz_origin[axis] + xyz_delta[axis]
					     *(array_dims[axis]-1)));
	}
}
