// input_grid.hh -- input grid for interpolation
// $Header: /numrelcvs/AEIThorns/TestInterpBase/src/input_grid.hh,v 1.3 2003/01/18 18:41:34 jthorn Exp $

//
// An  input_grid  object represents the input grid arrays and their
// subscripting for the interpolation.
//
// An  input_grid  object is almost a POD object, except that it has a
// constructor.  However, it's still basically a souped-up C struct, with
// lots of public data members...
//
class	input_grid
	{
public:
	//
	// ***** public data *****
	//

	// this describes our axes in terms of the Cactus grid's axes,
	// i.e. "ZX" means that our x = Cactus Z, our y = Cactus X
	const axis_list input_axes;

	int N_dims;

	// subscripting info for the Cactus data arrays on this processor
	// in terms of *our* xyz axes and ijk indices
	// ... subscripts are from 0 to dim-1 in each axis
	CCTK_INT array_dims   [max_N_dims];
	CCTK_INT array_strides[max_N_dims];
	fp xyz_origin[max_N_dims], xyz_delta[max_N_dims];

	// Cactus variable indices of input arrays
	int varindex_real, varindex_real4, varindex_real8,
	    varindex_complex, varindex_complex8, varindex_complex16;

	// pointers to the Cactus data arrays themselves on this processor
	// n.b. we do *not* own these data arrays!
	struct data_ptrs data_ptrs;


	//
	// ***** public member functions *****
	//

	// find xyz coordinates of given ijk point
	fp x_of_i(int i) const { return xyz_origin[0] + i*xyz_delta[0]; }
	fp y_of_j(int j) const { return xyz_origin[1] + j*xyz_delta[1]; }
	fp z_of_k(int k) const { return xyz_origin[2] + k*xyz_delta[2]; }

	// array subscripting
	// these functions compute a 1-D array index in the data arrays
public:
	int subscript(int i, int j, int k) const
		{
		assert(i >= 0);
		assert(i < array_dims[0]);
		assert(j >= 0);
		assert(j < array_dims[1]);
		assert(k >= 0);
		assert(k < array_dims[2]);
		return subscript_unchecked(i,j,k);
		}
private:
	int subscript_unchecked(int i, int j, int k) const
		{
		return   array_strides[0]*i
		       + array_strides[1]*j
		       + array_strides[2]*k;
		}



	//
	// ***** constructor *****
	//

public:
	// construct from Cactus grid on this processor,
	// permuting axes as specified by  axes_spec
	input_grid(const cGH* GH,
		   const CCTK_INT Cactus_grid_size[max_N_dims],
		   const fp Cactus_local_origin[], const fp Cactus_delta[],
		   const char axes_spec[]);

	// no need for destructor, compiler-synthesized default is ok here

	// we forbid copying and passing by value
	// by declaring the copy constructor and assignment operator
	// private, but never defining them
private:
	input_grid(const input_grid& rhs);
	input_grid& operator=(const input_grid& rhs);
	};
