// interp_grid.hh -- grid of interpolation points and output arrays
// $Header: /numrelcvs/AEIThorns/TestInterpBase/src/interp_grid.hh,v 1.2 2003/01/23 20:01:21 jthorn Exp $

//
// An  interp_grid  object represents and stores the grid of interpolation
// points, and the output arrays from the interpolation.
//
// An  interp_grid  object is almost a POD object, except that it has a
// constructor.  However, it's still basically a souped-up C struct, with
// lots of public data members...
//
class	interp_grid
	{
public:
	//
	// ***** public data *****
	//

	// number of dimensions in which we interpolate
	int N_dims;

	// number of interpolation points
	int N_points;

	// number of derivative opcodes
	int N_operations;

	// xyz coordinates of interpolation points
	fp* interp_x;
	fp* interp_y;
	fp* interp_z;

	//
	// for the interpolations, the output arrays are ordered as follows:
	//	CCTK_REAL	iop=0
	//	CCTK_REAL	iop=1
	//	CCTK_REAL	iop=2
	//	CCTK_REAL	...
	//	CCTK_REAL	iop=N_operations-1
	//	CCTK_REAL4	iop=0
	//	CCTK_REAL4	iop=1
	//	CCTK_REAL4	iop=2
	//	CCTK_REAL4	...
	//	CCTK_REAL4	iop=N_operations-1
	//	CCTK_REAL8	iop=0
	//	CCTK_REAL8	iop=1
	//	...		...
	//	CCTK_COMPLEX16	iop=N_operations-1
	//

	// --> array [N_operations]
	struct data_ptrs *output_arrays;


	//
	// ***** private data *****
	//
private:
	int interp_grid_size[max_N_dims];
	fp interp_grid_origin[max_N_dims], interp_grid_delta[max_N_dims];


	//
	// ***** public member functions *****
	//
public:
	void randomly_permute_interp_points();


	//
	// ***** constructor *****
	//

public:
	// construct from input grid on this processor,
	// refining grid spacing
	// specify local/global Cactus size/origin parameters as appropriate
	interp_grid(const CCTK_INT Cactus_grid_size[max_N_dims],
		    const CCTK_INT N_ghost_zones[max_N_dims],
		    const fp Cactus_grid_origin[], const fp Cactus_grid_delta[],
		    const input_grid& input_grid,
		    int N_extra_interp_points, int data_to_interp_dxyz_ratio,
		    int N_operations_in);

	~interp_grid();

	// helper functions for constructor
private:
	void compute_grid_shape
		(const CCTK_INT Cactus_global_grid_size[max_N_dims],
		 const CCTK_INT N_ghost_zones[max_N_dims],
		 const fp Cactus_global_origin[], const fp Cactus_delta[],
		 const input_grid& input_grid,  
		 int N_extra_interp_points, int data_to_interp_dxyz_ratio);

	// set xyz coordinates (lexicographic order)
	void set_interp_xyz();

	// we forbid copying and passing by value
	// by declaring the copy constructor and assignment operator
	// private, but never defining them
private:
	interp_grid(const interp_grid& rhs);
	interp_grid& operator=(const interp_grid& rhs);
	};
