// permutation.cc -- keep track of (random) permutations
// $Header: /numrelcvs/AEIThorns/TestInterpBase/src/permutation.cc,v 1.2 2003/01/17 15:23:13 jthorn Exp $
//
// permutation
//
// permute<T>
// inverse_permute<T>
//
// ***** template instantiations *****
//

#include <stdio.h>

#ifdef TESTINTERP_STANDALONE_BUILD
  #include "fake_cctk.h"
#else
  #include "cctk.h"
#endif

#include "stdc.h"
#include "fp.h"
#include "util.hh"

#include "permutation.hh"

// swap x and y, using temp */
#define SWAP(x,y,temp)		\
	temp = x;		\
	x = y;			\
	y = temp;	/* end macro */

//******************************************************************************
//******************************************************************************
//******************************************************************************

//
// This function constructs a permutation object describing a random
// permutation on N_in objects.
//
permutation::permutation(int N_in)
	: N_(N_in),
	  swaps_(new int[N_in])
{
	for (int i = 0 ; i < N_ ; ++i)
	{
	// random integer in the range [0,i]
	int j = int( (i+1)*jtutil::drandom() );
	swaps_[i] = j;
	}
}

//******************************************************************************
//******************************************************************************
//******************************************************************************

//
// This function applies the (forward) permutation to an N-element array v[].
//
template <typename T>
  void permutation::permute(T v[])
	const
{
T temp;
	for (int i = 0 ; i < N_ ; ++i)
	{
	const int j = swaps_[i];
	SWAP(v[i], v[j], temp);
	}
}

//******************************************************************************

//
// This function applies the inverse permutation to an N-element array v[].
//
template <typename T>
  void permutation::inverse_permute(T v[])
	const
{
T temp;
	for (int i = N_-1 ; i >= 0 ; --i)
	{
	const int j = swaps_[i];
	SWAP(v[i], v[j], temp);
	}
}

//******************************************************************************
//******************************************************************************
//******************************************************************************

//
// ***** template instantiations *****
//

//
// n.b. if CCTK_REAL is the same as (say) CCTK_REAL8, then C++ will
//	complain about duplicate template definnitions if we try to
//	instantiate templates for both of them :( :(
//
#ifdef CCTK_REAL_IS_DISTINCT
template void permutation::permute<CCTK_REAL>(CCTK_REAL v[]) const;
#endif
template void permutation::permute<CCTK_REAL4>(CCTK_REAL4 v[]) const;
template void permutation::permute<CCTK_REAL8>(CCTK_REAL8 v[]) const;
#ifdef CCTK_REAL_IS_DISTINCT
template void permutation::permute<CCTK_COMPLEX>(CCTK_COMPLEX v[]) const;
#endif
template void permutation::permute<CCTK_COMPLEX8>(CCTK_COMPLEX8 v[]) const;
template void permutation::permute<CCTK_COMPLEX16>(CCTK_COMPLEX16 v[]) const;

#ifdef CCTK_REAL_IS_DISTINCT
template void permutation::inverse_permute<CCTK_REAL>(CCTK_REAL v[]) const;
#endif
template void permutation::inverse_permute<CCTK_REAL4>(CCTK_REAL4 v[]) const;
template void permutation::inverse_permute<CCTK_REAL8>(CCTK_REAL8 v[]) const;
#ifdef CCTK_REAL_IS_DISTINCT
template void permutation::inverse_permute<CCTK_COMPLEX>
					  (CCTK_COMPLEX v[])
	const;
#endif
template void permutation::inverse_permute<CCTK_COMPLEX8>
					  (CCTK_COMPLEX8 v[])
	const;
template void permutation::inverse_permute<CCTK_COMPLEX16>
					  (CCTK_COMPLEX16 v[])
	const;
