// permutation.hh -- keep track of (random) permutations
// $Header: /numrelcvs/AEIThorns/TestInterpBase/src/permutation.hh,v 1.1 2003/01/06 12:57:21 jthorn Exp $

//
// A permutation object describes a (random) permutation of the N integers
// [0,N), usually thought of as the elements of various N-elements arrays.
//
// We represent a permutation by a set of N swaps applied in order, i.e.
// by the sequence of j values in the  permute()  algorithm
//	for (int i = 0 ; i < N ; ++i)
//	{
//	// loop invariant: elements [0,i) are already randomized
//	int j = random[0,i]
//	swap(v[i], v[j])
//	}
// Given this, the  inverse_permute()  algorithm is equally simple: just
// apply the same swaps in reverse order.
//
class	permutation
	{
public:
	// number of elements
	int N() const { return N_; }

	// apply permutation to array v[]
	template <typename T>
	  void permute(T v[]) const;

	// apply inverse permutation to array v[]
	template <typename T>
	  void inverse_permute(T v[]) const;

	//
	// ***** constructor, destuctor *****
	//
	permutation(int N_in);	// construct a random permutation on N elements
	~permutation() { delete swaps_; }

private:
	// we forbid copying and passing by value
	// by declaring the copy constructor and assignment operator
	// private, but never defining them
	permutation(const permutation& rhs);
	permutation& operator=(const permutation& rhs);

	//
	// ***** permutation representation *****
	//
private:
	int N_;
	int *swaps_;		// --> new[]-allocated vector giving j values
	};
