// random.cc - generate random numbers
// $Header: /numrelcvs/AEIThorns/TestInterpBase/src/random.cc,v 1.2 2003/01/17 15:23:13 jthorn Exp $
//
// drandom - machine-independent random floating point number in [a,b)
// drandom - machine-independent random floating point number in [0,1)
// get_random_seed - get random number seed
// set_random_seed - set random number seed
//
/// irandom - machine-independent random integer in [0, random_modulus())
//

#include <math.h>
#include <stdlib.h>
#include <limits.h>

#ifdef TESTINTERP_STANDALONE_BUILD
  #include "fake_cctk.h"
#else
  #include "cctk.h"
#endif

#include "stdc.h"
#include "fp.h"
#include "util.hh"

//******************************************************************************

//
// ***** stuff local to this file *****
//

// prototypes
namespace {
int irandom(int& x);
	  }

// magic numbers for random number generator
// ... don't change unless you know what you're doing!
namespace {
const int random_modulus    = 714025;
const int random_multiplier = 1366;
const int random_add        = 150889;
	  }

namespace {
// the seed itself
static int random_seed = 42;	// static ==> keeps state across calls
	  }

//******************************************************************************

//
// This function computes a random number drawn from a uniform
// distribution on [a,b).
//
namespace jtutil
	  {
double drandom(double a, double b)
{
return a + (b-a)*drandom();
}
	  }

//******************************************************************************

//
// This function computes a random number drawn from a uniform
// distribution on [0,1), using 3 calls on the machine-independent
// integer random number generator  irandom() .
//
namespace jtutil
	  {
double drandom()
{
const double dmax = double(random_modulus);
const double dmax3 = jtutil::pow3(dmax);

// random integer-valued floating-point numbers in [0, dmax)
const double d1 = double(irandom(random_seed));
const double d2 = double(irandom(random_seed));
const double d3 = double(irandom(random_seed));

// random integer-valued floating-point number in [0, dmax3)
// (at least modulo roundoff)
const double d = d1 + dmax*(d2 + dmax*d3);

// random floating-point number in [0, 1)
return d / dmax3;
}
	  }

//******************************************************************************

//
// This function gets the current seed value for the  drandom()  random
// number generators.
//
namespace jtutil
	  {
int get_random_seed()
{
return random_seed;
}
	  }

//******************************************************************************

//
// This function gets allows clients to set the current seed value for
// the  drandom()  random number generators.  It returns the old seed.
//
namespace jtutil
	  {
int set_random_seed(int new_seed)
{
const int old_seed = random_seed;
random_seed = new_seed % random_modulus;
return old_seed;
}
	  }

//******************************************************************************
//******************************************************************************
//******************************************************************************

//
// This function computes random integers in the range [0, random_modulus),
// using a machine-independent quick-n-dirty linear congruential method
// from section 7.1 of Numerical Recipes (2nd edition).  This gives a
// rather low quality random number generator, but that doesn't matter
// for our purposes.
//
// Note that the 3 "magic numbers" m, a, and c, are special -- don't
// change these!
//
// Arguments:
// x = (in out) On input, this is the previous random integer.
//		On output, this is the next random integer.
//		This should not be modified between calls to this function.
//
// Results:
// This function returns the updated value of  x .
//
namespace {
int irandom(int& x)
{
x = (x*random_multiplier + random_add) % random_modulus;
return x;
}
	  }
