// test_permutation.cc -- test driver for permutation class
// $Header: /numrelcvs/AEIThorns/TestInterpBase/src/test_permutation.cc,v 1.2 2003/01/17 15:23:13 jthorn Exp $
//
// main
/// initialize_perm
/// print_perm
/// perm_eq_initial
//

#include <stdio.h>
#include <assert.h>
#include <math.h>

#ifdef TESTINTERP_STANDALONE_BUILD
  #include "fake_cctk.h"
#else
  #include "cctk.h"
#endif

#include "stdc.h"
#include "fp.h"
#include "util.hh"

#include "permutation.hh"

//******************************************************************************

//
// ***** prototypes *****
//
namespace {
void initialize_perm(int N, fp v[]);
void print_perm(int N, fp v[]);
bool perm_eq_initial(int N, fp v[]);
	  }

//******************************************************************************

//
// Usage:
//	test_permutation N_elements N_try
//
// This program computes and prints  N_try  random permutations of the
//  N_elements  numbers 0.0, 1.0, 2.0, ..., N_elements-1 .  For each of
// these permutations, it also checks that applying the permutation
// followed by its inverse results in the identity operation.
// 
int main(int argc, const char* const argv[])
{
int N_elements, N_try;
if ( (argc == 3)
     && (sscanf(argv[1], "%d", &N_elements) == 1)
     && (sscanf(argv[2], "%d", &N_try) == 1) )
   then {
	// all ok ==> no-op here
	}
   else {
	fprintf(stderr, "usage: %s N_elements N_try\n", argv[0]);
	exit(1);
	}

fp *const v = new fp[N_elements];

	for (int i = 0 ; i < N_try ; ++i)
	{
	initialize_perm(N_elements, v);
	permutation P(N_elements);
	assert( P.N() == N_elements );
	P.permute(v);
	print_perm(N_elements, v);
	P.inverse_permute(v);
	if (! perm_eq_initial(N_elements, v))
	   then {
		printf("***** trouble with inverse permutation!\n");
		printf("***** (0...N_elements-1) --> perm --> inverse perm -->\n");
		print_perm(N_elements, v);
		fprintf(stderr, "***** trouble with inverse permutation!\n");
		exit(2);
		}
	}

return 0;
}

//******************************************************************************

namespace {
void initialize_perm(int N, fp v[])
{
	for (int i = 0 ; i < N ; ++i)
	{
	v[i] = fp(i);
	}
}
	  }

//******************************************************************************

namespace {
void print_perm(int N, fp v[])
{
	for (int i = 0 ; i < N ; ++i)
	{
	printf("%.0f%s", v[i], ((i == N-1) ? "\n" : " "));
	}
}
	  }

//******************************************************************************

namespace {
bool perm_eq_initial(int N, fp v[])
{
	for (int i = 0 ; i < N ; ++i)
	{
	if (v[i] != fp(i))
	   then return false;
	}
return true;
}
	  }
