c/*@@
c  @file      EOS_1D_Ideal_Polytrope.F
c  @date      May 2000
c  @author    Mark Miller
c  @desc 
c     Routines to calculate Ideal Fluid / Polytrope EOS through EOS_1D_Base
c  @enddesc 
c@@*/

#include "cctk.h"
#include "cctk_Parameters.h"

c/*@@
c  @routine    EOS_1D_IP_Pressurefromrho
c  @date       May 2000
c  @author     Mark Miller
c  @desc 
c    Calculate the pressure  P=(gamma-1)*eps*rho, P = k*rho**gamma
c  @enddesc 
c  @calls     
c  @calledby   
c  @history 
c
c  @endhistory 
c
c@@*/
      CCTK_REAL FUNCTION EOS_1D_IP_pressfromrho(
     .   rho)

      IMPLICIT NONE
      DECLARE_CCTK_PARAMETERS

      CCTK_REAL rho

      EOS_1D_IP_pressfromrho = eos_k * rho**eos_gamma

      END FUNCTION EOS_1D_IP_pressfromrho


c/*@@
c  @routine    EOS_1D_IP_Pressurefromeps
c  @date       May 2000
c  @author     Mark Miller
c  @desc 
c    Calculate the pressure  P=(gamma-1)*eps*rho, P = k*rho**gamma
c  @enddesc 
c  @calls     
c  @calledby   
c  @history 
c
c  @endhistory 
c
c@@*/
      CCTK_REAL FUNCTION EOS_1D_IP_pressfromeps(
     .   eps)

      IMPLICIT NONE
      DECLARE_CCTK_PARAMETERS

      CCTK_REAL eps,rho

      rho = ( (eos_gamma - 1.0d0)*eps/eos_k)**(1.0d0/(eos_gamma - 1.0d0))
      EOS_1D_IP_pressfromeps = (eos_gamma - 1.0d0) *
     .   rho*eps

      END FUNCTION EOS_1D_IP_pressfromeps




c/*@@
c  @routine    EOS_1D_IP_SpecificIEfrompress
c  @date       May 2000
c  @author     Mark Miller
c  @desc 
c    Calculate the specific internal energy  from
c       P=(gamma-1)*eps*rho, P = k*rho**gamma
c  @enddesc 
c  @calls     
c  @calledby   
c  @history 
c
c  @endhistory 
c
c@@*/
      CCTK_REAL FUNCTION EOS_1D_IP_epsfrompress(press)

      IMPLICIT NONE
      DECLARE_CCTK_PARAMETERS

      CCTK_REAL press,rho

      rho = (press/eos_k)**(1.0d0/eos_gamma)
      EOS_1D_IP_epsfrompress = press / 
     .   ( (eos_gamma - 1.0d0)*rho )

      END FUNCTION EOS_1D_IP_epsfrompress

c/*@@
c  @routine    EOS_1D_IP_SpecificIEfromrho
c  @date       May 2000
c  @author     Mark Miller
c  @desc 
c    Calculate the specific internal energy  from
c       P=(gamma-1)*eps*rho, P = k*rho**gamma
c  @enddesc 
c  @calls     
c  @calledby   
c  @history 
c
c  @endhistory 
c
c@@*/
      CCTK_REAL FUNCTION EOS_1D_IP_epsfromrho(rho)

      IMPLICIT NONE
      DECLARE_CCTK_PARAMETERS

      CCTK_REAL rho

      
      EOS_1D_IP_epsfromrho = (eos_k/(eos_gamma - 1.0d0))*
     .   rho**(eos_gamma-1.0d0)

      END FUNCTION EOS_1D_IP_epsfromrho




c/*@@
c  @routine    EOS_1D_IP_RestMassDensfrompress
c  @date       May 2000
c  @author     Mark Miller
c  @desc 
c    Calculate the rest mass density
c       P=(gamma-1)*eps*rho, P = k*rho**gamma
c  @enddesc 
c  @calls     
c  @calledby   
c  @history 
c
c  @endhistory 
c
c@@*/
      CCTK_REAL FUNCTION EOS_1D_IP_rhofrompress(press)

      IMPLICIT NONE
      DECLARE_CCTK_PARAMETERS

      CCTK_REAL press

      EOS_1D_IP_rhofrompress = 
     .   (press/eos_k)**(1.0d0/eos_gamma)

      END FUNCTION EOS_1D_IP_rhofrompress

c/*@@
c  @routine    EOS_1D_IP_RestMassDensfromeps
c  @date       May 2000
c  @author     Mark Miller
c  @desc 
c    Calculate the rest mass density
c       P=(gamma-1)*eps*rho, P = k*rho**gamma
c  @enddesc 
c  @calls     
c  @calledby   
c  @history 
c
c  @endhistory 
c
c@@*/
      CCTK_REAL FUNCTION EOS_1D_IP_rhofromeps(eps)

      IMPLICIT NONE
      DECLARE_CCTK_PARAMETERS

      CCTK_REAL eps

      EOS_1D_IP_rhofromeps = 
     .   ((eos_gamma-1.0d0)*eps/eos_k)**(1.0d0/(eos_gamma-1.0d0))

      END FUNCTION EOS_1D_IP_rhofromeps





