 /*@@
   @file      ParamCheck.c
   @date      November 1999
   @author    Gabrielle Allen
   @desc 
      Check parameters for ADM
   @enddesc 
 @@*/

#include <stdio.h>
#include <stdlib.h>

#include "cctk.h"
#include "cctk_Arguments.h"
#include "cctk_Parameters.h"

static const char *rcsid = "$Header: /cactusdevcvs/CactusEinstein/ADM/src/ParamCheck.c,v 1.12 2004/05/17 12:28:56 goodale Exp $";

CCTK_FILEVERSION(CactusEinstein_ADM_ParamCheck_c)

void ADM_ParamCheck(CCTK_ARGUMENTS);
void ADM_ConformalCheck(CCTK_ARGUMENTS);

 /*@@
   @routine    ParamChecker
   @date       November 1999
   @author     Gabrielle Allen
   @desc 
      Check parameters for ADM
   @enddesc 
   @calls     
   @history 
 
   @endhistory 

@@*/

void ADM_ParamCheck(CCTK_ARGUMENTS)
{
  DECLARE_CCTK_ARGUMENTS;
  DECLARE_CCTK_PARAMETERS;

  if(! CCTK_EQUALS(metric_type, "physical") &&
     ! CCTK_EQUALS(metric_type, "static conformal"))
  {
    CCTK_PARAMWARN("Unknown ADMBase::metric_type - known types are \"physical\" and \"static conformal\"");
  }

  if(CCTK_EQUALS(metric_type, "static conformal") &&
     ! CCTK_EQUALS(conformal_storage, "factor+derivs+2nd derivs"))
  {
    CCTK_PARAMWARN("ADM can currently only work with a physical metric or a static conformal metric with second derivatives");
  }
  

  CCTK_INFO("Evolve using the ADM system");

  /* Hack to use Cartoon boundary conditions for now */
  if (cartoon && !CCTK_IsThornCompiled("Cartoon2D"))
  {
    CCTK_WARN(0,"Cartoon requested but not compiled in");
  }

  if (minimal_communications)
  {
    CCTK_INFO("  using minimal communications");
  }


  if (CCTK_Equals(method,"stagleap")) 
  {
    CCTK_INFO("  with staggered leapfrog");
  }
  else if (CCTK_Equals(method,"leapfrog")) 
  {
    CCTK_INFO("  with double leapfrog");
  }
  else if ((CCTK_Equals(method,"ICN")) || (CCTK_Equals(method,"icn"))) 
  {
    CCTK_INFO("  with iterative Crank-Nicholson");
  }


  /* Check for problems with courant */

  if (CCTK_Equals(timestep_method,"courant_speed") 
      || CCTK_Equals(timestep_method,"courant_time")) 
  {
    if (!timestep_outonly) 
    {
      if (CCTK_Equals(method,"stagleap")) 
      {
        CCTK_WARN(0,"Staggered Leapfrog and Courant are Incompatible");
      }
      else if (CCTK_Equals(method,"leapfrog")) 
      {
        CCTK_WARN(2, "Leapfrog and Courant are untested");
      }
    }
    
  }

}

 /*@@
   @routine    ADM_ConformalCheck
   @date       Fri Apr 26 11:40:36 2002
   @author     Tom Goodale
   @desc 
   Check that the initial data has setup enough derivatives.
   @enddesc 
   @calls     
   @calledby   
   @history 
 
   @endhistory 
 
 @@*/
void ADM_ConformalCheck(CCTK_ARGUMENTS)
{
  DECLARE_CCTK_ARGUMENTS;
  DECLARE_CCTK_PARAMETERS;

  if(CCTK_EQUALS(metric_type, "physical"))
  {
    /* Use this internally so we don't need to check the parameter again. */
    *conformal_state = 0;
  }
  else if(CCTK_EQUALS(metric_type, "static conformal"))
  {
    if(*conformal_state < 3)
    {
      CCTK_WARN(0, "ADM needs second derivatives of the conformal factor when running with a static conformal metric");
    }
  }

}
