
#ifndef ALPSOURCES_GUTS
#define ALPSOURCES_GUTS

#include "CactusEinstein/ADMMacros/src/macro/TRK_guts.h"
#include "CactusEinstein/ADMMacros/src/macro/DDA_guts.h"

c     Source term for zero shift:   d alpha / dt = ADM_gauge*trK

      adms_alp(i,j,k) = ADM_gauge(i,j,k)*TRK_TRK 

c     Add flat-space dissipation.

      if (ADM_flatNablaAlpCoeff /= 0.0D0 ) then
         adms_alp(i,j,k) = adms_alp(i,j,k)
     .      + ADM_flatNablaAlpCoeff*(DDA_DXXDA+DDA_DYYDA+DDA_DZZDA)
      end if

c     Add curved-space dissipation.          
               
      if (ADM_nablaAlpCoeff /= 0.0D0) then

#include "CactusEinstein/ADMMacros/src/macro/NABA_guts.h"

c       put in proper dx and proper dt information.  This is
c       so ugly since ADM_nablaAlpCoeff has only dx*dx, which
c       has to be divided out.

        min_dxfac = min(gxx(i,j,k)*dx*dx,gyy(i,j,k)*dy*dy)
        min_dxfac = min(min_dxfac,gzz(i,j,k)*dz*dz)

        adms_alp(i,j,k) = adms_alp(i,j,k)
     .     + ADM_nablaAlpCoeff*min_dxfac*NABA_NABA/(dx*dx*alp(i,j,k))

      end if

c     Add shift terms.  Notice that here I add shift terms
c     in such a way that the time derivative of the lapse
c     remains proportional to the time derivative of the
c     volume element, even in the presence of a non-zero
c     shift vector.  This means that:
c
c                           /                 __      i \
c     d alpha/dt = f(alpha) | - alpha trK  +  \/  beta  |
c                           \                   i       /
c
c                            /         __      i           \
c                = ADM_gauge | trK  -  \/  beta  /  alpha  |
c                            \           i                 /
c
c     The divergence of the shift is calculated as:
c
c     __      i           i        i
c     \/  beta  =  d  beta  +  beta  d g  /  (2 g)
c       i           i                 i
c
c     with g the determinant of the 3-metric.  The second term in
c     this expression can be calculated either centered or upwinded.


      if (shift_state .ne. 0) then

         if (alp(i,j,k).ne.0.0D0) then

            ALPSOURCES_TEMP1 = - ADM_gauge(i,j,k)/alp(i,j,k)

c           Calculate determinant of spatial metric at
c           nearest neighbours (needed to find derivatives).

            ALPSOURCES_DETG = gxx(i,j,k)*gyy(i,j,k)*gzz(i,j,k)
     &         + 2.0D0*gxy(i,j,k)*gxz(i,j,k)*gyz(i,j,k)
     &         - gxx(i,j,k)*gyz(i,j,k)**2
     &         - gyy(i,j,k)*gxz(i,j,k)**2
     &         - gzz(i,j,k)*gxy(i,j,k)**2

            ALPSOURCES_DETG_IP = gxx(i+1,j,k)*gyy(i+1,j,k)*gzz(i+1,j,k)
     &         + 2.0D0*gxy(i+1,j,k)*gxz(i+1,j,k)*gyz(i+1,j,k)
     &         - gxx(i+1,j,k)*gyz(i+1,j,k)**2
     &         - gyy(i+1,j,k)*gxz(i+1,j,k)**2
     &         - gzz(i+1,j,k)*gxy(i+1,j,k)**2

            ALPSOURCES_DETG_IM = gxx(i-1,j,k)*gyy(i-1,j,k)*gzz(i-1,j,k)
     &         + 2.0D0*gxy(i-1,j,k)*gxz(i-1,j,k)*gyz(i-1,j,k)
     &         - gxx(i-1,j,k)*gyz(i-1,j,k)**2
     &         - gyy(i-1,j,k)*gxz(i-1,j,k)**2
     &         - gzz(i-1,j,k)*gxy(i-1,j,k)**2

            ALPSOURCES_DETG_JP = gxx(i,j+1,k)*gyy(i,j+1,k)*gzz(i,j+1,k)
     &         + 2.0D0*gxy(i,j+1,k)*gxz(i,j+1,k)*gyz(i,j+1,k)
     &         - gxx(i,j+1,k)*gyz(i,j+1,k)**2
     &         - gyy(i,j+1,k)*gxz(i,j+1,k)**2
     &         - gzz(i,j+1,k)*gxy(i,j+1,k)**2

            ALPSOURCES_DETG_JM = gxx(i,j-1,k)*gyy(i,j-1,k)*gzz(i,j-1,k)
     &         + 2.0D0*gxy(i,j-1,k)*gxz(i,j-1,k)*gyz(i,j-1,k)
     &         - gxx(i,j-1,k)*gyz(i,j-1,k)**2
     &         - gyy(i,j-1,k)*gxz(i,j-1,k)**2
     &         - gzz(i,j-1,k)*gxy(i,j-1,k)**2

            ALPSOURCES_DETG_KP = gxx(i,j,k+1)*gyy(i,j,k+1)*gzz(i,j,k+1)
     &         + 2.0D0*gxy(i,j,k+1)*gxz(i,j,k+1)*gyz(i,j,k+1)
     &         - gxx(i,j,k+1)*gyz(i,j,k+1)**2
     &         - gyy(i,j,k+1)*gxz(i,j,k+1)**2
     &         - gzz(i,j,k+1)*gxy(i,j,k+1)**2

            ALPSOURCES_DETG_KM = gxx(i,j,k-1)*gyy(i,j,k-1)*gzz(i,j,k-1)
     &         + 2.0D0*gxy(i,j,k-1)*gxz(i,j,k-1)*gyz(i,j,k-1)
     &         - gxx(i,j,k-1)*gyz(i,j,k-1)**2
     &         - gyy(i,j,k-1)*gxz(i,j,k-1)**2
     &         - gzz(i,j,k-1)*gxy(i,j,k-1)**2

c           FIXME:  Is this right ?
            if (conformal_state .ne. 0) then
c WAS       if (conformal_state /= CONFORMAL_METRIC) then

               ALPSOURCES_DETG = psi(i,j,k)**12*ALPSOURCES_DETG

               ALPSOURCES_DETG_IP = psi(i+1,j,k)**12*ALPSOURCES_DETG_IP
               ALPSOURCES_DETG_IM = psi(i-1,j,k)**12*ALPSOURCES_DETG_IM
               ALPSOURCES_DETG_JP = psi(i,j+1,k)**12*ALPSOURCES_DETG_JP
               ALPSOURCES_DETG_JM = psi(i,j-1,k)**12*ALPSOURCES_DETG_JM
               ALPSOURCES_DETG_KP = psi(i,j,k+1)**12*ALPSOURCES_DETG_KP
               ALPSOURCES_DETG_KM = psi(i,j,k-1)**12*ALPSOURCES_DETG_KM

            end if

c           Add advection term.

            ALPSOURCES_TEMP2 = 0.5D0*ALPSOURCES_TEMP1/ALPSOURCES_DETG

            if (CCTK_Equals(advection,'center').eq.1) then

               adms_alp(i,j,k) = adms_alp(i,j,k)
     &            + 0.5D0*ALPSOURCES_TEMP2
     &            *(betax(i,j,k)*(ALPSOURCES_DETG_IP - ALPSOURCES_DETG_IM)/dx
     &            + betay(i,j,k)*(ALPSOURCES_DETG_JP - ALPSOURCES_DETG_JM)/dy
     &            + betaz(i,j,k)*(ALPSOURCES_DETG_KP - ALPSOURCES_DETG_KM)/dz)

            else if ((CCTK_Equals(advection,'upwind1').eq.1).or.
     &         (i.eq.2).or.(i.eq.cctk_lsh(1)-1).or.
     &         (j.eq.2).or.(j.eq.cctk_lsh(2)-1).or.
     &         (k.eq.2).or.(k.eq.cctk_lsh(3)-1)) then

               if (betax(i,j,k).gt.0.0D0) then
                  adms_alp(i,j,k) = adms_alp(i,j,k)
     &               + ALPSOURCES_TEMP2*betax(i,j,k)
     &               *(ALPSOURCES_DETG_IP - ALPSOURCES_DETG)/dx
               else
                  adms_alp(i,j,k) = adms_alp(i,j,k)
     &               + ALPSOURCES_TEMP2*betax(i,j,k)
     &               *(ALPSOURCES_DETG - ALPSOURCES_DETG_IM)/dx
               end if

               if (betay(i,j,k).gt.0.0D0) then
                  adms_alp(i,j,k) = adms_alp(i,j,k)
     &               + ALPSOURCES_TEMP2*betay(i,j,k)
     &               *(ALPSOURCES_DETG_JP - ALPSOURCES_DETG)/dy
               else
                  adms_alp(i,j,k) = adms_alp(i,j,k)
     &               + ALPSOURCES_TEMP2*betay(i,j,k)
     &               *(ALPSOURCES_DETG - ALPSOURCES_DETG_JM)/dy
               end if

               if (betaz(i,j,k).gt.0.0D0) then
                  adms_alp(i,j,k) = adms_alp(i,j,k)
     &               + ALPSOURCES_TEMP2*betaz(i,j,k)
     &               *(ALPSOURCES_DETG_KP - ALPSOURCES_DETG)/dz
               else
                  adms_alp(i,j,k) = adms_alp(i,j,k)
     &               + ALPSOURCES_TEMP2*betaz(i,j,k)
     &               *(ALPSOURCES_DETG - ALPSOURCES_DETG_KM)/dz
               end if

            else if (CCTK_Equals(advection,'upwind2').eq.1) then

c              For second order upwind we need to find the
c              determinant of the metric two points away as well.

               ALPSOURCES_DETG_IPP = gxx(i+2,j,k)*gyy(i+2,j,k)*gzz(i+2,j,k)
     &            + 2.0D0*gxy(i+2,j,k)*gxz(i+2,j,k)*gyz(i+2,j,k)
     &            - gxx(i+2,j,k)*gyz(i+2,j,k)**2
     &            - gyy(i+2,j,k)*gxz(i+2,j,k)**2
     &            - gzz(i+2,j,k)*gxy(i+2,j,k)**2

               ALPSOURCES_DETG_IMM = gxx(i-2,j,k)*gyy(i-2,j,k)*gzz(i-2,j,k)
     &            + 2.0D0*gxy(i-2,j,k)*gxz(i-2,j,k)*gyz(i-2,j,k)
     &            - gxx(i-2,j,k)*gyz(i-2,j,k)**2
     &            - gyy(i-2,j,k)*gxz(i-2,j,k)**2
     &            - gzz(i-2,j,k)*gxy(i-2,j,k)**2

               ALPSOURCES_DETG_JPP = gxx(i,j+2,k)*gyy(i,j+2,k)*gzz(i,j+2,k)
     &            + 2.0D0*gxy(i,j+2,k)*gxz(i,j+2,k)*gyz(i,j+2,k)
     &            - gxx(i,j+2,k)*gyz(i,j+2,k)**2
     &            - gyy(i,j+2,k)*gxz(i,j+2,k)**2
     &            - gzz(i,j+2,k)*gxy(i,j+2,k)**2

               ALPSOURCES_DETG_JMM = gxx(i,j-2,k)*gyy(i,j-2,k)*gzz(i,j-2,k)
     &            + 2.0D0*gxy(i,j-2,k)*gxz(i,j-2,k)*gyz(i,j-2,k)
     &            - gxx(i,j-2,k)*gyz(i,j-2,k)**2
     &            - gyy(i,j-2,k)*gxz(i,j-2,k)**2
     &            - gzz(i,j-2,k)*gxy(i,j-2,k)**2

               ALPSOURCES_DETG_KPP = gxx(i,j,k+2)*gyy(i,j,k+2)*gzz(i,j,k+2)
     &            + 2.0D0*gxy(i,j,k+2)*gxz(i,j,k+2)*gyz(i,j,k+2)
     &            - gxx(i,j,k+2)*gyz(i,j,k+2)**2
     &            - gyy(i,j,k+2)*gxz(i,j,k+2)**2
     &            - gzz(i,j,k+2)*gxy(i,j,k+2)**2

               ALPSOURCES_DETG_KMM = gxx(i,j,k-2)*gyy(i,j,k-2)*gzz(i,j,k-2)
     &            + 2.0D0*gxy(i,j,k-2)*gxz(i,j,k-2)*gyz(i,j,k-2)
     &            - gxx(i,j,k-2)*gyz(i,j,k-2)**2
     &            - gyy(i,j,k-2)*gxz(i,j,k-2)**2
     &            - gzz(i,j,k-2)*gxy(i,j,k-2)**2

               if (betax(i,j,k).gt.0.0D0) then
                  adms_alp(i,j,k) = adms_alp(i,j,k)
     &               - 0.5D0*ALPSOURCES_TEMP2*betax(i,j,k)
     &               *(3.0D0*ALPSOURCES_DETG - 4.0D0*ALPSOURCES_DETG_IP
     &               + ALPSOURCES_DETG_IPP)/dx
               else
                  adms_alp(i,j,k) = adms_alp(i,j,k)
     &               + 0.5D0*ALPSOURCES_TEMP2*betax(i,j,k)
     &               *(3.0D0*ALPSOURCES_DETG - 4.0D0*ALPSOURCES_DETG_IM
     &               + ALPSOURCES_DETG_IMM)/dx
               end if

               if (betay(i,j,k).gt.0.0D0) then
                  adms_alp(i,j,k) = adms_alp(i,j,k)
     &               - 0.5D0*ALPSOURCES_TEMP2*betay(i,j,k)
     &               *(3.0D0*ALPSOURCES_DETG - 4.0D0*ALPSOURCES_DETG_JP
     &               + ALPSOURCES_DETG_JPP)/dy
               else
                  adms_alp(i,j,k) = adms_alp(i,j,k)
     &               + 0.5D0*ALPSOURCES_TEMP2*betay(i,j,k)
     &               *(3.0D0*ALPSOURCES_DETG - 4.0D0*ALPSOURCES_DETG_JM
     &               + ALPSOURCES_DETG_JMM)/dy
               end if

               if (betaz(i,j,k).gt.0.0D0) then
                  adms_alp(i,j,k) = adms_alp(i,j,k)
     &               - 0.5D0*ALPSOURCES_TEMP2*betaz(i,j,k)
     &               *(3.0D0*ALPSOURCES_DETG - 4.0D0*ALPSOURCES_DETG_KP
     &               + ALPSOURCES_DETG_KPP)/dz
               else
                  adms_alp(i,j,k) = adms_alp(i,j,k)
     &               + 0.5D0*ALPSOURCES_TEMP2*betaz(i,j,k)
     &               *(3.0D0*ALPSOURCES_DETG - 4.0D0*ALPSOURCES_DETG_KM
     &               + ALPSOURCES_DETG_KMM)/dz
               end if

            end if

c           Finally, add coordinate divergence of the shift.

#include "CactusEinstein/ADMMacros/src/macro/DB_guts.h"

            adms_alp(i,j,k) = adms_alp(i,j,k) + ALPSOURCES_TEMP1
     &         *(DXDB_DXDBX + DYDB_DYDBY + DZDB_DZDBZ)

         end if

      end if

#endif
