 /*@@
   @header    ConformalMinkowski.c
   @date      Sun Apr 28 22:58:03 2002
   @author    Gabrielle Allen
   @desc 
   Minkowski initial data with the conformal factor included
   @enddesc
   @version $Header: /cactusdevcvs/CactusEinstein/IDSimple/src/ConformalMinkowski.c,v 1.3 2004/05/17 12:28:57 goodale Exp $
 @@*/

#include "cctk.h"

#include "cctk_Arguments.h"
#include "cctk_Parameters.h"

static const char *rcsid = "$Header: /cactusdevcvs/CactusEinstein/IDSimple/src/ConformalMinkowski.c,v 1.3 2004/05/17 12:28:57 goodale Exp $";

CCTK_FILEVERSION(CactusEinstein_IDSimple_ConformalMinkowski_c)

/********************************************************************
 *********************     Local Data Types   ***********************
 ********************************************************************/

/********************************************************************
 ********************* Local Routine Prototypes *********************
 ********************************************************************/

/********************************************************************
 ***************** Scheduled Routine Prototypes *********************
 ********************************************************************/

void IDSimple_ConformalMinkowski(CCTK_ARGUMENTS);

/********************************************************************
 ********************* Other Routine Prototypes *********************
 ********************************************************************/

/********************************************************************
 *********************     Local Data   *****************************
 ********************************************************************/

/********************************************************************
 *********************     External Routines   **********************
 ********************************************************************/

/*@@
  @routine    IDSimple_ConformalMinkowski
  @date       Sun Apr 28 23:12:18 2002
  @author     Gabrielle Allen
  @desc 
  Scheduled routine to initialise the metric and extrinsic curvature to
  Minkowski space respecting the conformal factor
  @enddesc 
  @calls     
  @calledby   
  @history 
  
  @endhistory 
  
 @@*/
void IDSimple_ConformalMinkowski(CCTK_ARGUMENTS)
{
  int i;
  DECLARE_CCTK_ARGUMENTS;
  DECLARE_CCTK_PARAMETERS;

  for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
  {
    gxx[i] = 1.0;
    gyy[i] = 1.0;
    gzz[i] = 1.0;

    gxy[i] = 0.0;
    gxz[i] = 0.0;
    gyz[i] = 0.0;

    kxx[i] = 0.0;
    kyy[i] = 0.0;
    kzz[i] = 0.0;

    kxy[i] = 0.0;
    kxz[i] = 0.0;
    kyz[i] = 0.0;
  }

  if (CCTK_EQUALS(metric_type, "static conformal"))
  {

    if (CCTK_EQUALS(conformal_storage, "factor"))
    {
      *conformal_state = 1;
      for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
      {
        psi[i] = 1.0;
      }
    }
    else if (CCTK_EQUALS(conformal_storage, "factor+derivs"))
    {
      *conformal_state = 2;
      for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
      {
        psi[i]  = 1.0;
        psix[i] = 0.0;
        psiy[i] = 0.0;
        psiz[i] = 0.0;
      }
    }
    else if (CCTK_EQUALS(conformal_storage, "factor+derivs+2nd derivs"))
    {
      *conformal_state = 3;
      for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
      {
        psi[i]  = 1.0;
        psix[i] = 0.0;
        psiy[i] = 0.0;
        psiz[i] = 0.0;
        psixx[i] = 0.0;
        psixy[i] = 0.0;
        psixz[i] = 0.0;
        psiyy[i] = 0.0;
        psiyz[i] = 0.0;
        psizz[i] = 0.0;
      }
    }

  }

  if (CCTK_ActiveTimeLevels(cctkGH, "ADMBase::metric") > 1)
  {
    for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
    {
      gxx_p[i] = 1.0;
      gyy_p[i] = 1.0;
      gzz_p[i] = 1.0;
      
      gxy_p[i] = 0.0;
      gxz_p[i] = 0.0;
      gyz_p[i] = 0.0;
      
      kxx_p[i] = 0.0;
      kyy_p[i] = 0.0;
      kzz_p[i] = 0.0;
      
      kxy_p[i] = 0.0;
      kxz_p[i] = 0.0;
      kyz_p[i] = 0.0;
    }
  }
  

  if (CCTK_ActiveTimeLevels(cctkGH, "ADMBase::metric") > 2)
  {
    for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
    {
      gxx_p_p[i] = 1.0;
      gyy_p_p[i] = 1.0;
      gzz_p_p[i] = 1.0;
      
      gxy_p_p[i] = 0.0;
      gxz_p_p[i] = 0.0;
      gyz_p_p[i] = 0.0;
      
      kxx_p_p[i] = 0.0;
      kyy_p_p[i] = 0.0;
      kzz_p_p[i] = 0.0;
      
      kxy_p_p[i] = 0.0;
      kxz_p_p[i] = 0.0;
      kyz_p_p[i] = 0.0;
    }
  }
  
}

/********************************************************************
 *********************     Local Routines   *************************
 ********************************************************************/

