 /*@@
   @header    Lapse.c
   @date      Tue May 7 2002
   @author    Gabrielle Allen
   @desc 
   Set up easy lapse choices
   @enddesc
   @version $Header: /cactusdevcvs/CactusEinstein/IDSimple/src/Lapse.c,v 1.2 2004/05/17 12:28:57 goodale Exp $
 @@*/

#include <math.h>

#include "cctk.h"

#include "cctk_Arguments.h"
#include "cctk_Parameters.h"

static const char *rcsid = "$Header: /cactusdevcvs/CactusEinstein/IDSimple/src/Lapse.c,v 1.2 2004/05/17 12:28:57 goodale Exp $";

CCTK_FILEVERSION(CactusEinstein_IDSimple_Lapse_c)

/********************************************************************
 *********************     Local Data Types   ***********************
 ********************************************************************/

/********************************************************************
 ********************* Local Routine Prototypes *********************
 ********************************************************************/

/********************************************************************
 ***************** Scheduled Routine Prototypes *********************
 ********************************************************************/

void IDSimple_LapseGaussian(CCTK_ARGUMENTS);
void IDSimple_LapsePsiMinusTwo(CCTK_ARGUMENTS);
void IDSimple_LapseIsotropic(CCTK_ARGUMENTS);

/********************************************************************
 ********************* Other Routine Prototypes *********************
 ********************************************************************/

/********************************************************************
 *********************     Local Data   *****************************
 ********************************************************************/

/********************************************************************
 *********************     External Routines   **********************
 ********************************************************************/

/*@@
  @routine    IDSimple_LapseGaussian
  @date       Tue May 7 2002
  @author     Gabrielle Allen
  @desc 
  Initialize lapse to gaussian bump (flatty)
  @enddesc 
  @calls     
  @calledby   
  @history 
  
  @endhistory 
  
 @@*/
void IDSimple_LapseGaussian(CCTK_ARGUMENTS)
{
  int i;
  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS

  for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
  {
    alp[i] = 1.0 + gaussian_amplitude*exp(-r[i]*r[i]/gaussian_sigma2);    
  }
}

/*@@
  @routine    IDSimple_LapsePsiMinusTwo
  @date       Tue May 7 2002
  @author     Gabrielle Allen
  @desc 
  Here we use the following expression for the
  initial lapse:

  alp = ((1 - 2s + s*psi)/(psi-s)))^2

  where psi is the conformal factor, s is a cutoff
  value between 0 and 1 (to stop the lapse from becoming
  too small).

  Notice that if s=0, then alp=1/psi^2, while if
  s=1 then alp=1. 
  @enddesc 
  @calls     
  @calledby   
  @history 
  
  @endhistory 
  
 @@*/
void IDSimple_LapsePsiMinusTwo(CCTK_ARGUMENTS)
{
  int i;
  CCTK_REAL scut;

  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS

  scut = sqrt(psiminustwo_cut);

  for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
  {
    alp[i] =  pow((1.0 - 2.0*scut + scut*psi[i])
                  / (psi[i]-scut),2);
  }
}

/*@@
  @routine    IDSimple_LapseIsotropic
  @date       Tue May 7 2002
  @author     Gabrielle Allen
  @desc 
  The isotropic lapse has the form:

  alp = 2/psi - 1

  with psi the conformal factor.

  This is exact for one black hole in
  isotropic coordinates.
  @enddesc 
  @calls     
  @calledby   
  @history 
  
  @endhistory 
  
 @@*/
void IDSimple_LapseIsotropic(CCTK_ARGUMENTS)
{
  int i;

  DECLARE_CCTK_ARGUMENTS

  for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
  {
    alp[i] = 2.0/psi[i] - 1.0;
  }
}

/********************************************************************
 *********************     Local Routines   *************************
 ********************************************************************/

