 /*@@
   @file      ParamChecker.F
   @date      Sun 28 April 2002
   @author    Gabrielle Allen
   @desc 
   Check parameters for simple initial data 
   @enddesc 
   @version $Header: /cactusdevcvs/CactusEinstein/IDSimple/src/ParamChecker.c,v 1.4 2004/05/17 12:28:57 goodale Exp $
 @@*/

#include "cctk.h"

#include "cctk_Arguments.h"
#include "cctk_Parameters.h"

static const char *rcsid = "$Header: /cactusdevcvs/CactusEinstein/IDSimple/src/ParamChecker.c,v 1.4 2004/05/17 12:28:57 goodale Exp $";

CCTK_FILEVERSION(CactusEinstein_IDSimple_ParamChecker_c)

/********************************************************************
 *********************     Local Data Types   ***********************
 ********************************************************************/

/********************************************************************
 ********************* Local Routine Prototypes *********************
 ********************************************************************/

/********************************************************************
 ***************** Scheduled Routine Prototypes *********************
 ********************************************************************/

void ParamChecker(CCTK_ARGUMENTS);

/********************************************************************
 ********************* Other Routine Prototypes *********************
 ********************************************************************/

/********************************************************************
 *********************     Local Data   *****************************
 ********************************************************************/

/********************************************************************
 *********************     External Routines   **********************
 ********************************************************************/

 /*@@
   @routine    IDSimple_ParamChecker
   @date       Sun 28 April 2002
   @author     Gabrielle Allen
   @desc 
   Check parameters for simple initial data 
   @enddesc 
   @calls     
@@*/
void IDSimple_ParamChecker(CCTK_ARGUMENTS)
{
  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS

  /* Conformal Minkowski Initial Data */

  if (CCTK_EQUALS(initial_data, "conformal minkowski"))
  {
    /* Do we know how to deal with this type of metric ? */
    if(! (CCTK_EQUALS(metric_type, "physical") ||
          CCTK_EQUALS(metric_type, "static conformal")))
    {
      CCTK_PARAMWARN("Unknown ADMBase::metric_type - known types are \"physical\" and \"static conformal\"");
    }

    CCTK_INFO("Minkowski (flat) initial data for g and K");
  
    if (CCTK_EQUALS(metric_type, "static conformal"))
    {
      CCTK_INFO("  using conformal metric");
      if (CCTK_EQUALS(conformal_storage,"factor+derivs"))
      {
        CCTK_INFO("  and conformal first derivatives");
      }
      else if (CCTK_EQUALS(conformal_storage,"factor+derivs+2nd derivs"))
      {
        CCTK_INFO("  and conformal first and second derivatives");
      }
      else
      {
        CCTK_INFO("  but no conformal derivatives");
      }
    }
    else
    { 
      CCTK_INFO("  implements non-conformal metric");
    }   
  }

  if (CCTK_EQUALS(initial_lapse,"gaussian"))
  {
    CCTK_INFO("Gaussian initial lapse");
  }
  else if (CCTK_EQUALS(initial_lapse,"isotropic"))
  {
    CCTK_INFO("Isotropic initial lapse");
    /* Do we know how to deal with this type of metric ? */
    if(! CCTK_EQUALS(metric_type, "static conformal"))
    {
      CCTK_PARAMWARN("Unknown ADMBase::metric_type - Isotropic lapse needs\"static conformal\"");
    }
  }  
  else if (CCTK_EQUALS(initial_lapse,"psiminustwo"))
  {
    CCTK_INFO("Psiminustwo initial lapse");
    /* Do we know how to deal with this type of metric ? */
    if(! CCTK_EQUALS(metric_type, "static conformal"))
    {
      CCTK_PARAMWARN("Unknown ADMBase::metric_type - Psiminustwo lapse needs\"static conformal\"");
    }
  }
    
}

/********************************************************************
 *********************     Local Routines   *************************
 ********************************************************************/

