 /*@@
   @file      MaxHelps.c
   @date      Thu Apr 27 16:00:32 2000
   @author    Gerd Lanfermann
   @desc 
     collects routines that can be registered as the conditional
     Time-To-Slice functions. These functions are evaluated by Slicing.c
     and can be used to activate maximal slicing on certtain conditions
     as iteration, time, lapse behavior.
    
   @enddesc 
 @@*/

#include "cctk.h"

#include "cctk_Arguments.h"
#include "cctk_Parameters.h"

#include "Slicing.h"

static const char *rcsid = "$Header: /cactusdevcvs/CactusEinstein/Maximal/src/MaxHelps.c,v 1.6 2004/05/17 12:28:57 goodale Exp $";

CCTK_FILEVERSION(CactusEinstein_Maximal_MaxHelps_c)

/********************************************************************
 *********************     Local Data Types   ***********************
 ********************************************************************/

/********************************************************************
 ********************* Local Routine Prototypes *********************
 ********************************************************************/

/********************************************************************
 ***************** Scheduled Routine Prototypes *********************
 ********************************************************************/

int MaximalHelps(cGH *GH);

/********************************************************************
 ********************    External Routines   ************************
 ********************************************************************/

 /*@@
   @routine    MaximalHelps
   @date       Thu Apr 27 16:02:21 2000
   @author     Gerd Lanfermann
   @desc 
     This routine is registered by Einstein_RegisterTimeToSlice
     and attached to the maximal slicing. Maximal Slicing will be
     activated if this function returns SLICING_YES. In this case
     it depends on iterations.
   @enddesc
   @calls     
   @calledby   
   @history 
 
   @endhistory 

@@*/

int MaximalHelps(cGH *GH) 
{
  DECLARE_CCTK_PARAMETERS
    
  int tts=SLICING_NO;
  int mistart=0, miend=0;

  if (maxhelps_istart>-1) 
  {
    mistart=1;
  }
  if (maxhelps_iend  >-1) 
  {
    miend  =1;
  }

  if (mistart && miend)
  {
    if ((GH->cctk_iteration>=maxhelps_istart)    && 
        (GH->cctk_iteration<=maxhelps_iend)      &&
        (GH->cctk_iteration% maxhelps_ievery==0))
       tts=SLICING_YES;
  }
  else if (mistart)
  {  
    if ((GH->cctk_iteration>=maxhelps_istart)    && 
        (GH->cctk_iteration %maxhelps_ievery==0))
       tts=SLICING_YES;
  }
  else if (miend)
  {
    if ((GH->cctk_iteration<=maxhelps_iend)      && 
        (GH->cctk_iteration %maxhelps_ievery==0))
       tts=SLICING_YES;
  }
  else if (GH->cctk_iteration%maxhelps_ievery==0)
  {
       tts=SLICING_YES;
  }
  else
  {
       tts=SLICING_NO;
  }

  /* Implementing a time trigger works identically. iteration trigger
     should overwrite timetriggers */

  return(tts);
}


/********************************************************************
 *********************     Local Routines   *************************
 ********************************************************************/

