
#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"

      subroutine maximal(CCTK_ARGUMENTS)

      implicit none

      DECLARE_CCTK_ARGUMENTS
      DECLARE_CCTK_PARAMETERS

      integer i,j,k
      integer Mlin_index,Nsrc_index,field_index
      integer metpsi_index(7)
      integer CCTK_Equals
      integer Einstein_GetSlicingHandle
      integer ierr

      CCTK_REAL AbsTol(3),RelTol(3)

c     Check if it is our turn to do maximal.

      if (mod(cctk_iteration,int(max_every)).ne.0) then
         return
      end if

      if (active_slicing_handle.ne.Einstein_GetSlicingHandle("maximal")) then
         return
      end if

c     Perform some basic setup.

      call Maximal_Setup(CCTK_ARGUMENTS)

c     Prepare the call the linear elliptic, by deriving the
c     grid function indices.

      call CCTK_VarIndex (metpsi_index(1), "admbase::gxx")
      call CCTK_VarIndex (metpsi_index(2), "admbase::gxy")
      call CCTK_VarIndex (metpsi_index(3), "admbase::gxz")
      call CCTK_VarIndex (metpsi_index(4), "admbase::gyy")
      call CCTK_VarIndex (metpsi_index(5), "admbase::gyz")
      call CCTK_VarIndex (metpsi_index(6), "admbase::gzz")

      if (conformal_state > 0) then
         call CCTK_VarIndex (metpsi_index(7), "staticconformal::psi")
      end if

      call CCTK_VarIndex (field_index,     "admbase::alp")
      call CCTK_VarIndex (Mlin_index,      "maximal::Mlinear")
      call CCTK_VarIndex (Nsrc_index,      "maximal::Nsource")

      AbsTol(1)= maximal_thresh
      AbsTol(2)= -1
      AbsTol(3)= -1
      
      RelTol(1)= -1
      RelTol(2)= -1
      RelTol(3)= -1

c     Call the Linear Elliptic solver interface; depending on the 
c     chosen boudary for maximal, set the approriate values to the 
c     keys. We could do A LOT of error checking, which I do not do 
c     here: e.g. is a boundary provided?, is the solver provided?,
c     are the values supported?

c     Boundaries.

      if (CCTK_EQUALS(max_bound,"const")) then

         if (conformal_state > 0) then
            call Ell_SetRealKey(ierr,max_const_v0,"EllLinConfMetric::Bnd::Const::V0")
            if (ierr < 0) then
               call CCTK_WARN(1,"Error setting EllLinConfMetric::Bnd::Const::V0")
            end if
         else
            call Ell_SetRealKey(ierr,max_const_v0,"EllLinMetric::Bnd::Const::V0")
            if (ierr < 0) then
               call CCTK_WARN(1,"Error setting EllLinMetric::Bnd::Const::V0")
            end if
         end if

      else if (CCTK_EQUALS(max_bound,"robin")) then

         if (conformal_state > 0) then
            call Ell_SetStrKey(ierr, "yes", "EllLinConfMetric::Bnd::Robin")
            if (ierr < 0) then
               call CCTK_WARN(0,"Error setting EllLinConfMetric::Bnd::Robin")
            end if
            call Ell_SetIntKey(ierr,max_robin_falloff,"EllLinConfMetric::Bnd::Robin::falloff")
            if (ierr < 0) then
               call CCTK_WARN(0,"Error setting EllLinConfMetric::Bnd::Robin::falloff")
            end if
            call Ell_SetRealKey(ierr,max_robin_inf,"EllLinConfMetric::Bnd::Robin::inf")
            if (ierr < 0) then
               call CCTK_WARN(0,"Error setting EllLinConfMetric::Bnd::Robin::inf")
            end if
         else 
            call Ell_SetStrKey(ierr, "yes", "EllLinMetric::Bnd::Robin")
            if (ierr < 0) then
               call CCTK_WARN(0,"Error setting EllLinMetric::Bnd::Robin")
            end if
            call Ell_SetIntKey(ierr,max_robin_falloff,"EllLinMetric::Bnd::Robin::falloff")
            if (ierr < 0) then
               call CCTK_WARN(0,"Error setting EllLinMetric::Bnd::Robin::falloff")
            end if
            call Ell_SetRealKey(ierr,max_robin_inf,"EllLinMetric::Bnd::Robin::inf")
            if (ierr < 0) then
               call CCTK_WARN(0,"Error setting EllLinMetric::Bnd::Robin::inf")
            end if
         end if
      end if

c     Solver.

      call CCTK_SyncGroup(ierr,cctkGH,"admbase::lapse")
      call CCTK_SyncGroup(ierr,cctkGH,"admbase::metric")

      if (CCTK_EQUALS(max_solver,"sor")) then

c        Set any SOR specific keys
         call Ell_SetIntKey(ierr,sor_maxit,"Ell::SORmaxit");
         call Ell_SetStrKey(ierr,"const","Ell::SORaccel");

c        Call SOR solver

         if (conformal_state > 0) then
            call Ell_LinConfMetricSolver(ierr, cctkGH, 
     $           metpsi_index,field_index,Mlin_index,Nsrc_index,
     $           AbsTol,RelTol,"sor")
         else
            call Ell_LinMetricSolver(ierr, cctkGH, 
     $           metpsi_index,field_index,Mlin_index,Nsrc_index,
     $           AbsTol,RelTol,"sor")
         end if
      end if

      if (CCTK_EQUALS(max_solver,"bam")) then
         if (conformal_state > 0) then
            call Ell_LinConfMetricSolver(ierr, cctkGH, 
     $           metpsi_index,field_index,Mlin_index,Nsrc_index,
     $           AbsTol,RelTol,"bam")
         else
            call Ell_LinMetricSolver(ierr, cctkGH, 
     $           metpsi_index,field_index,Mlin_index,Nsrc_index,
     $           AbsTol,RelTol,"bam")
         end if
      end if

      if (CCTK_EQUALS(max_solver,"petsc")) then
         call Ell_LinConfMetricSolver(ierr, cctkGH, 
     $   metpsi_index, field_index,Mlin_index,Nsrc_index,
     $   AbsTol,RelTol,"petsc")
      end if

c     Synchronization and symmetry boundaries.

      call CCTK_SyncGroup(ierr,cctkGH,"admbase::lapse")
      call CartSymGN(ierr,cctkGH,"admbase::lapse")
      
      return
      end


      subroutine initialise_lapse(CCTK_ARGUMENTS)

      implicit none

      DECLARE_CCTK_ARGUMENTS
      DECLARE_CCTK_PARAMETERS

      integer i, j, k

      do k=1,cctk_lsh(3)
        do j=1,cctk_lsh(2)
          do i=1,cctk_lsh(1)
            alp(i,j,k) = 1.d0
          end do
        end do
      end do

      return
      end
