$Path = Union[$Path,{"~/SetTensor"}];
Needs["SetTensor`"];

Clear[foo1,foo2,foo3];
DefineTensor[foo1,{{1},1}];
DefineTensor[foo2,{{2,1},1}];
DefineTensor[foo3,{{2,1,3},1}];

x[1] = x; x[2] = y; x[3] = z;
Dimension = 3;
fsf[x1_] := FactorSquareFree[x1,Trig->False];
md = AddDeps[AddSym2["g"]];
mu = AddDeps[AddSym2["u"]];
InitializeMetric[md,mu,fsf];
SetSqrtDetg[1, Sqrt[AddDeps["det"]]]

(* the basis vectors... *)
Clear[v1];
DefineTensor[v1,{{1},1}];
SetTensor[v1[ua],AddDeps[AddIndex["v1"]]];
SetTensor[v1[la],v1[ub] Metricg[lb,la],Simplify];

Clear[v2];
DefineTensor[v2,{{1},1}];
SetTensor[v2[ua],AddDeps[AddIndex["v2"]]];
SetTensor[v2[la],v2[ub] Metricg[lb,la],Simplify];

Clear[v3];
DefineTensor[v3,{{1},1}];
SetTensor[v3[ua],AddDeps[AddIndex["v3"]]];
SetTensor[v3[la],v3[ub] Metricg[lb,la],Simplify];

fd = OpenWrite["ortho-norm.h"];
(* Step 1, normalize v1 *)
norm = Sqrt[Simplify[EvalMT[v1[ua] v1[la]]]];
Iter[foo1[ua],
	fwri[fd,v1[ua],v1[ua]/norm];
	];

(* Step 2, project off v1 from v2 *)
fac = Simplify[EvalMT[v1[ua] v2[la]]];
Iter[foo1[ua],
	fwri[fd,v2[ua],fsf[v2[ua]-fac v1[ua]]];
	];

(* Step 3, normalize v2 *)
norm = Sqrt[Simplify[EvalMT[v2[ua] v2[la]]]];
Iter[foo1[ua],
	fwri[fd,v2[ua],v2[ua]/norm];
	];

(* Step 4, assign v3 *)
Iter[foo1[ud],
	val = Epsilon[uc,la,lb] v1[ua] v2[ub];
	val = EvalMT[val,{uc->ud}];
	fwri[fd,v3[ud],fsf[val]];
	];
Close[fd];

Clear[Elec,Mag,Ric,Exc];
DefineTensor[Elec,"Elec",{{2,1},1}];
SetTensor[Elec[la,lb],AddDeps[AddSym2["elec"]]];
DefineTensor[Ric,"Ric",{{2,1},1}];
SetTensor[Ric[la,lb],AddDeps[AddSym2["ric"]]];
DefineTensor[Mag,"Mag",{{2,1},1}];
SetTensor[Mag[la,lb],AddDeps[AddSym2["mag"]]];
DefineTensor[Exc,"Exc",{{2,1},1}];
SetTensor[Exc[la,lb],AddDeps[AddSym2["h"]]];

Clear[divk];
DefineTensor[divk,"divk",{{2,1,3},1}];
tmp = AddSym2["h"];
tmp = AddIndex[Prepend,tmp];
tmp = AddStrs["cd",tmp];
tmp = AddDeps[tmp];
SetTensor[divk[la,lb,lc],tmp];
fd = OpenWrite["BandE.h"];
Iter[foo3[le,lf,lg],
	tmp = CD[Exc[la,lb],lc];
	tmp = EvalMT[tmp,{la->le,lb->lf,lc->lg}];
	fwri[fd,divk[le,lf,lg],tmp];
];

Iter[foo2[lf,lg],
	tmp = Ric[la,lb]-Exc[la,lc] Exc[lb,ld] Metricg[uc,ud]+
		AddDeps["trk"] Exc[la,lb];
	tmp = EvalMT[tmp,{la->lf,lb->lg}];
	eetmp[lf,lg]=tmp;
	fwri[fd,Elec[lf,lg],tmp];
	
	tmp = Epsilon[la,uc,ud] divk[ld,lb,lc];
	tmp = EvalMT[tmp,{la->lf,lb->lg}];
	tmp = fsf[tmp];
	mmtmp[lf,lg]=tmp;
	fwri[fd,Mag[lf,lg],tmp];
];
Close[fd];

fd = OpenWrite["Newman-Penrose.h"];
f[x1_,x2_] := Elec[x1,x2]+I Mag[x1,x2];
mv[x1_] := (v2[x1]-I v3[x1])/Sqrt[2];
mb[x1_] := (v2[x1]+I v3[x1])/Sqrt[2];
ef[x1_] := fsf[EvalMT[x1]];

psiStrs = ToString /@ {psi0,psi1,psi2,psi3,psi4};
psiVals = {
	f[la,lb] mv[ua] mv[ub],
	f[la,lb] mv[ua] v1[ub]/Sqrt[2],
	f[la,lb] v1[ua] v1[ub]/2,
	f[la,lb] mb[ua] v1[ub]/Sqrt[2],
	f[la,lb] mb[ua] mb[ub]
	};

For[i=1,i<=5,i++,
	val = EvalMT[psiVals[[i]] ];

	reval = ComplexExpand[val];
	reval = reval /. Complex[x1_,x2_]->x1;
	reval = reval /. {Im[_]->0,Re[x1_]->x1};

	imval = ComplexExpand[-I val];
	imval = imval /. Complex[x1_,x2_]->x1;
	imval = imval /. {Im[_]->0,Re[x1_]->x1};

	renm = ToExpression[psiStrs[[i]] <> "re"];
	imnm = ToExpression[psiStrs[[i]] <> "im"];
	fwri[fd,imnm,imval];
	fwri[fd,renm,reval];
	savr[i]=reval;
	savi[i]=imval;
]

ival = psi0[x,y,z] psi4[x,y,z]-4 psi1[x,y,z] psi3[x,y,z]+3 psi2[x,y,z]^2;
jval = Det[{
{psi0[x,y,z],psi1[x,y,z],psi2[x,y,z]},
{psi1[x,y,z],psi2[x,y,z],psi3[x,y,z]},
{psi2[x,y,z],psi3[x,y,z],psi4[x,y,z]}}];
isav = ival;
jsav = jval;
reim = {
	Im[x1_[x,y,z]] :> ToExpression[ToString[x1]<>"im[x,y,z]"],
	Re[x1_[x,y,z]] :> ToExpression[ToString[x1]<>"re[x,y,z]"]
	};
ival = ComplexExpand[ival] /. reim;
ival = ExpandAll[ival];
fwri[fd,icurvre,ival /. Complex[x1_,x2_]->x1];
ival = ExpandAll[-I*ival];
fwri[fd,icurvim,ival /. Complex[x1_,x2_]->x1];

jval = ComplexExpand[jval] /. reim;
jval = ExpandAll[jval];
fwri[fd,jcurvre,jval /. Complex[x1_,x2_]->x1];
jval = ExpandAll[-I*jval];
fwri[fd,jcurvim,jval /. Complex[x1_,x2_]->x1];

Close[fd];
