c=======================================================================
c pnd_sub.f 
c     part of PND routine   
c     common to stand-alone pnd9806.f and thorn_PsiKadelia
c     by Hisaaki Shinkai, shinkai@wurel.wustl.edu
c                            
c-----------------------------------------------------------------------
c  CACTUS version: file name      = pnd_sub.F   
c                  thorn_pninit   = version 0.9    19990618
c                  to be called   = pnd.F
c                  to call subs   = 
c
c  note that all variables are not double precision
c
c-----------------------------------------------------------------------
c=======================================================================
c input  ... 
c output ... Z1R,Z2R,Z3R,Z4R,Z1I,Z2I,Z3I,Z4I,iflag4
c 19980602
c-----------------------------------------------------------------------
#include "cctk.h"

      subroutine pnd_solve_quartic(
     &                  ps0,ps1,ps2,ps3,ps4,zeroc,tinyc,pi,
     &                  z1r,z2r,z3r,z4r,z1i,z2i,z3i,z4i,iflag)
      implicit NONE


      complex   ps0,ps1,ps2,ps3,ps4,zeroc,tinyc
      CCTK_REAL      pi
      CCTK_REAL      z1r,z2r,z3r,z4r,z1i,z2i,z3i,z4i
      integer   iflag

c locally defined
      complex   z1,z2,z3,z4


c############################################################ index 15 #
c  part 15.   Solution of eq. ps4*z^4+4*ps3*z^3+6*ps2*z^2+4*ps1*z+ps0=0
c#######################################################################
c=======================================================================
c   CASE 1a. (ps4.NEQ.0.0)   4-ji houteishiki 
c   Solution of eq.   ps4*z^4+4*ps3*z^3+6*ps2*z^2+4*ps1*z+ps0=0
c=======================================================================
      if(ps4.NE.zeroc) then
         write(*,*) " ps4.neq.0 : solve quartic : iflag=1",ps4
         call houteishiki4(ps4,ps3,ps2,ps1,ps0,zeroc,tinyc,pi,
     &                     z1,z2,z3,z4)
         iflag=1
c=======================================================================
c   CASE 1b. (ps0.NEQ.0.0)   4-ji houteishiki 
c   Solution of eq.   ps4*z^4+4*ps3*z^3+6*ps2*z^2+4*ps1*z+ps0=0
c=======================================================================
      elseif(ps0.NE.zeroc) then
         write(*,*) " ps0.neq.0 : solve quartic : iflag=2",ps0
         call houteishiki4(ps0,ps1,ps2,ps3,ps4,zeroc,tinyc,pi,
     &                     z1,z2,z3,z4)
         iflag=2
      else 
c=======================================================================
c   CASE 2. (ps0=ps4=0)     3-ji houteishiki 
c   Solution of eq.           4*ps3*z^3+6*ps2*z^2+4*ps1*z+ps0=0
c=======================================================================
         if(ps3.ne.zeroc) then 
           write(*,*) " ps0=ps4=0 and ps3 nonzero : psi3=",ps3
           iflag=1
           z1=(3.0*ps2+csqrt(9.0*ps2**2-16.0*ps1*ps3+tinyc))/4.0/ps3
           z2=(3.0*ps2-csqrt(9.0*ps2**2-16.0*ps1*ps3+tinyc))/4.0/ps3
           z1=(3.0*ps2+sqrt(9.0*ps2**2-16.0*ps1*ps3+tinyc))/4.0/ps3
           z2=(3.0*ps2-sqrt(9.0*ps2**2-16.0*ps1*ps3+tinyc))/4.0/ps3
           z3=zeroc
           z4=zeroc
         elseif(ps1.ne.zeroc) then 
           write(*,*) " ps0=ps4=0 and ps1 nonzero : psi1=",ps1
           iflag=2
           z1=(3.0*ps2+csqrt(9.0*ps2**2-16.0*ps1*ps3+tinyc))/4.0/ps1
           z2=(3.0*ps2-csqrt(9.0*ps2**2-16.0*ps1*ps3+tinyc))/4.0/ps1
           z1=(3.0*ps2+sqrt(9.0*ps2**2-16.0*ps1*ps3+tinyc))/4.0/ps1
           z2=(3.0*ps2-sqrt(9.0*ps2**2-16.0*ps1*ps3+tinyc))/4.0/ps1
           z3=zeroc
           z4=zeroc
         elseif(ps2.ne.zeroc) then 
           write(*,*) " ps0=ps1=ps3=ps4=0 typeD : psi2= ",ps2
           iflag=3
           z1=zeroc
           z2=zeroc
           z3=zeroc
           z4=zeroc
         else
           write(*,*) " ps0=ps1=ps2=ps3=ps4=0 flat"
           iflag=4
           z1=zeroc
           z2=zeroc
           z3=zeroc
           z4=zeroc
         endif
      endif

      z1r= real(z1)
      z1i=aimag(z1)
      z2r= real(z2)
      z2i=aimag(z2)
      z3r= real(z3)
      z3i=aimag(z3)
      z4r= real(z4)
      z4i=aimag(z4)
c$$$      print *,z1r,z1i,z2r,z2i,z3r,z3i,z4r,z4i

      return
      end

c=======================================================================
c=======================================================================
c-----------------------------------------------------------------------
c input  ... c4,c3,c2,c1,c0
c            c4*z**4 + 4*c3*z**3 + 6*c2*z**2 + 4*c1*z +c0 =0
c output ... z1,z2,z3,z4
c-----------------------------------------------------------------------

      subroutine houteishiki4(c4,c3,c2,c1,c0,zeroc,tinyc,pi,
     &                        z1,z2,z3,z4)

      implicit none

      complex c4,c3,c2,c1,c0,z1,z2,z3,z4,zeroc,tinyc
      CCTK_REAL pi
      
c locally defined
      complex pndPSI,pndPSJ,pndPSH,pndPSG,RM1,RM2,RM3,RE23,RE43,root,
     &        root3,ZALP,ZBET,ZGAM
      CCTK_REAL    RNR,RNI,RND,RND3,THET
      integer MORINO
      
c-----------------------------------------------------------------------
      pndPSI=c4*c0-4.0*c1*c3+3.0*c2*c2
      pndPSJ=c4*c2*c0+c3*c1*c2+c2*c3*c1-c2*c2*c2-c1*c1*c4-c3*c3*c0
      pndPSH=c4*c2-c3*c3
      pndPSG=c4*c4*c1-3.0*c4*c3*c2+2.0*c3*c3*c3

c-----------------------------------------------------------------------
c   Solution of eq.   RM^3 - PSI * RM + 2 PSJ =0
c   CL: Laurens  notation
c-----------------------------------------------------------------------
       if((pndPSI.EQ.zeroc).AND.(pndPSJ.EQ.zeroc)) then
c----------- 3-ji houteishiki no kai =0
         RM1=zeroc
         RM2=zeroc
         RM3=zeroc
       else
c----------- 3-ji houteishiki no kai wo motomeru  >>
         RE23=cmplx(cos(2.0*pi/3.0),sin(2.0*pi/3.0))
         RE43=cmplx(cos(4.0*pi/3.0),sin(4.0*pi/3.0))
c
         if(pndPSJ*pndPSJ-pndPSI*pndPSI*pndPSI/27.0.eq.zeroc) then
           root = pndPSJ
         else
           root = pndPSJ
     &        +csqrt(pndPSJ*pndPSJ-pndPSI*pndPSI*pndPSI/27.0+tinyc)
           root = pndPSJ
     &        +sqrt(pndPSJ*pndPSJ-pndPSI*pndPSI*pndPSI/27.0+tinyc)
         endif
         if(root.EQ.zeroc) then
c          komaru node toriaezu
           MORINO=1
           if(pndPSJ*pndPSJ-pndPSI*pndPSI*pndPSI/27.0.eq.zeroc) then
           root=-pndPSJ
           else
           root=-pndPSJ
     &          +csqrt(pndPSJ*pndPSJ-pndPSI*pndPSI*pndPSI/27.0+tinyc)
           root=-pndPSJ
     &          +sqrt(pndPSJ*pndPSJ-pndPSI*pndPSI*pndPSI/27.0+tinyc)
           endif
         else
           MORINO=0
         end if
c        PRINT *,"root=       ",root
c        PRINT *,"morino=     ",MORINO
c-------- ROOT=ROOT**(1.0/3.0)  --->>
          RNR= real(root)
          RNI=aimag(root)
          RND=sqrt(RNR*RNR+RNI*RNI)
          RND3=RND**(1.0/3.0)
          THET=acos(RNR/RND)
          root3=cmplx(cos(THET/3.0),sin(THET/3.0))
          root3=RND3*root3
c         print *,"ROOT^(1/3)=  ",root3
c-----------------------------------<<
        if(MORINO.EQ.0) then
          if(pndPSJ.eq.zeroc) then
               RM1=zeroc
            if(pndPSI.eq.zeroc) then
               RM2=zeroc
               RM3=zeroc
            else
               RM2= csqrt(pndPSI+tinyc)
               RM3=-csqrt(pndPSI+tinyc)
               RM2= sqrt(pndPSI+tinyc)
               RM3=-sqrt(pndPSI+tinyc)
            endif
          else
            RM1=-(     root3+     pndPSI/root3/3.0 )
            RM2=-(RE23*root3+RE43*pndPSI/root3/3.0 )
            RM3=-(RE43*root3+RE23*pndPSI/root3/3.0 )
          end if
        else
            RM1=-(     root3+     pndPSI/root3/3.0 )
            RM2=-(RE23*root3+RE43*pndPSI/root3/3.0 )
            RM3=-(RE43*root3+RE23*pndPSI/root3/3.0 )
        end if
       end if
c----------- 3-ji houteishiki no kai ga motomaru  <<
c       print *,"RM1 ",RM1
c       print *,"RM2 ",RM2
c       print *,"RM3 ",RM3 
c=======================================================================
c   Solution of eq.   PS4*z^4+4*PS3*z^3+6*PS2*z^2+4*PS1*z+PS0=0
c=======================================================================
       if(2.0*c4*RM1-4.0*pndPSH.eq.zeroc) then
          ZALP=zeroc
       else
          ZALP=csqrt(2.0*c4*RM1-4.0*pndPSH+tinyc)
          ZALP=sqrt(2.0*c4*RM1-4.0*pndPSH+tinyc)
       endif
       if(2.0*c4*RM2-4.0*pndPSH.eq.zeroc) then
          ZBET=zeroc
       else
          ZBET=csqrt(2.0*c4*RM2-4.0*pndPSH+tinyc)
          ZBET=sqrt(2.0*c4*RM2-4.0*pndPSH+tinyc)
       endif
       if(2.0*c4*RM3-4.0*pndPSH.eq.zeroc) then
          ZGAM=zeroc
       else
          ZGAM=csqrt(2.0*c4*RM3-4.0*pndPSH+tinyc)
          ZGAM=sqrt(2.0*c4*RM3-4.0*pndPSH+tinyc)
       endif
       
c signature check    ZALP*ZBET*ZGAM=4*pndPSG ??   
       if(cabs( ZALP*ZBET*ZGAM-4.0*pndPSG).gt.
     &    cabs(-ZALP*ZBET*ZGAM-4.0*pndPSG)) then
          ZGAM=-ZGAM
       endif
c$$$       print *,"ZALP*ZBET*ZGAM=",ZALP*ZBET*ZGAM
c$$$       print *,"    4.0*pndPSG=",4.0*pndPSG  
       
c       ZGAM=1.0*pndPSG/ZALP/ZBET
c       ZGAM=csqrt(-ZALP*ZALP-ZBET*ZBET-12.0*pndPSH+tinyc)
c?      if(pndPSG.LE.0.0) ZGAM=-ZGAM
c
        Z1=-(c3+0.5*( ZALP+ZBET+ZGAM))/c4
        Z2=-(c3+0.5*( ZALP-ZBET-ZGAM))/c4
        Z3=-(c3+0.5*(-ZALP+ZBET-ZGAM))/c4
        Z4=-(c3+0.5*(-ZALP-ZBET+ZGAM))/c4
c        print *," z =",z1,z2,z3,z4  

      return
      end

c#######################################################################
c input  ... z1r,z1i,...,iflag
c output ... pnd1X, ...
c   19980602
c-----------------------------------------------------------------------
      subroutine zi2pnd(z1r,z1i,z2r,z2i,z3r,z3i,z4r,z4i,iflag,
     &     pnd1t,pnd1x,pnd1y,pnd1z,pnd2t,pnd2x,pnd2y,pnd2z,
     &     pnd3t,pnd3x,pnd3y,pnd3z,pnd4t,pnd4x,pnd4y,pnd4z)

      implicit none
      CCTK_REAL pnd1t,pnd1x,pnd1y,pnd1z,pnd2t,pnd2x,pnd2y,pnd2z,
     &     pnd3t,pnd3x,pnd3y,pnd3z,pnd4t,pnd4x,pnd4y,pnd4z,
     &     z1r,z1i,z2r,z2i,z3r,z3i,z4r,z4i
      CCTK_REAL one,two,zero
      integer iflag

      one=1.0
      two=2.0
      zero=0.0

      if(iflag.eq.1) then
c based on p^A = zi iota^A + o^A
         pnd1t=z1r**2+z1i**2-one
         pnd1x= two*z1r
         pnd1y=-two*z1i
         pnd1z=one-z1r**2-z1i**2
c     
         pnd2t=z2r**2+z2i**2-one
         pnd2x= two*z2r
         pnd2y=-two*z2i
         pnd2z=one-z2r**2-z2i**2
c     
         pnd3t=z3r**2+z3i**2-one
         pnd3x= two*z3r
         pnd3y=-two*z3i
         pnd3z=one-z3r**2-z3i**2
c     
         pnd4t=z4r**2+z4i**2-one
         pnd4x= two*z4r
         pnd4y=-two*z4i
         pnd4z=one-z4r**2-z4i**2
      else if(iflag.eq.2) then
c based on p^A = iota^A + zi o^A
         pnd1t=z1r**2+z1i**2+one
         pnd1x= two*z1r
         pnd1y= two*z1i
         pnd1z=z1r**2+z1i**2-one
c     
         pnd2t=z2r**2+z2i**2+one
         pnd2x= two*z2r
         pnd2y= two*z2i
         pnd2z=z2r**2+z2i**2-one
c     
         pnd3t=z3r**2+z3i**2+one
         pnd3x= two*z3r
         pnd3y= two*z3i
         pnd3z=z3r**2+z3i**2-one
c     
         pnd4t=z4r**2+z4i**2+one
         pnd4x= two*z4r
         pnd4y= two*z4i
         pnd4z=z4r**2+z4i**2-one
      else if(iflag.eq.3) then
c type D  along l^a and n^a
         pnd1t=one
         pnd1x=zero
         pnd1y=zero
         pnd1z=one
         pnd2t=one
         pnd2x=zero
         pnd2y=zero
         pnd2z=-one
         pnd3t=zero
         pnd3x=zero
         pnd3y=zero
         pnd3z=zero
         pnd4t=zero
         pnd4x=zero
         pnd4y=zero
         pnd4z=zero
      else if(iflag.eq.4) then
c flat
         pnd1t=zero
         pnd1x=zero
         pnd1y=zero
         pnd1z=zero
         pnd2t=zero
         pnd2x=zero
         pnd2y=zero
         pnd2z=zero
         pnd3t=zero
         pnd3x=zero
         pnd3y=zero
         pnd3z=zero
         pnd4t=zero
         pnd4x=zero
         pnd4y=zero
         pnd4z=zero
      else
         call CCTK_WARN(0,"Something wrong in iflag: zi2pnd.F/thorn_PsiKadelia")
      endif

      return
      end



c#######################################################################
c input  ... pnd1X, ..., OX(1), ...
c output ... pnd1X, ...
c   19980602
c-----------------------------------------------------------------------
c23456      
      subroutine tetrad_express(pnd1X,pnd1Y,pnd1Z,pnd2X,pnd2Y,pnd2Z,
     &                          pnd3X,pnd3Y,pnd3Z,pnd4X,pnd4Y,pnd4Z,
     &                          OX1,OX2,OX3,OY1,OY2,OY3,OZ1,OZ2,OZ3)

      implicit none
      CCTK_REAL pnd1X,pnd1Y,pnd1Z,pnd2X,pnd2Y,pnd2Z,
     &                       pnd3X,pnd3Y,pnd3Z,pnd4X,pnd4Y,pnd4Z,
     &                       OX1,OX2,OX3,OY1,OY2,OY3,OZ1,OZ2,OZ3
      CCTK_REAL Ppnd1X,Ppnd1Y,Ppnd1Z,Ppnd2X,Ppnd2Y,Ppnd2Z
      CCTK_REAL Ppnd3X,Ppnd3Y,Ppnd3Z,Ppnd4X,Ppnd4Y,Ppnd4Z


         Ppnd1X=pnd1X
         Ppnd1Y=pnd1Y
         Ppnd1Z=pnd1Z
         Ppnd2X=pnd2X
         Ppnd2Y=pnd2Y
         Ppnd2Z=pnd2Z
         Ppnd3X=pnd3X
         Ppnd3Y=pnd3Y
         Ppnd3Z=pnd3Z
         Ppnd4X=pnd4X
         Ppnd4Y=pnd4Y
         Ppnd4Z=pnd4Z

         pnd1X= Ppnd1X*OX1+Ppnd1Y*OY1+Ppnd1Z*OZ1
         pnd1Y= Ppnd1X*OX2+Ppnd1Y*OY2+Ppnd1Z*OZ2
         pnd1Z= Ppnd1X*OX3+Ppnd1Y*OY3+Ppnd1Z*OZ3
         pnd2X= Ppnd2X*OX1+Ppnd2Y*OY1+Ppnd2Z*OZ1
         pnd2Y= Ppnd2X*OX2+Ppnd2Y*OY2+Ppnd2Z*OZ2
         pnd2Z= Ppnd2X*OX3+Ppnd2Y*OY3+Ppnd2Z*OZ3
         pnd3X= Ppnd3X*OX1+Ppnd3Y*OY1+Ppnd3Z*OZ1
         pnd3Y= Ppnd3X*OX2+Ppnd3Y*OY2+Ppnd3Z*OZ2
         pnd3Z= Ppnd3X*OX3+Ppnd3Y*OY3+Ppnd3Z*OZ3
         pnd4X= Ppnd4X*OX1+Ppnd4Y*OY1+Ppnd4Z*OZ1
         pnd4Y= Ppnd4X*OX2+Ppnd4Y*OY2+Ppnd4Z*OZ2
         pnd4Z= Ppnd4X*OX3+Ppnd4Y*OY3+Ppnd4Z*OZ3

         return
         end

c=======================================================================
c#######################################################################
c input  ... z1r,z1i
c output ... Ppnd1X,Ppnd1Y,Ppnd1Z
c
c  Result is interpreted as
c       Ppnd1X * x^a + Ppnd1Y * y^a  + Ppnd1Z * z^a
c  
      subroutine pnd_direc(z1r,z1i,Ppnd1X,Ppnd1Y,Ppnd1Z,tinyR,pi)

      implicit none
      CCTK_REAL z1r,z1i,Ppnd1X,Ppnd1Y,Ppnd1Z,tinyR,pi
      CCTK_REAL zr,zrsq,zang,zan

     
      zr=z1r**2+z1i**2
      if(zr.eq.0.0) zrsq=0.0
      if(zr.ne.0.0) zrsq=sqrt(zr+1.0e-15)

      if(zrsq.LE.tinyR) then
        Ppnd1X=0.0
        Ppnd1Y=0.0
        Ppnd1Z=0.0
      else
        if(z1r.EQ.0.0) then
          if(z1i.GE.0.0) zan= pi/2.0
          if(z1i.LT.0.0) zan=-pi/2.0
        else
             zang=abs(z1i/z1r)
             if((z1r.GT.0.0).AND.(z1i.GE.0.0)) zan= atan(zang)
             if((z1r.GT.0.0).AND.(z1i.LT.0.0)) zan=-atan(zang)
             if((z1r.LT.0.0).AND.(z1i.GE.0.0)) zan=-atan(zang)+pi
             if((z1r.LT.0.0).AND.(z1i.LT.0.0)) zan= atan(zang)-pi
        end if
        Ppnd1X= 2.0*zrsq*cos(zan)/(1.0+zr)
        Ppnd1Y=-2.0*zrsq*sin(zan)/(1.0+zr)
        Ppnd1Z=(1.0-zr)/(1.0+zr)
      end if

      return
      end


c=======================================================================
c########################################################### 19980618 ##
c input  ... ps0, .., ps4,petrov_eps
c output ... npetrov,PETROV
      subroutine petrov_judge(ps0,ps1,ps2,ps3,ps4,petrov_eps,
     &                        npetrov, PETROV)

      implicit none

c input output
      character*6  PETROV
      integer      npetrov
      CCTK_REAL         petrov_eps

      complex   ps0,ps1,ps2,ps3,ps4

c locally defined
  
      complex   pndPSI,pndPSJ,pndPSH,pndPSG,PSL,PSK,PSN,pchk1
      CCTK_REAL      PSIAB,PSJAB,PSKAB,PSLAB,PSNAB


C############################################################ index 13 #
C  part 13.   Penrose Scalar Invariants PSI,PSJ,PSH,PSG
C#######################################################################
      pndPSI=PS4*PS0-4.0*PS1*PS3+3.0*PS2*PS2
      pndPSJ=PS4*PS2*PS0+PS3*PS1*PS2+PS2*PS3*PS1
     $   -PS2*PS2*PS2-PS1*PS1*PS4-PS3*PS3*PS0
      pndPSH=PS4*PS2-PS3*PS3
      pndPSG=PS4*PS4*PS1-3.0*PS4*PS3*PS2+2.0*PS3*PS3*PS3

c$$$      if(y.eq.0.AND.z.eq.0) write(98,520) x,aimag(pndPSI)
c$$$      if(y.eq.0.AND.z.eq.0) write(99,520) x,real(pndPSI)
c############################################################ index 14 #
c  part 14.   Petrov type classification
c             R. A. d Inverno and R. A. Russel-Clark, 
c             J. Math. Phys 12 (1971) 1258
c#######################################################################
c  PETROV:  I  II  D  III  N  O(flat)
c npetrov:  1   2  5    3  4  0
c
      PSL=pndPSH
      PSK=pndPSG
      PSN=12.0*PSL*PSL-PS4*PS4*pndPSI
c
      npetrov=9
c       print *,pndPSI**pndPSI*pndPSI-27.0*pndPSJ**2
        pchk1=pndPSI*pndPSI*pndPSI-27.0*pndPSJ*pndPSJ
      if(cabs(pchk1).GT.petrov_eps) then
        PETROV="     I"
        npetrov=1
      else 
        PSIAB=cabs(pndPSI)
        PSJAB=cabs(pndPSJ)
        PSKAB=cabs(PSK)
        PSLAB=cabs(PSL)
        PSNAB=cabs(PSN)

        if((PSIAB.LE.petrov_eps).AND.(PSJAB.LE.petrov_eps)) then
          if((PSKAB.LE.petrov_eps).AND.(PSLAB.LE.petrov_eps)) then
            PETROV="     N"
            npetrov=4
          else
            PETROV="   III"
            npetrov=3
          end if
        else
          if((PSKAB.LE.petrov_eps).AND.(PSNAB.LE.petrov_eps)) then
            PETROV="     D"
            npetrov=5
          else
            PETROV="    II"
            npetrov=2
          end if
        end if
      end if
      if(npetrov.eq.9) then
         write(*,*) "no petrov type selection: pnd_sub/petrov_judge"
         stop
      endif

      return
      end

c=======================================================================
c########################################################### 19980618 ##
c input  ... npetrov,x,y,z,dim
c output ... 
      subroutine petrov_output_f90(npetrov,x,y,z,dim)

      implicit none
      integer npetrov,i_lasttime,dim
      CCTK_REAL x,y,z

c  PETROV:  I  II  D  III  N  O(flat)
c npetrov:  1   2  5    3  4  0
      if(dim.eq.2) then
             if(npetrov.eq.0) then
                write(20,'("Text[O,{",f15.6," , ",f15.6," }]")') x,y
         elseif(npetrov.eq.1) then 
                write(20,'("Text[1,{",f15.6," , ",f15.6," }]")') x,y
         elseif(npetrov.eq.2) then
                write(20,'("Text[2,{",f15.6," , ",f15.6," }]")') x,y
         elseif(npetrov.eq.3) then
                write(20,'("Text[3,{",f15.6," , ",f15.6," }]")') x,y
         elseif(npetrov.eq.4) then
                write(20,'("Text[N,{",f15.6," , ",f15.6," }]")') x,y
         elseif(npetrov.eq.5) then
                write(20,'("Text[D,{",f15.6," , ",f15.6," }]")') x,y
         endif 
      elseif(dim.eq.3) then
             if(npetrov.eq.0) then
       write(20,'("Text[O,{",f15.6," , ",f15.6," , ",f15.6," }]")') x,y,z
         elseif(npetrov.eq.1) then 
       write(20,'("Text[1,{",f15.6," , ",f15.6," , ",f15.6," }]")') x,y,z
         elseif(npetrov.eq.2) then
       write(20,'("Text[2,{",f15.6," , ",f15.6," , ",f15.6," }]")') x,y,z
         elseif(npetrov.eq.3) then
       write(20,'("Text[3,{",f15.6," , ",f15.6," , ",f15.6," }]")') x,y,z
         elseif(npetrov.eq.4) then
       write(20,'("Text[N,{",f15.6," , ",f15.6," , ",f15.6," }]")') x,y,z
         elseif(npetrov.eq.5) then
       write(20,'("Text[D,{",f15.6," , ",f15.6," , ",f15.6," }]")') x,y,z
         endif

      else
         write(*,*) "wrong output dimension: pnd_sub/petrov_output"
         stop
      endif
         
      return
      end
