////////////////////////////////////////////////////////////////////////////////////////////////////
//
// $Id: NetGeodesics.cc,v 1.14 2004/12/30 13:34:27 schnetter Exp $
// $Log: NetGeodesics.cc,v $
// Revision 1.14  2004/12/30 13:34:27  schnetter
// Add missing #include <stdio.h> statement.
//
// Revision 1.13  2003/10/28 13:48:24  schnetter
// Change the way in which pointers are passed to and from Fortran.
//
// Revision 1.12  2001/05/08 21:20:28  allen
//
// CCTK_FILEVERSION
//
// Revision 1.11  2000/07/19 12:24:46  allen
//
// Fixing compiler warnings
//
// Revision 1.10  2000/06/19 13:41:59  werner
// make it compileable under IRIX again
//
// Revision 1.9  2000/05/25 17:03:36  werner
// "math.h" is not required, and conflicts with C++ on SunOS
//
// Revision 1.8  2000/05/10 12:01:52  allen
//
// New names for symmetry boundary conditions
//
// New macros for calling fortran from C
//
// Revision 1.7  2000/04/17 14:52:53  goodale
//
// Moving DECLARE_CCTK_PARAMETERS outside an ifdef.
//
// Tom
//
// Revision 1.6  2000/04/04 10:22:42  allen
//
// MPI -> CCTK_MPI
//
// Revision 1.5  2000/02/10 13:46:30  werner
// bugfixed Thomas' bugfix - wrong array was gathered.
//
// Revision 1.4  2000/02/04 11:57:48  tradke
//
// Bugfixes:
//
// 1) number of local geodesics was passed as an CCTK_INT but expected as INTEGER
// 2) MPI_Gather() got incompatible datatypes (which made the code crashing on T3E)
// 3) fixed warnings for uninitialized variables if no thorn Exact is there
//
// Geodesics should be fine now for multiprocessor case.
//
// But I've observed code instabilities for very small grid sizes or if you do
// a lot of iterations. Under Linux and on origin the code continues, T3E crashes
// with numeric execption.
// Anyway, the parameter file provided by TimeGeodesic runs through.
//
//
// Thomas
//
// Revision 1.3  2000/01/27 07:46:14  allen
//
// cctk_parameters -> cctk_Parameters
// cctk_arguments -> cctk_Arguments
//
// Revision 1.2  1999/12/14 13:23:35  werner
// Support for timelike downsampling of netgeodesics (out_every).
//
// Revision 1.1.1.1  1999/11/12 14:13:53  allen
// TimeGeodesic
//
// Revision 1.6  1999/11/12 11:00:37  goodale
//
// Fixed a small bug in uniprocessor code.
//
// The MPI version now produces the same results as the uniprocessor one
// when run with one processor.  With more than one processor there
// are slight differences in the last decimal place.
//
// Tom
//
// Revision 1.5  1999/11/11 21:44:14  werner
// Slightly reordering of output, somewhat simpler now.
//
// Revision 1.4  1999/11/11 16:52:03  goodale
//
// Parallelised version.  Works the same as the old version
// when compiled without MPI.  Seems to have some data
// corruption in the data sending when compiled with MPI.
// Will try further to track this down later.
//
// Tom
//
// Revision 1.3  1999/11/11 12:37:01  werner
// This probably fixes the memory leak problem (memory is automatically released
// by the destructor, no need to do this explicetly) and the geodesic counting.
// Some comments how to parallelize
//
// Revision 1.2  1999/11/09 23:27:12  werner
// Streaming geodesics out to the Viz implemented and activated, works!
//
// Revision 1.1  1999/11/08 18:35:48  werner
// Preliminiary interface to network extensions.
// Still couple of open technical questions here.
//
//
/////////////////////////////////////////////////////////////////////////////////////////////////////

//#define DEBUG_OUTPUT

#include <stdio.h>

#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"
#include "cctk_FortranString.h"
#include "cctk_DefineThorn.h"

#ifdef CCTK_MPI
#include "mpi.h"
#endif

static char *rcsid = "$Header: /cactusdevcvs/CactusEinstein/TimeGeodesic/src/NetGeodesics.cc,v 1.14 2004/12/30 13:34:27 schnetter Exp $";

CCTK_FILEVERSION(CactusEinstein_TimeGeodesic_NetGeodesics_cc)


/*
  On sun's, math.h conflicts with the definition of `exception' in C++.
*/
#ifndef	 sun
#include <math.h>
#endif

#ifdef	CACTUSNET_HTTP
#include "CactusNet/http/src/http_GHExtensions.hpp"
#endif

#ifdef	CACTUSNET_NETGEOMETRY
#include "CactusNet/NetGeometry/src/NetGeometry_GHExtensions.hpp"

#include <Geometry.hpp>
#endif	

static void OutputGeodesics(cGH *GH, 
                            const CCTK_INT lgeos, 
                            const CCTK_REAL*geotau, 
                            const CCTK_REAL*XS, const CCTK_REAL*YS, const CCTK_REAL*ZS, 
                            const CCTK_REAL*PXS, const CCTK_REAL*PYS, const CCTK_REAL*PZS, 
                            const CCTK_REAL*geoflag);

static void CollectGeodesics(cGH *GH, 
                             CCTK_INT lgeos, 
                             CCTK_REAL*geotau, 
                             CCTK_REAL*XS, CCTK_REAL*YS, CCTK_REAL*ZS, 
                             CCTK_REAL*PXS, CCTK_REAL*PYS, CCTK_REAL*PZS, 
                             CCTK_REAL*geoflag,
                             CCTK_INT  &total_geos, 
                             CCTK_REAL *&total_geotau, 
                             CCTK_REAL *&total_XS, CCTK_REAL *&total_YS, CCTK_REAL *&total_ZS, 
                             CCTK_REAL *&total_PXS, CCTK_REAL *&total_PYS, CCTK_REAL *&total_PZS, 
                             CCTK_REAL *&total_geoflag);

/// Just called before the integration starts
extern "C" void CCTK_FCALL CCTK_FNAME(UpdateGeodesics)(cGH**GH, CCTK_INT*lgeos, CCTK_REAL*geotau, 
				CCTK_REAL*XS, CCTK_REAL*YS, CCTK_REAL*ZS, 
				CCTK_REAL*PXS, CCTK_REAL*PYS, CCTK_REAL*PZS, 
				CCTK_REAL*geoflag);

/// Just called after integration has been done
extern "C" void  CCTK_FCALL CCTK_FNAME(OutputGeodesics)(cGH**GH, CCTK_INT*lgeos, CCTK_REAL*geotau, 
				CCTK_REAL*XS, CCTK_REAL*YS, CCTK_REAL*ZS, 
				CCTK_REAL*PXS, CCTK_REAL*PYS, CCTK_REAL*PZS, 
				CCTK_REAL*geoflag);

void  CCTK_FCALL CCTK_FNAME(UpdateGeodesics)(cGH**GH, CCTK_INT*lgeos, CCTK_REAL*geotau, 
				CCTK_REAL*XS, CCTK_REAL*YS, CCTK_REAL*ZS, 
				CCTK_REAL*PXS, CCTK_REAL*PYS, CCTK_REAL*PZS, 
				CCTK_REAL*geoflag)
{

  /*
#ifdef	CACTUSNET_HTTP
http_GHExtension*httpGH = http_GH_Ext(GH);
#endif 
  */

/*
  Some code to come here to receive a `Command' structure from http's Command stack
  and to set up / change the mentioned geodesics. Possibly involves broadcasting the
  current command structure to other procs.
  However, the available commands are not clear yet, so we delay this to somwhen later 
  todo.
   WB
 */

}



 /*@@
   @routine    OutputGeodesics
   @date       Wed Nov 10 13:31:42 1999
   @author     Tom Goodale
   @desc 
   Collects all the geodesic data to processor zero and outputs it.
   @enddesc 
   @calls     
   @calledby   
   @history 
 
   @endhistory 

@@*/
extern "C" void CCTK_FCALL CCTK_FNAME(OutputGeodesics)(cGH**pGH, CCTK_INT*lgeos, CCTK_REAL*geotau, 
		     CCTK_REAL*XS, CCTK_REAL*YS, CCTK_REAL*ZS, 
		     CCTK_REAL*PXS, CCTK_REAL*PYS, CCTK_REAL*PZS, 
		     CCTK_REAL*geoflag)
{
	DECLARE_CCTK_PARAMETERS
        cGH* GH=*pGH;

	if ( (GH->cctk_iteration % out_every) != 0)
		return;

  CCTK_INT total_lgeos;
  CCTK_REAL *total_geotau;
  CCTK_REAL *total_XS, *total_YS, *total_ZS;
  CCTK_REAL *total_PXS, *total_PYS, *total_PZS;
  CCTK_REAL *total_geoflag;

  CollectGeodesics(GH, 
                   *lgeos, geotau, XS, YS, ZS, PXS, PYS, PZS, geoflag,
                   total_lgeos, total_geotau, total_XS, total_YS, total_ZS, 
                   total_PXS, total_PYS, total_PZS, total_geoflag);

  if(CCTK_MyProc(GH) == 0)
  {

#ifdef DEBUG_OUTPUT
    for(int i=0; i < total_lgeos; i++)
    {
      printf("%d: %f %f %f %f %f %f %f %f\n", i, total_geotau[i], 
	                             total_XS[i], total_YS[i], total_ZS[i],
	                             total_PXS[i], total_PYS[i], total_PZS[i],
	                             total_geoflag[i]);
    }
#endif

    OutputGeodesics(GH, total_lgeos, total_geotau, total_XS, total_YS, total_ZS, 
                    total_PXS, total_PYS, total_PZS, total_geoflag);
  }

  delete [] total_geotau;
  delete [] total_XS;
  delete [] total_YS;
  delete [] total_ZS;
  delete [] total_PXS; 
  delete [] total_PYS;
  delete [] total_PZS;

}


 /*@@
   @routine    CollectGeodesics
   @date       Wed Nov 10 13:31:04 1999
   @author     Tom Goodale
   @desc 
   Collects the geodesics onto processor zero.
   @enddesc 
   @calls     
   @calledby   
   @history 
 
   @endhistory 

@@*/
#ifdef CCTK_INTEGER_PRECISION_8
#define NET_MPI_INT  (sizeof (int) == 8 ? MPI_INT : MPI_LONG)
#elif  CCTK_INTEGER_PRECISION_4
#define NET_MPI_INT  (sizeof (int) == 8 ? MPI_SHORT : MPI_INT)
#elif  CCTK_INTEGER_PRECISION_2
#define NET_MPI_INT  MPI_SHORT
#endif

#define NET_MPI_REAL4     MPI_FLOAT
#ifdef SINGLE_PRECISION
#define NET_MPI_REAL      MPI_FLOAT
#else
#define NET_MPI_REAL      MPI_DOUBLE
#endif

#ifdef CCTK_MPI

#define NET_MPI_ERROR(xf)  do {int errcode; \
                                    if((errcode = xf) != MPI_SUCCESS)                     \
				    {                                                     \
				      char mpi_error_string[MPI_MAX_ERROR_STRING+1];      \
				      int resultlen;                                      \
				      MPI_Error_string(errcode, mpi_error_string, &resultlen);\
				      fprintf(stderr, "MPI Call %s returned error code %d (%s)\n", \
                                      #xf, errcode, mpi_error_string);                    \
				      fprintf(stderr, "At line %d of file %s\n",                   \
					     __LINE__, __FILE__);                         \
				    }                                                     \
				  } while (0)
#endif

static void CollectGeodesics(cGH *GH, 
                             CCTK_INT lgeos, 
                             CCTK_REAL*geotau, 
                             CCTK_REAL*XS, CCTK_REAL*YS, CCTK_REAL*ZS, 
                             CCTK_REAL*PXS, CCTK_REAL*PYS, CCTK_REAL*PZS, 
                             CCTK_REAL*geoflag,
                             CCTK_INT  &total_geos, 
                             CCTK_REAL *&total_geotau, 
                             CCTK_REAL *&total_XS, CCTK_REAL *&total_YS, CCTK_REAL *&total_ZS, 
                             CCTK_REAL *&total_PXS, CCTK_REAL *&total_PYS, CCTK_REAL *&total_PZS, 
                             CCTK_REAL *&total_geoflag)
{

  int *ngeos_array   = new int [CCTK_nProcs(GH)];
  int *ngeos_offsets = new int [CCTK_nProcs(GH)];

  total_geos = lgeos;
  // Find the total number of geodesics
#ifdef CCTK_MPI 

#if 1
  {
  CCTK_INT *ngeos_array_CCTK_INT = new CCTK_INT [CCTK_nProcs(GH)]; 
	NET_MPI_ERROR(MPI_Gather(&lgeos, 1, NET_MPI_INT, ngeos_array_CCTK_INT, 1, NET_MPI_INT, 0, MPI_COMM_WORLD)); 

	/* copy the CCTK_INTs into an int-array which MPI_Gatherv() expects later */ 
	for (int i = 0; i < CCTK_nProcs(GH); i++)
		ngeos_array [i] = ngeos_array_CCTK_INT [i]; 
	delete [] ngeos_array_CCTK_INT;
  } 
#else 
  //  This version should do the same as above, but is yet unchecked on T3E.
  int lgeos_int = lgeos; 
      NET_MPI_ERROR(MPI_Gather(&lgeos_int, 1, MPI_INT, ngeos_array, 1, MPI_INT, 0, MPI_COMM_WORLD)); 
#endif

  ngeos_offsets[0] = 0;

  if(CCTK_MyProc(GH) == 0)
  {
    for(int i=1; i < CCTK_nProcs(GH); i++)
    {
      total_geos += ngeos_array[i];
      ngeos_offsets[i] = ngeos_offsets[i-1]+ngeos_array[i-1];
    }
  }

#endif

  total_geotau  = new CCTK_REAL [total_geos];
  total_XS      = new CCTK_REAL [total_geos];
  total_YS      = new CCTK_REAL [total_geos];
  total_ZS      = new CCTK_REAL [total_geos];
  total_PXS     = new CCTK_REAL [total_geos];
  total_PYS     = new CCTK_REAL [total_geos];
  total_PZS     = new CCTK_REAL [total_geos];
  total_geoflag = new CCTK_REAL [total_geos];

#ifdef CCTK_MPI

  NET_MPI_ERROR(MPI_Gatherv(geotau, lgeos, NET_MPI_REAL, 
			    total_geotau, ngeos_array, ngeos_offsets,
			    NET_MPI_REAL, 0, MPI_COMM_WORLD));
  
  NET_MPI_ERROR(MPI_Gatherv(XS, lgeos, NET_MPI_REAL, 
			    total_XS, ngeos_array, ngeos_offsets,
			    NET_MPI_REAL, 0, MPI_COMM_WORLD));

  NET_MPI_ERROR(MPI_Gatherv(YS, lgeos, NET_MPI_REAL, 
			    total_YS, ngeos_array, ngeos_offsets,
			    NET_MPI_REAL, 0, MPI_COMM_WORLD));
  
  NET_MPI_ERROR(MPI_Gatherv(ZS, lgeos, NET_MPI_REAL, 
			    total_ZS, ngeos_array, ngeos_offsets,
			    NET_MPI_REAL, 0, MPI_COMM_WORLD));
  
  NET_MPI_ERROR(MPI_Gatherv(PXS, lgeos, NET_MPI_REAL, 
			    total_PXS, ngeos_array, ngeos_offsets,
			    NET_MPI_REAL, 0, MPI_COMM_WORLD));
  
  NET_MPI_ERROR(MPI_Gatherv(PYS, lgeos, NET_MPI_REAL, 
			    total_PYS, ngeos_array, ngeos_offsets,
			    NET_MPI_REAL, 0, MPI_COMM_WORLD));
  
  NET_MPI_ERROR(MPI_Gatherv(PZS, lgeos, NET_MPI_REAL, 
			    total_PZS, ngeos_array, ngeos_offsets,
			    NET_MPI_REAL, 0, MPI_COMM_WORLD));
  
  NET_MPI_ERROR(MPI_Gatherv(geoflag, lgeos, NET_MPI_REAL, 
			    total_geoflag, ngeos_array, ngeos_offsets,
			    NET_MPI_REAL, 0, MPI_COMM_WORLD));
  
#else
  
  for(int i=0; i < lgeos; i++)
  {
    total_geotau[i] = geotau[i];
    total_XS[i]      = XS[i];
    total_YS[i]      = YS[i];
    total_ZS[i]      = ZS[i];
    total_PXS[i]     = PXS[i];
    total_PYS[i]     = PYS[i];
    total_PZS[i]     = PZS[i];
    total_geoflag[i] = geoflag[i];
  }

  delete [] ngeos_array;
  delete [] ngeos_offsets;
#endif
  
}

#define isValidGeodesicPoint(j) \
		( ( geoflag[j] > 0) && 		\
			!isnan( geotau[j] ) &&	\
			!isnan( XS    [j] ) &&	\
			!isnan( YS    [j] ) &&	\
			!isnan( ZS    [j] ) )


 /*@@
   @routine    OutputGeodesics
   @date       Wed Nov 10 13:32:17 1999
   @author     Werner Benger
   @desc 
   Sends geodesic data down a socket.
   @enddesc 
   @calls     
   @calledby   
   @history 
 
   @endhistory 

@@*/
static void OutputGeodesics(cGH *GH, 
                            const CCTK_INT lgeos, 
                            const CCTK_REAL*geotau, 
                            const CCTK_REAL*XS, const CCTK_REAL*YS, const CCTK_REAL*ZS, 
                            const CCTK_REAL*PXS, const CCTK_REAL*PYS, const CCTK_REAL*PZS, 
                            const CCTK_REAL*geoflag)
{
                   
	DECLARE_CCTK_PARAMETERS

#ifdef	CACTUSNET_NETGEOMETRY
	


	  // Get DataSender structure

NetGeometry_GHExtension*NetGeomGH = NetGeometry_GH_Extension(GH); 
DataSender*DS = NetGeomGH ? NetGeomGH->datasocket : 0; 

	if (!DS)
		return; 

	  /// 1. check how many geodesics actually are available 

int	Ngeodesics = 0;

	for(int i=0; i < lgeos; i++)
	{
		if ( isValidGeodesicPoint(i) )
			Ngeodesics++;
	} 


Geometry    OutData; 

	strcpy( OutData.gridfunc, "geodesics");

	OutData.timeslice = (GH)->cctk_iteration;

	  //
	  // reserve memory for (t,x,y,z) for all available geodesics 
	  //
float	*Vertices4D = new CCTK_REAL4[ 4*Ngeodesics ]; 

	  //
	  // also for array of available indices 
	  // 
CCTK_INT4*avl = new CCTK_INT4[Ngeodesics]; 

	  //
	  // These values are not interpreted for geodesics on the viz side. 
	  // Maybe we will give them some meaning somewhen later 
	  //
	OutData.instance = out_every; 
	OutData.data_min = 0; 
	OutData.data_max = 1; 
	OutData.level    = 0.5; 

	  //
	  // Now collect data from all arrays into the (t,x,y,z)[] structure 
	  // and set the curveindices array: This is actually an 
	  // array containing all the numbers of available geodesics, 
	  // for which the Vertices4D have been sent. 
	  //
int	g = 0;
	for(int j=0; j < lgeos; j++)	
	{
		if ( isValidGeodesicPoint(j) )
		{
			Vertices4D[4*g + 0] = geotau[j]; 
			Vertices4D[4*g + 1] = XS    [j]; 
			Vertices4D[4*g + 2] = YS    [j]; 
			Vertices4D[4*g + 3] = ZS    [j]; 
			avl[g] = j;

#if 0 // debug mod
			printf("%d: (%g,%g,%g,%g)\n", j,
			       Vertices4D[4*g + 0],
			       Vertices4D[4*g + 1],
			       Vertices4D[4*g + 2],
			       Vertices4D[4*g + 3] );
#endif

			g++;
		}
	} 

	  //
	  // Now actually send the geodesic vertices to the client 
	  //
	OutData.geom = new GeometryData<CCTK_REAL4>(Vertices4D, 4*Ngeodesics); 
	OutData.meaning = Geometry::Vertices4D; 

	if (!OutData.send( DS ) )
	{
		puts("*** ERROR *** in geodesic vertices send!!!");
	} 

	  // 
	  // actually release temporarily required memory 
	  //
	delete OutData.geom; 


	  // 
	  // Now actually send which geodesics are available
	  //
	OutData.geom = new GeometryData<CCTK_INT4>(avl, Ngeodesics); 
	OutData.meaning = Geometry::CurveIndices; 

	  //
	  // Now actually send the geodesic indices to the client 
	  //
	if (!OutData.send( DS ) )
	{
		puts("*** ERROR *** in geodesic indices send!!!");
	} 	

	  //
	  // memory from OutData is automatically released by the 
	  // destructor ~OutData()
	  //
#endif
}

