#include "ArrayDistribution.h"

/********************************
 *      ArrayDistribution       *
 ********************************/
Boolean ArrayDistribution::equal(ArrayDistribution *) { return NO; }

int ArrayDistribution::distribution_type()
{
  printf("In ArrayDistributon: distribution_type, shouldn't be called\n");
  return -1;
}

ArrayDistribution::ArrayDistribution()
{
  num_of_chunks_ = 0;
  chunk_list_ = new List();
  current_cell_ = NULL;
}

ArrayDistribution::ArrayDistribution(int **schema_buf) 
{
  printf("In ArrayDistributon: init, shouldn't be called\n");
}

void ArrayDistribution::add_last(Chunk *new_chunk)
{
  chunk_list_->add_last(new_chunk);
  num_of_chunks_++;
}

ArrayDistribution::~ArrayDistribution()
{
  Cell *list_ptr;
  if (chunk_list_) {
    list_ptr = chunk_list_->head_;
    while (list_ptr) {
      delete list_ptr->item();
      list_ptr = list_ptr->next();
    }
    delete chunk_list_;
    chunk_list_ = NULL; num_of_chunks_ = 0;
  }
}

void ArrayDistribution::pack(int **schema_buf)
{
  printf("In ArrayDistributon: pack, shouldn't be called\n");
}

int ArrayDistribution::get_next_index(Chunk *&chunk, int old_val, 
				      int io_node_num,
                                      int num_io_nodes, int max)
{
  printf("In ArrayDistributon: get_next_index shouldn't be called\n");
  return max;
}

List *ArrayDistribution::chunk_list()
{
  return chunk_list_;
}

Chunk* ArrayDistribution::get_next_chunk()
{
  if (current_cell_) current_cell_ = current_cell_->next();
  else current_cell_ = chunk_list_->head_;

  if (current_cell_) return ((Chunk *)current_cell_->item());
  return NULL;
}

int ArrayDistribution::total_elements()
{
  printf("In ArrayDistributon: total_elements shouldn't be called\n");
  return 0;
}

ChunkAllocPolicy ArrayDistribution::alloc_policy()
{
  printf("In ArrayDistributon: alloc_policy, shouldn't be called\n");
  return ROUND_ROBIN;
}

void ArrayDistribution::list_clear() { current_cell_ = NULL; }

/********************************
 *      RegularDistribution     *
 ********************************/
RegularDistribution::RegularDistribution(int rank, ArrayLayout *layout,
                                         Distribution *dist,
                                         ChunkAllocPolicy alloc_policy,
                                         Block_Distribution block_dist)
  : ArrayDistribution()
{
  layout_ = new ArrayLayout(layout);
  rank_ = rank;
  alloc_policy_ = alloc_policy;
  dist_ = copy_distribution(rank_, dist);
  block_dist_ = block_dist;
}

RegularDistribution::RegularDistribution(int **schema_buf) : ArrayDistribution()
{
  int* ptr = *schema_buf;
  layout_ = new ArrayLayout(&ptr);
  rank_ = *ptr++;
  dist_ = new_distribution(&ptr, rank_);
  alloc_policy_ = (ChunkAllocPolicy)*ptr++;
  block_dist_ = (Block_Distribution)*ptr++;
  *schema_buf = ptr;
}

RegularDistribution::~RegularDistribution()
{
  if (layout_) { delete layout_; layout_ = NULL; }
  if (dist_ ) { free(dist_); dist_ = NULL; }
}

Boolean RegularDistribution::equal(ArrayDistribution *that)
{
  if (!that) return NO;

  RegularDistribution *tmp;
  if (that->distribution_type() == Regular)
    tmp = (RegularDistribution *)that;
  else return NO;

  if (layout_->equal(tmp->layout_) &&
      equal_distribution(rank_, dist_, tmp->dist_)) return YES;
  return NO;
}

ArrayLayout *RegularDistribution::layout()
{
  return layout_;
}

Distribution *RegularDistribution::distribution()
{
  return dist_;
}

void RegularDistribution::pack(int **schema_buf)
{
  int* ptr = *schema_buf;

  *ptr++ = (int)Regular;
  layout_->pack(&ptr);
  *ptr++ = rank_;
  pack_distribution(&ptr, rank_, dist_);
  *ptr++ = (int)alloc_policy_;
  *ptr++ = block_dist_;
  *schema_buf = ptr;
}

int RegularDistribution::distribution_type()
{
  return Regular;
}

int RegularDistribution::total_elements()
{
  return layout_->total_elements();
}

ChunkAllocPolicy RegularDistribution::alloc_policy() { return alloc_policy_; }

int RegularDistribution::get_next_index(Chunk *&chunk, int old_val, 
					int io_node_num,
                                        int num_io_nodes, int max)
{
  if (old_val == -1) return io_node_num;
  else return (old_val + num_io_nodes);
}

Block_Distribution RegularDistribution::block_dist() { return block_dist_; }

/********************************
 *    IrregularDistribution     *
 ********************************/
int IrregularDistribution::distribution_type()
{
  return Irregular;
}

int IrregularDistribution::total_elements()
{
  return num_of_chunks_;
}

int IrregularDistribution::get_next_index(Chunk *&chunk, 
                                          int old_val, int io_node_num,
                                          int num_io_nodes, int max)
{
  chunk = get_next_chunk();
  if (chunk == NULL) return max;
  return chunk->chunk_id();
}

IrregularDistribution::IrregularDistribution(int num, Chunk **chunk_list)
			: ArrayDistribution()
{
  for (int i=0; i<num; i++) add_last(chunk_list[i]); 
}
