#include "definitions.h"
#include "ArrayLayout.h"

ArrayLayout::ArrayLayout(int Rank, int *sizearray):Template(Rank, sizearray){}

/* Create an arraylayout object using info stored in the schema buffer */
ArrayLayout::ArrayLayout(int **schema_buf)
{
  int* ptr = *schema_buf;
  
  rank_ = *ptr++;
  size_ = (int *) malloc(sizeof(int)*rank_);
  for(int i=0; i < rank_; i++)
	size_[i] = *ptr++;

  *schema_buf = ptr;
}

/* Make a copy of an existing ArrayLayout object */
ArrayLayout::ArrayLayout(ArrayLayout *old_layout)
{
   rank_ = old_layout->rank();
   size_ = copy_int_list(rank_, old_layout->size());
}

/* Use the destructor of  the Template object */
ArrayLayout::~ArrayLayout()
{
}

/* converts a chunk index to a number */
int ArrayLayout::convert_from_index_to_number(int *indices)
{
   int result=0, temp_product=1;
   for(int i=rank_-1; i>=0; i--)
   {
      result += temp_product * indices[i];
      temp_product *= size_[i];
   }
   return result;
}
  
/* converts a number to the appropriate chunk index */
void  ArrayLayout::convert_from_number_to_index(int num, int *result)
{
  int  temp_product=1;
  int  i, j;
  
  for(i = 0; i< rank_; i++)
  {
     temp_product = 1;
     for(j = i+1 ; j < rank_; j++)
         temp_product *= size_[j];
     result[i] = num / temp_product;
     num -= num/temp_product *temp_product;
  }
}
    
/* converts a number to the appropriate chunk index */  
int* ArrayLayout::convert_from_number_to_index(int num)
{
  int* result = (int *) malloc(sizeof(int)*rank_);
  convert_from_number_to_index(num, result);
  return result;
}

/* Check if the input indices are valid. Assumes that the rank of *
 * input indices are the same as rank of the layout               */
Boolean ArrayLayout::valid_index(int *indices)
{
  if (indices == NULL) return NO;
  else for(int i=0; i<rank_; i++)
       {  
           if ((indices[i] < 0) || (indices[i] >= size_[i]))
		return NO;
       }
  return YES;
}

/* Checks if the specified input distribution of the array is   *
 * compatible with the layout. It is compatible only if the     *
 * number of dimensions in which the array is distributed in a  *
 * BLOCK or CYCLIC fashion is equal to the rank of the layout   */
Boolean ArrayLayout::valid_distribution(int array_rank, Distribution* dist)
{
  if ((array_rank <= 0) || (dist == NULL)) return NO;
  else {
         int block_or_cyclic=0, i;
         for (i=0;i<array_rank;i++)
		if ((dist[i]==BLOCK)||(dist[i]==CYCLIC)) block_or_cyclic++;
         if (block_or_cyclic != rank_)
                return NO;
         else 
		return YES;
       }
}


int ArrayLayout::size(int i){return size_[i];}
       



/* This function is used to return a linked list of numbers (representing the
 * indices of the compute node chunks which overlap with the io node chunk)
 * given the base, size of the overlapping layout (??).
 *
 * The function assumes that the input is valid
 */
void  ArrayLayout::indices_list(int *index_base, int *index_size, 
				int *num, int *ret_list) 
{
   int *ptr=ret_list;
   int size=1;
   for(int i=0; i < rank(); i++)
	size *= index_size[i];
   *num = size;
   calculate_indices(index_base, index_size, rank(), 0, &ptr);
}

/* Recursive function to convert a layout into a list of numbers */
void ArrayLayout::calculate_indices(int *index_base, int *index_size, 
					int my_rank, int sum, int **buf_ptr)
{
  int prod=1, i;
  int *ptr;

#ifdef DEBUG
  printf("In calculate indices rank=%d sum=%d *buf=%ld\n", my_rank, sum, *buf_ptr);
#endif
  if (my_rank > 1)
  {
     for(i=rank()-1; i > (rank() - my_rank) ; i--)
	prod *= size_[i];
     for(i=0 ; i < index_size[rank()-my_rank]; i++)
	calculate_indices(index_base, index_size, my_rank-1,
		sum + (index_base[rank()-my_rank]+i)*prod, buf_ptr);
  }
  else
  {
    for(i=0; i < index_size[rank()-my_rank]; i++)
    {
        ptr = *buf_ptr;
	*ptr++ = sum + (index_base[rank()-my_rank]+i);
#ifdef DEBUG
        printf("In calculate indices *buf=%ld val=%d\n", *buf_ptr, **buf_ptr);
#endif
	*buf_ptr = ptr;
    }
  }
}

/* Pack the info into the schema buffer */
void ArrayLayout::pack(int **schema_buf)
{
  int* ptr = *schema_buf;
   
  *ptr++ = rank_;
  for(int i=0; i< rank_; i++)
     *ptr++ = size_[i];

  *schema_buf = ptr;
}

/* Check if the two layouts are equal */
Boolean ArrayLayout::equal(ArrayLayout *layout)
{
  if (rank_ != layout->rank()) return NO;
  for(int i=0; i<rank_; i++)
	if (size_[i] != layout->size(i)) return NO;
  return YES;
}



int* ArrayLayout::size(){return size_;}



