#include "definitions.h"
#include "Panda.h"


/* Currently the sequential case is unsupported */

extern MPIFS *MPIFS_global_obj;
int  global_system_type_;

Panda::Panda()
{
  file_system_type_ = UNIX_SYSTEM;
  file_system_ = NULL;
}

/* This is the interface for regular Panda (i.e no part-time io nodes) */
Panda::Panda(int node_type, int app_num , int relative_rank,int app_size,
		int* world_ranks)
{
  global_system_type_ = file_system_type_  = MPI_SYSTEM;
  file_system_ = new MPIFS(node_type, app_num, relative_rank, app_size,
			world_ranks);
}

/* This is the interface for regular Panda (i.e no part-time io nodes) */
Panda::Panda(int node_type, int app_num , int relative_rank,int app_size,
		int* world_ranks, Boolean shared_flag)
{
  global_system_type_ = file_system_type_  = MPI_SYSTEM;
  file_system_ = new MPIFS(node_type, app_num, relative_rank, app_size,
			world_ranks, shared_flag);
}

/* This is the interface for part-time io nodes */
Panda::Panda(int node_type, int comp_rank, int comp_size, int *comp_world_ranks,
	int io_rank, int io_size, int *io_world_ranks)
{
  global_system_type_ = file_system_type_  = MPI_SYSTEM;
  file_system_ = new MPIFS(node_type, comp_rank, comp_size, comp_world_ranks,
		io_rank, io_size, io_world_ranks);
}

/* This is the simplest interface for initializing the I/O and compute nodes.
 * It can be used only for regular and shread i/o (i.e it cannot be used for
 * part-time i/o nodes. The constructor assumes that MPIRUN library has been 
 * installed and you have distinct applications at the mpirun level 
 */

/*
Panda::Panda(int node_type)
{
  int app_size, app_rank, *world_ranks, leader;

  file_system_type_ = MPI_SYSTEM;
  if ((node_type == COMPUTE_NODE) || (node_type == IO_NODE)){
    MPI_Comm_size(MPIRUN_APP_COMM, &app_size);
    MPI_Comm_rank(MPIRUN_APP_COMM, &app_rank);
    leader = MPIRUN_APP_LEADERS[MPIRUN_APP_ID];
    world_ranks = (int *) malloc(sizeof(int)*app_size);
    for(int i=0; i<app_size;i++)
      world_ranks[i] = leader + i;
    file_system_ = new MPIFS(node_type, MPIRUN_APP_ID, app_rank, app_size,
			     world_ranks);
    free(world_ranks);
    world_ranks = NULL;
  } else {
    printf("Error: Invalid constructor for this node_type %d\n", node_type);
    exit(1);
  }
}
*/

/* This is the simplest interface for the part-time i/o nodes. Here the number
 * of i/o nodes is specified. The first <num_io_nodes> are designated as part
 * time i/o nodes and the remaining as part-time compute. This requires mpirun
 * library to be initialized and there should be only one mpirun application.
 */
/*
Panda::Panda(int node_type, int num_io_nodes)
{
  int app_rank, app_size, *io_ranks, *world_ranks;

  file_system_type_ = MPI_SYSTEM;
  if ((node_type == COMPUTE_NODE) || (node_type == IO_NODE)){
    printf("Error: Invalid constructor\n");
    exit(1);
  } else if (MPIRUN_NUM_APPS == 1){
    MPI_Comm_size(MPIRUN_APP_COMM, &app_size);
    MPI_Comm_rank(MPIRUN_APP_COMM, &app_rank);
    world_ranks = (int *)malloc(sizeof(int)*app_size);
    io_ranks = (int *) malloc(sizeof(int)*num_io_nodes);
    for(int i=0;i<app_size; i++) world_ranks[i] = i;
    for(i=0;i<num_io_nodes;i++) io_ranks[i] =i;
    if (app_rank < num_io_nodes){
      file_system_ = new MPIFS(PART_TIME_IO, app_rank, app_size, world_ranks,
			       app_rank, num_io_nodes, io_ranks);
    } else {
      file_system_ = new MPIFS(PART_TIME_COMPUTE, app_rank, app_size, world_ranks,
			       -1, num_io_nodes, io_ranks);
    }
  } else {
    printf("Error: Part-time I/O nodes - More than one mpirun app running\n");
    exit(1);
  }
}
*/
  
Panda::~Panda()
{
  if (file_system_) delete file_system_;
  file_system_ = NULL;

}

void Panda::global_barrier()
{
  if (file_system_type_ == MPI_SYSTEM)
  {
     ((MPIFS *) file_system_)->user_commands(GLOBAL_BARRIER);
  }
}

void Panda::app_barrier()
{
  if (file_system_type_ == MPI_SYSTEM)
  {
     ((MPIFS *) file_system_)->user_commands(APP_BARRIER);
  }
}

void Panda::flushfiles()
{
  if (file_system_type_ == MPI_SYSTEM)
  {
     ((MPIFS *) file_system_)->user_commands(FLUSHFILES);
  }
}

void Panda::cleanfiles()
{
  if (file_system_type_ == MPI_SYSTEM)
  {
     ((MPIFS *) file_system_)->user_commands(CLEANFILES);
  }
}

void Panda::createfiles()
{
  if (file_system_type_ == MPI_SYSTEM)
  {
     ((MPIFS *) file_system_)->user_commands(CREATEFILES);
  }
}
