/*@@
@file      Whisky_EOSResetHydro.F90
@date      Sat Jan 26 01:36:57 2002
@author     Ian Hawke
@desc 
   This routine will reset the specific internal energy using the polytropic
   EOS that will be used at evolution. This is necessary if the EoS changes
   between setting up the initial data and evolving
  @enddesc 
  @@*/
  
#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"
 
 /*@@
   @routine    Whisky_EOSResetHydro
   @date       Sat Jan 26 01:38:12 2002
   @author     Ian Hawke
   @desc 
   see above
   @enddesc 
   @calls     
   @calledby   
   @history 
   
   @endhistory 

@@*/


subroutine Whisky_EoSChangeGamma(CCTK_ARGUMENTS)

  USE EOS_Polytrope_Scalars

  implicit none

  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS

  CCTK_INT :: i, j, k
  CCTK_REAL :: det
  
  CCTK_REAL :: local_gamma
  
#include "CactusEOS/EOS_Base/src/EOS_Base.inc"

!!$  Set up the fluid constants

  local_Gamma = 1.d0 + EOS_Pressure(whisky_polytrope_handle, 1.d0, 1.d0) / &
       EOS_SpecificIntEnergy(whisky_polytrope_handle, 1.d0, 1.d0)

!!$  Change the pressure and specific internal energy

  press = p_geom_factor * eos_k_cgs * &
       (rho * rho_geom_factor_inv)**local_Gamma 
  eps = press / (rho * (local_Gamma - 1.d0))

!!$  Get the conserved variables. Hardwired polytrope EoS!!!
!!$  Note that this call also sets pressure and eps
    
  do k = 1, cctk_lsh(3)
    do j = 1, cctk_lsh(2)
      do i = 1, cctk_lsh(1)
        
        call SpatialDeterminant(gxx(i,j,k),gxy(i,j,k),gxz(i,j,k),&
             gyy(i,j,k),gyz(i,j,k),gzz(i,j,k),det)
        call prim2conpolytype(whisky_polytrope_handle,gxx(i,j,k),gxy(i,j,k),&
             gxz(i,j,k),gyy(i,j,k),gyz(i,j,k),gzz(i,j,k),&
             det, dens(i,j,k),sx(i,j,k),sy(i,j,k),sz(i,j,k),&
             tau(i,j,k),rho(i,j,k),velx(i,j,k),vely(i,j,k),velz(i,j,k),&
             eps(i,j,k),press(i,j,k),w_lorentz(i,j,k))

      end do
    end do
  end do


end subroutine Whisky_EoSChangeGamma
