 /*@@
   @file      DC_ApplyCorrection.F90
   @date      Mon May 17 07:05:55 2004
   @author    Ian Hawke
   @desc 
   Apply the correction term.
   @enddesc 
 @@*/


#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"

 /*@@
   @routine    DC_ApplyCorrection
   @date       Sun May 23 21:36:28 2004
   @author     Ian Hawke
   @desc 
   Apply the correction term.
   @enddesc 
   @calls     
   @calledby   
   @history 
 
   @endhistory 

@@*/

subroutine DC_ApplyCorrection(CCTK_ARGUMENTS)

  implicit none

  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS

  CCTK_REAL :: correction, rho, radial_factor, xpos, ypos
  CCTK_INT :: i, j, k
  character(len=100) :: info_message

  if (dc_iteration > 0) then

    correction = dc_p_factor * (dc_error + &
         dc_i_factor * dc_int_error + &
         dc_d_factor * dc_deriv_error)

    if (verbose .ne. 0) then

      write(info_message,'(a19,f10.5)') 'Correction factor ', &
           correction
      call CCTK_INFO (trim(info_message))

    end if
    
    do k = 1, cctk_lsh(3)
      do j = 1, cctk_lsh(2)
        do i = 1, cctk_lsh(1)

          xpos = CCTK_ORIGIN_SPACE(1) + &
               CCTK_DELTA_SPACE(1) * (cctk_lbnd(1) + i - 1)
          ypos = CCTK_ORIGIN_SPACE(2) + &
               CCTK_DELTA_SPACE(2) * (cctk_lbnd(2) + j - 1)

          rho = sqrt(xpos**2 + ypos**2)

          radial_factor = 0.5d0 * &
               ( 1.d0 - tanh((rho - dc_shave_r0) / dc_shave_width) )

          ADM_BS_sbetax(i,j,k) = ADM_BS_sbetax(i,j,k) - &
               ypos * radial_factor * correction
          ADM_BS_sbetay(i,j,k) = ADM_BS_sbetay(i,j,k) + &
               xpos * radial_factor * correction 
!!$          ADM_BS_sBx(i,j,k) = ADM_BS_sBx(i,j,k) - &
!!$               ypos * radial_factor * correction
!!$          ADM_BS_sBy(i,j,k) = ADM_BS_sBy(i,j,k) + &
!!$               xpos * radial_factor * correction 

        end do
      end do
    end do
    
  end if

end subroutine DC_ApplyCorrection
