 /*@@
   @file      DC_CalculateCorrection.F90
   @date      Mon May 17 07:05:55 2004
   @author    Ian Hawke
   @desc 
   Work out the error terms from the centroid.
   @enddesc 
 @@*/


#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"

 /*@@
   @routine    DC_CalculateCorrection
   @date       Sun May 23 21:36:28 2004
   @author     Ian Hawke
   @desc 
   Work out the error terms from the centroid.
   @enddesc 
   @calls     
   @calledby   
   @history 
 
   @endhistory 

@@*/

subroutine DC_CalculateCorrection(CCTK_ARGUMENTS)

  implicit none

  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS
  character(len=100) :: info_message

  CCTK_REAL :: angle

  if (dc_flag .ne. 0) then
  
    angle = atan( dc_y / (abs(dc_x) + 1.d-10) )

    if (apply_time_deriv .eq. 0) then

      if (dc_iteration > 0) then
        dc_deriv_error = angle - dc_error
      end if
      
      dc_error = angle
      
      if (CCTK_TIME > dc_int_error_initial_time) then
        dc_int_error = dc_int_error + angle
      end if

    else

      if (dc_iteration > 1) then
        dc_deriv_error = angle - dc_int_error - dc_error
      end if
      
      if (dc_iteration > 0) then
        dc_error = angle - dc_int_error
      end if
      
      dc_int_error = angle

    end if
    
    dc_iteration = dc_iteration + 1    

    if (verbose .ne. 0) then

      write(info_message,'(a6,f10.5,a14,3f10.5)') 'Angle ', angle, &
           ' gives errors ', dc_error, dc_int_error, dc_deriv_error
      call CCTK_INFO (trim(info_message))

    end if

    dc_omega = dc_omega +  dc_p_factor * (dc_error + &
         dc_i_factor * dc_int_error + &
         dc_d_factor * dc_deriv_error)
    
  end if
  
  dc_flag = 0

end subroutine DC_CalculateCorrection
