 /*@@
   @file      DC_SetCentroid.c
   @date      Mon May 20 09:52:33 2002
   @author    Ian Hawke
   @desc 
   The aliased function that sets the location of the centre of the NS.
   @enddesc 
   @version   $Header: /EUNetwork/EUHydro/Whisky_DriftCorrect/src/DC_SetCentroid.c,v 1.1.1.1 2004/05/28 15:02:26 hawke Exp $
 @@*/

#include "cctk.h"
#include "cctk_Arguments.h"
#include "cctk_Parameters.h"

static const char *rcsid = "$Header: /EUNetwork/EUHydro/Whisky_DriftCorrect/src/DC_SetCentroid.c,v 1.1.1.1 2004/05/28 15:02:26 hawke Exp $";

CCTK_FILEVERSION(EUHydro_Whisky_DriftCorrect_DC_SetCentroid_c);

/********************************************************************
 *********************     Local Data Types   ***********************
 ********************************************************************/

/********************************************************************
 ********************* Local Routine Prototypes *********************
 ********************************************************************/

/********************************************************************
 ***************** Scheduled Routine Prototypes *********************
 ********************************************************************/

/********************************************************************
 ********************* Other Routine Prototypes *********************
 ********************************************************************/

void DC_SetCentroid(CCTK_POINTER_TO_CONST GH_Pointer, CCTK_REAL x,
                              CCTK_REAL y, CCTK_REAL z);

/********************************************************************
 *********************     Local Data   *****************************
 ********************************************************************/

/********************************************************************
 *********************     External Routines   **********************
 ********************************************************************/

 /*@@
   @routine    DC_SetCentroid
   @date       Mon May 20 09:54:39 2002
   @author     Ian Hawke
   @desc 
   The aliased function that sets the centroid locations.
   Copied pretty much verbatim from DriftCorrect.
   @enddesc 
   @calls     
   @calledby   
   @history 
 
   @endhistory 

@@*/

void DC_SetCentroid(CCTK_POINTER_TO_CONST GH_Pointer, CCTK_REAL x,
                    CCTK_REAL y, CCTK_REAL z)
{

  DECLARE_CCTK_PARAMETERS;
  
  const cGH* GH = (const cGH*) GH_Pointer;
  CCTK_REAL *dc_centroid_x, *dc_centroid_y, *dc_centroid_z;
  CCTK_INT *dc_flag;

  dc_centroid_x = (CCTK_REAL *) CCTK_VarDataPtr(GH, 0,
                  "Whisky_DriftCorrect::dc_x");
  dc_centroid_y = (CCTK_REAL *) CCTK_VarDataPtr(GH, 0,
                  "Whisky_DriftCorrect::dc_y");
  dc_centroid_z = (CCTK_REAL *) CCTK_VarDataPtr(GH, 0,
                  "Whisky_DriftCorrect::dc_z");
  dc_flag = (CCTK_INT *) CCTK_VarDataPtr(GH, 0,
            "Whisky_DriftCorrect::dc_flag");

  *dc_centroid_x = x;
  *dc_centroid_y = y;
  *dc_centroid_z = z;

  if (verbose) 
  {  
    CCTK_VInfo(CCTK_THORNSTRING,
               "Setting the centroid to (%g,%g,%g)",
               x,y,z);
  }
  
  (*dc_flag)++;
 
}

/********************************************************************
 *********************     Local Routines   *************************
 ********************************************************************/
