/*@@
@file       Whisky_Dust_Evolve.F90
@routine    Whisky_Dust_Evolve
@date       Wed Apr 9 16:13:46 2003
@author     Luca Baiotti
@desc 
   provides MoL with upwind update rhs's; to be used instead of Whisky's Riemann solvers.
@enddesc 
@calls     
@calledby  scheduler 
@history 

@endhistory 
@@*/

#include "cctk.h"
#include "cctk_Arguments.h"
#include "cctk_Parameters.h"

#include "cctk_Functions.h"

subroutine Whisky_Dust_Evolve(CCTK_ARGUMENTS)

  implicit none

  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS
  DECLARE_CCTK_FUNCTIONS

  CCTK_INT i,j,k
  CCTK_REAL det,w,vlowx,vlowy,vlowz,alp_l_x,alp_l_y,alp_l_z,alp_r_x,alp_r_y,alp_r_z 
  

!!$  !Convert to conservative variables NOT NEEDED during evolution
!!$
!!$  do k = whisky_stencil,cctk_lsh(3)-whisky_stencil+1
!!$    do j = whisky_stencil,cctk_lsh(2)-whisky_stencil+1
!!$      do i = whisky_stencil,cctk_lsh(1)-whisky_stencil+1
!!$
!!$        call SpatialDeterminant(gxx(i,j,k),gxy(i,j,k),gxz(i,j,k), &
!!$             gyy(i,j,k),gyz(i,j,k),gzz(i,j,k),det)
!!$        call prim2conpolytype(whisky_eos_handle,gxx(i,j,k),gxy(i,j,k), &
!!$             gxz(i,j,k),gyy(i,j,k),gyz(i,j,k),gzz(i,j,k), &
!!$             det, dens(i,j,k),sx(i,j,k),sy(i,j,k),sz(i,j,k), &
!!$             tau(i,j,k),rho(i,j,k),velx(i,j,k),vely(i,j,k),velz(i,j,k), &
!!$             eps(i,j,k),press(i,j,k),w_lorentz(i,j,k))
!!$        
!!$      end do
!!$    end do
!!$  end do



  do k = whisky_stencil + 1, cctk_lsh(3) - whisky_stencil
    do j = whisky_stencil + 1, cctk_lsh(2) - whisky_stencil
      do i = whisky_stencil + 1, cctk_lsh(1) - whisky_stencil


        densflux_x(i,j,k) = alp(i,j,k) * dens(i,j,k) * (velx(i,j,k) - betax(i,j,k) / alp(i,j,k))
        densflux_y(i,j,k) = alp(i,j,k) * dens(i,j,k) * (vely(i,j,k) - betay(i,j,k) / alp(i,j,k))
        densflux_z(i,j,k) = alp(i,j,k) * dens(i,j,k) * (velz(i,j,k) - betaz(i,j,k) / alp(i,j,k))
        sxflux_x(i,j,k)   = alp(i,j,k) *   sx(i,j,k) * (velx(i,j,k) - betax(i,j,k) / alp(i,j,k))
        sxflux_y(i,j,k)   = alp(i,j,k) *   sx(i,j,k) * (vely(i,j,k) - betay(i,j,k) / alp(i,j,k))
        sxflux_z(i,j,k)   = alp(i,j,k) *   sx(i,j,k) * (velz(i,j,k) - betaz(i,j,k) / alp(i,j,k))
        syflux_x(i,j,k)   = alp(i,j,k) *   sy(i,j,k) * (velx(i,j,k) - betax(i,j,k) / alp(i,j,k))
        syflux_y(i,j,k)   = alp(i,j,k) *   sy(i,j,k) * (vely(i,j,k) - betay(i,j,k) / alp(i,j,k))
        syflux_z(i,j,k)   = alp(i,j,k) *   sy(i,j,k) * (velz(i,j,k) - betaz(i,j,k) / alp(i,j,k))
        szflux_x(i,j,k)   = alp(i,j,k) *   sz(i,j,k) * (velx(i,j,k) - betax(i,j,k) / alp(i,j,k))
        szflux_y(i,j,k)   = alp(i,j,k) *   sz(i,j,k) * (vely(i,j,k) - betay(i,j,k) / alp(i,j,k))
        szflux_z(i,j,k)   = alp(i,j,k) *   sz(i,j,k) * (velz(i,j,k) - betaz(i,j,k) / alp(i,j,k))
        tauflux_x(i,j,k)  = alp(i,j,k) *  tau(i,j,k) * (velx(i,j,k) - betax(i,j,k) / alp(i,j,k))
        tauflux_y(i,j,k)  = alp(i,j,k) *  tau(i,j,k) * (vely(i,j,k) - betay(i,j,k) / alp(i,j,k))
        tauflux_z(i,j,k)  = alp(i,j,k) *  tau(i,j,k) * (velz(i,j,k) - betaz(i,j,k) / alp(i,j,k))


      end do
    end do
  end do

  
  !UPWIND

  do k = whisky_stencil + 1, cctk_lsh(3) - whisky_stencil
    do j = whisky_stencil + 1, cctk_lsh(2) - whisky_stencil
      do i = whisky_stencil + 1, cctk_lsh(1) - whisky_stencil

        densrhs(i,j,k) = densrhs(i,j,k) - ( &
             (densflux_x(i+1,j,k) - densflux_x(i,j,k)) / CCTK_DELTA_SPACE(1) + &
             (densflux_y(i,j+1,k) - densflux_y(i,j,k)) / CCTK_DELTA_SPACE(2) + &
             (densflux_z(i,j,k+1) - densflux_z(i,j,k)) / CCTK_DELTA_SPACE(3)   )

        sxrhs(i,j,k) = sxrhs(i,j,k) - (&
             (sxflux_x(i+1,j,k) - sxflux_x(i,j,k)) / CCTK_DELTA_SPACE(1) + &
             (sxflux_y(i,j+1,k) - sxflux_y(i,j,k)) / CCTK_DELTA_SPACE(2) + &
             (sxflux_z(i,j,k+1) - sxflux_z(i,j,k)) / CCTK_DELTA_SPACE(3))

        syrhs(i,j,k) = syrhs(i,j,k) - (&
             (syflux_x(i+1,j,k) - syflux_x(i,j,k)) / CCTK_DELTA_SPACE(1) + &
             (syflux_y(i,j+1,k) - syflux_y(i,j,k)) / CCTK_DELTA_SPACE(2) + &
             (syflux_z(i,j,k+1) - syflux_z(i,j,k)) / CCTK_DELTA_SPACE(3))

        szrhs(i,j,k) = szrhs(i,j,k) - (&
             (szflux_x(i+1,j,k) - szflux_x(i,j,k)) / CCTK_DELTA_SPACE(1) + &
             (szflux_y(i,j+1,k) - szflux_y(i,j,k)) / CCTK_DELTA_SPACE(2) + &
             (szflux_z(i,j,k+1) - szflux_z(i,j,k)) / CCTK_DELTA_SPACE(3))

        taurhs(i,j,k) = taurhs(i,j,k) - (&
             (tauflux_x(i+1,j,k) - tauflux_x(i,j,k)) / CCTK_DELTA_SPACE(1) + &
             (tauflux_y(i,j+1,k) - tauflux_y(i,j,k)) / CCTK_DELTA_SPACE(2) + &
             (tauflux_z(i,j,k+1) - tauflux_z(i,j,k)) / CCTK_DELTA_SPACE(3))


      end do
    end do
  end do

  return
  
end subroutine Whisky_Dust_Evolve
