 /*@@
   @file      Meudon.C.F90
   @date      Wed Sep 18 10:08:54 2002
   @author    Luca Baiotti
   @desc 
   Reads the Meudon data files for binary NS. Adapted from M. Koppitz BH reader.
   @enddesc 
 @@*/

#include <bin_ns.h>

#include <algorithm>

#include "cctk.h"
#include "cctk_Arguments.h"
#include "cctk_Parameters.h"

#ifdef __cplusplus

extern "C" {

#endif

  void SetupMeudonNS(CCTK_ARGUMENTS);

#ifdef __cplusplus

}

#endif

  void SetupMeudonNS(CCTK_ARGUMENTS)
  {
    DECLARE_CCTK_ARGUMENTS;
    DECLARE_CCTK_PARAMETERS;
    
    int local_grid_size = cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2];
    int i,j,k,i3D;
    int error=0;
    char name[100];
    FILE *fp;

    // check ID filename and path
    if ((fp=fopen(ID_file,"r"))==NULL){
      fprintf(stderr,"can't open %s \n",ID_file);
      exit(1);
    }
    else{
      printf("Reading Meudon data file:\n %s \n",ID_file); 
      fclose(fp);
    }
    
    // convert grid units from Cactus to Lorene (Km)
    for(i=0; i<cctk_lsh[0]; i++)
      for(j=0; j<cctk_lsh[1]; j++)
	for(k=0; k<cctk_lsh[2]; k++)
	  {
	    i3D=CCTK_GFINDEX3D(cctkGH, i, j, k);
	    
	    x[i3D] = x[i3D]*1.47664;
	    y[i3D] = y[i3D]*1.47664;
	    z[i3D] = z[i3D]*1.47664;
	  }
  
    // call the Lorene routine to transform spectral data to cartesian
    Bin_NS binary_system(local_grid_size, x, y, z, ID_file);

    // convert back to Cactus units
    for(i=0; i<cctk_lsh[0]; i++)
      for(j=0; j<cctk_lsh[1]; j++)
	for(k=0; k<cctk_lsh[2]; k++)
	  {
	    i3D=CCTK_GFINDEX3D(cctkGH, i, j, k);
	    
	    x[i3D] = x[i3D]/1.47664;
	    y[i3D] = y[i3D]/1.47664;
	    z[i3D] = z[i3D]/1.47664;
	  }

    CCTK_INFO("Done reading data file");

    const CCTK_REAL rho_atmosphere = whisky_rho_central * 
      rho_rel_min * initial_atmosphere_factor;

    // Units. See EOS_Polytrope.
    const CCTK_REAL m_solar_cgs = 1.987e33;
    const CCTK_REAL c_cgs = 2.9979e10;
    const CCTK_REAL G_cgs = 6.6732e-8;
    const CCTK_REAL m_solar_geom = G_cgs / c_cgs / c_cgs * m_solar_cgs;

    // Note that to get the correct EOS take the K that Lorene outputs
    // to screen and divide by 2.68727e-04.
    // NOTE THAT THIS ONLY WORKS FOR A Gamma = 2 POLYTROPE!

    for (int i=0; i < local_grid_size; i++)
    {
      gxx[i] = binary_system.g_xx[i];
      gxy[i] = binary_system.g_xy[i];
      gxz[i] = binary_system.g_xz[i];
      gyy[i] = binary_system.g_yy[i];
      gyz[i] = binary_system.g_yz[i];
      gzz[i] = binary_system.g_zz[i];
      betax[i] = binary_system.beta_x[i];
      betay[i] = binary_system.beta_y[i];
      betaz[i] = binary_system.beta_z[i];
      kxx[i] = binary_system.k_xx[i];
      kxy[i] = binary_system.k_xy[i];
      kxz[i] = binary_system.k_xz[i];
      kyy[i] = binary_system.k_yy[i];
      kyz[i] = binary_system.k_yz[i];
      kzz[i] = binary_system.k_zz[i];
      alp[i] = binary_system.nnn[i];
      rho[i] = binary_system.nbar[i];
      eps[i] = binary_system.ener_spec[i];
      velx[i] = binary_system.u_euler_x[i];
      vely[i] = binary_system.u_euler_y[i];
      velz[i] = binary_system.u_euler_z[i];

      // Unit conversion

      rho[i] *= 1.618996722e-21;

      rho[i] = max(rho[i], rho_atmosphere);

      kxx[i] *= 1.47664;
      kxy[i] *= 1.47664; 
      kxz[i] *= 1.47664; 
      kyy[i] *= 1.47664; 
      kyz[i] *= 1.47664; 
      kzz[i] *= 1.47664; 


      CCTK_REAL det;

      CCTK_INT handle = 0;
      
      SpatialDet(gxx[i],gxy[i],gxz[i],
		 gyy[i],gyz[i],gzz[i],&det);
      Prim2ConPoly(handle,gxx[i],gxy[i],gxz[i],
		   gyy[i],gyz[i],gzz[i],det,
		   dens+i,sx+i,sy+i,sz+i,tau+i,
		   rho[i],velx[i],vely[i],velz[i],
		   eps[i],press[i],w_lorentz[i]);

      alp_p[i] = alp[i];
      betax_p[i] = betax[i];
      betay_p[i] = betay[i];
      betaz_p[i] = betaz[i];

      gxx_p[i] = gxx[i];
      gxy_p[i] = gxy[i];
      gxz_p[i] = gxz[i];
      gyy_p[i] = gyy[i];
      gyz_p[i] = gyz[i];
      gzz_p[i] = gzz[i];
      kxx_p[i] = kxx[i];
      kxy_p[i] = kxy[i];
      kxz_p[i] = kxz[i];
      kyy_p[i] = kyy[i];
      kyz_p[i] = kyz[i];
      kzz_p[i] = kzz[i];

      rho_p[i] = rho[i];
      velx_p[i] = velx[i];
      vely_p[i] = vely[i];
      velz_p[i] = velz[i];
      press_p[i] = press[i];
      w_lorentz_p[i] = w_lorentz[i];
      eps_p[i] = eps[i];

      dens_p[i] = dens[i];
      sx_p[i] = sx[i];
      sy_p[i] = sy[i];
      sz_p[i] = sz[i];
      tau_p[i] = tau[i];

      alp_p_p[i] = alp[i];
      betax_p_p[i] = betax[i];
      betay_p_p[i] = betay[i];
      betaz_p_p[i] = betaz[i];

      gxx_p_p[i] = gxx[i];
      gxy_p_p[i] = gxy[i];
      gxz_p_p[i] = gxz[i];
      gyy_p_p[i] = gyy[i];
      gyz_p_p[i] = gyz[i];
      gzz_p_p[i] = gzz[i];
      kxx_p_p[i] = kxx[i];
      kxy_p_p[i] = kxy[i];
      kxz_p_p[i] = kxz[i];
      kyy_p_p[i] = kyy[i];
      kyz_p_p[i] = kyz[i];
      kzz_p_p[i] = kzz[i];

      rho_p_p[i] = rho[i];
      velx_p_p[i] = velx[i];
      vely_p_p[i] = vely[i];
      velz_p_p[i] = velz[i];
      press_p_p[i] = press[i];
      w_lorentz_p_p[i] = w_lorentz[i];
      eps_p_p[i] = eps[i];

      dens_p_p[i] = dens[i];
      sx_p_p[i] = sx[i];
      sy_p_p[i] = sy[i];
      sz_p_p[i] = sz[i];
      tau_p_p[i] = tau[i];      
      
      *shift_state = 1;

    }
    
    return;
  }
