 /*@@
   @file      Whisky_Steer.c
   @date      Wed Apr 23 12:54:05 2003
   @author    Ian Hawke
   @desc 
   Routines to steer parameters.
   @enddesc 
 @@*/

#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"

 /*@@
   @routine    Whisky_SteerFunction
   @date       Wed Apr 23 12:54:30 2003
   @author     Ian Hawke
   @desc 
   A routine that steers the checkpointing parameters based on
   the existence of a horizon.
   @enddesc 
   @calls     
   @calledby   
   @history 
 
   @endhistory 

@@*/

/* 
   This external function is defined in thorn HTTPD.
*/

int HTTP_SteerQueue(const char *thorn, const char *parameter, const char *value);

void Whisky_SteerFunction(CCTK_ARGUMENTS);

void Whisky_SteerFunction(CCTK_ARGUMENTS)
{

  DECLARE_CCTK_ARGUMENTS;
  DECLARE_CCTK_PARAMETERS;

  CCTK_INT *checkpoint_next;
  CCTK_INT ahf_findevery, ahf_this_iteration;

  CCTK_REAL AH_mass;

  if (CCTK_MyProc(cctkGH) == 0)
  {

    /* Very lazy... */

    if (cctk_iteration == 0)
    {
      (*old_AH_mass) = 0;
    }
    
    /* There is no obvious reason for doing this other than
       checking that the thorn is active. */
    
    if (CCTK_IsThornActive("IOHDF5"))
    {
      checkpoint_next = (CCTK_INT*) CCTK_ParameterGet("checkpoint_next", 
                                                     "IOHDF5", NULL);
    }
    else
    {
      CCTK_WARN(0, "Must have thorn IOHDF5 active");
    }
    
    if (CCTK_IsThornActive("AHFinder"))
    {
      AH_mass = *((CCTK_REAL*) CCTK_VarDataPtr(cctkGH, 0, 
                                               "AHFinder::out_mass"));
      ahf_findevery = *((CCTK_INT*) CCTK_ParameterGet("ahf_findevery",
                                                      "AHFinder", NULL));
    }
    else
    {
      CCTK_WARN(0, "Must have thorn AHFinder active");
    }
    
    /* Do we have an event horizon? */
    
    if (AH_mass > 1.e-10)
    {

      /* Did we calculate a new horizon on the last step? */
      
      if ((cctk_iteration % ahf_findevery) == 1)
      {

        CCTK_INFO("Forcing a checkpoint");
        /*
        CCTK_ParameterSet("checkpoint_next", "IOHDF5", "yes");
        */

        HTTP_SteerQueue("IOHDF5", "checkpoint_next", "yes");

        /* Was the change in the mass small enough that we 
           should stop? */
        
        if (fabs(AH_mass - (*old_AH_mass)) / AH_mass < 1.e-2)
        {
          CCTK_INFO("Forcing a code termination!!!");
          /*
          CCTK_ParameterSet("terminate_next", "Cactus", "yes");
          */

          HTTP_SteerQueue("Cactus", "terminate_next", "yes");

        }
        else
        {
          (*old_AH_mass) = AH_mass;
        }
        
      }    
      
    }
    else
    {
      /*
      CCTK_ParameterSet("checkpoint_next", "IOHDF5", "no");
      */

      HTTP_SteerQueue("IOHDF5", "checkpoint_next", "no");
    }
    
  }
  
  return;
  
}
